using System;
using System.Text;
using System.IO;
using Assetlink;
using Assetlink.IOProxy;


namespace Assetlink.FileAccessLayer
{
    public class WindowsFileSystem : IFileSystem
    {

        public String MediaServerUrl
        {
            set{}
        }

        //----------------------- File Operations---------------------------------
        /// <summary>
        /// <CreatedDate> 02-January-2006 </CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Creates a file on the Windows File System.
        /// </Purpose>
        /// </summary>
        /// <param name="SourcePath">Source path from where file has to be taken</param>
        /// <param name="DestinationPath">Location and file name where file has to be created.</param>
        /// <returns>An object of Type OMError</returns>
        public OMError CreateFile(String SourcePath, String DestinationPath)
        {
            OMError ErrObj = null;
            
            try
            {
                if (File.Exists(DestinationPath) == true)
                {
                    File.Delete(DestinationPath);
                    File.Copy(SourcePath, DestinationPath);
                }
                else
                    File.Copy(SourcePath, DestinationPath);
            }
            catch (Exception ExcepObj)
            {
                ErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILECREATION_FAILED,
                                                "File Creation Failed",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system",OMErrorType.EXCEPTION,
                                                ExcepObj, null);
                return ErrObj;
            }
            return null;
        }

        /// <summary>
        /// <CreatedDate> 02-January-2006 </CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Moves a file on the Windows File System.
        /// </Purpose>
        /// </summary>
        /// <param name="SourcePath">Source path from where file has to be taken</param>
        /// <param name="DestinationPath">Location and file name where file has to be moved.</param>
        /// <returns>An object of Type OMError</returns>
        public OMError MoveFile(String SourcePath, String DestinationPath)
        {
            OMError ErrObj = null;

            try
            {
                if (!File.Exists(SourcePath))
                {
                    ErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                "File Move Failed. Source File: " + SourcePath + " not found.",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.ERROR,
                                                null, null);
                    return ErrObj;
                }
                File.Move(SourcePath, DestinationPath);
            }
            catch (Exception ExcepObj)
            {
                ErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEMOVE_FAILED,
                                                "File Move Failed",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.EXCEPTION,
                                                ExcepObj, null);
                return ErrObj;
            }
            return null;
        }

        /// <summary>
        /// <CreatedDate> 02-January-2006 </CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Deletes a file from the Windows File System.
        /// </Purpose>
        /// </summary>
        /// <param name="FilePath">Source path from where file has to be deleted</param>
        /// <returns>An object of Type OMError</returns>
        public OMError DeleteFile(String FilePath)
        {
            OMError ErrObj = null;

            try
            {
                if (!File.Exists(FilePath))
                {
                    ErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                "File Delete Failed. Source File: " + FilePath + " not found.",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.ERROR,
                                                null, null);
                    return ErrObj;
                }
                //If File has Read-only attributes, remove those attributes
                FileInfo fileInfo        = new FileInfo(FilePath);

                if(fileInfo.IsReadOnly)
                    fileInfo.Attributes = FileAttributes.Normal;
                fileInfo = null;

                File.Delete(FilePath);
            }
            catch (Exception ExcepObj)
            {
                ErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                "File Delete Failed",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.EXCEPTION,
                                                ExcepObj, null);
                return ErrObj;
            }
          
            return null;
        }

        /// <summary>
        /// <CreatedDate> 02-January-2006 </CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Copies a file from the Windows File System.
        /// </Purpose>
        /// </summary>
        /// <param name="SourcePath">Source path from where file has to be taken</param>
        /// <param name="DestinationPath">Location and file name where file has to be copied</param>
        /// <returns>An object of Type OMError</returns>
        public OMError CopyFile(String SourcePath, String DestinationPath)
        {
            OMError ErrObj = null;

            try
            {
                if (!File.Exists(SourcePath))
                {
                    ErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEDELETE_FAILED,
                                                "File Copy Failed. Source File: " + SourcePath + " not found.",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.ERROR,
                                                null, null);
                    return ErrObj;
                }
                File.Copy(SourcePath, DestinationPath,true);
            }
            catch (Exception ExcepObj)
            {
                ErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILECOPY_FAILED,
                                                "File Copy Failed",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.EXCEPTION,
                                                ExcepObj, null);
                return ErrObj;
            }
            return null;
        }

        /// <summary>
        /// <CreatedDate> 05-Sep-2006 </CreatedDate>
        /// <Author>Mr. Yuvraj Deshmukh</Author>
        /// <Purpose>
        /// To Check if file exists on NTFS
        /// </Purpose>
        /// </summary>
        /// <param name="FilePath">NTFS Path of the file, Which need to be check</param>
        /// <returns>It will return true if file is present else returns false</returns>
        public Boolean IsFileExists(String FilePath)
        {
            return File.Exists(FilePath);
        }


        /// <summary>
        /// <CreatedDate>25-August-2008 </CreatedDate>
        /// <Author>Mr. Yuvraj Deshmukh</Author>
        /// <Purpose>
        /// To get the Size of File contents.
        /// </Purpose>
        /// </summary>
        /// <param name="FilePath">NTFS Path of the file, of which size has to retrieved </param>
        /// <param name="FileSize">returns the File Size </param>
        /// <returns>on Success returns Null else returns  </returns>
        public OMError GetFileSize(String FilePath, out ulong FileSize)
        {
            OMError ErrObj  = null;
            FileSize        = 0;

            try
            {
                FileInfo fileInfoObj = new FileInfo(FilePath);
                FileSize = (ulong)fileInfoObj.Length;
            }
            catch (Exception ExcepObj)
            {
                ErrObj = new OMError(OMErrorCodes.FILESYSTEM_FILEGETSIZE_FAILED,
                                                "File GetSize Failed",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.EXCEPTION,
                                                ExcepObj, null);
                return ErrObj;
            }
            return null;
        }

        /// <summary>
        /// <CreatedDate> 02-January-2006 </CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Retrieves a file from the Windows File System.
        /// </Purpose>
        /// </summary>
        /// <param name="FilePath">Source path from where file has to be taken</param>
        /// <returns>An object of Type OMError</returns>
        public OMError RetrieveFile(String FilePath)
        {
          
            return null;
        }

        //----------------------- Folder Operations---------------------------------

        /// <summary>
        /// <CreatedDate> 02-January-2006 </CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Create folder on the Windows File System. It does take care of folder hierarchical creation.
        /// </Purpose>
        /// </summary>
        /// <param name="FolderPath">The path on file system where folder has to be created.</param>
        /// <returns>An object of Type OMError</returns>
        public OMError CreateFolder(String FolderPath)
        {
            OMError ErrObj = null;

            try
            {
                Directory.CreateDirectory(FolderPath);
            }
            catch (Exception ExcepObj)
            {
                ErrObj = new OMError(OMErrorCodes.FILESYSTEM_DIRCREATION_FAILED,
                                                "Directory Creation Failed",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.EXCEPTION,
                                                ExcepObj, null);
                return ErrObj;
            }
            return null;
        }

        /// <summary>
        /// <CreatedDate> 02-January-2006 </CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Create folder on the Windows File System. It does take care of folder hierarchical creation.
        /// </Purpose>
        /// </summary>
        /// <param name="SourcePath">The path on file system where folder has to be created.</param>
        /// <param name="DestinationPath">The path on file system where folder has to be created.</param>
        /// <returns>An object of Type OMError</returns>
        public OMError MoveFolder(String SourcePath, String DestinationPath)
        {
            OMError ErrObj = null;

            try
            {
                Directory.Move(SourcePath, DestinationPath);
            }
            catch (Exception ExcepObj)
            {
                ErrObj = new OMError(OMErrorCodes.FILESYSTEM_DIRMOVE_FAILED,
                                                "Directory Move Failed",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.EXCEPTION,
                                                ExcepObj, null);
                return ErrObj;
            }
            return null;
        }

        /// <summary>
        /// <CreatedDate> 02-January-2006 </CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Deletes folder on the Windows File System. It does take care of folder hierarchical deletion.
        /// </Purpose>
        /// </summary>
        /// <param name="FolderPath">The path on file system where folder has to be deleted.</param>
        /// <returns>An object of Type OMError</returns>
        public OMError DeleteFolder(String FolderPath)
        {
            OMError ErrObj = null;

            try
            {
                Context ObjContext = new Context(0, 0, String.Empty,String.Empty, "Assetlink.FileAccessLayer.WindowsFileSystem.DeleteFolder(FolderPath)");
                if (!SASMOMDirectory.DeleteFolder(FolderPath, ObjContext))
                {
                    ErrObj = new OMError(OMErrorCodes.FILESYSTEM_DIRPATH_IN_AVOID_DELETE_LIST,
                                                "Directory path is in Avoid delete list",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.WARNING,
                                                null, null);
                    return ErrObj;
                }
            }
            catch (Exception ExcepObj)
            {
                ErrObj = new OMError(OMErrorCodes.FILESYSTEM_DIRDELETE_FAILED,
                                                "Directory Delete Failed",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.EXCEPTION,
                                                ExcepObj, null);
                return ErrObj;
            }
            return null;
        }


        /// <summary>
        /// <CreatedDate> 25-August-2008  </CreatedDate>
        /// <Author>Mr. Yuvraj Deshmukh</Author>
        /// <Purpose>
        /// Deletes folder recursively on the Windows File System.
        /// </Purpose>
        /// </summary>
        /// <param name="FolderPath">The path on file system where folder has to be deleted.</param>
        /// <returns>An object of Type OMError</returns>
        public OMError DeleteFolder(String FolderPath, Boolean Recursive)
        {
            OMError ErrObj = null;

            try
            {
                Context ObjContext = new Context(0, 0, String.Empty,String.Empty, "Assetlink.FileAccessLayer.WindowsFileSystem.DeleteFolder(FolderPath,Recursive)");
                if (!SASMOMDirectory.DeleteFolder(FolderPath, Recursive, ObjContext))
                {
                    ErrObj = new OMError(OMErrorCodes.FILESYSTEM_DIRPATH_IN_AVOID_DELETE_LIST,
                                                "Directory path in Avoid delete list",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.WARNING,
                                                null, null);
                    return ErrObj;
                }
            }
            catch (Exception ExcepObj)
            {
                ErrObj = new OMError(OMErrorCodes.FILESYSTEM_DIRDELETE_FAILED,
                                                "Directory Delete Failed",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.EXCEPTION,
                                                ExcepObj, null);
                return ErrObj;
            }
            return null;
        }

        /// <summary>
        /// <CreatedDate> 02-January-2006 </CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Copies folder on the Windows File System. It does take care of folder hierarchical deletion.
        /// </Purpose>
        /// </summary>
        /// <param name="SourcePath">The path on file system from where folder has to be copied.</param>
        /// <param name="DestinationPath">The path on file system where folder has to be copied.</param>
        /// <returns>An object of Type OMError</returns>
        public OMError CopyFolder(String SourcePath, String DestinationPath)
        {
            OMError         ErrObj          = null;
            DirectoryInfo   SourceDirObj    = null;
            DirectoryInfo[] DirList         = null;
            FileInfo[]      FileList        = null;
            try
            {
                SourceDirObj = new DirectoryInfo(SourcePath);

                if (Directory.Exists(DestinationPath) != true)
                    Directory.CreateDirectory(DestinationPath);

                FileList = SourceDirObj.GetFiles();

                foreach (FileInfo FileObj in FileList)
                {
                    File.Copy(FileObj.FullName , DestinationPath + "\\" + FileObj.Name);
                }

                DirList = SourceDirObj.GetDirectories();

                foreach (DirectoryInfo DirObj in DirList)
                {
                    CopyFolder(DirObj.FullName , DestinationPath + "\\" + DirObj.Name);
                }


            }
            catch (Exception ExcepObj)
            {
                ErrObj = new OMError(OMErrorCodes.FILESYSTEM_DIRCOPY_FAILED,
                                                "Directory Copy Failed",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.EXCEPTION,
                                                ExcepObj, null);
                return ErrObj;
            }

            finally
            {
                SourceDirObj = null;
            }
            return null;
        }

        /// <summary>
        /// <CreatedDate> 02-January-2006 </CreatedDate>
        /// <Author>Mr. Debasish Pramanik</Author>
        /// <Purpose>
        /// Get the count of files in a specified folder.
        /// </Purpose>
        /// </summary>
        /// <param name="FolderPath">The path on file system whose file count is required</param>
        /// <param name="FileCount">Number of files to be returned by the method.</param>
        /// <returns>An object of Type OMError</returns>
        public OMError GetFileCount(String FolderPath, ref ulong FileCount)
        {
            OMError     ErrObj      = null;
            String[]    FileArray   = null;
            try
            {
                FileArray = Directory.GetFiles(FolderPath);
                FileCount = Convert.ToUInt64(FileArray.Length);
            }
            catch (Exception ExcepObj)
            {
                ErrObj = new OMError(OMErrorCodes.FILESYSTEM_DIRFILECOUNT_FAILED,
                                                "Directory File Count Failed",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.EXCEPTION,
                                                ExcepObj, null);
                return ErrObj;
            }
            return null;
        }
        public OMError RetrieveAllFiles(String FolderPath)
        {
            return null;
        }
        public OMError RetrieveFileFromFolder(String FolderPath, String FileName)
        {
            return null;
        }

        /// <summary>
        /// This function checks whether a folder path exists or not.
        /// </summary>
        /// <param name="FolderPath">Path to be checked</param>
        /// <returns>TRUE if Folder exists or FALSE otherwise</returns>
        public Boolean IsFolderExists(String FolderPath)
        {
            if (Directory.Exists(FolderPath))
                return true;
            else
                return false; 
        }

        /// <summary>
        /// <CreatedDate> 25-August-2008 </CreatedDate>
        /// <Author>Mr. Yuvraj Deshmukh</Author>
        /// <Purpose>
        /// To get the Size of folder contents.
        /// </Purpose>
        /// </summary>
        /// <param name="FolderPath">NTFS Path of the folder, of which size has to retrieved </param>
        /// <param name="FolderSize">returns the Specifed folder Size</param>
        /// <returns>returns folder Size</returns>
        public OMError GetFolderSize(String FolderPath, out ulong FolderSize)
        {
            OMError ErrObj  = null;
            FolderSize      = 0;
            try
            {
                DirectoryInfo DirectoryInfoObj = new DirectoryInfo(FolderPath);
                FolderSize = GetFolderSize(DirectoryInfoObj);
            }
            catch (Exception ExcepObj)
            {
                ErrObj = new OMError(OMErrorCodes.FILESYSTEM_DIRGETSIZE_FAILED,
                                                "Directory GetFolderSize Failed",
                                                OMErrorObjectType.FILESYSTEM,
                                                "Windows File system", OMErrorType.EXCEPTION,
                                                ExcepObj, null);
                return ErrObj;
            }
            return null;
        }

        private ulong GetFolderSize(DirectoryInfo DirectoryInfoObj)
        {
            ulong size = 0;

            #region Calculating File Size
            foreach (FileInfo file in DirectoryInfoObj.GetFiles())
            {
                size += (ulong)file.Length;
            }
            #endregion

            #region Calulating Sub Folder Size
            foreach (DirectoryInfo directoryinfo in DirectoryInfoObj.GetDirectories())
            {
                size += GetFolderSize(directoryinfo);
            }
            #endregion
            return size;
        }
    }
}
