using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.IO;
using System.Xml;
using System.Net.Mail;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.Reflection;
using System.Configuration;

using Assetlink.Framework;
using Assetlink.WebServices.ALInfrastructureWS;
using Assetlink.Services.Engines;
using Assetlink.Services.WindowsService;
using Assetlink;

using log4net;
using log4net.Config;

using Assetlink.WebServices.ALSearchWS;

using Assetlink.TextProcessors;

namespace ALTextCataloger
{
    using XML = System.String;
    using ERRORXML = System.String;

    public class TextCatalogerManager
    {
        private static readonly ILog log = LogManager.GetLogger("TextCataloger");
        String InstalledPath = null;
        Dictionary<string, AssemblyClassNames> TextProcessorAssemblyNames = null;
        String AssetlinkConfigXMLPath = null;
        public String ImpersonationToken = null;
        public ulong curSessionID = 0;
        public ulong curUserID = 0;
        public String curLanguage = null;
        public string DeploymentMode = null;

        public TextCatalogerManager()
        {
            InstalledPath = ALCommon.GetInstallationDir();
            AssetlinkConfigXMLPath = ALCommon.GetConfigurationXMLPath() + "Assetlink.xml";

            log.DebugFormat("Path for AL Config XML: {0}", AssetlinkConfigXMLPath);

            LoadTextProcessors();
        }

        private void LoadTextProcessors()
        {
            string AssemblyName = null;
            XmlNodeList nodeList = null; //Temporary XML Node list
            string DefinedExtensions = null;
            string ClassName = null;

            XmlDocument AssetlinkConfigXMLDocument = new XmlDocument();
            AssetlinkConfigXMLDocument.Load(AssetlinkConfigXMLPath);

            nodeList = AssetlinkConfigXMLDocument.SelectNodes("Assetlink/TEXTCATALOGER/TEXTPROCESSORS/TEXTPROCESSOR");
            if (nodeList.Count > 0)
            {
                TextProcessorAssemblyNames = new Dictionary<string, AssemblyClassNames>(nodeList.Count);
                foreach (XmlNode TextProcessorNode in nodeList)
                {
                    DefinedExtensions = TextProcessorNode.SelectSingleNode("EXTENSION").InnerText;
                    AssemblyName = TextProcessorNode.SelectSingleNode("ASSEMBLYNAME").InnerText;
                    ClassName = TextProcessorNode.SelectSingleNode("CLASSNAME").InnerText;

                    log.InfoFormat("TextProcessor Info - Extensions: {0}. Assembly: {1}. ClassName: {2}", 
                                    DefinedExtensions, AssemblyName, ClassName);

                    string[] Extensions = DefinedExtensions.Split(',');

                    foreach (string Extension in Extensions)
                    {
                        TextProcessorAssemblyNames.Add(Extension.ToUpper(), new AssemblyClassNames(AssemblyName, ClassName));
                    }

                }
                nodeList = null;
            }


        }

        public ITextProcessor GetTextProcessor(string FileExtension)
        {
            string AssemblyPath = null;
            Type ClassType = null;
            Assembly LocalAssembly = null;

            if (!TextProcessorAssemblyNames.ContainsKey(FileExtension))
                return null;

            AssemblyPath = string.Format("{0}Binaries\\{1}.dll", InstalledPath, TextProcessorAssemblyNames[FileExtension].AssemblyName);
            log.InfoFormat("GetTextProcessor AssemblyPath: {0}", AssemblyPath);
            LocalAssembly = Assembly.LoadFrom(AssemblyPath);

            ClassType = GetAssemblyType(LocalAssembly, TextProcessorAssemblyNames[FileExtension].ClassName);

            if (ClassType == null)
            {
                return null;
            }

            return (ITextProcessor)Activator.CreateInstance(ClassType);
        }

        private Type GetAssemblyType(Assembly Assembly, string TextProcessorName)
        {

            foreach (Type ClassType in Assembly.GetTypes())
            {
                if (ClassType.Name == TextProcessorName)
                    return ClassType;
            }
            return null;
        }

        public void ProcessSearchableFilesForAllTenants()
        {
            String TenantList = getAllTenants();
            if (TenantList != null)
            {
                string[] TenantArr = TenantList.Split(',');
                int TenantCount = Convert.ToInt32((TenantArr.Length - 1));

                for (int i = 0; i < TenantCount; i++)
                {
                    string TenantName = TenantArr[i];

                    log.InfoFormat("Started processing searchable files for tenant {0}", TenantName);
                    
                    ProcessSearchableFileAlertsForTenant(TenantName);

                    log.InfoFormat("Completed processing searchable files for tenant {0}", TenantName);
                }
            }
            else
            {
                //If we do not have any tenant then what to do ?
                log.Error("No tenant found");
                return;
            }

        }

        private ERRORXML ProcessSearchableFileAlertsForTenant(string TenantName)
        {
            //Get the list of all alerts for this tenant
            string SearchableFileAlertsXML = null;
            string FileExtension = null;
            string FileText = null;
            string CompleteFilePath = null;
            StringBuilder sbErrorCollection = new StringBuilder(200);
            bool ErrorOccurred = false;

            ERRORXML ErrorXml = null;

            OMError ErrorObj = null;

            try
            {
                ErrorXml = DoImpersonateAdmin(TenantName);
                if (!string.IsNullOrEmpty(ErrorXml))
                {
                    log.ErrorFormat("Error: {0}. Details: {1}", "DoImpersonateAdmin", ErrorXml);
                    throw new OMErrorOccurred("Error in Impersonating Admin.", ErrorXml);
                }

                //GetSearchableFileAlerts("qa", ref SearchableFileAlertsXML);
                ErrorXml = GetSearchableFileAlerts(TenantName, ref SearchableFileAlertsXML);
                if (!string.IsNullOrEmpty(ErrorXml))
                {
                    log.ErrorFormat("Error: {0}. Details: {1}", "GetSearchableFileAlerts", ErrorXml);
                    return ErrorXml;
                }

                XmlDocument SearchableFileAlertsXMLDoc = new XmlDocument();
                SearchableFileAlertsXMLDoc.LoadXml(SearchableFileAlertsXML);

                XmlNodeList SearchableFileAlertsNodeList = SearchableFileAlertsXMLDoc.SelectNodes("SEARCHABLE_FILE_ALERTCOLLECTION/SEARCHABLE_FILE_ALERT");

                log.InfoFormat("Tenant {1} - Found {0} Searchable File Alerts", SearchableFileAlertsNodeList.Count, TenantName);

                string ObjectName, ObjectType, FieldName, FileName;
                string OnMediaServer, MediaServerURL;
                long ObjectId, FileId, SchemaId;

                sbErrorCollection.Append("<ERRORCOLLECTION>");

                //for each alert
                foreach (XmlNode SearchableFileAlertNode in SearchableFileAlertsNodeList)
                {
                    ObjectName = SearchableFileAlertNode.SelectSingleNode("OBJECTNAME").InnerText;
                    ObjectType = SearchableFileAlertNode.SelectSingleNode("OBJECTTYPE").InnerText;
                    ObjectId = Convert.ToInt64(SearchableFileAlertNode.SelectSingleNode("OBJECTID").InnerText);
                    FileId = Convert.ToInt64(SearchableFileAlertNode.SelectSingleNode("FILEID").InnerText);
                    SchemaId = Convert.ToInt64(SearchableFileAlertNode.SelectSingleNode("SCHEMAID").InnerText);
                    FieldName = SearchableFileAlertNode.SelectSingleNode("FIELDNAME").InnerText;
                    FileName = SearchableFileAlertNode.SelectSingleNode("FILENAME").InnerText;
                    OnMediaServer = SearchableFileAlertNode.SelectSingleNode("ONMEDIASERVER").InnerText;
                    MediaServerURL = SearchableFileAlertNode.SelectSingleNode("MEDIASERVERURL").InnerText;

                    log.InfoFormat("Started processing file {0}", FileName);

                    //retrieve the file
                    try
                    {
                        ErrorXml = CopyFileToTemporaryLocation(TenantName, ObjectId, ObjectType, ObjectName, FileId,
                                                    SchemaId, FieldName, FileName, OnMediaServer, MediaServerURL, ref CompleteFilePath);

                        log.InfoFormat("File copied to {0}", CompleteFilePath);

                        if (!string.IsNullOrEmpty(ErrorXml))
                        {
                            log.ErrorFormat("Error: {0}. Details: {1}", "CopyFileToTemporaryLocation", ErrorXml);
                            log.ErrorFormat("Skipping file. Details: {0}{1}{2}{3}{4}{5}{6}",
                                            TenantName, ObjectId, ObjectType, ObjectName, FileId, SchemaId, CompleteFilePath);
                            sbErrorCollection.Append(ErrorXml);
                            ErrorOccurred = true;

                            continue;//We'll skip this file and proceed to the next
                        }
                    }
                    catch (Exception e)
                    {
                        log.ErrorFormat("Error: {0}. Details: {1}:{2}", "ProcessAlertsForTenant-CopyFileToTemporaryLocation", e.Message, e.StackTrace);

                        ErrorXml = MarkAlert(TenantName, ObjectId, ObjectType, ObjectName, FileId, "ERROR");

                        if (!string.IsNullOrEmpty(ErrorXml))
                        {
                            log.ErrorFormat("Error: {0}. Details: {1}", "MarkAlert", ErrorXml);
                            log.ErrorFormat("Skipping file. Details: {0}{1}{2}{3}{4}{5}{6}",
                                            TenantName, ObjectId, ObjectType, ObjectName, FileId, SchemaId, CompleteFilePath);

                            sbErrorCollection.Append(ErrorXml);
                            ErrorOccurred = true;

                        }

                        continue;//We'll skip this file and proceed to the next
                    }

                    FileExtension = CompleteFilePath.Substring(CompleteFilePath.LastIndexOf(".") + 1).ToUpper();

                    //from the config, invoke the appropriate text processing library through the interface
                    ITextProcessor itp = GetTextProcessor(FileExtension);

                    if (itp != null)
                    {
                        try
                        {
                            //call the interface function
                            ErrorObj = itp.ExtractText(CompleteFilePath, ref FileText);

                            if (ErrorObj != null)
                            {
                                ErrorXml = ErrorObj.Serialize();

                                log.ErrorFormat("Error: {0}. Details: {1}", "ExtractText", ErrorXml);
                                log.ErrorFormat("Skipping file. Details: {0}{1}{2}{3}{4}{5}{6}",
                                                TenantName, ObjectId, ObjectType, ObjectName, FileId, SchemaId, CompleteFilePath);

                                sbErrorCollection.Append(ErrorXml);
                                ErrorOccurred = true;

                                continue;//We'll skip this file and proceed to the next
                            }

                            CleanText(ref FileText);
                        }
                        catch(Exception e)
                        {
                            log.ErrorFormat("Error: {0}. Details: {1}:{2}", "ProcessAlertsForTenant", e.Message, e.StackTrace);

                            ErrorXml = MarkAlert(TenantName, ObjectId, ObjectType, ObjectName, FileId, "PENDING");

                            if (!string.IsNullOrEmpty(ErrorXml))
                            {
                                log.ErrorFormat("Error: {0}. Details: {1}", "MarkAlert", ErrorXml);
                                log.ErrorFormat("Skipping file. Details: {0}{1}{2}{3}{4}{5}{6}",
                                                TenantName, ObjectId, ObjectType, ObjectName, FileId, SchemaId, CompleteFilePath);

                                sbErrorCollection.Append(ErrorXml);
                                ErrorOccurred = true;
                                
                            }
                            //Delete file from temporary location
                            try
                            {
                                DeleteFileFromTemporaryLocation(CompleteFilePath);
                            }
                            catch (Exception ex)
                            {
                                log.ErrorFormat("Error: {0}. Details: {1}:{2}", "ProcessAlertsForTenant-DeleteFileFromTemporaryLocation", ex.Message, ex.StackTrace);
                            }

                            continue;//We'll skip this file and proceed to the next
                        }

                        //take the retrieved text and fire the update into the facet
                        ErrorXml = InsertNewTextIntoFacet(TenantName, ObjectId, ObjectType, ObjectName, FileId, FileText);

                        if (!string.IsNullOrEmpty(ErrorXml))
                        {
                            log.ErrorFormat("Error: {0}. Details: {1}", "InsertNewTextIntoFacet", ErrorXml);
                            log.ErrorFormat("Skipping file. Details: {0}{1}{2}{3}{4}{5}{6}",
                                            TenantName, ObjectId, ObjectType, ObjectName, FileId, SchemaId, CompleteFilePath);

                            sbErrorCollection.Append(ErrorXml);
                            ErrorOccurred = true;

                            continue;//We'll skip this file and proceed to the next
                        }

                        //Mark the record as indexed
                        ErrorXml = MarkAlert(TenantName, ObjectId, ObjectType, ObjectName, FileId, "SUCCESS");

                        if (!string.IsNullOrEmpty(ErrorXml))
                        {
                            log.ErrorFormat("Error: {0}. Details: {1}", "MarkAlert", ErrorXml);
                            log.ErrorFormat("Skipping file. Details: {0}{1}{2}{3}{4}{5}{6}",
                                            TenantName, ObjectId, ObjectType, ObjectName, FileId, SchemaId, CompleteFilePath);

                            sbErrorCollection.Append(ErrorXml);
                            ErrorOccurred = true;

                            continue;//We'll skip this file and proceed to the next
                        }
                    }
                    else
                    {
                        log.ErrorFormat("Error: {0}. Details: Returned NULL for file {1}",
                                            "GetTextProcessor", CompleteFilePath);
                        ErrorOccurred = true;
                        sbErrorCollection.AppendFormat("<ERROR>No text processor found for file. Details: {0}{1}{2}{3}{4}{5}{6}</ERROR>",
                                                        TenantName, ObjectId, ObjectType, 
                                                        ObjectName, FileId, SchemaId, CompleteFilePath);

                        //Mark the record as indexed
                        ErrorXml = MarkAlert(TenantName, ObjectId, ObjectType, ObjectName, FileId, "NO TEXTPROCESSOR");

                        if (!string.IsNullOrEmpty(ErrorXml))
                        {
                            log.ErrorFormat("Error: {0}. Details: {1}", "MarkAlert", ErrorXml);
                            log.ErrorFormat("Skipping file. Details: {0}{1}{2}{3}{4}{5}{6}",
                                            TenantName, ObjectId, ObjectType, ObjectName, FileId, SchemaId, CompleteFilePath);

                            sbErrorCollection.Append(ErrorXml);
                            ErrorOccurred = true;

                            continue;//We'll skip this file and proceed to the next
                        }

                    }

                    log.InfoFormat("Deleting file {0}", CompleteFilePath);

                    try
                    {
                        DeleteFileFromTemporaryLocation(CompleteFilePath);
                    }
                    catch (Exception e)
                    {
                        log.ErrorFormat("Error: {0}. Details: {1}:{2}", "ProcessAlertsForTenant-DeleteFileFromTemporaryLocation", e.Message, e.StackTrace);

                        continue;//We'll skip this file and proceed to the next
                    }

                    log.InfoFormat("Finished processing file {0}", FileName);
                }
            }

            catch (Exception e)
            {
                log.ErrorFormat("Error: {0}. Details: {1}:{2}", "ProcessAlertsForTenant", e.Message, e.StackTrace);
                ErrorOccurred = true;
                sbErrorCollection.Append(ErrorXml);
            }

            finally
            {
                sbErrorCollection.Append("</ERRORCOLLECTION>");
            }

            //a DB job will reindex the table incrementally or completely
            if (ErrorOccurred)
                return sbErrorCollection.ToString();
            else
                return null;
        }

        private ERRORXML MarkAlert(string TenantName, long ObjectId, string ObjectType, 
                                            string ObjectName, long FileId, string NewStatus)
        {
            string UpdateSearchableFileAlertXML = string.Format(@"<InputXML>
                                  <Transaction>false</Transaction>
                                  <Operation>
                                    <OBJECTNAME>ENTITY</OBJECTNAME>
                                    <METHOD>UPDATEDATA</METHOD>
                                    <ACCESSCONTROLCHECKREQUIRED>FALSE</ACCESSCONTROLCHECKREQUIRED>
                                    <PARAM Name='InputXML'>
                                      <SEARCHABLE_FILE_ALERT>
                                        <FILEID>{0}</FILEID>
                                        <OBJECTID>{1}</OBJECTID>
                                        <OBJECTTYPE>{2}</OBJECTTYPE>
                                        <OBJECTNAME>{3}</OBJECTNAME>
                                        <STATUS>{4}</STATUS>
                                      </SEARCHABLE_FILE_ALERT >
                                    </PARAM>
                                  </Operation>
                                </InputXML>", FileId, ObjectId, ObjectType, ObjectName, NewStatus);

            OMInterface OMIObj = null;
            ERRORXML ErrorXML = null;

            try
            {
                OMIObj = new OMInterface(curUserID, curLanguage, TenantName);
                ErrorXML = OMIObj.Execute(ref UpdateSearchableFileAlertXML);

                if (ErrorXML != null)
                {
                    return ErrorXML;
                }
            }
            catch
            {
                throw;
            }

            return null;
        }

        /// <summary>
        /// Clean the text from the file by substituting XML tags, etc.
        /// </summary>
        /// <param name="TextToBeCleaned">The Text that is to be cleaned</param>
        private void CleanText(ref string TextToBeCleaned)
        {
            if (TextToBeCleaned == null) 
                return;

            StringBuilder newString = new StringBuilder();
            char ch;

            for (int i = 0; i < TextToBeCleaned.Length; i++)
            {
                ch = TextToBeCleaned[i];
                // remove any characters outside the valid UTF-8 range as well as all control characters
                // except tabs and new lines
                if (ch < 0x00FD && ch > 0x001F)
                {
                    if (ch == '\t' || ch == '\n' || ch == '\r')
                    {
                        newString.Append(' ');
                    }
                    else
                    {
                        newString.Append(ch);
                    }
                }
                else
                {
                    //Add blank space to separate words at line ends 
                    newString.Append(' ');
                }
            }

            newString.Replace("&", "&amp;");
            newString.Replace("<", "&lt;");
            newString.Replace(">", "&gt;");
            newString.Replace("\"", "&quot;");
            newString.Replace("'", "&apos;");

            TextToBeCleaned = newString.ToString();

        }

        private void DeleteFileFromTemporaryLocation(string CompleteFilePath)
        {
            string DirectoryName = null;
            try
            {
                /*
                 * Note:  This Cleanup is not required, since Outbox gets cleared by ALCleanup Utility and cleanup of this file will be taken care by that.
                 
                File.Delete(CompleteFilePath);

                //Get the parent of the parent of the CompleteFilePath
                DirectoryName = Directory.GetParent(Directory.GetParent(CompleteFilePath).FullName).FullName;

                Directory.Delete(DirectoryName, true);
                */
            }
            catch (Exception e)
            {
                log.ErrorFormat("Could not delete file {0}. Reason {1}", CompleteFilePath, e.Message);
            }
        }

        private ERRORXML InsertNewTextIntoFacet(string TenantName, long ObjectId, string ObjectType, string ObjectName, 
                                            long FileId, string FileText)
        {
            //Transaction is false so that in case of the search text facet being added for the first time,
            //the transaction should not fail as it does not exist right now
            string SearchableFacetOperationsXML = string.Format(@"<InputXML>
                                  <Transaction>false</Transaction>
                                  <Operation>
                                    <OBJECTNAME>FACET</OBJECTNAME>
                                    <METHOD>DELETE</METHOD>
                                    <Param Name='FacetID'>{0}</Param>
		                            <Param Name='FacetTypeName'>SEARCHABLE_FILE_TEXT</Param>
                                  </Operation>

                                  <Operation>
                                    <OBJECTNAME>FACET</OBJECTNAME>
                                    <METHOD>CREATE</METHOD>
                                    <PARAM Name='InputXML'>
                                      <SEARCHABLE_FILE_TEXT>
                                        <FILEID>{0}</FILEID>
                                        <OBJECTID>{1}</OBJECTID>
                                        <OBJECTTYPE>{2}</OBJECTTYPE>
                                        <OBJECTNAME>{3}</OBJECTNAME>
                                        <FILETEXT>{4}</FILETEXT>
                                      </SEARCHABLE_FILE_TEXT >
                                    </PARAM>
                                  </Operation>
                                </InputXML>", FileId, ObjectId, ObjectType, ObjectName, FileText);
            
            ERRORXML ErrorXML = null;
            
            try
            {
                ErrorXML = MakeExecuteRequest(TenantName, ref SearchableFacetOperationsXML);

                if (ErrorXML != null)
                {
                    return ErrorXML;
                }
            }
            catch
            {
                throw;
            }

            return null;
        }

        private ERRORXML MakeSearchRequest(string TenantName, string InputXML, ref string OutputXML)
        {
            OMInterface OMIObj = null;
            ERRORXML ErrorXML = null;

            try
            {
                OMIObj = new OMInterface(curUserID, curLanguage, TenantName);
                ErrorXML = OMIObj.GetHierarchialData(InputXML, ref OutputXML);

                if (ErrorXML != null)
                {
                    return ErrorXML;
                }
            }
            catch
            {
                throw;
            }

            return null;
        }
        
        private ERRORXML MakeExecuteRequest(string TenantName, ref string CommandXML)
        {
            OMInterface OMIObj = null;
            ERRORXML ErrorXML = null;

            try
            {
                OMIObj = new OMInterface(curUserID, curLanguage, TenantName);
                ErrorXML = OMIObj.Execute(ref CommandXML);

                if (ErrorXML != null)
                {
                    return ErrorXML;
                }
            }
            catch
            {
                throw;
            }

            return null;
        }

        private ERRORXML DoImpersonateAdmin(string TenantName)
        {
            ImpersonateAdmin IAObj = null;
            String EncryptedSystemUserName = null;
            String EncryptedSystemUserPwd = null;
            StringBuilder sbSearchXML = new StringBuilder();

            ERRORXML ErrorXml = null;

            EncryptedSystemUserName = ALCommon.Encrypt(ImpersonateAdmin.SystemUserName);
            EncryptedSystemUserPwd = ALCommon.Encrypt(ImpersonateAdmin.SystemUserPwd);
            ErrorXml = Infrastructure.SystemConnect(EncryptedSystemUserName, EncryptedSystemUserPwd, ref ImpersonationToken);
            if (ErrorXml != null)
            {
                throw new OMErrorOccurred("Error in System Connect.", ErrorXml);
            }
            else
            { //error in SystemConnect
            }//error in SystemConnect

            IAObj = new ImpersonateAdmin(TenantName, ImpersonationToken);
            ErrorXml = IAObj.InitStandardParam(ref curLanguage, ref curSessionID, ref curUserID);

            IAObj = null;

            return ErrorXml;
        }

        private ERRORXML CopyFileToTemporaryLocation(string TenantName, long ObjectId, string ObjectType, 
                                                string ObjectName, long FileId, long SchemaId, 
                                                string FieldName, string FileName,string OnMediaServer,string MediaServerURL, ref string CompleteFilePath)
        
        {
            StringBuilder sbRequestFileXML = new StringBuilder();
            ERRORXML ErrorXml = null;
            ERRORXML MarkPending = null;

            try
            {
                sbRequestFileXML.AppendFormat("<FILESYSTEM ObjectID=\"{0}\">", ObjectId);
                sbRequestFileXML.Append("<FILE FileRequired=\"Yes\">");
                sbRequestFileXML.AppendFormat("<FIELDNAME>{0}</FIELDNAME>", FieldName);
                //Bug Fix 6733:Special character handling needed for file name.
                if ((FileName != null) && (FileName != ""))
                    FileName = ALCommon.TrimAndReplaceSpecialChars(FileName);

                sbRequestFileXML.AppendFormat("<NAME>{0}</NAME>", FileName);
                sbRequestFileXML.AppendFormat("<OBJECTID>{0}</OBJECTID>", ObjectId);
                sbRequestFileXML.AppendFormat("<SCHEMAID>{0}</SCHEMAID>", SchemaId);
                sbRequestFileXML.AppendFormat("<ONMEDIASERVER>{0}</ONMEDIASERVER>", OnMediaServer);
                sbRequestFileXML.AppendFormat("<MEDIASERVERURL>{0}</MEDIASERVERURL>", MediaServerURL);
                sbRequestFileXML.Append("</FILE>");
                sbRequestFileXML.Append("</FILESYSTEM>");

                string RequestFileXML = sbRequestFileXML.ToString();

                ALFile f = new ALFile(curUserID, curLanguage, TenantName);

                ErrorXml = f.GetFile(ref RequestFileXML);

                if (!String.IsNullOrEmpty(ErrorXml))
                {
                    MarkPending = MarkAlert(TenantName, ObjectId, ObjectType, ObjectName, FileId, "PENDING");
                    throw new OMErrorOccurred("Error in GetFile operation", ErrorXml);
                }

                if (Convert.ToBoolean(OnMediaServer))
                {
                    XmlDocument FileSystemDocument = new XmlDocument();
                    FileSystemDocument.LoadXml(RequestFileXML);

                    XmlNode FileNode = FileSystemDocument.SelectSingleNode("FILESYSTEM/FILE");

                    Assetlink.Services.Engines.DirectoryEngine.MediaServerOperations MediaServerOperationsObj = null;
                    //No need to pull file for LoadBalance environment as file is directly accessible.
                    //This is done as the outbox is a shared location in case of LoadBalance eventhough file was on MediaServer
                    if (FileNode.SelectSingleNode("ONMEDIASERVER") != null && Boolean.Parse(FileNode.SelectSingleNode("ONMEDIASERVER").InnerText) && (DeploymentMode.ToUpper() != "LOADBALANCE"))
                    {
                        //file is on Media Server and need to pull on OM (Services Machine)
                        MediaServerOperationsObj = new Assetlink.Services.Engines.DirectoryEngine.MediaServerOperations(curUserID, curLanguage, TenantName);
                        MediaServerOperationsObj.PullFileFromMediaServer(ref FileNode);
                    }

                }

                //Extract the complete file path
                XmlDocument FileSystemXMLDocument = new XmlDocument();
                FileSystemXMLDocument.LoadXml(RequestFileXML);

                XmlNode PathNode = null;

                PathNode = FileSystemXMLDocument.SelectSingleNode("FILESYSTEM/FILE/PATH");

                if (PathNode != null)
                {
                    CompleteFilePath = PathNode.InnerText;

                    return null;
                }
                else
                {
                    throw new OMErrorOccurred("Full File path could not be found", RequestFileXML);
                }
            }
            catch
            {
                MarkPending = MarkAlert(TenantName, ObjectId, ObjectType, ObjectName, FileId, "PENDING");
                throw;
            }
               
        }

        private ERRORXML GetSearchableFileAlerts(string TenantName, ref string SearchableFileAlertsXML)
        {
            StringBuilder sbSearchXML = new StringBuilder();
            ERRORXML ErrorXml = null;

            int NumberOfFilesToProcess;//The number of file alerts to process for this run
            try
            {
                NumberOfFilesToProcess = Convert.ToInt32(ConfigurationManager.AppSettings["FileCountToProcess"]);
            }
            catch
            {
                NumberOfFilesToProcess = 50;//default
            }

            sbSearchXML.Append("<SEARCH>");
            sbSearchXML.AppendFormat("<SESSIONINFO SessionID='{0}' UserID='{1}' TenantName='{2}' Language='{3}' />", curSessionID, curUserID, TenantName, curLanguage);
            sbSearchXML.Append("<TARGETOBJECT>SEARCHABLE_FILE_ALERT</TARGETOBJECT>");
            sbSearchXML.Append("<TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>");
            sbSearchXML.Append("<AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>");

            if (NumberOfFilesToProcess > 0)
                sbSearchXML.AppendFormat("<MAXRESULTSIZE>{0}</MAXRESULTSIZE>", NumberOfFilesToProcess);

            sbSearchXML.Append("<TARGETENTITY><FILTERLIST><FILTER><EXPRESSION>STATUS</EXPRESSION>");
            sbSearchXML.Append("<FILTERID>1</FILTERID>");
            sbSearchXML.Append("<OPERATOR>EQUALTO</OPERATOR>");
            sbSearchXML.Append("<EXPRESSIONVALUE>NEW</EXPRESSIONVALUE>");
            sbSearchXML.Append("<ISCASESENSITIVE>False</ISCASESENSITIVE>");
            sbSearchXML.Append("</FILTER></FILTERLIST>");
            sbSearchXML.Append("</TARGETENTITY></SEARCH>");

            try
            {
                ErrorXml = MakeSearchRequest(TenantName, sbSearchXML.ToString(), ref SearchableFileAlertsXML);
            }
            catch
            {
                throw;
            }

            if (ErrorXml != null)
            {
                throw new OMErrorOccurred("Error in getting NEW Searchable File Alerts.", ErrorXml);
            }

            return null;
        }

        private String getAllTenants()
        {
            String TenantList = null;
            try
            {
                log.Info("Trying to access " + AssetlinkConfigXMLPath);

                if (File.Exists(AssetlinkConfigXMLPath))
                {
                    XmlDocument mainDoc = new XmlDocument();
                    mainDoc.Load(AssetlinkConfigXMLPath);
                    XmlNodeList tenantNodes = mainDoc.SelectNodes("//Tenant");

                    foreach (XmlNode tNode in tenantNodes)
                    {
                        String tName = ((XmlElement)tNode).GetAttribute("Name");

                        Boolean bSupport = IsFullTextSearchSupported(tName);
                        if (bSupport == true)
                            TenantList = TenantList + tName + ",";

                    }
                    //Deployment Mode is required to determine whether pulling file from MediaServer is required or not
                    XmlNode DeploymentNode = mainDoc.SelectSingleNode("//Settings/add[@name = 'DeploymentMode']");
                    if (DeploymentNode != null)
                    {
                        DeploymentMode = DeploymentNode.Attributes["value"].Value;
                    }
                }
                else
                {
                    throw new FileNotFoundException();
                }
                return TenantList;
            }
            catch
            {
                throw;
            }


        }//getAllTenants

        /// <summary>
        /// This function will return true if the DataBaseType for the tenant is SQLServer or else it will return false.
        /// This is because Full Text Search is supported only on SQL Server.
        /// </summary>
        /// <Author>Hemant Kirad</Author>
        /// <Date>20 December 2010</Date>
        /// <param name="tName">Tenant Name</param>
        /// <returns></returns>
        private Boolean IsFullTextSearchSupported(String tName)
        {
            String                  DBTypeVal                   = null;
            InfrastructureService   InfrastructureServiceObject = new InfrastructureService();

            InfrastructureServiceObject.GetTenantSetting(tName, "DATABASE/DATABASETYPE", ref DBTypeVal);

            if ((DBTypeVal.ToUpper() != "TERADATA"))
                return true;
            else 
                return false;
        }
    }



    public class OMErrorOccurred : Exception
    {
        public String configMessage = null;
        public String ErrorXML = null;
        public OMErrorOccurred(String temp, String tErrorXML)
        {
            configMessage = temp;
            ErrorXML = tErrorXML;
        }
        public override string ToString()
        {
            return (configMessage);
        }
    }//OMErrorOccurred

    struct AssemblyClassNames
    {
        public string AssemblyName;
        public string ClassName;

        public AssemblyClassNames(string pAssemblyName, string pClassName)
        {
            AssemblyName = pAssemblyName;
            ClassName = pClassName;
        }
    }

    class FileNotFoundException : Exception
    {
        public override string ToString()
        {
            return "File does not exist.";
        }
    }

    class ImpersonateAdmin
    {
        public const ulong SystemUserID = 111111122211111;
        public const String SystemUserName = "MePhIsToPhElEs";
        public const String SystemUserPwd = "ErAtOsThEnEs";
        private const String str_CallerIdentity = "ALTEXTCATALOGER";
        private ulong SessionID = 0;
        private ulong UserID = 0;
        private String TenantName = null;
        private String ImpersonationToken = null;
        public Assetlink.Services.Engines.ALServiceHeader RequestHeader = null;

        public ImpersonateAdmin(String tempTenant, String tImpersonationToken)
        {
            TenantName = tempTenant;
            ImpersonationToken = tImpersonationToken;
        }

        public String InitALServiceHeader(ref Assetlink.Services.Engines.ALServiceHeader RequestHeader)
        {
            try
            {
                String UserXML = null;
                String ErrorXML = Infrastructure.ImpersonateUser("Admin", TenantName, ImpersonationToken, 
                    str_CallerIdentity, ref SessionID, ref UserXML);
                if ((ErrorXML == null) && (UserXML != null))
                {
                    XmlDocument UserXMLDoc = new XmlDocument();
                    UserXMLDoc.LoadXml(UserXML);
                    UserID = Convert.ToUInt64(UserXMLDoc.SelectSingleNode("//USERID").InnerText);

                    RequestHeader.SessionID = SessionID;
                    RequestHeader.UserID = UserID;
                    RequestHeader.TenantName = TenantName;
                    RequestHeader.Language = "English";
                    return null;
                }
                else
                    return ErrorXML;

            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }//InitALServiceHeader

        public String InitStandardParam(ref String tLanguage, ref ulong tSessionID, ref ulong tUserID)
        {
            try
            {
                bool bIsSessionValid = false;
                if (tSessionID != 0)
                    Infrastructure.IsSessionValid(tUserID, tSessionID, tLanguage, TenantName, ref bIsSessionValid);

                if (!bIsSessionValid)
                {
                    String UserXML = null;
                    String ErrorXML = Infrastructure.ImpersonateUser("Admin", TenantName, ImpersonationToken,
                        str_CallerIdentity,ref tSessionID, ref UserXML);
                    if ((ErrorXML == null) && (UserXML != null))
                    {
                        XmlDocument UserXMLDoc = new XmlDocument();
                        UserXMLDoc.LoadXml(UserXML);
                        tUserID = Convert.ToUInt64(UserXMLDoc.SelectSingleNode("//USERID").InnerText);
                        tLanguage = "English";
                        return null;
                    }
                    else
                        return ErrorXML;
                }
                return null;

            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }

        }//InitStandardParam
    }//ImpersonateAdmin
}
