#region System Include
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.XPath;
using System.IO;
using System.Collections.Specialized;
#endregion

#region Assetlink Include
using Assetlink.Services.Engines;
using Assetlink.Framework;
using Assetlink.Diagnostic;
using Assetlink.ALWebServiceInterface;
using Assetlink.IOProxy;
#endregion

namespace Assetlink.Services.Engines.ArtworkProducerEngine
{
    using XML = System.String;
    using ERRORXML = System.String;
    using EntityID = System.UInt64;

    public class ArtworkEngine : ALBaseEngine
    {
        #region variable
        ALErrorCommon objErrorCommon = null;
        String ReturnErrorXML = null;
        StringDictionary strdict_AdditionalInfo = null;
        public enum ErrorObjectType
        {
            Artwork,
            MediaJob
        }
        #endregion

        #region Constructor
        public ArtworkEngine(): base()
        {
            objErrorCommon = new ALErrorCommon();
        }

        public ArtworkEngine(UInt64 UserID, String Language, String TenantName)
            : base(UserID, Language, TenantName)
        {
            objErrorCommon = new ALErrorCommon();
        }

        public ArtworkEngine(UInt64 SessionID, UInt64 UserID, String Language, String TenantName, String SolutionName)
            : base(SessionID, UserID, Language, TenantName, SolutionName)
        {
            objErrorCommon = new ALErrorCommon();
        }

        #endregion

        #region Public Methods
        /// <alsummary>
        /// <createddate>16 November 2006</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>This method is used to create new Artwork based on the input XML.</purpose>
        ///  <xmlstructure><![CDATA[
        /// 
        ///     Input XML :         |-  [ARTWORK]
        ///                                       - $Attributes$
        ///                                       |-    [$RelationShipName$ @TYPE="RELATION"  @DISASSOCIATE="TRUE/FALSE"]text()=$Associated Entity/Facet Id$     // Associate or Disassociate existing instance with TargetEntity                    
        ///                                                  |-  [$Associated Entity/Facet Name$]        //it will create new instance of associated entity 
        ///                                                           |-  $Attributes                       // or facet , and associate it with target entity
        /// 
        /// ]]></xmlstructure>
        /// <param name="ArtworkXML"></param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        public ERRORXML CreateArtwork(ref XML ArtworkXML)
        {
            ERRORXML ErrorXML = null;
            ArtworkProducerCache objAPCache = new ArtworkProducerCache(UserID, Language, TenantName);
            OMInterface OMI = null;
            OperationQueue InputXML = null;
            Operation OperationNode = null;
            XML OMInputXML = null;
            ArtworkProducerCommon ArtworkProducerCommonObj = null;

            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine CreateArtwork Starts");
                ArtworkProducerCommonObj = new ArtworkProducerCommon(UserID, Language, TenantName);
                String FormatID = ALCommon.GetInnerValue(ref ArtworkXML, "<FORMAT_ARTWORK TYPE=\"RELATION\">", "</FORMAT_ARTWORK>");
                StringBuilder CreateAuthorizationXML = null;
                string ArtworkID = null;

                objErrorCommon = new ALErrorCommon();
                ErrorXML = ArtworkProducerEngine.CreateAndUpdateData(ref ArtworkXML, "CREATEDATA", UserID, Language, TenantName);

                //Increment Format's GenerationCount value
                if (String.IsNullOrEmpty(ErrorXML))
                    ErrorXML = ArtworkProducerCommonObj.IncrementGenerationCount(FormatID, ArtworkProducerCommon.EntityType.Format, true, out OperationNode);

                //Add authorization entry for artwork
                if (String.IsNullOrEmpty(ErrorXML))
                {
                    ArtworkID = ALCommon.GetInnerValue(ref ArtworkXML, "<ARTWORKID>", "</ARTWORKID>");

                    #region Prepare Authorization XML for Creator of Book
                    CreateAuthorizationXML = new StringBuilder();
                    CreateAuthorizationXML.Append("<AUTHORIZATION>");
                    CreateAuthorizationXML.Append("<OBJECTINSTANCE>");
                    CreateAuthorizationXML.AppendFormat("<OBJECTID>{0}</OBJECTID>", ArtworkID);
                    CreateAuthorizationXML.Append("<OBJECTTYPE>ENTITY</OBJECTTYPE>");
                    CreateAuthorizationXML.Append("<OBJECTTYPENAME>ARTWORK</OBJECTTYPENAME>");
                    CreateAuthorizationXML.Append("<ACCESSCONTROLLIST>");
                    CreateAuthorizationXML.Append("<ACCESSCONTROLENTRY>");
                    CreateAuthorizationXML.AppendFormat("<PRIVILEGENAME>DELETE</PRIVILEGENAME>");
                    CreateAuthorizationXML.Append("<ACCESSORTYPE>USER</ACCESSORTYPE>");
                    CreateAuthorizationXML.AppendFormat("<ACCESSORID>{0}</ACCESSORID>", UserID);
                    CreateAuthorizationXML.Append("</ACCESSCONTROLENTRY>");
                    CreateAuthorizationXML.Append("</ACCESSCONTROLLIST>");
                    CreateAuthorizationXML.Append("</OBJECTINSTANCE>");
                    CreateAuthorizationXML.Append("</AUTHORIZATION>");
                    #endregion

                    OMI = new OMInterface(UserID, Language, TenantName);
                    InputXML = new OperationQueue(true);
                    OperationNode = new Operation("AUTHORIZATION", "CREATE");
                    OperationNode.AddParameter("INPUTXML", CreateAuthorizationXML.ToString());
                    InputXML.AddOperation(OperationNode);
                    OMInputXML = InputXML.Xml;
                    ErrorXML = OMI.Execute(ref OMInputXML);
                }

                if (ErrorXML != null)
                {
                    objErrorCommon.AppendError("APENGINE_ARTWORK_CREATEARTWORK_FAILED", this.GetType().ToString(), "CreateArtwork", null, ErrorXML);
                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                }
                else
                {
                    ErrorXML = GenerateArtwork_Cache_Section_TextElements(FormatID, ref ArtworkXML);
                }

                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_ARTWORK_CREATEARTWORK_EXCEPTION", this.GetType().ToString(), "CreateArtwork", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine CreateArtwork Ends");
            }
        }

        public ERRORXML SaveArtwork(String ArtworkID, String ArtworkHTML)
        {
            FormatEngine FormatEngineObj = null;
            ERRORXML ErrorXML = null;
            XML FileCollectionNode;
            String UploadFileName = null;
            String UploadFilePath = null;
            String FormatID = null;
            String FormatXML = null;
            String TenantTempPath = null;
            String FileAttribute = null;
            String TextTagValues = null;
            String ElementGroupXML = null;
            String TextElementId = null;
            String TextElementValueId = null;
            String GraphicElementId = null;
            String GraphicElementValueId = null;
            String ArtworkCachePath = null;
            String strURLContent = null;
            String ShowURL = null;
            String URL = null;
            String URLTarget = null;
            String URLType = null;
            String URLProtocol = null;
            StringBuilder sbFileCollectionNode = new StringBuilder();
            StringBuilder sbFormatXML = null;
            StringBuilder sbTextTagXML = null;
            StringBuilder sbUpdateArtworkXML = null;
            StringBuilder sbGraphicTagXML = null;
            XPathDocument xpDoc_TextTags = null;
            XPathDocument xpDoc_OrgFile = null;
            XPathDocument xpDoc_GraphicTags = null;
            XPathNavigator xpNav_TextTags = null;
            XPathNavigator TEValuenode = null;
            XPathNavigator xpNav_OrgFile = null;
            XPathNavigator xpNav_Tag = null;
            XPathNavigator xpNav_GraphicTags = null;
            XPathNodeIterator xpTEIterator = null;
            XPathNodeIterator xpItr_TagList = null;
            XPathNodeIterator xpGEIterator = null;

            #region copy the file to the Integration folder under download files
            GetFormatIDForArtwork(ArtworkID, out FormatID);
            TenantTempPath = ALCommon.GetTempPath() + "\\Integration\\" + SessionID + "\\" + FormatID;

            UploadFileName = "Main.html";
            UploadFilePath = TenantTempPath + "\\Main.html";

            if (!Directory.Exists(TenantTempPath))
                Directory.CreateDirectory(TenantTempPath);
            System.IO.File.WriteAllText(UploadFilePath, ArtworkHTML);
            #endregion

            FileAttribute = "Operation='UPDATE'";
            sbFileCollectionNode.Append("<FILESYSTEM>");

            sbFileCollectionNode.AppendFormat("<FILE {0}><NAME>{1}</NAME><DISPLAYNAME>{2}</DISPLAYNAME><PATH>{3}</PATH><FIELDNAME>ORIGINALFILE</FIELDNAME></FILE>",
            FileAttribute, ALCommon.TrimAndReplaceSpecialChars(UploadFileName), ALCommon.TrimAndReplaceSpecialChars(UploadFileName),
            ALCommon.TrimAndReplaceSpecialChars(UploadFilePath));

            sbFileCollectionNode.Append("</FILESYSTEM>");

            FileCollectionNode = sbFileCollectionNode.ToString();
            sbFormatXML = new StringBuilder(30);
            sbFormatXML.AppendFormat("<FORMAT><FORMATID>{0}</FORMATID>", FormatID);
            sbFormatXML.Append(sbFileCollectionNode);
            sbFormatXML.Append("</FORMAT>");
            FormatXML = sbFormatXML.ToString();
            FormatEngineObj = new FormatEngine(UserID, Language, TenantName);
            ErrorXML = FormatEngineObj.UpdateFormat(ref FormatXML, true);

            ArtworkProducerCache objAPCache = new ArtworkProducerCache(UserID, Language, TenantName);
            if (!Directory.Exists(ArtworkProducerCache.GetCacheFolderPath(TenantName, FormatID, ArtworkProducerCache.CacheType.FormatCache) + "\\Original"))
                ErrorXML = objAPCache.PrepareFormatCache(FormatID);

            #region Updating Main.html of Artwork Cache
            objAPCache.UpdateArtworkCacheOriginalFile(ArtworkID);
            #endregion

            ErrorXML = GetTextTagValues(ArtworkID, FormatID, out TextTagValues);

            #region XML for text element
            sbUpdateArtworkXML = new StringBuilder(30);
            sbTextTagXML = new StringBuilder(30);
            sbUpdateArtworkXML.Append("<PAGE><GROUPELEMENT>");

            xpDoc_TextTags = new XPathDocument(new StringReader(TextTagValues));
            xpNav_TextTags = xpDoc_TextTags.CreateNavigator();
            xpNav_TextTags = xpNav_TextTags.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK");
            xpTEIterator = xpNav_TextTags.Select("FORMAT_ARTWORK/FORMAT/FORMAT_TEXTELEMENT/TEXTELEMENT");

            xpDoc_OrgFile = new XPathDocument(new StringReader(ArtworkHTML));
            xpNav_OrgFile = xpDoc_OrgFile.CreateNavigator();

            String strContent = null;
            String CurrentTagName = null;

            while (xpTEIterator.MoveNext())
            {
                CurrentTagName = null;
                TextElementId = null;
                TextElementValueId = null;
                sbTextTagXML.Remove(0, sbTextTagXML.Length);
                sbTextTagXML.Append("<TEXTELEMENT MODIFIED='TRUE'>");

                TextElementId = xpTEIterator.Current.SelectSingleNode("TEXTELEMENTID").Value.ToString();
                sbTextTagXML.AppendFormat("<TEXTELEMENTID>{0}</TEXTELEMENTID>", TextElementId);

                CurrentTagName = xpTEIterator.Current.SelectSingleNode("TEXTTAGNAME").Value.ToString();
                sbTextTagXML.Append("<TEXTELEMENT_TEXTELEMENTVALUE TYPE='RELATION'>");

                TEValuenode = xpNav_TextTags.SelectSingleNode("ARTWORK_TEXTELEMENTVALUE/TEXTELEMENTVALUE [TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENT/TEXTELEMENTID/text()='" + TextElementId + "']");
                if (TEValuenode != null)
                {
                    TextElementValueId = TEValuenode.SelectSingleNode("TEXTELEMENTVALUEID").Value;
                    sbTextTagXML.Append("<TEXTELEMENTVALUE MODIFIED='TRUE'>");
                    sbTextTagXML.AppendFormat("<TEXTELEMENTVALUEID>{0}</TEXTELEMENTVALUEID>", TextElementValueId);
                }
                else //To create new node for Text Element
                    sbTextTagXML.Append("<TEXTELEMENTVALUE>");

                if (!String.IsNullOrEmpty(CurrentTagName))
                {
                    xpNav_Tag = xpNav_OrgFile.SelectSingleNode("//*[(starts-with(translate(@id,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'),'" + CurrentTagName.ToUpper().ToString() + "'))]");

                    strContent = null;
                    if (xpNav_Tag != null)
                        strContent = xpNav_Tag.InnerXml;

                    strContent = ALSpecialCharHandling.DecodeSpecialCharacters(strContent);
                    ALSpecialCharHandling.EncodeSpecialCharacters(ref strContent);
                    sbTextTagXML.AppendFormat("<TEXTELEMENTACTUALVALUE>{0}</TEXTELEMENTACTUALVALUE>", strContent);
                }
                sbTextTagXML.Append("</TEXTELEMENTVALUE>");
                sbTextTagXML.Append("</TEXTELEMENT_TEXTELEMENTVALUE>");
                sbTextTagXML.Append("</TEXTELEMENT>");
                sbUpdateArtworkXML.Append(sbTextTagXML);
            }

            #endregion

            #region XML for graphic elements

            sbGraphicTagXML = new StringBuilder(30);
            xpDoc_GraphicTags = new XPathDocument(new StringReader(TextTagValues));
            xpNav_GraphicTags = xpDoc_GraphicTags.CreateNavigator();
            xpNav_GraphicTags = xpNav_GraphicTags.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK");
            xpGEIterator = xpNav_TextTags.Select("FORMAT_ARTWORK/FORMAT/FORMAT_GRAPHICELEMENT/GRAPHICELEMENT");
            XPathNavigator GEValuenode = null;

            while (xpGEIterator.MoveNext())
            {
                CurrentTagName = null;
                GraphicElementId = null;
                GraphicElementValueId = null;
                sbGraphicTagXML.Remove(0, sbGraphicTagXML.Length);
                sbGraphicTagXML.Append("<GRAPHICELEMENT MODIFIED='TRUE'>");

                GraphicElementId = xpGEIterator.Current.SelectSingleNode("GRAPHICELEMENTID").Value.ToString();
                sbGraphicTagXML.AppendFormat("<GRAPHICELEMENTID>{0}</GRAPHICELEMENTID>", GraphicElementId);

                CurrentTagName = xpGEIterator.Current.SelectSingleNode("GRAPHICTAGNAME").Value.ToString();
                sbGraphicTagXML.Append("<GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO TYPE='RELATION'>");

                GEValuenode = xpNav_GraphicTags.SelectSingleNode("ARTWORK_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE [GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENT/GRAPHICELEMENTID/text()='" + GraphicElementId + "']");
                if (GEValuenode != null)
                {
                    GraphicElementValueId = GEValuenode.SelectSingleNode("GRAPHICELEMENTVALUEID").Value;
                    sbGraphicTagXML.Append("<GRAPHICELEMENTVALUE MODIFIED='TRUE'>");
                    sbGraphicTagXML.AppendFormat("<GRAPHICELEMENTVALUEID>{0}</GRAPHICELEMENTVALUEID>", GraphicElementValueId);
                }
                else//To create new node for Graphic Element
                    sbGraphicTagXML.Append("<GRAPHICELEMENTVALUE>");

                if (!String.IsNullOrEmpty(CurrentTagName))
                {
                    strURLContent = null;
                    ShowURL = null;
                    URL = null;
                    URLTarget = null;
                    URLType = null;
                    URLProtocol = null;

                    xpItr_TagList = xpDoc_OrgFile.CreateNavigator().Select("//*[(starts-with(translate(@id,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ'),'" + "A_" + CurrentTagName.ToUpper().ToString() + "'))]");
                    while (xpItr_TagList.MoveNext())
                        strURLContent = xpItr_TagList.Current.OuterXml;

                    if (String.IsNullOrEmpty(strURLContent))
                    {
                        ShowURL = "0";
                        URL = "";
                        URLTarget = "";
                        URLProtocol = "";
                        URLType = "NONE";

                        sbGraphicTagXML.AppendFormat("<SHOWURL>{0}</SHOWURL>", ShowURL);
                        sbGraphicTagXML.AppendFormat("<URLPROTOCOL>{0}</URLPROTOCOL>", URLProtocol);
                        sbGraphicTagXML.AppendFormat("<URL>{0}</URL>", URL);
                        sbGraphicTagXML.AppendFormat("<URLTARGET>{0}</URLTARGET>", URLTarget);
                        sbGraphicTagXML.AppendFormat("<URLTYPE>{0}</URLTYPE>", URLType);
                    }
                    else
                    {
                        ShowURL = "1";
                        ShowURL = ALSpecialCharHandling.DecodeSpecialCharacters(ShowURL);
                        ALSpecialCharHandling.EncodeSpecialCharacters(ref ShowURL);

                        sbGraphicTagXML.AppendFormat("<SHOWURL>{0}</SHOWURL>", ShowURL);

                        URLProtocol = strURLContent.Substring(strURLContent.ToUpper().IndexOf("HREF=\"") + 6, strURLContent.ToUpper().IndexOf("://") - strURLContent.ToUpper().IndexOf("HREF=\"") - 6);
                        URLProtocol = URLProtocol + "://";
                        URLProtocol = ALSpecialCharHandling.DecodeSpecialCharacters(URLProtocol);
                        ALSpecialCharHandling.EncodeSpecialCharacters(ref URLProtocol);
                        if (!String.IsNullOrEmpty(URLProtocol.Trim()))
                            sbGraphicTagXML.AppendFormat("<URLPROTOCOL>{0}</URLPROTOCOL>", URLProtocol);

                        int urlLastIndex = strURLContent.ToUpper().IndexOf("\" TARGET=");
                        if (urlLastIndex == -1)
                            urlLastIndex = strURLContent.ToUpper().IndexOf("\">");

                        URL = strURLContent.Substring(strURLContent.ToUpper().IndexOf(" HREF=") + 7, urlLastIndex - strURLContent.ToUpper().IndexOf(" HREF=") - 7);
                        URL = ALSpecialCharHandling.DecodeSpecialCharacters(URL);
                        ALSpecialCharHandling.EncodeSpecialCharacters(ref URL);
                        if (!String.IsNullOrEmpty(URL.Trim()))
                            sbGraphicTagXML.AppendFormat("<URL>{0}</URL>", URL);

                        URLTarget = strURLContent.Substring(strURLContent.ToUpper().LastIndexOf("TARGET=\"") + 8, strURLContent.ToUpper().IndexOf("\">") - strURLContent.ToUpper().LastIndexOf("TARGET=\"") - 8);
                        if (URLTarget.ToUpper().ToString().Equals("_BLANK"))
                            URLTarget = "NEW";
                        else
                            URLTarget = "SAME";
                        sbGraphicTagXML.AppendFormat("<URLTARGET>{0}</URLTARGET>", URLTarget);

                        URLType = "TRACKING";
                        URLType = ALSpecialCharHandling.DecodeSpecialCharacters(URLType);
                        ALSpecialCharHandling.EncodeSpecialCharacters(ref URLType);
                        if (!String.IsNullOrEmpty(URLType.Trim()))
                            sbGraphicTagXML.AppendFormat("<URLTYPE>{0}</URLTYPE>", URLType);
                    }
                }

                sbGraphicTagXML.Append("</GRAPHICELEMENTVALUE>");
                sbGraphicTagXML.Append("</GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO>");
                sbGraphicTagXML.Append("</GRAPHICELEMENT>");
                sbUpdateArtworkXML.Append(sbGraphicTagXML);
            }

            #endregion

            sbUpdateArtworkXML.Append("</GROUPELEMENT></PAGE>");
            ElementGroupXML = sbUpdateArtworkXML.ToString();

            #region UpdateArtwork Call
            ErrorXML = UpdateArtwork(ArtworkID, ref ElementGroupXML, false);
            //PrepareArtworkCache(ArtworkID);
            #endregion

            return ErrorXML;
        }

        public ERRORXML GetTextTagValues(String ArtworkID, String FormatId, out String TextTagValues)
        {
            ERRORXML ErrorXML = null;
            TextTagValues = null;
            StringBuilder sbSearchXml = null;
            OMInterface OMIObj = null;

            sbSearchXml = new StringBuilder(255);
            sbSearchXml.Append(@"<SEARCH>
                            <TARGETOBJECT>ARTWORK</TARGETOBJECT>
                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                           <TARGETENTITY>
                              <PRIVILEGENAME>READ</PRIVILEGENAME>
                              <FILTERLIST>
                                <FILTER>
                                  <EXPRESSION>ARTWORKID</EXPRESSION>
                                  <OPERATOR>EQUALTO</OPERATOR>
                                  <EXPRESSIONVALUE>" + ArtworkID + @"</EXPRESSIONVALUE>
                                  <FILTERID>1</FILTERID>
                                  <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                </FILTER>
                              </FILTERLIST>
                              <SELECTLIST>
                                <EXPRESSION>
                                  <FIELDNAME>ARTWORKID</FIELDNAME>
                                  <ALIAS>ARTWORKID</ALIAS>
                                </EXPRESSION>
                              </SELECTLIST>
                             <RELATIONSHIP NAME='ARTWORK_TEXTELEMENTVALUE'>
                                <SELECTLIST>
                                  <EXPRESSION>
                                    <FIELDNAME>TEXTELEMENTVALUEID</FIELDNAME>
                                    <ALIAS>TEXTELEMENTVALUEID</ALIAS>
                                  </EXPRESSION>
                                  <EXPRESSION>
                                    <FIELDNAME>TEXTELEMENTID</FIELDNAME>
                                    <ALIAS>TEXTELEMENTID</ALIAS>
                                  </EXPRESSION>
                                </SELECTLIST>
                                <RELATIONSHIP NAME='TEXTELEMENT_TEXTELEMENTVALUE'>
                                  <SELECTLIST>
                                    <EXPRESSION>
                                      <FIELDNAME>TEXTELEMENTID</FIELDNAME>
                                      <ALIAS>TEXTELEMENTID</ALIAS>
                                    </EXPRESSION>
                                  </SELECTLIST>
                                </RELATIONSHIP>
                              </RELATIONSHIP>
                              <RELATIONSHIP NAME='ARTWORK_GRAPHICELEMENTVALUEINFO'>
                                <RELATIONSHIP NAME='GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO'>
                                  <SELECTLIST>
                                    <EXPRESSION>
                                      <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                      <ALIAS>GRAPHICELEMENTID</ALIAS>
                                    </EXPRESSION>
                                  </SELECTLIST>
                                </RELATIONSHIP>
                              </RELATIONSHIP>
                              <RELATIONSHIP NAME='ARTWORK_GRAPHICELEMENT'>
                                <SELECTLIST>
                                  <EXPRESSION>
                                    <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                    <ALIAS>GRAPHICELEMENTID</ALIAS>
                                  </EXPRESSION>
                                </SELECTLIST>
                              </RELATIONSHIP>
                              <RELATIONSHIP NAME='FORMAT_ARTWORK'>
                                <SELECTLIST>
                                  <EXPRESSION>
                                    <FIELDNAME>FORMATID</FIELDNAME>
                                    <ALIAS>FORMATID</ALIAS>
                                  </EXPRESSION>
                                </SELECTLIST>
                                <FILTERLIST>
                                  <FILTER>
                                    <EXPRESSION>FORMATID</EXPRESSION>
                                    <OPERATOR>EQUALTO</OPERATOR>
                                    <EXPRESSIONVALUE>" + FormatId + @"</EXPRESSIONVALUE>
                                    <FILTERID>1</FILTERID>
                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                  </FILTER>
                                </FILTERLIST>
                                <RELATIONSHIP NAME='FORMAT_TEXTELEMENT'>
                                  <SELECTLIST>
                                    <EXPRESSION>
                                      <FIELDNAME>TEXTELEMENTID</FIELDNAME>
                                      <ALIAS>TEXTELEMENTID</ALIAS>
                                    </EXPRESSION>
                                    <EXPRESSION>
                                      <FIELDNAME>TEXTTAGNAME</FIELDNAME>
                                      <ALIAS>TEXTTAGNAME</ALIAS>
                                    </EXPRESSION>
                                  </SELECTLIST>                                  
                                </RELATIONSHIP>
                                <RELATIONSHIP NAME='FORMAT_GRAPHICELEMENT'>
                                  <SELECTLIST>
                                    <EXPRESSION>
                                      <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                      <ALIAS>GRAPHICELEMENTID</ALIAS>
                                    </EXPRESSION>
                                    <EXPRESSION>
                                      <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
                                      <ALIAS>GRAPHICTAGNAME</ALIAS>
                                    </EXPRESSION>
                                  </SELECTLIST>                                  
                                </RELATIONSHIP>
                              </RELATIONSHIP>
                            </TARGETENTITY>
                          </SEARCH>");
            OMIObj = new OMInterface(UserID, Language, TenantName);
            ErrorXML = OMIObj.GetHierarchialData(sbSearchXml.ToString(), ref TextTagValues);

            return ErrorXML;
        }

        /// <summary>
        /// Below code is to be called when their is need to generate an artwork having no format i.e. Blank Artwork
        /// </summary>
        /// <param name="ArtworkXML"></param>
        /// <returns></returns>
        public ERRORXML CreateBlankArtwork(ref XML ArtworkXML)
        {
            ERRORXML ErrorXML = null;
            ArtworkProducerCache objAPCache = new ArtworkProducerCache(UserID, Language, TenantName);
            OMInterface OMI = null;
            OperationQueue InputXML = null;
            Operation OperationNode = null;
            XML OMInputXML = null;
            ArtworkProducerCommon ArtworkProducerCommonObj = null;

            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine CreateBlankArtwork Starts");
                ArtworkProducerCommonObj = new ArtworkProducerCommon(UserID, Language, TenantName);
                StringBuilder CreateAuthorizationXML = null;
                string ArtworkID = null;

                objErrorCommon = new ALErrorCommon();
                ErrorXML = ArtworkProducerEngine.CreateAndUpdateData(ref ArtworkXML, "CREATEDATA", UserID, Language, TenantName);

                //Add authorization entry for artwork
                if (String.IsNullOrEmpty(ErrorXML))
                {
                    ArtworkID = ALCommon.GetInnerValue(ref ArtworkXML, "<ARTWORKID>", "</ARTWORKID>");

                    #region Prepare Authorization XML for Creator of Book
                    CreateAuthorizationXML = new StringBuilder();
                    CreateAuthorizationXML.Append("<AUTHORIZATION>");
                    CreateAuthorizationXML.Append("<OBJECTINSTANCE>");
                    CreateAuthorizationXML.AppendFormat("<OBJECTID>{0}</OBJECTID>", ArtworkID);
                    CreateAuthorizationXML.Append("<OBJECTTYPE>ENTITY</OBJECTTYPE>");
                    CreateAuthorizationXML.Append("<OBJECTTYPENAME>ARTWORK</OBJECTTYPENAME>");
                    CreateAuthorizationXML.Append("<ACCESSCONTROLLIST>");
                    CreateAuthorizationXML.Append("<ACCESSCONTROLENTRY>");
                    CreateAuthorizationXML.AppendFormat("<PRIVILEGENAME>DELETE</PRIVILEGENAME>");
                    CreateAuthorizationXML.Append("<ACCESSORTYPE>USER</ACCESSORTYPE>");
                    CreateAuthorizationXML.AppendFormat("<ACCESSORID>{0}</ACCESSORID>", UserID);
                    CreateAuthorizationXML.Append("</ACCESSCONTROLENTRY>");
                    CreateAuthorizationXML.Append("</ACCESSCONTROLLIST>");
                    CreateAuthorizationXML.Append("</OBJECTINSTANCE>");
                    CreateAuthorizationXML.Append("</AUTHORIZATION>");
                    #endregion

                    OMI = new OMInterface(UserID, Language, TenantName);
                    InputXML = new OperationQueue(true);
                    OperationNode = new Operation("AUTHORIZATION", "CREATE");
                    OperationNode.AddParameter("INPUTXML", CreateAuthorizationXML.ToString());
                    InputXML.AddOperation(OperationNode);
                    OMInputXML = InputXML.Xml;
                    ErrorXML = OMI.Execute(ref OMInputXML);
                }

                if (ErrorXML != null)
                {
                    objErrorCommon.AppendError("APENGINE_ARTWORK_CREATEARTWORK_FAILED", this.GetType().ToString(), "CreateBlankArtwork", null, ErrorXML);
                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                }
                else
                {
                    ErrorXML = null;
                    ArtworkID = ALCommon.GetInnerValue(ref ArtworkXML, "<ARTWORKID>", "</ARTWORKID>");

                    objErrorCommon = new ALErrorCommon();

                    #region Create Artwork Section
                    String tempArtworkName = null;
                    String ArtworkName = ALCommon.GetInnerValue(ref ArtworkXML, "<ARTWORKNAME>", "</ARTWORKNAME>");
                    ArtworkProducerCommon ArtCommonObj = new ArtworkProducerCommon(UserID, Language, TenantName);
                    String SectionID = null;
                    ArtCommonObj.Mode = ArtworkProducerCommon.ModeType.Generation;

                    tempArtworkName = ArtworkName;
                    ErrorXML = ArtworkProducerEngine.ConvertToAlphaNumericString(ref tempArtworkName);

                    if (String.IsNullOrEmpty(ErrorXML))
                        ErrorXML = ArtCommonObj.CreateSection(tempArtworkName + "_" + ArtworkID, null, out SectionID);
                    else
                        ErrorXML = ArtCommonObj.CreateSection(ArtworkName + "_" + ArtworkID, null, out SectionID);

                    if (ErrorXML != null)
                    {
                        objErrorCommon.AppendError("APENGINE_ARTWORK_CREATEARTWORK_CREATESECTION_FAILED", this.GetType().ToString(), "CreateBlankArtwork", null, ErrorXML);
                        ErrorXML = objErrorCommon.ErrorCollectionXML;
                    }

                    StringBuilder sbAssoSectionArtwork = new StringBuilder(50);
                    sbAssoSectionArtwork.AppendFormat(@"<ARTWORK><ARTWORKID>{1}</ARTWORKID><ARTWORK_SECTION TYPE='RELATION'>{0}</ARTWORK_SECTION></ARTWORK>", SectionID, ArtworkID);
                    String UpdateXML = sbAssoSectionArtwork.ToString();
                    ErrorXML = UpdateArtwork(ref UpdateXML, false);

                    #endregion
                }

                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_ARTWORK_CREATEARTWORK_EXCEPTION", this.GetType().ToString(), "CreateBlankArtwork", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                return ReturnErrorXML;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine CreateBlankArtwork Ends");
            }
        }

        /// <alsummary>
        /// <createddate>15 Feb 2010</createddate>
        /// <author>Anand Mahadik</author>
        /// <purpose>This method is used to create multiple Artworks for a Artwork job based on the input XML.</purpose>
        /// <param name="JobID"></param>
        /// <param name="ArtworkXML">
        ///  <xmlstructure><![CDATA[
        ///                     <ARTWORKCOLLECTION>
        ///                     <ARTWORK>
        ///                         <ARTWORKNAME/>
        ///                         <ARTWORKDESCRIPTION/>
        ///                         <FORMATID/>
        ///                     </ARTWORK>
        ///                     </ARTWORKCOLLECTION>
        /// ]]></xmlstructure>
        ///  </param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        public ERRORXML CreateBatchArtworks(String JobID, ref XML ArtworkCollectionXML)
        {
            ERRORXML ErrorXML = null;
            XmlDocument ArtworkCollectionXMLDoc = null;
            EntityID ArtworkID = 0;
            XmlNode ArtworkIDNode = null;
            XmlNode TempNode = null;
            XmlNodeList ArtworkNodes = null;
            OperationQueue QMQueueObj = null;
            Operation OperObj = null;
            OMInterface OMIObj = null;
            XML OperationXML = null;
            StringBuilder SB_ArtworkXML = null;
            String ArtworkName = null;
            String ArtworkDesc = null;
            String FormatID = null;
            String ErrorMessage = null;
            String ArtworkInfoXML = null;
            ArtworkJobEngine ArtworkJobEngineObj = null;
            ArtworkProducerCommon ArtworkProducerCommonObj = null;
            ArtworkJobEngine.BatchJobStatus JobStatus;
            ArtworkJobEngine.BatchJobStatus PreviousJobStatus;
            String SectionID = null;
            String ArtworkSectionRelation = null;
            Assetlink.Services.Engines.ArtworkProducerEngine.ArtworkProducerCommon ArtCommonObj = null;
            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine CreateBatchArtworks Starts");
                ArtworkJobEngineObj = new ArtworkJobEngine(UserID, Language, TenantName);
                strdict_AdditionalInfo = new StringDictionary();
                ArtworkCollectionXMLDoc = new XmlDocument();
                SB_ArtworkXML = new StringBuilder(255);
                QMQueueObj = new OperationQueue(true);
                ArtworkProducerCommonObj = new ArtworkProducerCommon(UserID, Language, TenantName);

                ArtworkCollectionXMLDoc.LoadXml(ArtworkCollectionXML);
                ArtworkNodes = ArtworkCollectionXMLDoc.DocumentElement.SelectNodes("ARTWORK");
                foreach (XmlNode ArtworkNode in ArtworkNodes)
                {
                    SB_ArtworkXML.Remove(0, SB_ArtworkXML.Length);

                    UniqueID.Read(ref ArtworkID);

                    FormatID = ArtworkNode.SelectSingleNode("FORMATID").InnerText;
                    ArtworkIDNode = ArtworkNode.SelectSingleNode("ARTWORKID");
                    ((XmlElement)ArtworkNode).SetAttribute("Status", "Pending");

                    if (ArtworkIDNode == null)
                    {
                        ArtworkIDNode = ArtworkCollectionXMLDoc.CreateElement("ARTWORKID");
                        ArtworkNode.AppendChild(ArtworkIDNode);
                    }

                    ArtworkIDNode.InnerText = ArtworkID.ToString();
                    ArtworkName = ArtworkNode.SelectSingleNode("ARTWORKNAME").InnerText;
                    TempNode = ArtworkNode.SelectSingleNode("ARTWORKDESCRIPTION");
                    if (TempNode != null)
                        ArtworkDesc = TempNode.InnerText;
                    else
                        ArtworkDesc = String.Empty;

                    if (ArtworkNode.SelectSingleNode("WIDTH") != null || ArtworkNode.SelectSingleNode("HEIGHT") != null)
                    {
                        ArtworkInfoXML = @"<ARTWORK_ARTWORKINFO TYPE='RELATION'>
                                                <ARTWORKINFO>"
                                                + @"<CUSTOM_HEIGHT>" + ArtworkNode.SelectSingleNode("HEIGHT").InnerText + "</CUSTOM_HEIGHT>"
                                                + @"<CUSTOM_WIDTH>" + ArtworkNode.SelectSingleNode("WIDTH").InnerText + "</CUSTOM_WIDTH>"
                                            + @"</ARTWORKINFO>
                                            </ARTWORK_ARTWORKINFO>";
                    }
                    else
                        ArtworkInfoXML = String.Empty;

                    //Create section
                    ArtworkProducerEngine.ConvertToAlphaNumericString(ref ArtworkName);
                    ArtCommonObj = new Assetlink.Services.Engines.ArtworkProducerEngine.ArtworkProducerCommon(UserID, Language, TenantName);
                    ArtCommonObj.Mode = Assetlink.Services.Engines.ArtworkProducerEngine.ArtworkProducerCommon.ModeType.Generation;
                    ErrorXML = ArtCommonObj.CreateSection(ArtworkName + "_" + ArtworkID.ToString(), null, out SectionID);

                    if (ErrorXML == null)
                        ArtworkSectionRelation = "<ARTWORK_SECTION TYPE='RELATION'>" + SectionID + "</ARTWORK_SECTION>";

                    SB_ArtworkXML.AppendFormat(@"<ARTWORK>
                                                     <ARTWORKID>{0}</ARTWORKID>
                                                     <ARTWORKNAME>{1}</ARTWORKNAME>
                                                     <ARTWORKDESCRIPTION>{2}</ARTWORKDESCRIPTION>
                                                     <ARTWORKSTATUS>Files Pending</ARTWORKSTATUS>
                                                     <DISPLAYSTATUS>Draft</DISPLAYSTATUS>
                                                     <CREATEDFROM>BATCH ARTWORK GENERATION</CREATEDFROM>
                                                     <CREATEDON>{3}</CREATEDON>
                                                     <MODIFIEDON>{3}</MODIFIEDON>
                                                        <ARTWORK_CREATOR TYPE='RELATION'>{4}</ARTWORK_CREATOR>
                                                        <ARTWORK_MODIFIER TYPE='RELATION'>{4}</ARTWORK_MODIFIER>
                                                        <FORMAT_ARTWORK TYPE='RELATION' PARENTID='TRUE'>{5}</FORMAT_ARTWORK>
                                                        <ARTWORKJOB_ARTWORK TYPE='RELATION' PARENTID='TRUE'>{6}</ARTWORKJOB_ARTWORK>
                                                        {7}
                                                        {8}
                                                 </ARTWORK>", ArtworkID, ALCommon.TrimAndReplaceSpecialChars(ArtworkName),
                                                              ALCommon.TrimAndReplaceSpecialChars(ArtworkDesc),
                                                              ALCommon.CurrentDateTimeToXml(), UserID, FormatID, JobID, ArtworkInfoXML, ArtworkSectionRelation);

                    OperObj = new Operation("ENTITY", "CREATEDATA");
                    OperObj.AddParameter("INPUTXML", SB_ArtworkXML.ToString());
                    QMQueueObj.AddOperation(OperObj);

                    //Increment Format's GenerationCount value
                    ErrorXML = ArtworkProducerCommonObj.IncrementGenerationCount(FormatID, ArtworkProducerCommon.EntityType.Format, false, out OperObj);
                    QMQueueObj.AddOperation(OperObj);
                }

                OperationXML = QMQueueObj.Xml;
                OMIObj = new OMInterface(UserID, Language, TenantName);
                ErrorXML = OMIObj.Execute(ref OperationXML);

                if (!String.IsNullOrEmpty(ErrorXML))
                {
                    objErrorCommon.AppendError("APENGINE_ARTWORK_BATCH_CREATION_FAILED", this.GetType().ToString(), "CreateBatchArtworks", null, ErrorXML);
                    return objErrorCommon.ErrorCollectionXML;
                }

                ArtworkJobEngineObj.CalculateJobProgress(JobID, out PreviousJobStatus, out JobStatus);

                ArtworkCollectionXML = ArtworkCollectionXMLDoc.InnerXml;

                return null;
            }
            catch (Exception ExcepObj)
            {
                strdict_AdditionalInfo.Add("APINAME", "CreateBatchArtworks()");
                strdict_AdditionalInfo.Add("MESSAGE", ExcepObj.ToString());
                ErrorMessage = "Exception thrown in batch Artwork creation. Exception details: " + ExcepObj.Message;
                ArtworkJobEngineObj.LogBatchJobError(JobID, "APENGINE_ARTWORK_EXCEPTION_OCCURED", ErrorMessage, null, ErrorType.EXCEPTION, strdict_AdditionalInfo);

                objErrorCommon.AppendError("APENGINE_ARTWORK_CREATEARTWORK_EXCEPTION", this.GetType().ToString(), "CreateBatchArtworks", ExcepObj, null);
                return objErrorCommon.ErrorCollectionXML;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine CreateBatchArtworks Ends");
            }
        }

        /// <alsummary>
        /// <createddate>27 July 2011</createddate>
        /// <author>Amit chaudhari</author>
        /// <purpose>This method is overwritten method for UpDateArtwork with Async flag parameter.</purpose>
        ///  <xmlstructure><![CDATA[
        /// ReturnXML       :  <PAGE>
        ///                     <GROUPELEMENT>
        ///                     	<TEXTELEMENT MODIFIED="TRUE">
        ///                         		<TEXTELEMENTID>131087275470205</TEXTELEMENTID>
        ///                             	<TEXTELEMENT_TEXTELEMENTVALUE TYPE="RELATION">
        ///                     			    <TEXTELEMENTVALUE @MODIFIED="TRUE">
        ///                                             <TEXTELEMENTVALUEID/>               //[Modified attribute and TextElementValudID is given in case of Update only.]
        ///                             		    	<TEXTELEMENTACTUALVALUE/>  
        ///                     			    </TEXTELEMENTVALUE>
        ///                         		</TEXTELEMENT_TEXTELEMENTVALUE>
        ///                     	</TEXTELEMENT>
        ///                     	<ARTWORK_GRAPHICELEMENT TYPE="RELATION">
        ///                 		    <GRAPHICELEMENT>
        ///                 			    <FILESYSTEM>
        ///                 				    <FILE>
        ///     			            		    <NAME>orchidplant.jpg</NAME>
        ///                 					    <PATH>D:\Assetlink58\mom\UploadedFiles\210079462842129\orchidplant.jpg</PATH>
        ///                 					    <FIELDNAME>ORIGINALFILE</FIELDNAME>
        ///                 				    </FILE>
        ///         				        </FILESYSTEM>
        ///             			        <GRAPHICELEMENT_REPLACEDFILE>222828869939788</GRAPHICELEMENT_REPLACEDFILE>
        ///                 		    </GRAPHICELEMENT>
        ///                 	    </ARTWORK_GRAPHICELEMENT>
        ///                     </GROUPELEMENT>
        ///                     <GROUPELEMENT>
        ///                     </GROUPELEMENT>
        ///                     </<PAGE>
        ///     
        /// ]]></xmlstructure>
        /// <param name="ArtworkID">Artwork Id</param>
        ///  <param name="TextAndGraphicElementValueXML"></param>
        ///  <param name="Async">Async call for getting Graphic element value</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        public ERRORXML UpdateArtwork(String ArtworkID, ref XML TextAndGraphicElementValueXML)
        {
            return UpdateArtwork(ArtworkID, ref TextAndGraphicElementValueXML, true);
        }

        /// <alsummary>
        /// <createddate>2 December 2006</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>This method is used to create or update Graphic Element value and Text Element value associated with Artwork.</purpose>
        ///  <xmlstructure><![CDATA[
        /// ReturnXML       :  <PAGE>
        ///                     <GROUPELEMENT>
        ///                     	<TEXTELEMENT MODIFIED="TRUE">
        ///                         		<TEXTELEMENTID>131087275470205</TEXTELEMENTID>
        ///                             	<TEXTELEMENT_TEXTELEMENTVALUE TYPE="RELATION">
        ///                     			    <TEXTELEMENTVALUE @MODIFIED="TRUE">
        ///                                             <TEXTELEMENTVALUEID/>               //[Modified attribute and TextElementValudID is given in case of Update only.]
        ///                             		    	<TEXTELEMENTACTUALVALUE/>  
        ///                     			    </TEXTELEMENTVALUE>
        ///                         		</TEXTELEMENT_TEXTELEMENTVALUE>
        ///                     	</TEXTELEMENT>
        ///                     	<ARTWORK_GRAPHICELEMENT TYPE="RELATION">
        ///                 		    <GRAPHICELEMENT>
        ///                 			    <FILESYSTEM>
        ///                 				    <FILE>
        ///     			            		    <NAME>orchidplant.jpg</NAME>
        ///                 					    <PATH>D:\Assetlink58\mom\UploadedFiles\210079462842129\orchidplant.jpg</PATH>
        ///                 					    <FIELDNAME>ORIGINALFILE</FIELDNAME>
        ///                 				    </FILE>
        ///         				        </FILESYSTEM>
        ///             			        <GRAPHICELEMENT_REPLACEDFILE>222828869939788</GRAPHICELEMENT_REPLACEDFILE>
        ///                 		    </GRAPHICELEMENT>
        ///                 	    </ARTWORK_GRAPHICELEMENT>
        ///                     </GROUPELEMENT>
        ///                     <GROUPELEMENT>
        ///                     </GROUPELEMENT>
        ///                     </<PAGE>
        ///     
        /// ]]></xmlstructure>
        /// <param name="ArtworkID">Artwork Id</param>
        ///  <param name="TextAndGraphicElementValueXML"></param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        public ERRORXML UpdateArtwork(String ArtworkID, ref XML TextAndGraphicElementValueXML, bool MakeAsyncCall)
        {
            #region variable

            ERRORXML ErrorXML = null;
            String BookID = null;
            XML strBookUpdateXML = null;
            String DisplayStatus = null;
            ArtworkProducerCommon helperObj = null;
            String FormatID = null;
            ArtworkProducerCache objAPCache = null;
            #endregion

            try
            {
                helperObj = new ArtworkProducerCommon(UserID, Language, TenantName, SolutionName);

                #region Update Book Status to "Draft"
                if (IsBookArtwork(UserID, Language, TenantName, ArtworkID, out BookID))
                {
                    GetFormatIDForArtwork(ArtworkID, out FormatID);
                    objAPCache = new ArtworkProducerCache(UserID, Language, TenantName);
                    bool bretVal = objAPCache.ValidateFormatCache(FormatID);
                    if (bretVal)
                    {
                        bretVal = objAPCache.ValidateArtworkCacheWithFormatCache(ArtworkID, FormatID);
                        if (!bretVal)
                            objAPCache.PrepareArtworkCache(ArtworkID, "EDIT");
                    }
                    else
                    {
                        objAPCache.PrepareFormatCache(FormatID);
                        objAPCache.PrepareArtworkCache(ArtworkID, "EDIT");
                    }

                    strBookUpdateXML = "<BOOK><BOOKID>" + BookID + "</BOOKID><STATUS>Draft</STATUS><DISPLAYSTATUS>" + helperObj.GetDisplayStatus("Book", "Draft", out DisplayStatus) + "</DISPLAYSTATUS><MODIFIEDON>" + ALCommon.CurrentDateTimeToXml() + "</MODIFIEDON></BOOK>";
                    ErrorXML = ArtworkProducerEngine.CreateAndUpdateData(ref strBookUpdateXML, "UPDATEDATA", UserID, Language, TenantName);
                    if (!String.IsNullOrEmpty(ErrorXML))
                    {
                        objErrorCommon.AppendError("APENGINE_BOOK_STATUS_UPDATE_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                        return objErrorCommon.ErrorCollectionXML;
                    }
                }
                #endregion

                #region Update Artwork in order to associat it with GE and TE
                if (ErrorXML == null)
                {
                    ErrorXML = UpdateArtworkOnly(ArtworkID, ref TextAndGraphicElementValueXML, MakeAsyncCall);
                }
                return null;

                #endregion
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_EXCPETION", this.GetType().ToString(), "UpdateArtwork", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                return ReturnErrorXML;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine UpdateArtwork Ends");
            }
        }

        public ERRORXML UpdateArtworkOnly(String ArtworkID, ref XML TextAndGraphicElementValueXML, bool MakeAsyncCall)
        {
            #region variable
            StringReader srXPathXML = null;
            XPathDocument XPathDoc = null;
            XPathNavigator XPathNav = null;
            XPathNavigator XPathNode = null;
            XPathNodeIterator XPNodeItr = null;
            XPathNodeIterator XPathNodeItr = null;
            ERRORXML ErrorXML = null;
            //StringBuilder               MainErrorXML        =   null;
            TextElementEngine TextElementEngineObj = null;
            XML TextElementXML = null;
            StringBuilder ArtworkXML = null;
            String TextElementValueID = null;
            String InputArtworkXML = null;
            GraphicElementEngine GEEngineObj = null;
            XML GraphicElementValueXML = null;
            String GraphicElementID = null;
            XML TempXML = null;
            XPathDocument GEXPathDoc = null;
            XPathNavigator GEXPathNav = null;
            StringReader GEStringReaderObj = null;
            XPathNavigator GEXPathNode = null;
            ArtworkProducerCache objAPCache = null;
            ArtworkProducerCommon helperObj = null;
            String DisplayStatus = null;
            XPathNavigator xpNav_Dimension = null;
            String str_Dimension = null;
            String FilePath = null;
            String Org_GEID = null;
            #endregion

            try
            {
                helperObj = new ArtworkProducerCommon(UserID, Language, TenantName, SolutionName);

                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine UpdateArtwork Starts");

                objErrorCommon = new ALErrorCommon();
                TextElementEngineObj = new TextElementEngine(UserID, Language, TenantName);
                GEEngineObj = new GraphicElementEngine(UserID, Language, TenantName);
                objAPCache = new ArtworkProducerCache(UserID, Language, TenantName);
                #region Update Artwork in order to associat it with GE and TE

                ArtworkXML = new StringBuilder();
                ArtworkXML.Append("<ARTWORK MODIFIED='TRUE'>");
                ArtworkXML.AppendFormat("<ARTWORKID>{0}</ARTWORKID>", ArtworkID);
                ArtworkXML.Append("<ARTWORKSTATUS>Draft</ARTWORKSTATUS><DISPLAYSTATUS>" + helperObj.GetDisplayStatus("Artwork", "Draft", out DisplayStatus) + "</DISPLAYSTATUS>");

                #region To add 'ModifiedOn' date, if ArtworkXML doesn't contain it.
                ArtworkXML.AppendFormat("<MODIFIEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></MODIFIEDON>"
                                                            , DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour, DateTime.Now.Minute, DateTime.Now.Second);

                #endregion

                #region Create/Update GE and TE and associate it with artwork

                srXPathXML = new StringReader(TextAndGraphicElementValueXML);
                XPathDoc = new XPathDocument(srXPathXML);
                XPathNav = XPathDoc.CreateNavigator();
                #region Disassociate Old modifier
                XPathNode = XPathNav.SelectSingleNode("//ARTWORK_MODIFIER");
                if ((XPathNode != null) && (XPathNode.GetAttribute("DISASSOCIATE", "") == "TRUE"))
                {
                    ArtworkXML.AppendFormat("<ARTWORK_MODIFIER TYPE='RELATION' DISASSOCIATE='TRUE' >{0}</ARTWORK_MODIFIER>", XPathNode.InnerXml);
                }
                else
                {
                    String LastModifierID = null;
                    ErrorXML = GetArtworkLastModifier(ArtworkID, out LastModifierID);
                    if (!String.IsNullOrEmpty(LastModifierID))
                    {
                        ArtworkXML.AppendFormat("<ARTWORK_MODIFIER TYPE='RELATION' DISASSOCIATE='TRUE' >{0}</ARTWORK_MODIFIER>", LastModifierID);
                    }
                }
                XPathNode = null;
                #endregion


                #region Associate Text Element Value with Artwork
                XPathNodeItr = XPathNav.Select("//TEXTELEMENT");
                while (XPathNodeItr.MoveNext())
                {
                    TextElementXML = XPathNodeItr.Current.OuterXml;
                    TempXML = ALCommon.GetInnerValue(ref TextElementXML, "<TEXTELEMENTVALUEID>", "</TEXTELEMENTVALUEID>");
                    ErrorXML = TextElementEngineObj.UpdateTextElement(ref TextElementXML);
                    if (ErrorXML == null)
                    {
                        if (TempXML == null)
                        {
                            TextElementValueID = ALCommon.GetInnerValue(ref TextElementXML, "<TEXTELEMENTVALUEID>", "</TEXTELEMENTVALUEID>");
                            if (TextElementValueID != null)
                                ArtworkXML.AppendFormat("<ARTWORK_TEXTELEMENTVALUE TYPE='RELATION'>{0}</ARTWORK_TEXTELEMENTVALUE>", TextElementValueID);
                        }

                        String TagValue = XPathNodeItr.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENTVALUE/TEXTELEMENTACTUALVALUE").InnerXml;
                        String TEID = XPathNodeItr.Current.SelectSingleNode("TEXTELEMENTID").InnerXml;
                        if (!objAPCache.ModifyTextTagValue(ArtworkID, TEID, TagValue))
                        {
                            objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_UPDATECACHE_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                            ErrorXML = objErrorCommon.ErrorCollectionXML;
                            return ErrorXML;
                        }
                    }
                    else
                    {
                        objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_UPDATETEXTELEMENT_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                        ErrorXML = objErrorCommon.ErrorCollectionXML;
                        return ErrorXML;
                        //if (MainErrorXML == null)
                        //{
                        //    MainErrorXML = new StringBuilder();
                        //    MainErrorXML.Append("<GROUPERROR>");
                        //}
                        //MainErrorXML.Append(ErrorXML);
                    }

                    ErrorXML = null;
                    TextElementXML = null;
                    TextElementValueID = null;
                    TempXML = null;


                }
                XPathNodeItr = null;

                #endregion

                #region Process Update cache node
                XPathNodeItr = XPathNav.Select("//UPDATECACHE/TE");
                while (XPathNodeItr.MoveNext())
                {
                    String TagValue = XPathNodeItr.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEVALUE/TEXTELEMENTACTUALVALUE").InnerXml;
                    String TEID = XPathNodeItr.Current.SelectSingleNode("TEXTELEMENTID").InnerXml;
                    if (!objAPCache.ModifyTextTagValue(ArtworkID, TEID, TagValue))
                    {
                        objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_UPDATECACHE_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                        ErrorXML = objErrorCommon.ErrorCollectionXML;
                        return ErrorXML;
                    }
                }
                XPathNodeItr = null;
                #endregion

                #region Assoicate Graphic Element with Artwork
                XPathNodeItr = XPathNav.Select("//GROUPELEMENT/GRAPHICELEMENT");
                if (XPathNodeItr.Count > 0)
                {
                    while (XPathNodeItr.MoveNext())
                    {
                        bool bCacheUpdated = false;
                        GraphicElementValueXML = XPathNodeItr.Current.OuterXml;
                        String Org_GraphicElementValueXML = GraphicElementValueXML;
                        ErrorXML = GEEngineObj.UpdateGraphicElement(ref GraphicElementValueXML);
                        if (ErrorXML == null)
                        {
                            GEStringReaderObj = new StringReader(Org_GraphicElementValueXML);
                            GEXPathDoc = new XPathDocument(GEStringReaderObj);
                            GEXPathNav = GEXPathDoc.CreateNavigator();

                            XPathDocument xpDoc_NewGEValues = new XPathDocument(new StringReader(GraphicElementValueXML));

                            #region Graphic Element Value
                            GEXPathNode = GEXPathNav.SelectSingleNode("GRAPHICELEMENT/GRAPHICELEMENT_REPLACEDFILE/GRAPHICELEMENT");
                            if (GEXPathNode != null)
                            {
                                GraphicElementID = xpDoc_NewGEValues.CreateNavigator().SelectSingleNode("GRAPHICELEMENT/GRAPHICELEMENT_REPLACEDFILE/GRAPHICELEMENT/GRAPHICELEMENTID").InnerXml;
                                if ((GEXPathNode.GetAttribute("MODIFIED", "") == "") || (GEXPathNode.GetAttribute("MODIFIED", "") == null))
                                {
                                    if (GraphicElementID != null)
                                        ArtworkXML.AppendFormat("<ARTWORK_GRAPHICELEMENT TYPE='RELATION'>{0}</ARTWORK_GRAPHICELEMENT>", GraphicElementID);

                                }

                                //GraphicElementID = GEXPathNode.SelectSingleNode("GRAPHICELEMENTID").InnerXml;
                                XPathNavigator xpNav_Temp = GEXPathNav.SelectSingleNode("GRAPHICELEMENT/GRAPHICELEMENT_REPLACEDFILE/GRAPHICELEMENT/FILESYSTEM/FILE/PATH");
                                if (xpNav_Temp != null)
                                {
                                    FilePath = GEXPathNav.SelectSingleNode("GRAPHICELEMENT/GRAPHICELEMENT_REPLACEDFILE/GRAPHICELEMENT/FILESYSTEM/FILE/PATH").Value;
                                    Org_GEID = XPathNodeItr.Current.SelectSingleNode("GRAPHICELEMENTID").InnerXml;
                                    xpNav_Dimension = XPathNodeItr.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/DIMENSIONS");
                                    if (xpNav_Dimension != null)
                                    {
                                        str_Dimension = xpNav_Dimension.Value;
                                        objAPCache.FileDimensions = str_Dimension;
                                    }

                                    if (!objAPCache.ModifyGraphicTagValue(ArtworkID, Org_GEID, FilePath, ArtworkProducerCache.GraphicTagUpdateType.UPLOAD, ArtworkProducerCache.GraphicTagUpdateStatus.PENDING, MakeAsyncCall))
                                    {
                                        objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_UPDATECACHE_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                                        ErrorXML = objErrorCommon.ErrorCollectionXML;
                                        return ErrorXML;
                                    }
                                    bCacheUpdated = true;
                                    objAPCache.FileDimensions = null;
                                }
                                else //if the same external file's dimensions are changed
                                {
                                    xpNav_Dimension = XPathNodeItr.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/DIMENSIONS");
                                    if (xpNav_Dimension != null)
                                    {
                                        str_Dimension = xpNav_Dimension.Value;
                                        objAPCache.FileDimensions = str_Dimension;

                                        XPathNavigator xpNav_ExtGEID = GEXPathNav.SelectSingleNode("GRAPHICELEMENT/GRAPHICELEMENT_REPLACEDFILE/GRAPHICELEMENT/GRAPHICELEMENTID");
                                        if (xpNav_ExtGEID != null)
                                        {
                                            //get file and process it 
                                            Org_GEID = XPathNodeItr.Current.SelectSingleNode("GRAPHICELEMENTID").InnerXml;
                                            FilePath = getExternalFileFromDB(xpNav_ExtGEID.Value);
                                            if (!String.IsNullOrEmpty(FilePath))
                                            {
                                                if (!objAPCache.ModifyGraphicTagValue(ArtworkID, Org_GEID, FilePath, ArtworkProducerCache.GraphicTagUpdateType.UPLOAD, ArtworkProducerCache.GraphicTagUpdateStatus.PENDING, MakeAsyncCall))
                                                {
                                                    objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_UPDATECACHE_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                                                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                                                    return ErrorXML;
                                                }
                                                bCacheUpdated = true;
                                            }
                                            objAPCache.FileDimensions = null;
                                        }
                                    }
                                }
                            }

                            GEXPathNode = GEXPathNav.SelectSingleNode("GRAPHICELEMENT/GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE");
                            if (GEXPathNode != null)
                            {
                                if ((GEXPathNode.GetAttribute("MODIFIED", "") == "") || (GEXPathNode.GetAttribute("MODIFIED", "") == null))
                                {
                                    String ValueID = xpDoc_NewGEValues.CreateNavigator().SelectSingleNode("GRAPHICELEMENT/GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/GRAPHICELEMENTVALUEID").InnerXml;
                                    ArtworkXML.AppendFormat("<ARTWORK_GRAPHICELEMENTVALUEINFO TYPE='RELATION'>{0}</ARTWORK_GRAPHICELEMENTVALUEINFO>", ValueID);
                                }
                            }

                            #endregion

                            #region Dam  Item (Associate as well as disassociate with dam item)
                            XPNodeItr = GEXPathNav.Select("GRAPHICELEMENT/GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE");

                            bool DisAssociateArtworkDigitalAsset;
                            XPathNavigator XPNode2;

                            while (XPNodeItr.MoveNext())
                            {
                                DisAssociateArtworkDigitalAsset = true;
                                XPNode2 = XPNodeItr.Current;
                                GEXPathNode = XPNode2.SelectSingleNode("GRAPHICELEMENTVALUEINFO_DIGITALASSET[(@DISASSOCIATE='') or not(@DISASSOCIATE) or ((translate(@DISASSOCIATE,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='FALSE'))]");

                                if (GEXPathNode != null)
                                {
                                    GraphicElementID = GEXPathNode.InnerXml;
                                    ArtworkXML.AppendFormat("<ARTWORK_DIGITALASSET TYPE='RELATION'>{0}</ARTWORK_DIGITALASSET>", GraphicElementID);
                                    Org_GEID = XPathNodeItr.Current.SelectSingleNode("GRAPHICELEMENTID").InnerXml;
                                    if (!objAPCache.ModifyGraphicTagValue(ArtworkID, Org_GEID, GEXPathNode.InnerXml, ArtworkProducerCache.GraphicTagUpdateType.ASSET, ArtworkProducerCache.GraphicTagUpdateStatus.PENDING, MakeAsyncCall, out DisAssociateArtworkDigitalAsset))
                                    {
                                        objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_UPDATECACHE_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                                        ErrorXML = objErrorCommon.ErrorCollectionXML;
                                        return ErrorXML;
                                    }
                                    bCacheUpdated = true;
                                }

                                GEXPathNode = XPNode2.SelectSingleNode("GRAPHICELEMENTVALUEINFO_DIGITALASSET[(translate(@DISASSOCIATE,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='TRUE')]");
                                if ((GEXPathNode != null) && DisAssociateArtworkDigitalAsset)
                                {
                                    GraphicElementID = GEXPathNode.InnerXml;
                                    ArtworkXML.AppendFormat("<ARTWORK_DIGITALASSET TYPE='RELATION' DISASSOCIATE='TRUE'>{0}</ARTWORK_DIGITALASSET>", GraphicElementID);
                                }
                            }

                            XPNodeItr = null;
                            GEXPathNode = null;
                            #endregion

                            #region No Photo
                            GEXPathNode = GEXPathNav.SelectSingleNode("GRAPHICELEMENT/GRAPHICELEMENT_NOPHOTO");
                            if (GEXPathNode == null)
                                GEXPathNode = GEXPathNav.SelectSingleNode("GRAPHICELEMENT/GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE [NOIMAGE/text()='1']");

                            if (GEXPathNode != null)
                            {
                                FilePath = "";
                                Org_GEID = XPathNodeItr.Current.SelectSingleNode("GRAPHICELEMENTID").InnerXml;
                                if (!objAPCache.ModifyGraphicTagValue(ArtworkID, Org_GEID, FilePath, ArtworkProducerCache.GraphicTagUpdateType.NOIMAGE, ArtworkProducerCache.GraphicTagUpdateStatus.DONE, MakeAsyncCall))
                                {
                                    objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_UPDATECACHE_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                                    return ErrorXML;
                                }
                                bCacheUpdated = true;
                            }
                            #endregion

                            #region Deleting GraphicElement Value
                            XPNodeItr = GEXPathNav.Select("GRAPHICELEMENT/DELETE_GRAPHICELEMENTVALUE");
                            String GEValueIDList = null;
                            while (XPNodeItr.MoveNext())
                            {
                                if (GEValueIDList == null)
                                    GEValueIDList = XPNodeItr.Current.InnerXml;
                                else
                                    GEValueIDList += ";" + XPNodeItr.Current.InnerXml;
                            }
                            if (GEValueIDList != null)
                            {
                                ErrorXML = GEEngineObj.DeleteGraphicElement(GEValueIDList);
                                if (ErrorXML != null)
                                {
                                    objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_DELETEGRAPHICELEMENT_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                                    return ErrorXML;
                                }

                            }
                            #endregion
                        }
                        else
                        {
                            objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_UPDATEGRAPHICELEMENT_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                            ErrorXML = objErrorCommon.ErrorCollectionXML;
                            return ErrorXML;
                            //if (MainErrorXML == null)
                            //{
                            //    MainErrorXML = new StringBuilder();
                            //    MainErrorXML.Append("<GROUPERROR>");
                            //}
                            //MainErrorXML.Append(ErrorXML);
                        }
                        if (!bCacheUpdated)
                        {
                            if (XPathNodeItr.Current.SelectSingleNode("DONTUPDATECACHE") == null)
                            {
                                Org_GEID = XPathNodeItr.Current.SelectSingleNode("GRAPHICELEMENTID").InnerXml;
                                xpNav_Dimension = XPathNodeItr.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/DIMENSIONS");
                                if (xpNav_Dimension != null)
                                {
                                    str_Dimension = xpNav_Dimension.Value;
                                    objAPCache.FileDimensions = str_Dimension;
                                }

                                if (!objAPCache.ModifyGraphicTagValue(ArtworkID, Org_GEID, null, ArtworkProducerCache.GraphicTagUpdateType.DEFAULT, ArtworkProducerCache.GraphicTagUpdateStatus.PENDING, MakeAsyncCall))
                                {
                                    objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_UPDATECACHE_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                                    return ErrorXML;
                                }

                                objAPCache.FileDimensions = null;
                            }
                        }
                    }
                }

                #endregion
                #endregion

                #region To add 'Modifier' , if ArtworkXML doesn't contain it.

                ArtworkXML.AppendFormat("<ARTWORK_MODIFIER TYPE=\"RELATION\">{0}</ARTWORK_MODIFIER>", UserID);
                #endregion


                ArtworkXML.Append("</ARTWORK>");
                InputArtworkXML = ArtworkXML.ToString();

                ErrorXML = UpdateArtwork(ref InputArtworkXML);
                TextAndGraphicElementValueXML = InputArtworkXML;
                if (ErrorXML != null)
                {
                    objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                    return ErrorXML;
                    //if (MainErrorXML == null)                    
                    //    MainErrorXML = new StringBuilder();

                    //MainErrorXML.Append(ErrorXML);
                }

                //ErrorXML = objErrorCommon.ErrorCollectionXML;
                //if (String.IsNullOrEmpty(ErrorXML))
                //    return null;
                //else
                //    return ErrorXML;
                return null;

                //if (MainErrorXML != null)
                //{
                //    MainErrorXML.Append("</GROUPERROR>");
                //    return MainErrorXML.ToString();
                //}
                //else
                //    return null;
                #endregion



            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_EXCPETION", this.GetType().ToString(), "UpdateArtwork", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
            finally
            {
                TextElementEngineObj = null;
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine UpdateArtwork Ends");
            }
        }

        /// <alsummary>
        /// <createddate>16 November 2006</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>This method is used to update Artwork based on the input XML.</purpose>
        ///  <xmlstructure><![CDATA[
        /// 
        ///     Input XML :         |-  [ARTWORK @MODIFIED="TRUE"]
        ///                                       - $Attributes$
        ///                                       |-    [$RelationShipName$ @TYPE="RELATION"  @DISASSOCIATE="TRUE/FALSE"]text()=$Associated Entity/Facet Id$     // Associate or Disassociate existing instance with TargetEntity                    
        ///                                                  |-  [$Associated Entity/Facet Name$  @MODIFIED="TRUE"]        //it will create new instance of associated entity or facet
        ///                                                                                                                 //,if Modified is not specified
        ///                                                           |-  $Attributes                                       // and associates it with target entity
        /// 
        /// ]]></xmlstructure>
        /// <param name="ArtworkXML"></param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        public ERRORXML UpdateArtwork(ref XML ArtworkXML)
        {
            #region variable
            ERRORXML ErrorXML = null;
            #endregion
            try
            {
                objErrorCommon = new ALErrorCommon();
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine UpdateArtwork Starts");

                ErrorXML = UpdateArtwork(ref ArtworkXML, true);
                if (ErrorXML != null)
                {
                    objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                }
                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_EXCEPTION", this.GetType().ToString(), "UpdateArtwork", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine UpdateArtwork Ends");
            }
        }

        public ERRORXML UpdateArtwork(ref XML ArtworkXML, bool VerifyCache)
        {
            #region variable
            ERRORXML ErrorXML = null;
            String NewArtworkName = null;
            String ArtworkID = null;
            ArtworkProducerCommon ArtCmnObj = null;
            #endregion
            try
            {
                objErrorCommon = new ALErrorCommon();
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine UpdateArtwork Starts");

                ErrorXML = ArtworkProducerEngine.CreateAndUpdateData(ref ArtworkXML, "UPDATEDATA", UserID, Language, TenantName);
                if (ErrorXML == null)
                {
                    ArtworkID = ALCommon.GetInnerValue(ref ArtworkXML, "<ARTWORKID>", "</ARTWORKID>");
                    if (VerifyCache)
                    {
                        ArtworkProducerCache objAPCache = new ArtworkProducerCache(UserID, Language, TenantName);
                        String FormatID = null;
                        try
                        {
                            FormatID = objAPCache.GetFormatID(TenantName, ArtworkID);
                        }
                        catch
                        {
                            ErrorXML = GetFormatIDForArtwork(ArtworkID, out FormatID);
                            if (ErrorXML != null)
                            {
                                objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_GETFORMATIDFORARTWORK_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                                ErrorXML = objErrorCommon.ErrorCollectionXML;
                                return ErrorXML;
                            }

                            if (String.IsNullOrEmpty(FormatID))
                            {
                                objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_GETFORMATID_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                                ErrorXML = objErrorCommon.ErrorCollectionXML;
                            }
                        }
                        bool bretVal = objAPCache.ValidateFormatCache(FormatID);
                        if (bretVal)
                        {
                            bretVal = objAPCache.ValidateArtworkCacheWithFormatCache(ArtworkID, FormatID);
                            if (!bretVal)
                                objAPCache.PrepareArtworkCache(ArtworkID, "EDIT");
                            else
                                ErrorXML = objAPCache.UpdateArtworkDetails(ArtworkXML, ArtworkID);
                        }
                        else
                        {
                            objAPCache.PrepareFormatCache(FormatID);
                            objAPCache.PrepareArtworkCache(ArtworkID, "EDIT");
                        }
                    }
                    //Check whether Update call including Rename of Artwork
                    if (ArtworkXML.Contains("<ARTWORKNAME>"))
                    {
                        NewArtworkName = ALSpecialCharHandling.DecodeSpecialCharacters(ALCommon.GetInnerValue(ref ArtworkXML, "<ARTWORKNAME>", "</ARTWORKNAME>"));
                        if (NewArtworkName != null && ArtworkID != null)
                        {
                            ArtCmnObj = new ArtworkProducerCommon(UserID, Language, TenantName);
                            ErrorXML = ArtworkProducerEngine.ConvertToAlphaNumericString(ref NewArtworkName);
                            if (String.IsNullOrEmpty(ErrorXML))
                            {
                                ErrorXML = ArtCmnObj.RenameSection(NewArtworkName, ArtworkID);
                                if (ErrorXML != null)
                                {
                                    objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_RENAMESECTION_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                                }
                            }
                        }

                    }
                }
                else
                {
                    objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_FAILED", this.GetType().ToString(), "UpdateArtwork", null, ErrorXML);
                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                }
                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_EXCEPTION", this.GetType().ToString(), "UpdateArtwork", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine UpdateArtwork Ends");
            }
        }

        /// <alsummary>
        /// <createddate>15 Feb 2010</createddate>
        /// <author>Anand Mahadik</author>
        /// <purpose>This method is used to create or update Graphic Element value and Text Element value associated with Artwork. If Text and Graphic elements are
        /// successfully updated then the API will stamp media job for Thumbnail/Preview generation.</purpose>
        ///  <param name="ArtworkXML">
        ///  <xmlstructure><![CDATA[
        ///                     <ARTWORK>
        ///                         <ARTWORKID/>
        ///                         <ARTWORKNAME/>
        ///                         <FORMATID/>
        ///                     <PAGE>
        ///                     <GROUPELEMENT>
        ///                     	<TEXTELEMENT MODIFIED="TRUE">
        ///                         		<TEXTELEMENTID>131087275470205</TEXTELEMENTID>
        ///                             	<TEXTELEMENT_TEXTELEMENTVALUE TYPE="RELATION">
        ///                     			    <TEXTELEMENTVALUE @MODIFIED="TRUE">
        ///                                             <TEXTELEMENTVALUEID/>               //[Modified attribute and TextElementValudID is given in case of Update only.]
        ///                             		    	<TEXTELEMENTACTUALVALUE/>  
        ///                     			    </TEXTELEMENTVALUE>
        ///                         		</TEXTELEMENT_TEXTELEMENTVALUE>
        ///                     	</TEXTELEMENT>
        ///                     	<ARTWORK_GRAPHICELEMENT TYPE="RELATION">
        ///                 		    <GRAPHICELEMENT>
        ///                 			    <FILESYSTEM>
        ///                 				    <FILE>
        ///     			            		    <NAME>orchidplant.jpg</NAME>
        ///                 					    <PATH>D:\Assetlink58\mom\UploadedFiles\210079462842129\orchidplant.jpg</PATH>
        ///                 					    <FIELDNAME>ORIGINALFILE</FIELDNAME>
        ///                 				    </FILE>
        ///         				        </FILESYSTEM>
        ///             			        <GRAPHICELEMENT_REPLACEDFILE>222828869939788</GRAPHICELEMENT_REPLACEDFILE>
        ///                 		    </GRAPHICELEMENT>
        ///                 	    </ARTWORK_GRAPHICELEMENT>
        ///                     </GROUPELEMENT>
        ///                     </<PAGE>
        ///                 </ARTWORK>
        ///     
        /// ]]></xmlstructure>
        ///  </param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        public ERRORXML UpdateTEGEForArtworkAndStampMediaJob(ref XML ArtworkXML)
        {
            ERRORXML ErrorXML = null;
            XmlDocument ArtworkXMLDoc = null;
            XmlNode ArtworkNode = null;
            String FormatID = null;
            String ArtworkID = null;
            String ArtworkStatus = null;
            StringBuilder sbSearchXml = null;
            OMInterface OMIObj = null;
            XmlNode ArtworkStatusAttrNode = null;
            String ArtworkTEXml = null;
            XPathDocument ArtworkTEXmlDoc = null;
            XPathNavigator XPathNav = null;
            XPathNodeIterator TextElementNodes = null;
            XPathNavigator TextElementValueNodeNav = null;
            XmlNode TextElementNode = null;
            XmlNode TextElementValueNode = null;
            String TextElementValueID = null;
            String TextElementID = null;
            XmlNode TextElementValueIDNode = null;
            XML TextAndGraphicElementValueXML = null;
            XML FormatXML = null;
            FormatEngine FormatEngineObj = null;
            String FormatType = null;
            String ArtworkName = null;
            String ZipFilePath = null;
            StringBuilder SB_MediaJobXML = null;
            String JobDetailsXML = null;
            MediaJobEngine MediaJobEngineObj = null;
            String ErrorMessage = null;
            ArtworkJobEngine ArtworkJobEngineObj = null;
            String ArtworkJobID = null;
            ArtworkJobEngine.BatchJobStatus NewJobStatus;
            ArtworkJobEngine.BatchJobStatus PreviousJobStatus;
            ArtworkProducerCommon helperObj = null;
            String DisplayStatus = null;

            try
            {
                objErrorCommon = new ALErrorCommon();
                helperObj = new ArtworkProducerCommon(UserID, Language, TenantName, SolutionName);
                strdict_AdditionalInfo = new StringDictionary();
                ArtworkXMLDoc = new XmlDocument();
                ArtworkXMLDoc.LoadXml(ArtworkXML);
                ArtworkNode = ArtworkXMLDoc.DocumentElement;
                ArtworkStatusAttrNode = ArtworkNode.Attributes.GetNamedItem("Status", "");

                if (ArtworkStatusAttrNode != null)
                    ArtworkStatus = ArtworkStatusAttrNode.InnerText;

                FormatID = ALCommon.GetInnerValue(ref ArtworkXML, "<FORMATID>", "</FORMATID>");
                ArtworkID = ALCommon.GetInnerValue(ref ArtworkXML, "<ARTWORKID>", "</ARTWORKID>");
                ArtworkName = ALCommon.GetInnerValue(ref ArtworkXML, "<ARTWORKNAME>", "</ARTWORKNAME>");

                if (ArtworkStatus.ToUpper() == "PENDING")
                {
                    ErrorXML = GenerateArtwork_Cache_Section_TextElements(FormatID, ref ArtworkXML);
                    if (!String.IsNullOrEmpty(ErrorXML))
                    {
                        strdict_AdditionalInfo.Clear();
                        strdict_AdditionalInfo.Add("ARTWORKNAME", ArtworkName);
                        LogArtworkError(ArtworkID, ErrorObjectType.Artwork, "APENGINE_ARTWORK_CACHE_UPDATE_AND_ASSOCIATING_DEFAULT_TE_OPTION_FAILED", null, ErrorXML, ErrorType.ERROR, strdict_AdditionalInfo, true);

                        objErrorCommon.AppendError("APENGINE_ARTWORK_CACHE_UPDATE_AND_ASSOCIATING_DEFAULT_TE_OPTION_FAILED", this.GetType().ToString(), "UpdateTEGEForArtworkAndStampMediaJob", null, ErrorXML);
                        return objErrorCommon.ErrorCollectionXML;
                    }
                }

                #region Get Artwork Details
                sbSearchXml = new StringBuilder(255);
                sbSearchXml.Append(@"<SEARCH>
                            <TARGETOBJECT>ARTWORK</TARGETOBJECT>
                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                            <TARGETENTITY>
                                    <PRIVILEGENAME>READ</PRIVILEGENAME>
                                    <FILTERLIST>
                                        <FILTER>
                                            <EXPRESSION>ARTWORKID</EXPRESSION>
                                             <OPERATOR>EQUALTO</OPERATOR>
                                            <EXPRESSIONVALUE>" + ArtworkID + @"</EXPRESSIONVALUE>
                                            <FILTERID>1</FILTERID>
                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                        </FILTER>
                                     </FILTERLIST>
                                     <SELECTLIST>
                                        <EXPRESSION>
                                            <FIELDNAME>ARTWORKID</FIELDNAME>
                                            <ALIAS>ARTWORKID</ALIAS>
                                        </EXPRESSION>
                                        <EXPRESSION>
                                            <FIELDNAME>ARTWORKNAME</FIELDNAME>
                                            <ALIAS>ARTWORKNAME</ALIAS>
                                        </EXPRESSION>
                                        <EXPRESSION>
                                            <FIELDNAME>ARTWORKDESCRIPTION</FIELDNAME>
                                            <ALIAS>ARTWORKDESCRIPTION</ALIAS>
                                        </EXPRESSION>
                                    </SELECTLIST>
                                    <RELATIONSHIP NAME='ARTWORK_TEXTELEMENTVALUE'>
                                        <SELECTLIST>
                                            <EXPRESSION>
                                                <FIELDNAME>TEXTELEMENTVALUEID</FIELDNAME>
                                                <ALIAS>TEXTELEMENTVALUEID</ALIAS>
                                            </EXPRESSION>
                                            <EXPRESSION>
                                                <FIELDNAME>TEXTELEMENTACTUALVALUE</FIELDNAME>
                                                <ALIAS>TEXTELEMENTACTUALVALUE</ALIAS>
                                            </EXPRESSION>
                                        </SELECTLIST>
                                        <RELATIONSHIP NAME='TEXTELEMENT_TEXTELEMENTVALUE'>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>TEXTELEMENTID</FIELDNAME>
                                                    <ALIAS>TEXTELEMENTID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>TEXTTAGNAME</FIELDNAME>
                                                    <ALIAS>TEXTTAGNAME</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                        </RELATIONSHIP>
                                    </RELATIONSHIP>
                                </TARGETENTITY>
                          </SEARCH>");
                OMIObj = new OMInterface(UserID, Language, TenantName);
                ErrorXML = OMIObj.GetHierarchialData(sbSearchXml.ToString(), ref ArtworkTEXml);
                if (!String.IsNullOrEmpty(ErrorXML))
                {
                    strdict_AdditionalInfo.Clear();
                    strdict_AdditionalInfo.Add("ARTWORKNAME", ArtworkName);
                    LogArtworkError(ArtworkID, ErrorObjectType.Artwork, "APENGINE_ARTWORK_READ_FAILED", null, ErrorXML, ErrorType.ERROR, strdict_AdditionalInfo, true);

                    objErrorCommon.AppendError("APENGINE_ARTWORK_READ_FAILED", this.GetType().ToString(), "UpdateTEGEForArtworkAndStampMediaJob", null, ErrorXML);
                    return objErrorCommon.ErrorCollectionXML;
                }
                #endregion

                #region Check whether TextElementValue exists and based on it decide whether to "Create" or "Modify" TextElementValue
                ArtworkTEXmlDoc = new XPathDocument(new StringReader(ArtworkTEXml));
                XPathNav = ArtworkTEXmlDoc.CreateNavigator();
                TextElementNodes = XPathNav.Select("ARTWORKCOLLECTION/ARTWORK/ARTWORK_TEXTELEMENTVALUE/TEXTELEMENTVALUE/TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENT");
                while (TextElementNodes.MoveNext())
                {
                    TextElementID = TextElementNodes.Current.SelectSingleNode("TEXTELEMENTID").Value;
                    TextElementValueNodeNav = TextElementNodes.Current;
                    TextElementValueNodeNav.MoveToParent();
                    TextElementValueNodeNav.MoveToParent();
                    TextElementValueID = TextElementValueNodeNav.SelectSingleNode("TEXTELEMENTVALUEID").Value;

                    TextElementNode = ArtworkNode.SelectSingleNode("PAGE/GROUPELEMENT/TEXTELEMENT[TEXTELEMENTID=" + TextElementID + "]");
                    if (TextElementNode != null)
                    {
                        TextElementValueNode = TextElementNode.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENTVALUE");
                        ((XmlElement)TextElementValueNode).SetAttribute("MODIFIED", "TRUE");
                        TextElementValueIDNode = ArtworkXMLDoc.CreateElement("TEXTELEMENTVALUEID");
                        TextElementValueIDNode.InnerText = TextElementValueID.ToString();
                        TextElementValueNode.AppendChild(TextElementValueIDNode);
                    }

                }
                #endregion

                #region Update Artwork with Text and Graphic elements
                TextAndGraphicElementValueXML = ArtworkNode.SelectSingleNode("PAGE").OuterXml;
                ErrorXML = UpdateArtwork(ArtworkID, ref TextAndGraphicElementValueXML);
                if (!String.IsNullOrEmpty(ErrorXML))
                {
                    strdict_AdditionalInfo.Clear();
                    strdict_AdditionalInfo.Add("ARTWORKNAME", ArtworkName);
                    LogArtworkError(ArtworkID, ErrorObjectType.Artwork, "APENGINE_ARTWORK_FAILED_TO_UPDATE_TEXT_GRAPHIC_ELEMENTS", null, ErrorXML, ErrorType.ERROR, strdict_AdditionalInfo, true);

                    objErrorCommon.AppendError("APENGINE_ARTWORK_FAILED_TO_UPDATE_TEXT_GRAPHIC_ELEMENTS", this.GetType().ToString(), "UpdateTEGEForArtworkAndStampMediaJob", null, ErrorXML);
                    return objErrorCommon.ErrorCollectionXML;
                }
                #endregion

                #region Get Format details
                FormatEngineObj = new FormatEngine(UserID, Language, TenantName);
                ErrorXML = FormatEngineObj.GetFormatDetails(FormatID, ref FormatXML);
                if (!String.IsNullOrEmpty(ErrorXML))
                {
                    strdict_AdditionalInfo.Clear();
                    strdict_AdditionalInfo.Add("ARTWORKNAME", ArtworkName);
                    LogArtworkError(ArtworkID, ErrorObjectType.Artwork, "APENGINE_ARTWORK_FAILED_TO_RETRIEVE_FORMAT_TYPE", null, ErrorXML, ErrorType.ERROR, strdict_AdditionalInfo, true);

                    objErrorCommon.AppendError("APENGINE_ARTWORK_FAILED_TO_RETRIEVE_FORMAT_TYPE", this.GetType().ToString(), "UpdateTEGEForArtworkAndStampMediaJob", null, ErrorXML);
                    return objErrorCommon.ErrorCollectionXML;
                }
                FormatType = ALCommon.GetInnerValue(ref FormatXML, "<FORMATTYPE>", "</FORMATTYPE>");
                #endregion

                #region Stamp Media job
                switch (FormatType.ToUpper())
                {
                    case "INDESIGN":
                    case "ILLUSTRATOR": SB_MediaJobXML = new StringBuilder(150);
                        helperObj.GetDisplayStatus("Artwork", "Artwork Generation In Progress", out DisplayStatus);
                        SB_MediaJobXML.AppendFormat(@"<MEDIAJOB>
	                                                                        <JOBNAME>{4} Job</JOBNAME>
	                                                                        <CREATEDON>{0}</CREATEDON>
	                                                                        <MODIFIEDON>{0}</MODIFIEDON>
	                                                                        <SUBMITTEDON>{0}</SUBMITTEDON>
	                                                                        <JOBTYPE>ARTWORK GENERATION JOB</JOBTYPE>
	                                                                        <JOBHANDLER>{1}</JOBHANDLER>
	                                                                        <JOBSTATUS>New</JOBSTATUS>
	                                                                        <MEDIAJOB_CREATOR TYPE='RELATION'>{2}</MEDIAJOB_CREATOR>
	                                                                        <ARTWORK_MEDIAJOB TYPE='RELATION'>{3}</ARTWORK_MEDIAJOB>
	                                                                        <ARTWORK_MEDIAJOB TYPE='RELATION'>
		                                                                        <ARTWORK MODIFIED='TRUE'>
			                                                                        <ARTWORKID>{3}</ARTWORKID>
			                                                                        <ARTWORKSTATUS>Artwork Generation In Progress</ARTWORKSTATUS>
                                                                                    <DISPLAYSTATUS>{5}</DISPLAYSTATUS>
		                                                                        </ARTWORK>
	                                                                        </ARTWORK_MEDIAJOB>
                                                                        </MEDIAJOB>
                                                                        ", ALCommon.CurrentDateTimeToXml(), FormatType, UserID, ArtworkID, ArtworkName, DisplayStatus);
                        JobDetailsXML = SB_MediaJobXML.ToString();
                        MediaJobEngineObj = new MediaJobEngine(SessionID, UserID, Language, TenantName);

                        if (String.IsNullOrEmpty(SolutionName))
                            ErrorXML = MediaJobEngineObj.StampMediaJob(FormatID, ArtworkID, "ArtworkProducer", ref JobDetailsXML);
                        else
                            ErrorXML = MediaJobEngineObj.StampMediaJob(FormatID, ArtworkID, SolutionName, ref JobDetailsXML);

                        if (!String.IsNullOrEmpty(ErrorXML))
                        {
                            strdict_AdditionalInfo.Clear();
                            strdict_AdditionalInfo.Add("ARTWORKNAME", ArtworkName);
                            LogArtworkError(ArtworkID, ErrorObjectType.Artwork, "APENGINE_ARTWORK_MEDIAJOB_STAMPPING_FAILED", null, ErrorXML, ErrorType.ERROR, strdict_AdditionalInfo, true);

                            objErrorCommon.AppendError("APENGINE_ARTWORK_MEDIAJOB_STAMPPING_FAILED", this.GetType().ToString(), "UpdateTEGEForArtworkAndStampMediaJob", null, ErrorXML);
                            return objErrorCommon.ErrorCollectionXML;
                        }
                        break;

                    case "HTML":
                        if (String.IsNullOrEmpty(SolutionName))
                            ErrorXML = CreateArtwork_HTML(ArtworkID, ArtworkName, FormatID, "ArtworkProducer", "ARTWORK GENERATION JOB", out ZipFilePath);
                        else
                            ErrorXML = CreateArtwork_HTML(ArtworkID, ArtworkName, FormatID, SolutionName, "ARTWORK GENERATION JOB", out ZipFilePath);

                        if (!String.IsNullOrEmpty(ErrorXML))
                        {
                            strdict_AdditionalInfo.Clear();
                            strdict_AdditionalInfo.Add("ARTWORKNAME", ArtworkName);
                            LogArtworkError(ArtworkID, ErrorObjectType.Artwork, "APENGINE_ARTWORK_GENERATION_FAILED", null, ErrorXML, ErrorType.ERROR, strdict_AdditionalInfo, true);

                            objErrorCommon.AppendError("APENGINE_ARTWORK_GENERATION_FAILED", this.GetType().ToString(), "UpdateTEGEForArtworkAndStampMediaJob", null, ErrorXML);
                            return objErrorCommon.ErrorCollectionXML;
                        }

                        if (ArtworkEngine.IsArtworkPartOfBatch(UserID, Language, TenantName, ArtworkID, out ArtworkJobID))
                        {
                            ArtworkJobEngineObj = new ArtworkJobEngine(SessionID, UserID, Language, TenantName, SolutionName);
                            ErrorXML = ArtworkJobEngineObj.CalculateJobProgress(ArtworkJobID, out PreviousJobStatus, out NewJobStatus);
                            if ((!(PreviousJobStatus == Assetlink.Services.Engines.ArtworkProducerEngine.ArtworkJobEngine.BatchJobStatus.Done
                                  || PreviousJobStatus == Assetlink.Services.Engines.ArtworkProducerEngine.ArtworkJobEngine.BatchJobStatus.Error))
                                  && (NewJobStatus == Assetlink.Services.Engines.ArtworkProducerEngine.ArtworkJobEngine.BatchJobStatus.Done
                               || NewJobStatus == Assetlink.Services.Engines.ArtworkProducerEngine.ArtworkJobEngine.BatchJobStatus.Error))
                            {
                                ArtworkJobEngineObj.HandlePostProcessing(ArtworkJobID);
                            }
                            if (!String.IsNullOrEmpty(ErrorXML))
                            {
                                strdict_AdditionalInfo.Clear();
                                strdict_AdditionalInfo.Add("ARTWORKNAME", ArtworkName);
                                LogArtworkError(ArtworkID, ErrorObjectType.Artwork, "APENGINE_ARTWORK_CALCULATE_BATCHJOB_PROGRESS_FAILED", null, ErrorXML, ErrorType.ERROR, strdict_AdditionalInfo, true);

                                objErrorCommon.AppendError("APENGINE_ARTWORK_CALCULATE_BATCHJOB_PROGRESS_FAILED", this.GetType().ToString(), "CreateBatchArtworks", null, ErrorXML);
                                return objErrorCommon.ErrorCollectionXML;
                            }
                        }
                        break;
                }
                #endregion

            }
            catch (Exception ExcepObj)
            {
                strdict_AdditionalInfo.Add("APINAME", "UpdateTEGEForArtworkAndStampMediaJob()");
                strdict_AdditionalInfo.Add("MESSAGE", ExcepObj.ToString());
                strdict_AdditionalInfo.Add("ARTWORKNAME", ArtworkName);

                ErrorMessage = "Exception thrown in UpdateTEGEForArtworkAndStampMediaJob() while Artwork generation. Exception details: " + ExcepObj.Message;
                LogArtworkError(ArtworkID, ErrorObjectType.Artwork, "APENGINE_ARTWORK_EXCEPTION_OCCURED", ErrorMessage, null, ErrorType.ERROR, strdict_AdditionalInfo, true);

                objErrorCommon.AppendError("APENGINE_ARTWORK_GENERATION_FAILED", this.GetType().ToString(), "UpdateTEGEForArtworkAndStampMediaJob", ExcepObj, ReturnErrorXML);
                return objErrorCommon.ErrorCollectionXML;
            }
            return null;
        }

        /// <alsummary>
        /// <createddate>16 November 2006</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>This method is used to Delete Artworks based on the ArtworkIDList.</purpose>
        ///  <xmlstructure><![CDATA[
        /// ]]></xmlstructure>
        /// <param name="ArtworkIDList">Semicolon (;) separated Artwork ID list</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        public ERRORXML DeleteArtworks(String ArtworkIDList)
        {
            #region Local Variables
            OperationQueue OMQueueObj = null;
            OperationQueue DelTVOperQueueObj = null;
            Operation OperObj = null;
            Operation DelTVOperObj = null;
            OMInterface OMIObj = null;
            XML DelTVXML = null;
            String OperationXML = null;
            ERRORXML ErrorXML = null;

            XML ArworkDetails = null;
            StringBuilder SearchInputXML = null;

            XPathDocument XPathDoc = null;
            XPathNavigator XPathNav = null;
            StringReader StringReaderObj = null;
            XPathNodeIterator XPathNodeItr = null;
            //ERRORXML                MainError       = null;
            ArtworkProducerCommon ArtCommonObj = null;
            ArtworkJobEngine ArtworkJobEngineObj = null;
            ArtworkJobEngine.BatchJobStatus NewJobStatus;
            ArtworkJobEngine.BatchJobStatus PreviousJobStatus;
            String ArtworkJobID = null;

            #endregion

            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine DeleteArtworks Starts");

                objErrorCommon = new ALErrorCommon();
                String[] ArrArtworkList = ArtworkIDList.Split(new String[] { ";" }, StringSplitOptions.RemoveEmptyEntries);

                SearchInputXML = new StringBuilder();

                for (int cnt = 0; cnt < ArrArtworkList.Length; cnt++)
                {
                    #region Prepare Operation Queue to delete Associated Text Element values
                    //Get related Text element values
                    OMQueueObj = new OperationQueue(false);
                    DelTVOperQueueObj = new OperationQueue(false);
                    SearchInputXML.AppendFormat(@"<SEARCH>
	                                                <TARGETOBJECT>ARTWORK</TARGETOBJECT>
	                                                <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                    <TARGETENTITY>
		                                                <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                        <FILTERLIST>
				                                            <FILTER>
					                                            <EXPRESSION>ARTWORKID</EXPRESSION>
					                                            <OPERATOR>EQUALTO</OPERATOR>
					                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
					                                            <FILTERID>1</FILTERID>
					                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
				                                            </FILTER>
			                                            </FILTERLIST>
                                                       	<RELATIONSHIP NAME='ARTWORK_TEXTELEMENTVALUE'/>
                                                </TARGETENTITY>
                                                </SEARCH>", ArrArtworkList[cnt].ToString());


                    ErrorXML = GetArtworkDetails_NSearch(SearchInputXML.ToString(), out ArworkDetails);
                    if (ErrorXML == null)
                    {
                        StringReaderObj = new StringReader(ArworkDetails);
                        XPathDoc = new XPathDocument(StringReaderObj);
                        XPathNav = XPathDoc.CreateNavigator();
                        XPathNodeItr = XPathNav.Select("//ARTWORK_TEXTELEMENTVALUE/TEXTELEMENTVALUE/TEXTELEMENTVALUEID");
                        while (XPathNodeItr.MoveNext())
                        {
                            DelTVOperObj = new Operation("ENTITY", "DELETE");
                            DelTVOperObj.AddParameter("ENTITYID", XPathNodeItr.Current.InnerXml);
                            DelTVOperObj.AddParameter("ENTITYTYPENAME", "TEXTELEMENTVALUE");
                            DelTVOperQueueObj.AddOperation(DelTVOperObj);
                            DelTVOperObj = null;
                        }
                        XPathDoc = null;
                        XPathNav = null;
                        XPathNodeItr = null;
                        StringReaderObj = null;
                    }
                    else
                    {
                        objErrorCommon.AppendError("APENGINE_ARTWORK_DELETEARTWORKS_GETARTWORKDETAILS_FAILED", this.GetType().ToString(), "DeleteArtworks", null, ErrorXML);
                        ErrorXML = objErrorCommon.ErrorCollectionXML;
                        return ErrorXML;
                    }

                    ErrorXML = null;
                    SearchInputXML.Remove(0, SearchInputXML.Length);

                    #endregion


                    #region Prepare Operation Queue to delete Artworks
                    OperObj = new Operation("ENTITY", "DELETE");
                    OperObj.AddParameter("ENTITYID", ArrArtworkList[cnt]);
                    OperObj.AddParameter("ENTITYTYPENAME", "ARTWORK");
                    OMQueueObj.AddOperation(OperObj);
                    OperObj = null;
                    #endregion

                    OperationXML = OMQueueObj.Xml;
                    DelTVXML = DelTVOperQueueObj.Xml;

                    OMIObj = new OMInterface(UserID, Language, TenantName);

                    //Delete all associated Text Element Value
                    ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Before DeleteTextElementValue");
                    ErrorXML = OMIObj.Execute(ref DelTVXML);
                    ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine DeleteArtworks Starts");

                    if (ErrorXML != null)
                    {
                        objErrorCommon.AppendError("APENGINE_ARTWORK_DELETEARTWORKS_DELETETEVALUES_FAILED", this.GetType().ToString(), "DeleteArtworks", null, ErrorXML);
                        ErrorXML = objErrorCommon.ErrorCollectionXML;
                        return ErrorXML;
                        //MainError   =   "<ERROR>"+   ErrorXML+"</ERROR>";
                    }

                    //Delete Artwork
                    ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Before DeleteArtwork");
                    ErrorXML = OMIObj.Execute(ref OperationXML);
                    ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "After DeleteArtwork");

                    if (ErrorXML != null)
                    {
                        //return ErrorXML;
                        //if (MainError == null)
                        //    MainError = "<ERROR>" + ErrorXML + "</ERROR>";
                        //else
                        //    MainError = MainError.Insert(MainError.IndexOf("</ERROR>"), ErrorXML);
                        objErrorCommon.AppendError("APENGINE_ARTWORK_DELETEARTWORKS_FAILED", this.GetType().ToString(), "DeleteArtworks", null, ErrorXML);
                        ErrorXML = objErrorCommon.ErrorCollectionXML;
                        return ErrorXML;
                    }
                    else
                    {
                        if (ArtworkEngine.IsArtworkPartOfBatch(UserID, Language, TenantName, ArrArtworkList[cnt], out ArtworkJobID))
                        {
                            ArtworkJobEngineObj = new ArtworkJobEngine(SessionID, UserID, Language, TenantName, SolutionName);
                            ErrorXML = ArtworkJobEngineObj.CalculateJobProgress(ArtworkJobID, out PreviousJobStatus, out NewJobStatus);
                            if (!String.IsNullOrEmpty(ErrorXML))
                            {
                                objErrorCommon.AppendError("APENGINE_ARTWORK_CALCULATE_BATCHJOB_PROGRESS_FAILED", this.GetType().ToString(), "DeleteArtworks", null, ErrorXML);
                                return objErrorCommon.ErrorCollectionXML;
                            }
                        }

                        ArtCommonObj = new ArtworkProducerCommon(UserID, Language, TenantName);

                        ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Before DeleteSection");
                        ErrorXML = ArtCommonObj.DeleteSection(ArrArtworkList[cnt]);
                        ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "After DeleteSection");

                        if (ErrorXML != null)
                        {
                            objErrorCommon.AppendError("APENGINE_ARTWORK_DELETEARTWORKS_DELETESECTION_FAILED", this.GetType().ToString(), "DeleteArtworks", null, ErrorXML);
                            ErrorXML = objErrorCommon.ErrorCollectionXML;
                            return ErrorXML;
                        }

                        ArtworkProducerCache objAPCache = new ArtworkProducerCache(UserID, Language, TenantName);
                        ErrorXML = objAPCache.DeleteArtworkCache(ArrArtworkList[cnt]);
                        if (ErrorXML != null)
                        {
                            objErrorCommon.AppendError("APENGINE_ARTWORK_DELETEARTWORKS_DELETEARTWORKCACHE_FAILED", this.GetType().ToString(), "DeleteArtworks", null, ErrorXML);
                            ErrorXML = objErrorCommon.ErrorCollectionXML;
                            return ErrorXML;
                        }
                    }
                    OMQueueObj = null;
                }

                return null;
                //ErrorXML = objErrorCommon.ErrorCollectionXML;
                //if (String.IsNullOrEmpty(ErrorXML))
                //    return null;
                //else
                //    return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_ARTWORK_DELETEARTWORKS_EXCEPTION", this.GetType().ToString(), "DeleteArtworks", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
            finally
            {
                OMQueueObj = null;
                OperObj = null;
                OMIObj = null;
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine DeleteArtworks Ends");
            }

        }

        /// <alsummary>
        /// <createddate>16 November 2006</createddate>
        /// <author>Pallavi Pitla</author>
        /// <purpose>This method is used to Get Artworks based on the FilterXML and FieldCollectionXML.</purpose>
        ///  <xmlstructure><![CDATA[
        /// 
        /// FilterXML       :   |-  [FILTERLIST]
        ///                            |-  [FILTER]
        ///                                         |-  [FIELDNAME]
        ///                                         |-  [OPERATOR]
        ///                                         |-  [FIELDVALUE]
        ///                                         |-  [NAVIGATIONPATH]
        ///                                         |-  [ISCASESENSITIVE]
        ///                            |-   ...
        ///                            |_   ...
        /// 
        /// 
        /// FieldCollectiionXML :   |-[FIELDCOLLECTION]
        ///                                 |-  [TARGETENTITYFIELDS]
        ///                                             |-  [FIELD]
        ///                                             |-  ...
        ///                                 |-  [RELATEDENTITYFIELDS @NavigationPath] 
        ///                                             |-  [FIELD]
        ///                                             |-  ...
        ///                                 |-  [RELATEDFACETFIELDS @NavigationPath]           
        ///                                             |-  [FIELD]
        ///                                             |-  ...
        ///                                 |-      ...
        /// 
        /// ArtworkListXML      :   -[ARTWORKCOLLECTION]
        ///                                 |-  [ARTWORK]
        ///                                 |-  ...
        /// ]]></xmlstructure>
        /// <param name="FilterXML">Specifies Filters , used in search</param>
        /// <param name="FieldCollectionXML">Specifies Target ,Related entity /facet fields</param>
        /// <param name="ArtworkListXML">Collection of Artwork list</param>
        /// <returns>ErrorXML if Error occured else null</returns>
        /// </alsummary>
        public ERRORXML GetAllArtworks(XML FilterXML, XML FieldCollectionXML, out XML ArtworkListXML)
        {
            #region variable
            OMInterface OMIObj = null;
            ERRORXML ErrorXML = null;
            XML OutputXML = null;
            #endregion
            #region Get item List by calling DoSearch method of Assetlink Search
            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine GetAllArtworks Starts");

                objErrorCommon = new ALErrorCommon();
                OMIObj = new OMInterface(UserID, Language, TenantName);

                if (FieldCollectionXML != null && FieldCollectionXML != "")
                    ErrorXML = OMIObj.GetData(FilterXML, FieldCollectionXML, ref OutputXML);
                else
                    ErrorXML = OMIObj.GetData(FilterXML, null, ref OutputXML);

                if (ErrorXML != null)
                {
                    ArtworkListXML = "";
                    objErrorCommon.AppendError("APENGINE_ARTWORK_GETALLARTWORKS_FAILED", this.GetType().ToString(), "GetAllArtworks", null, ErrorXML);
                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                    return ErrorXML;
                }

                ArtworkListXML = OutputXML;
                return null;
            }
            catch (Exception ExcepObj)
            {
                ArtworkListXML = null;
                objErrorCommon.AppendError("APENGINE_ARTWORK_GETALLARTWORKS_EXCEPTION", this.GetType().ToString(), "GetAllArtworks", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
            finally
            {
                OMIObj = null;
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine GetAllArtworks Ends");
            }
            #endregion
        }

        /// <alsummary>    
        /// <createddate>2 December 2006</createddate>
        /// <author>Pranjali Bhide</author>
        /// <purpose>This method is used to Get all related entities & facets of Artwork based on ArtworkID .</purpose>
        /// </summary>
        /// <param name="ArtworkID"></param>
        /// <param name="ArtworkDetailsXML"></param>
        /// <returns></returns>
        /// </alsummary>
        public ERRORXML GetAllRelatedObjects_ForArtwork(String ArtworkID, XML AssociationXML, out XML ArtworkDetailsXML)
        {

            #region Variable
            String Search_OutputXML = null;
            String FilterXML = null;
            String ErrorXML = null;
            #endregion

            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine GetAllRelatedObjects_ForArtwork Starts");

                objErrorCommon = new ALErrorCommon();
                #region Create Input XML & get the details
                ServiceOperations objServiceOperation = new ServiceOperations(UserID, Language, TenantName);
                FilterXML = objServiceOperation.CreateFilterXML("ARTWORKID", ArtworkID, "ARTWORK");
                ErrorXML = objServiceOperation.NSearch(FilterXML, AssociationXML, ref Search_OutputXML);

                if (ErrorXML != null)
                {
                    ArtworkDetailsXML = null;
                    objErrorCommon.AppendError("APENGINE_ARTWORK_GETALLRELATEDOBJECTS_FORARTWORK_FAILED", this.GetType().ToString(), "GetAllRelatedObjects_ForArtwork", null, ErrorXML);
                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                    return ErrorXML;
                }
                ArtworkDetailsXML = Search_OutputXML;
                return ErrorXML;
                #endregion
            }
            catch (Exception ExcepObj)
            {
                #region Exception Handling
                ArtworkDetailsXML = null;
                objErrorCommon.AppendError("APENGINE_ARTWORK_GETALLRELATEDOBJECTS_FORARTWORK_EXCEPTION", this.GetType().ToString(), "GetAllRelatedObjects_ForArtwork", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
                #endregion
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine GetAllRelatedObjects_ForArtwork Ends");
            }
        }

        public ERRORXML GetArtworkDetails_NSearch(XML NSearchInputXML, out XML NSearchOutputXML)
        {
            OMInterface OMIObject = null;

            ERRORXML ErrorXML = null;
            NSearchOutputXML = null;

            try
            {
                objErrorCommon = new ALErrorCommon();
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine GetArtworkDetails_NSearch Starts");

                OMIObject = new OMInterface(UserID, Language, TenantName);

                if (NSearchInputXML != null && NSearchInputXML != "")
                    ErrorXML = OMIObject.GetHierarchialData(NSearchInputXML, ref NSearchOutputXML);

                if (ErrorXML != null)
                {
                    NSearchOutputXML = null;
                    objErrorCommon.AppendError("APENGINE_ARTWORK_GETARTWORKDETAILS_NSEARCH_FAILED", this.GetType().ToString(), "GetArtworkDetails_NSearch", null, ErrorXML);
                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                    return ErrorXML;
                }

                return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                NSearchOutputXML = null;
                objErrorCommon.AppendError("APENGINE_ARTWORK_GETARTWORKDETAILS_NSEARCH_EXCEPTION", this.GetType().ToString(), "GetArtworkDetails_NSearch", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                //objErrorCommon.LogError(TenantName, ReturnErrorXML);
                return ReturnErrorXML;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine GetArtworkDetails_NSearch Ends");
            }
        }

        public ERRORXML GetArtworks(String InputXML, int ResultSize, out String ArtworkList)
        {
            ArtworkList = null;
            AssetlinkSearch objSearch = new AssetlinkSearch(UserID, Language, TenantName);
            String ErrorXML = null;
            StringBuilder sbArtworkXml = new StringBuilder();

            ErrorXML = objSearch.GetHierarchialData(InputXML, ref ArtworkList);

            if (ResultSize > 0)
            {
                XPathDocument xpDocObj = null;
                xpDocObj = new XPathDocument(new StringReader(ArtworkList));
                XPathNavigator xpNavObj = xpDocObj.CreateNavigator();
                XPathNodeIterator ArtworkNodeList = xpNavObj.Select("ARTWORKCOLLECTION/ARTWORK");
                int NodeCount = 0;
                sbArtworkXml.Append("<ARTWORKCOLLECTION>");
                while (ArtworkNodeList.MoveNext())
                {
                    NodeCount++;
                    if (NodeCount <= ResultSize)
                    {
                        sbArtworkXml.Append(ArtworkNodeList.Current.OuterXml);
                    }
                    else
                        break;
                }
                sbArtworkXml.Append("</ARTWORKCOLLECTION>");
                ArtworkList = sbArtworkXml.ToString();
                xpDocObj = null;
            }

            return ErrorXML;
        }

        public ERRORXML CreateArtwork_HTML(String ArtworkID, String ArtworkName, String FormatID, String SolutionName, String JobType, out String ZipFilePath)
        {
            ZipFilePath = null;
            String JobID = null;
            String str_DestinationPath = null;
            XPathNavigator xpNav_AddFileName = null;
            ArtworkProducerCommon helperObj = null;
            String DisplayStatus = null;
            Context ContextObj = null;
            try
            {
                StringBuilder ContextAdditionalInfo = null;
                ContextAdditionalInfo = new StringBuilder(20);
                ContextAdditionalInfo.AppendFormat("<ADDITIOALINFO><THREADID>{4}</THREADID><APINAME>ArtworkProducerEngine.ArtworkEngine.CreateArtwork_HTML</APINAME><ARTWORKID>{0}</ARTWORKID><ARTWORKNAME>{1}</ARTWORKNAME><FORMATID>{2}</FORMATID><JOBTYPE>{3}</JOBTYPE></ADDITIOALINFO>", ArtworkID, ArtworkName, FormatID, JobType, System.Threading.Thread.CurrentThread.ManagedThreadId.ToString());

                ContextObj = new Context(UserID, SessionID, TenantName, SolutionName, ContextAdditionalInfo.ToString());
                #region Stamp Media job
                helperObj = new ArtworkProducerCommon(UserID, Language, TenantName, SolutionName);
                helperObj.GetDisplayStatus("Artwork", "Artwork Generation In Progress", out DisplayStatus);

                MediaJobEngine MJEngObj = new MediaJobEngine(SessionID, UserID, Language, TenantName);
                StringBuilder CreateJobXML = new StringBuilder();
                CreateJobXML.AppendFormat(@"<MEDIAJOB>");
                if (ArtworkName != null)
                    CreateJobXML.AppendFormat("<JOBNAME>{0}</JOBNAME>", ALSpecialCharHandling.TrimAndReplaceSpecialChars(ArtworkName) + " Job");
                else
                    CreateJobXML.AppendFormat("<JOBNAME>{0}</JOBNAME>", " Job");

                CreateJobXML.AppendFormat(@"<CREATEDON>
                                            <DAY>{0}</DAY>
                                            <MONTH>{1}</MONTH>
                                            <YEAR>{2}</YEAR>
                                            <HOUR>{3}</HOUR>
                                            <MINUTE>{4}</MINUTE>
                                            <SECOND>{5}</SECOND>
                                        </CREATEDON>
                                        <MODIFIEDON>
                                            <DAY>{0}</DAY>
                                            <MONTH>{1}</MONTH>
                                            <YEAR>{2}</YEAR>
                                            <HOUR>{3}</HOUR>
                                            <MINUTE>{4}</MINUTE>
                                            <SECOND>{5}</SECOND>
                                        </MODIFIEDON>
                                        <SUBMITTEDON>
                                            <DAY>{0}</DAY>
                                            <MONTH>{1}</MONTH>
                                            <YEAR>{2}</YEAR>
                                            <HOUR>{3}</HOUR>
                                            <MINUTE>{4}</MINUTE>
                                            <SECOND>{5}</SECOND>
                                        </SUBMITTEDON>
                                        <JOBTYPE>ARTWORK GENERATION JOB</JOBTYPE>
                                        <JOBHANDLER>HTML</JOBHANDLER>
                                        <JOBSTATUS>New</JOBSTATUS>
                                        <MEDIAJOB_CREATOR TYPE='RELATION'>{6}</MEDIAJOB_CREATOR>
                                        <ARTWORK_MEDIAJOB TYPE='RELATION'>{7}</ARTWORK_MEDIAJOB>
                                        <ARTWORK_MEDIAJOB TYPE='RELATION'>
                                            <ARTWORK MODIFIED='TRUE'>
                                                <ARTWORKID>{7}</ARTWORKID>
                                                <ARTWORKSTATUS>Artwork Generation In Progress</ARTWORKSTATUS>
                                                <DISPLAYSTATUS>{8}</DISPLAYSTATUS>
                                            </ARTWORK>
                                        </ARTWORK_MEDIAJOB>
                                    </MEDIAJOB>", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour,
                                                    DateTime.Now.Minute, DateTime.Now.Second, UserID, ArtworkID, DisplayStatus);

                String str_CreateJobXML = CreateJobXML.ToString();
                String ErrorXML = MJEngObj.StampMediaJob(FormatID, ArtworkID, SolutionName, ref str_CreateJobXML);

                if (ErrorXML != null)
                    return ErrorXML;

                JobID = ALCommon.GetInnerValue(ref str_CreateJobXML, "<JOBID>", "</JOBID>");

                #endregion

                #region Get Text Element Details

                StringBuilder sbSearch = new StringBuilder();
                OMInterface oOMI = new OMInterface(UserID, Language, TenantName);
                String str_TextDetails = null;
                String str_GEValueDetails = null;

                sbSearch.AppendFormat(@"<SEARCH>
                                        <TARGETOBJECT>TEXTELEMENT</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <AUTHORIZATIONREQUIRED>False</AUTHORIZATIONREQUIRED>
                                        <TARGETENTITY>
                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                            <FILTERLIST>
                                                <FILTER>
                                                    <EXPRESSION>FORMATID</EXPRESSION>
                                                     <OPERATOR>EQUALTO</OPERATOR>
                                                    <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                    <FILTERID>1</FILTERID>
                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                    <RELATIONSHIPNAME>FORMAT_TEXTELEMENT</RELATIONSHIPNAME>
                                                </FILTER>
                                             </FILTERLIST>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>TEXTELEMENTID</FIELDNAME>
                                                    <ALIAS>TEXTELEMENTID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                    <FIELDNAME>DATATYPE</FIELDNAME>
                                                    <ALIAS>DATATYPE</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                        </TARGETENTITY>
                                  </SEARCH>", FormatID);

                ErrorXML = oOMI.GetHierarchialData(sbSearch.ToString(), ref str_TextDetails);
                if (ErrorXML != null)
                    return ErrorXML;

                sbSearch = new StringBuilder();
                sbSearch.AppendFormat(@"<SEARCH>
                                        <TARGETOBJECT>ARTWORK</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <AUTHORIZATIONREQUIRED>False</AUTHORIZATIONREQUIRED>
                                        <TARGETENTITY>
                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                            <FILTERLIST>
                                                <FILTER>
                                                    <EXPRESSION>ARTWORKID</EXPRESSION>
                                                     <OPERATOR>EQUALTO</OPERATOR>
                                                    <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                    <FILTERID>1</FILTERID>
                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                </FILTER>
                                             </FILTERLIST>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>ARTWORKID</FIELDNAME>
                                                    <ALIAS>ARTWORKID</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                            <RELATIONSHIP NAME='ARTWORK_GRAPHICELEMENTVALUEINFO'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>SHOWURL</FIELDNAME>
                                                        <ALIAS>SHOWURL</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>URL</FIELDNAME>
                                                        <ALIAS>URL</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>URLTARGET</FIELDNAME>
                                                        <ALIAS>URLTARGET</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>URLTYPE</FIELDNAME>
                                                        <ALIAS>URLTYPE</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>URLPROTOCOL</FIELDNAME>
                                                        <ALIAS>URLPROTOCOL</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                                <RELATIONSHIP NAME='GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO'>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                                            <ALIAS>GRAPHICELEMENTID</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                </RELATIONSHIP>
                                            </RELATIONSHIP>
                                            <RELATIONSHIP NAME='FORMAT_ARTWORK'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME FileRequired='Yes'>ADDITIONALFILES</FIELDNAME>
                                                        <ALIAS>ADDITIONALFILES</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                            </RELATIONSHIP>
                                        </TARGETENTITY>
                                  </SEARCH>", ArtworkID);

                ErrorXML = oOMI.GetHierarchialData(sbSearch.ToString(), ref str_GEValueDetails);
                if (ErrorXML != null)
                    return ErrorXML;

                #endregion

                XPathDocument xpDoc_TEDetails = new XPathDocument(new StringReader(str_TextDetails));
                XPathNavigator xpNav_TEDetailsRoot = xpDoc_TEDetails.CreateNavigator();

                XPathDocument xpDoc_GEValDetails = new XPathDocument(new StringReader(str_GEValueDetails));
                XPathNavigator xpNav_GEValDetails = xpDoc_GEValDetails.CreateNavigator();

                String OrgFilePath = ArtworkProducerCache.GetArtworkOriginalFilePath(TenantName, ArtworkID);
                String DirPath = ArtworkProducerCache.GetCacheFolderPath(TenantName, ArtworkID, ArtworkProducerCache.CacheType.ArtworkCache) + "\\Output\\";
                if (Directory.Exists(DirPath))
                {
                    //Directory.Delete(DirPath, true);
                    SASMOMDirectory.DeleteFolder(DirPath, true, ContextObj);
                }
                String OrgFilePath_Output = DirPath + Path.GetFileName(OrgFilePath);
                Directory.CreateDirectory(DirPath);

                File.Copy(OrgFilePath, OrgFilePath_Output);

                XmlDocument xDoc_OrgFile = new XmlDocument();
                xDoc_OrgFile.XmlResolver = null;

                Boolean SmallLetters = false;
                String ExtraString = null;
                String HTMLSource = File.ReadAllText(OrgFilePath_Output);
                int start = HTMLSource.IndexOf("<html", 0);

                if (start < 0)
                    start = HTMLSource.IndexOf("<HTML", 0);
                else
                    SmallLetters = true;

                if (start >= 0)
                    start += 5;

                if (start > 0)
                {
                    int end = HTMLSource.IndexOf(">", start);

                    if (end > start)
                    {
                        ExtraString = HTMLSource.Substring(start, end - start);
                        HTMLSource = HTMLSource.Remove(start, end - start);
                    }
                }

                //xDoc_OrgFile.Load(OrgFilePath_Output);
                xDoc_OrgFile.LoadXml(HTMLSource);

                String ImagePath = DirPath + "\\images\\";
                if (Directory.Exists(ImagePath))
                {
                    //Directory.Delete(ImagePath, true);
                    SASMOMDirectory.DeleteFolder(DirPath, true, ContextObj);
                }
                Directory.CreateDirectory(ImagePath);

                ArtworkProducerCache objCache = new ArtworkProducerCache(UserID, Language, TenantName);
                String TagValuesXML = null;
                ErrorXML = objCache.GetAllTagValues(ArtworkID, out TagValuesXML);

                XPathDocument xpDoc_TagValues = new XPathDocument(new StringReader(TagValuesXML));
                XPathNavigator xpNav_Root = xpDoc_TagValues.CreateNavigator();
                XPathNodeIterator xpIter_Tags = xpNav_Root.Select("ARTWORK/TEXTTAGS/TAG");

                while (xpIter_Tags.MoveNext())
                {
                    XPathNavigator xpNav_Curr = xpIter_Tags.Current;
                    String Name = xpNav_Curr.GetAttribute("NAME", "");
                    String ID = xpNav_Curr.GetAttribute("ID", "");
                    XPathNavigator xpNav_TE = xpNav_TEDetailsRoot.SelectSingleNode("TEXTELEMENTCOLLECTION/TEXTELEMENT [TEXTELEMENTID/text()='" + ID + "']");
                    if (xpNav_TE == null)
                        continue;
                    String TEDataType = xpNav_TE.SelectSingleNode("DATATYPE").InnerXml;

                    String TagName = Name.ToUpper();
                    XmlNodeList xListTags = xDoc_OrgFile.SelectNodes("//*[translate(@id,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='" + TagName + "']");
                    foreach (XmlElement xNode in xListTags)
                    {
                        if (TEDataType == "HTML")
                        {
                            String val = xpNav_Curr.Value;
                            if (!String.IsNullOrEmpty(val) && val.Contains("<a ") || val.Contains("<A "))
                            {
                                XmlDocument xDoc_Val = new XmlDocument();

                                val = val.Replace("&", "$amp;");

                                xDoc_Val.LoadXml("<ROOT>" + val + "</ROOT>");
                                XmlNodeList xList = xDoc_Val.SelectNodes("//a");
                                foreach (XmlElement xTENode in xList)
                                {
                                    if ((!xTENode.GetAttribute("altransitionpage").ToUpper().Equals("UNSUBSCRIBE"))
                                        && (!xTENode.GetAttribute("altransitionpage").ToUpper().Equals("VIEWEMAILONLINE")) &&
                                        (!xTENode.GetAttribute("altransitionpage").ToUpper().Equals("SENDTOFRIEND")))
                                        xTENode.SetAttribute("altransitionpage", "clickthrough");
                                }

                                xList = xDoc_Val.SelectNodes("//A");
                                foreach (XmlElement xTENode in xList)
                                {
                                    if ((!xTENode.GetAttribute("altransitionpage").ToUpper().Equals("UNSUBSCRIBE"))
                                        && (!xTENode.GetAttribute("altransitionpage").ToUpper().Equals("VIEWEMAILONLINE"))
                                        && (!xTENode.GetAttribute("altransitionpage").ToUpper().Equals("SENDTOFRIEND")))
                                        xTENode.SetAttribute("altransitionpage", "clickthrough");
                                }

                                val = xDoc_Val.CreateNavigator().SelectSingleNode("ROOT").InnerXml;
                                xNode.InnerXml = val.Replace("$amp;", "&");
                            }
                            else
                                xNode.InnerXml = xpNav_Curr.Value;
                        }
                        else
                            xNode.InnerXml = xpNav_Curr.InnerXml;
                    }
                }

                xpIter_Tags = xpNav_Root.Select("ARTWORK/GRAPHICTAGS/TAG");

                while (xpIter_Tags.MoveNext())
                {
                    XPathNavigator xpNav_Curr = xpIter_Tags.Current;
                    String OrgPath = xpNav_Curr.Value;
                    String FileName = Path.GetFileName(OrgPath);
                    String NewPath = ImagePath + FileName;

                    if (File.Exists(OrgPath))
                    {
                        File.Copy(OrgPath, NewPath, true);
                    }
                    else
                    {
                        NewPath = "";
                    }

                    NewPath = ImagePath + System.Web.HttpUtility.UrlEncode(FileName).Replace("+", " ");

                    String Name = xpNav_Curr.GetAttribute("NAME", "");
                    String TagName = Name.ToUpper();
                    XmlNodeList xListTags = xDoc_OrgFile.SelectNodes("//*[translate(@id,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='" + TagName + "']");
                    foreach (XmlElement xNode in xListTags)
                    {
                        NewPath = NewPath.Replace(DirPath + "\\", "");
                        NewPath = NewPath.Replace("\\", "/");
                        if (xNode.Name.ToUpper() == "IMG")
                        {
                            xNode.SetAttribute("src", NewPath);
                        }
                        else
                        {
                            if (xNode.HasAttribute("style", ""))
                            {
                                String StyleAttrValue = xNode.GetAttribute("style", "");
                                if (StyleAttrValue.Contains("background-image"))
                                {
                                    String strBGImage = StyleAttrValue.Substring(StyleAttrValue.IndexOf("background-image"));
                                    if (strBGImage.Contains(";"))
                                    {
                                        String strFirstSubString = StyleAttrValue.Substring(0, StyleAttrValue.IndexOf("background-image"));
                                        strBGImage = strBGImage.Remove(strBGImage.IndexOf("background-image"), strBGImage.IndexOf(";"));
                                        StyleAttrValue = strFirstSubString + strBGImage;
                                        StyleAttrValue = StyleAttrValue + ";background-image:url(" + NewPath + ");";
                                    }
                                    else
                                    {
                                        StyleAttrValue = StyleAttrValue.Remove(StyleAttrValue.IndexOf("background-image"), StyleAttrValue.Length);
                                        StyleAttrValue = StyleAttrValue + ";background-image:url(" + NewPath + ");";
                                    }
                                }
                                else
                                {
                                    StyleAttrValue = StyleAttrValue + ";background-image:url(" + NewPath + ");";
                                }
                                xNode.SetAttribute("style", StyleAttrValue);
                            }
                            else
                            {
                                String StyleAttrValue = "background-image:url(" + NewPath + ");";
                                xNode.SetAttribute("style", StyleAttrValue);
                            }
                        }


                        String TagID = xpNav_Curr.GetAttribute("ID", "");
                        XPathNavigator xpNav_GEVal = xpNav_GEValDetails.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/ARTWORK_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE [GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENT/GRAPHICELEMENTID/text()='" + TagID + "']");
                        if (xpNav_GEVal != null)
                        {
                            String val = xpNav_GEVal.SelectSingleNode("SHOWURL").InnerXml;
                            String URLType = xpNav_GEVal.SelectSingleNode("URLTYPE").InnerXml;
                            //if ((val == "1") || (val == "0"))
                            if (URLType.ToUpper().ToString() != "NONE")
                            {
                                String URL = xpNav_GEVal.SelectSingleNode("URL").InnerXml;
                                String Target = xpNav_GEVal.SelectSingleNode("URLTARGET").InnerXml;
                                String URLProtocol = xpNav_GEVal.SelectSingleNode("URLPROTOCOL").InnerXml;

                                URL = URLProtocol + URL;

                                String AlTransitionPageValue = null;
                                if (String.Compare(URLType, "UNSUBSCRIBE", true) == 0)
                                {
                                    AlTransitionPageValue = "unsubscribe";
                                }
                                else if (String.Compare(URLType, "TRACKING", true) == 0)
                                {
                                    AlTransitionPageValue = "clickthrough";
                                }
                                else if (String.Compare(URLType, "SENDTOFRIEND", true) == 0)
                                {
                                    AlTransitionPageValue = "SendToFriend";
                                }
                                else if (String.Compare(URLType, "VIEWEMAILONLINE", true) == 0)
                                {
                                    AlTransitionPageValue = "ViewEmailOnline";
                                }
                                else
                                    AlTransitionPageValue = null;

                                if (xNode.Name.ToUpper() == "IMG")
                                {
                                    if (xNode.ParentNode.Name != "a")
                                    {
                                        XmlElement xAnchor = xDoc_OrgFile.CreateElement("a");
                                        XmlAttribute xAttr1 = xDoc_OrgFile.CreateAttribute("href");
                                        xAttr1.Value = URL;

                                        if (!String.IsNullOrEmpty(AlTransitionPageValue))
                                        {
                                            xAnchor.SetAttribute("altransitionpage", AlTransitionPageValue);
                                            //xAnchor.SetAttribute("alaction", AlActionValue);
                                        }

                                        XmlAttribute xAttr2 = xDoc_OrgFile.CreateAttribute("target");
                                        switch (Target)
                                        {
                                            case "SAME":
                                                xAttr2.Value = "_self";
                                                break;

                                            case "NEW":
                                                xAttr2.Value = "_blank";
                                                break;

                                            default:
                                                xAttr2.Value = "_blank";
                                                break;
                                        }

                                        xAnchor.Attributes.Append(xAttr1);
                                        xAnchor.Attributes.Append(xAttr2);

                                        xNode.ParentNode.ReplaceChild(xAnchor, xNode);
                                        xAnchor.AppendChild(xNode);
                                    }
                                    else
                                    {
                                        if (xNode.ParentNode.Attributes["id"] != null && String.Compare(xNode.ParentNode.Attributes["id"].Value, "A_" + xNode.Attributes["id"].Value, true) == 0)
                                        {
                                            //XmlAttribute xAttr = xDoc_OrgFile.CreateAttribute("alhref");
                                            //xAttr.Value = xNode.ParentNode.Attributes["href"].Value;
                                            //xNode.ParentNode.Attributes.Append(xAttr);
                                            //xNode.ParentNode.Attributes.Remove(xNode.ParentNode.Attributes["href"]);

                                            //if (!URL.StartsWith("http"))
                                            //    URL = "http://" + URL;

                                            if (xNode.ParentNode.Attributes["href"] == null)
                                            {
                                                XmlAttribute xAttr = xDoc_OrgFile.CreateAttribute("href");
                                                xNode.ParentNode.Attributes.Append(xAttr);
                                            }

                                            xNode.ParentNode.Attributes["href"].Value = URL;
                                            if (!String.IsNullOrEmpty(AlTransitionPageValue))
                                            {
                                                ((XmlElement)xNode.ParentNode).SetAttribute("altransitionpage", AlTransitionPageValue);
                                                //((XmlElement)xNode.ParentNode).SetAttribute("alaction", AlActionValue);
                                            }

                                            if (xNode.ParentNode.Attributes["target"] == null)
                                            {
                                                XmlAttribute xAttr = xDoc_OrgFile.CreateAttribute("target");
                                                xNode.ParentNode.Attributes.Append(xAttr);
                                            }

                                            switch (Target)
                                            {
                                                case "SAME":
                                                    xNode.ParentNode.Attributes["target"].Value = "_self";
                                                    break;

                                                case "NEW":
                                                    xNode.ParentNode.Attributes["target"].Value = "_blank";
                                                    break;

                                                default:
                                                    xNode.ParentNode.Attributes["target"].Value = "_blank";
                                                    break;
                                            }
                                        }
                                    }
                                }
                                else
                                {
                                    String UrlTarget = null;
                                    if (xNode.Attributes["onclick"] == null)
                                    {
                                        XmlAttribute xAttr = xDoc_OrgFile.CreateAttribute("onclick");
                                        xNode.Attributes.Append(xAttr);
                                    }

                                    switch (Target)
                                    {
                                        case "SAME":
                                            UrlTarget = "_self";
                                            break;

                                        case "NEW":
                                            UrlTarget = "_blank";
                                            break;

                                        default:
                                            UrlTarget = "_blank";
                                            break;
                                    }

                                    xNode.Attributes["onclick"].Value = "window.open('" + URL + "','" + UrlTarget + "','');";
                                    if (!String.IsNullOrEmpty(AlTransitionPageValue))
                                    {
                                        xNode.SetAttribute("altransitionpage", AlTransitionPageValue);
                                        //xNode.SetAttribute("alaction", AlActionValue);
                                    }
                                }
                            }
                            //else
                            //{
                            //    if (xpNav_GEVal.SelectSingleNode("URLTYPE") != null)
                            //    {
                            //        String URLType = xpNav_GEVal.SelectSingleNode("URLTYPE").InnerXml;
                            //        String AlTransitionPageValue = null;
                            //        if (!String.IsNullOrEmpty(URLType))
                            //        {
                            //            if (String.Compare(URLType, "SENDTOFRIEND", true) == 0)
                            //            {
                            //                AlTransitionPageValue = "SendToFriend";
                            //            }
                            //            else if (String.Compare(URLType, "VIEWEMAILONLINE", true) == 0)
                            //            {
                            //                AlTransitionPageValue = "ViewEmailOnline";
                            //            }
                            //            else AlTransitionPageValue = null;
                            //            if (!String.IsNullOrEmpty(AlTransitionPageValue))
                            //            {
                            //                xNode.SetAttribute("altransitionpage", AlTransitionPageValue);
                            //            }
                            //        }
                            //    }
                            //}
                        }
                        //if (xNode.ParentNode.Name == "a")
                        //{
                        //    if (xNode.ParentNode.Attributes["id"] == null && String.Compare(xNode.ParentNode.Attributes["id"].Value, "A_" + xNode.Attributes["id"].Value, true) == 0)
                        //    {
                        //        XmlAttribute xAttr = xDoc_OrgFile.CreateAttribute("alhref");
                        //        xAttr.Value = xNode.ParentNode.Attributes["href"].Value;
                        //        xNode.ParentNode.Attributes.Append(xAttr);
                        //        xNode.ParentNode.Attributes.Remove(xNode.ParentNode.Attributes["href"]);
                        //    }
                        //}
                    }
                }

                #region get Additional files to images folder
                XPathNavigator xpNav_AdditionalFiles = xpNav_GEValDetails.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/FORMAT_ARTWORK/FORMAT/FILESYSTEM/BINDER[FIELDNAME='ADDITIONALFILES']");
                if (xpNav_AdditionalFiles != null)
                {
                    foreach (XPathNavigator xpNav_File in xpNav_AdditionalFiles.Select("FILE"))
                    {
                        xpNav_AddFileName = xpNav_File.SelectSingleNode("DISPLAYNAME");
                        if (xpNav_AddFileName != null && !String.IsNullOrEmpty(xpNav_AddFileName.Value))
                            str_DestinationPath = ImagePath + "\\" + xpNav_AddFileName.Value;
                        else
                            str_DestinationPath = ImagePath + "\\" + xpNav_File.SelectSingleNode("NAME").Value;

                        File.Copy(xpNav_File.SelectSingleNode("PATH").Value, str_DestinationPath, true);
                    }
                }
                #endregion

                HTMLSource = xDoc_OrgFile.OuterXml;

                if (!String.IsNullOrEmpty(ExtraString))
                {
                    if (SmallLetters)
                        HTMLSource = HTMLSource.Replace("<html>", "<html" + ExtraString + ">");
                    else
                        HTMLSource = HTMLSource.Replace("<HTML>", "<HTML" + ExtraString + ">");
                }

                File.WriteAllText(OrgFilePath_Output, HTMLSource, Encoding.UTF8);

                /*NotRequired*/
                //File.WriteAllText(Path.GetDirectoryName(OrgFilePath_Output) + "\\" + Path.GetFileNameWithoutExtension(OrgFilePath_Output) + ".txt", FinalTXT, Encoding.UTF8);

                //xDoc_OrgFile.Save(OrgFilePath_Output);


                #region Zip the Output folder

                Assetlink.Services.Plugins.ManagedPlugInsWrapper.ZipWrapper ZipObj = new Assetlink.Services.Plugins.ManagedPlugInsWrapper.ZipWrapper();
                ZipObj.PreservePaths = true;
                String ZipFileName = ArtworkName + "_Zip";
                ArtworkProducerEngine.ConvertToAlphaNumericString(ref ZipFileName);
                ErrorXML = ZipObj.ZipFiles(DirPath, DirPath, ZipFileName);

                #endregion

                ZipFilePath = DirPath + "\\" + ZipFileName + ".zip";

                #region Save Zip file

                String CopyOfZip = ALCommon.GetTempPath() + "Artwork Preview Files\\";// +Guid.NewGuid().ToString() + ".zip";
                if (!Directory.Exists(CopyOfZip))
                    Directory.CreateDirectory(CopyOfZip);
                CopyOfZip += Guid.NewGuid().ToString() + ".zip";

                if (File.Exists(ZipFilePath))
                {
                    File.Copy(ZipFilePath, CopyOfZip, true);
                }

                List<String> objData = new List<string>();
                objData.Add(SolutionName);
                objData.Add(ArtworkID);
                objData.Add(ArtworkName);
                objData.Add(FormatID);
                objData.Add(JobType);
                objData.Add(JobID);
                objData.Add(CopyOfZip);

                //OrgIdentity = System.Security.Principal.WindowsIdentity.GetCurrent();
                //System.Threading.ThreadPool.QueueUserWorkItem(SaveHTMLArtwork, objData);
                SaveHTMLArtwork(objData);

                #endregion

                return null;
            }
            catch (Exception ExcepObj)
            {
                if (!String.IsNullOrEmpty(JobID))
                    ArtworkProducerCommon.UpdateJobStatus(UserID, Language, TenantName, SessionID, "Error", JobID);
                ArtworkProducerCommon.UpdateArtworkStatus(UserID, Language, TenantName, SolutionName, "Error", ArtworkID);
                objErrorCommon.AppendError("APENGINE_ARTWORK_CREATEARTWORK_HTML_EXCEPTION", this.GetType().ToString(), "CreateArtwork_HTML", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                return ReturnErrorXML;
            }
        }

        public ERRORXML GetTagValuesForArtwork(String ArtworkID, out String TagValuesXML)
        {
            TagValuesXML = null;
            ArtworkProducerCache objAPCache = new ArtworkProducerCache(UserID, Language, TenantName);
            ERRORXML ErrorXML = objAPCache.GetAllTagValues(ArtworkID, out TagValuesXML);
            if (ErrorXML != null)
                return ErrorXML;

            XmlDocument xDoc_TagValues = new XmlDocument();
            xDoc_TagValues.LoadXml(TagValuesXML);

            XmlNodeList xGEList = xDoc_TagValues.SelectNodes("ARTWORK/GRAPHICTAGS/TAG");
            String TempPath = ALCommon.GetTempPath();
            foreach (XmlElement xGENode in xGEList)
            {
                String FilePath = xGENode.InnerText;
                if (File.Exists(FilePath))
                {
                    String Newpath = TempPath + Guid.NewGuid().ToString() + Path.GetExtension(FilePath);
                    File.Copy(FilePath, Newpath, true);
                    xGENode.InnerXml = Newpath;
                }
            }
            TagValuesXML = xDoc_TagValues.OuterXml;
            return null;
        }

        public static bool IsArtworkPartOfBatch(ulong UserID, String Language, String TenantName, String ArtworkID, out String BatchJobID)
        {
            BatchJobID = null;
            return ArtworkProducerCache.IsArtworkPartOfBatch(UserID, Language, TenantName, ArtworkID, out BatchJobID);
        }

        public static bool IsBookArtwork(ulong UserID, String Language, String TenantName, String ArtworkID, out String BookID)
        {
            BookID = null;
            return ArtworkProducerCache.IsBookArtwork(UserID, Language, TenantName, ArtworkID, out BookID);
        }

        public ERRORXML LogArtworkError(String ObjectID, ErrorObjectType ObjectType, String ErrorCode, String ErrorDetails, String InnerError, ErrorType ErrorTypeValue, StringDictionary strdict_AdditionalInfo, bool bUpdateArtwork)
        {
            String BatchJobID = null;

            if (bUpdateArtwork && ObjectType == ErrorObjectType.Artwork)
            {
                ArtworkProducerCommon.UpdateArtworkStatus(UserID, Language, TenantName, SolutionName, "Error", ObjectID);
            }

            bool isPartofBatch = IsArtworkPartOfBatch(UserID, Language, TenantName, ObjectID, out BatchJobID);
            if (isPartofBatch)
            {
                ArtworkJobEngine.BatchJobStatus NewJobStatus;
                ArtworkJobEngine.BatchJobStatus PreviousJobStatus;
                ArtworkJobEngine oJobEng = new ArtworkJobEngine(SessionID, UserID, Language, TenantName, SolutionName);
                oJobEng.CalculateJobProgress(BatchJobID, out PreviousJobStatus, out NewJobStatus);
                if ((!(PreviousJobStatus == ArtworkJobEngine.BatchJobStatus.Done
                    || (PreviousJobStatus == ArtworkJobEngine.BatchJobStatus.Error)))
                    && (NewJobStatus == ArtworkJobEngine.BatchJobStatus.Error
                    || NewJobStatus == ArtworkJobEngine.BatchJobStatus.Done))
                {
                    oJobEng.HandlePostProcessing(BatchJobID);
                }
            }

            ArtworkProducerCommon oCommon = new ArtworkProducerCommon(UserID, Language, TenantName);
            if (ObjectType == ErrorObjectType.Artwork)
                oCommon.LogError(ErrorCode, "ArtworkProducer", ErrorDetails, InnerError, ErrorTypeValue, strdict_AdditionalInfo, ObjectID, "ARTWORK_JOBERROR");
            else
                oCommon.LogError(ErrorCode, "ArtworkProducer", ErrorDetails, InnerError, ErrorTypeValue, strdict_AdditionalInfo, ObjectID, "MEDIAJOBWATCHER_JOBERROR");

            return null;
        }

        public ERRORXML AutoPopulateOptions(String ArtworkID, String TextElementFilterXML, String TaxonomyDataXML, Boolean IsRelatedToBook, String OptionMode, out Boolean OptionsPopulated)
        {
            ERRORXML ErrorXML = null;
            String FormatID = null;
            String FormatType = null;
            String Usage = null;

            OptionsPopulated = false;
            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine AutoPopulateOptions Starts");
                ErrorXML = PopulateOptions(ArtworkID, TextElementFilterXML, TaxonomyDataXML, IsRelatedToBook, OptionMode, out FormatID, out FormatType, out Usage, out OptionsPopulated);
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_ARTWORK_AUTOPOPULATEOPTIONS_EXCEPTION", this.GetType().ToString(), "AutoPopulateOptions", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                return ReturnErrorXML;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine AutoPopulateOptions Ends");
            }
            return ErrorXML;
        }

        public ERRORXML PopulateOptionsAndGenerateArtwork(String ArtworkID, String TaxonomyDataXML, Boolean IsRelatedToBook, String OptionMode, out Boolean IsJobStamped)
        {
            ERRORXML ErrorXML = null;
            Boolean OptionsPopulated = false;
            String FormatID = null;
            String FormatType = null;
            String Usage = null;

            IsJobStamped = false;
            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine PopulateOptionsAndGenerateArtwork Starts");
                ErrorXML = PopulateOptions(ArtworkID, null, TaxonomyDataXML, IsRelatedToBook, OptionMode, out FormatID, out FormatType, out Usage, out OptionsPopulated);
                if (ErrorXML == null && OptionsPopulated)
                {
                    if (String.Compare(FormatType, "HTML", true) != 0)
                        ErrorXML = GenerateArtwork(FormatID, FormatType, Usage, ArtworkID);
                    if (ErrorXML == null)
                        IsJobStamped = true;
                }
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_ARTWORK_POPULATEOPTIONSANDGENERATEARTWORK_EXCEPTION", this.GetType().ToString(), "PopulateOptionsAndGenerateArtwork", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                return ReturnErrorXML;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine PopulateOptionsAndGenerateArtwork Ends");
            }
            return ErrorXML;
        }

        public ERRORXML CopyArtwork(String ArtworkID, String NewArtworkName, String NewFormatId, out EntityID NewArtworkID)
        {
            ERRORXML ErrorXML = null;
            XML OutputXML = null;
            ArtworkProducerCommon helperObj = null;
            XML AssociatedObjectXML = null;
            StringBuilder SearchInputXML = null;
            XPathNavigator XPathNav = null;
            XPathDocument AssociatedFacetTypeXMLDoc = null;
            XPathDocument xpDoc_TextTagForNewFormat = null;
            XPathNodeIterator RelationshipTypeNodes = null;
            XPathNodeIterator xpTextTagNodeItr = null;
            XPathNodeIterator xpGraphicNodeItr = null;
            String RelationshipName = null;
            XmlDocument ArtworkCollectionXMLDoc = null;
            String PrimaryKeyName = null;
            String SectionID = null;
            ALEngineError ALEngineErrorLocalObj = null;
            XmlNode ColumnNode = null;
            String FormatID = null;
            XmlNode Artwork_TEVRelNode = null;
            XmlNode Artwork_GEVRelNode = null;
            XmlNode Artwork_GERelNode = null;
            XmlNode ArtworkTagNode = null;
            XmlNodeList TextElementValueNodes = null;
            XmlNodeList GraphicElementValueNodes = null;
            XmlNodeList GraphicElementNodes = null;
            XmlNode ArtworkNode = null;
            OMInterface OMIObj = null;
            String DisplayStatus = null;
            String OldGraphicElementId = null;
            String NewGraphicElementId = null;
            StringBuilder CreateAuthorizationXML = null;
            OperationQueue OMQueueObj = null;
            Operation OperObj = null;
            XML OperationXML = null;
            XmlNode ArtworkDigitalAssetRelNode = null;
            XmlNode ReplacedFileNode = null;
            XmlNodeList DigitalAssetIDNodeList = null;
            //String                          DigitalAssetID                      = null;
            List<String> DigitalAssetIDList = null;
            ArtworkProducerCommon ArtworkProducerCommonObj = null;
            XmlElement Artwork_DigitalAsset = null;
            XPathNavigator xpNav_TagsForNewFormat = null;
            String FormatType = null;
            NewArtworkID = 0;
            try
            {
                ALEngineErrorLocalObj = new ALEngineError();
                SearchInputXML = new StringBuilder(100);
                OMIObj = new OMInterface(UserID, Language, TenantName);
                ArtworkProducerCommonObj = new ArtworkProducerCommon(UserID, Language, TenantName);

                #region Get facets types that can be associated with Artwork

                helperObj = new ArtworkProducerCommon(UserID, Language, TenantName, SolutionName);
                ErrorXML = helperObj.GetAssociatedFacetTypeHierarchy("ARTWORK", ref AssociatedObjectXML);
                if (ErrorXML != null)
                    return ErrorXML;


                AssociatedFacetTypeXMLDoc = new XPathDocument(new StringReader(AssociatedObjectXML));
                XPathNav = AssociatedFacetTypeXMLDoc.CreateNavigator();
                RelationshipTypeNodes = XPathNav.Select("descendant::RELATEDFACETTYPECOLLECTION/TYPE");

                #endregion

                #region Get Artwork along with the required Facets and TE/GE

                SearchInputXML.AppendFormat(@"<SEARCH>
                                                    <TARGETOBJECT>ARTWORK</TARGETOBJECT>
                                                    <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                    <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                                    <SESSIONINFO UserID='{0}' TenantName='{1}' Language='{2}'/>
                                                    <TARGETENTITY>
                                                        <FILTERLIST>
                                                            <FILTER>
	                                                            <EXPRESSION>ARTWORKID</EXPRESSION>
	                                                            <OPERATOR>EQUALTO</OPERATOR>
	                                                            <EXPRESSIONVALUE>{3}</EXPRESSIONVALUE>
	                                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                            </FILTER>
                                                        </FILTERLIST>
                                                        <RELATIONSHIP NAME='FORMAT_ARTWORK'>
                                                            <SELECTLIST>
                                                                <EXPRESSION>
                                                                    <FIELDNAME>FORMATID</FIELDNAME>
                                                                    <ALIAS>FORMATID</ALIAS>
                                                                </EXPRESSION>
                                                                <EXPRESSION>
                                                                    <FIELDNAME>FORMATTYPE</FIELDNAME>
                                                                    <ALIAS>FORMATTYPE</ALIAS>
                                                                </EXPRESSION>
                                                            </SELECTLIST>
                                                        </RELATIONSHIP>
                                                        <RELATIONSHIP NAME='ARTWORK_PREVIEWASSET'>
                                                           <SORTLIST>
							                                    <SORTEXPRESSION>
								                                    <SORTORDER>DESC</SORTORDER>
								                                    <COLUMNNAME>CREATEDON</COLUMNNAME>
							                                    </SORTEXPRESSION>
						                                   </SORTLIST>
                                                           <SELECTLIST>
                                                                <EXPRESSION>
                                                                    <FIELDNAME>DIGITALASSETID</FIELDNAME>
                                                                    <ALIAS>DIGITALASSETID</ALIAS>
                                                                </EXPRESSION>
                                                            </SELECTLIST>
                                                            <FILTERLIST>
                                                                <FILTER>
                                                                    <EXPRESSION>MARKEDASDELETED</EXPRESSION>
                                                                    <OPERATOR>EQUALTO</OPERATOR>
                                                                    <EXPRESSIONVALUE>0</EXPRESSIONVALUE>
                                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                                </FILTER>
                                                            </FILTERLIST>
                                                        </RELATIONSHIP>
                                                        <RELATIONSHIP NAME='ARTWORK_TEXTELEMENTVALUE'>
                                                            <RELATIONSHIP NAME='TEXTELEMENT_TEXTELEMENTVALUE'>
                                                                 <SELECTLIST>
                                                                    <EXPRESSION>
                                                                        <FIELDNAME>TEXTELEMENTID</FIELDNAME>
                                                                        <ALIAS>TEXTELEMENTID</ALIAS>
                                                                    </EXPRESSION>
                                                                    <EXPRESSION>
                                                                        <FIELDNAME>TEXTTAGNAME</FIELDNAME>
                                                                        <ALIAS>TEXTTAGNAME</ALIAS>
                                                                    </EXPRESSION>
                                                                </SELECTLIST>
                                                            </RELATIONSHIP>
                                                        </RELATIONSHIP>
                                                        <RELATIONSHIP NAME='ARTWORK_GRAPHICELEMENTVALUEINFO'>
                                                            <RELATIONSHIP NAME='TEXTELEMENT_TEXTELEMENTVALUE'>
                                                                 <SELECTLIST>
                                                                    <EXPRESSION>
                                                                        <FIELDNAME>TEXTELEMENTID</FIELDNAME>
                                                                        <ALIAS>TEXTELEMENTID</ALIAS>
                                                                    </EXPRESSION>
                                                                </SELECTLIST>
                                                            </RELATIONSHIP>
                                                            <RELATIONSHIP NAME='GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO'>
                                                                 <SELECTLIST>
                                                                    <EXPRESSION>
                                                                        <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                                                        <ALIAS>GRAPHICELEMENTID</ALIAS>
                                                                    </EXPRESSION>
                                                                    <EXPRESSION>
                                                                        <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
                                                                        <ALIAS>GRAPHICTAGNAME</ALIAS>
                                                                    </EXPRESSION>
                                                                </SELECTLIST>
                                                            </RELATIONSHIP>
                                                            <RELATIONSHIP NAME='GRAPHICELEMENTVALUEINFO_DIGITALASSET'>
                                                                 <SELECTLIST>
                                                                    <EXPRESSION>
                                                                        <FIELDNAME>DIGITALASSETID</FIELDNAME>
                                                                        <ALIAS>DIGITALASSETID</ALIAS>
                                                                    </EXPRESSION>
                                                                </SELECTLIST>
                                                            </RELATIONSHIP>
                                                        </RELATIONSHIP>                                                        
                                                        <RELATIONSHIP NAME='ARTWORK_GRAPHICELEMENT'>
                                                            <RELATIONSHIP NAME='GRAPHICELEMENT_REPLACEDFILE' HIERARCHY='BOTTOMUP'>
                                                                 <SELECTLIST>
                                                                    <EXPRESSION>
                                                                        <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                                                        <ALIAS>GRAPHICELEMENTID</ALIAS>
                                                                    </EXPRESSION>
                                                                </SELECTLIST>
                                                            </RELATIONSHIP>
                                                        </RELATIONSHIP>
                                                        <RELATIONSHIP NAME='ARTWORK_EXTRACTTEMPLATE'>
                                                            <SELECTLIST>
                                                                <EXPRESSION>
                                                                    <FIELDNAME>EXTRACTTEMPLATEID</FIELDNAME>
                                                                    <ALIAS>EXTRACTTEMPLATEID</ALIAS>
                                                                </EXPRESSION>
                                                            </SELECTLIST>
                                                        </RELATIONSHIP>
                                                        ", UserID, TenantName, Language, ArtworkID);

                while (RelationshipTypeNodes.MoveNext())
                {
                    RelationshipName = RelationshipTypeNodes.Current.GetAttribute("RelationshipName", "");
                    SearchInputXML.AppendFormat(" <RELATIONSHIP NAME='{0}'/>", RelationshipName);
                }

                SearchInputXML.Append(@"        </TARGETENTITY>
                                            </SEARCH>");

                ErrorXML = OMIObj.GetHierarchialData(SearchInputXML.ToString(), ref OutputXML);
                if (ErrorXML != null)
                {
                    ALEngineErrorLocalObj.AppendError("ARTWORKPRODUCERENGINE_ARTWORK_READ_FAILED", this.GetType().ToString(), "CopyArtwork", null, ErrorXML);
                    return ALEngineErrorLocalObj.ErrorCollectionXML;
                }

                ArtworkCollectionXMLDoc = new XmlDocument();
                ArtworkCollectionXMLDoc.LoadXml(OutputXML);

                if (!(String.IsNullOrEmpty(NewFormatId)))
                {
                    String TagValuesForNewFormat = null;
                    StringBuilder sbSearchXml = null;

                    sbSearchXml = new StringBuilder(255);
                    sbSearchXml.Append(@"<SEARCH>
                                          <TARGETOBJECT>FORMAT</TARGETOBJECT>
                                          <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                          <TARGETENTITY>
                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                            <FILTERLIST>
                                              <FILTER>
                                                <EXPRESSION>FORMATID</EXPRESSION>
                                                <OPERATOR>EQUALTO</OPERATOR>
                                                <EXPRESSIONVALUE>" + NewFormatId + @"</EXPRESSIONVALUE>
                                                <FILTERID>1</FILTERID>
                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                              </FILTER>
                                            </FILTERLIST>
                                            <SELECTLIST>
                                              <EXPRESSION>
                                                <FIELDNAME>FORMATID</FIELDNAME>
                                                <ALIAS>FORMATID</ALIAS>
                                              </EXPRESSION>
                                            </SELECTLIST>
                                            <RELATIONSHIP NAME='FORMAT_TEXTELEMENT'>
                                              <SELECTLIST>
                                                <EXPRESSION>
                                                  <FIELDNAME>TEXTELEMENTID</FIELDNAME>
                                                  <ALIAS>TEXTELEMENTID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                  <FIELDNAME>TEXTTAGNAME</FIELDNAME>
                                                  <ALIAS>TEXTTAGNAME</ALIAS>
                                                </EXPRESSION>
                                              </SELECTLIST>
                                            </RELATIONSHIP>
                                            <RELATIONSHIP NAME='FORMAT_GRAPHICELEMENT'>
                                              <SELECTLIST>
                                                <EXPRESSION>
                                                  <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                                  <ALIAS>GRAPHICELEMENTID</ALIAS>
                                                </EXPRESSION>
                                                <EXPRESSION>
                                                  <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
                                                  <ALIAS>GRAPHICTAGNAME</ALIAS>
                                                </EXPRESSION>
                                              </SELECTLIST>
                                            </RELATIONSHIP>
                                          </TARGETENTITY>
                                        </SEARCH>");
                    OMIObj = new OMInterface(UserID, Language, TenantName);
                    ErrorXML = OMIObj.GetHierarchialData(sbSearchXml.ToString(), ref TagValuesForNewFormat);

                    if (ErrorXML == null)
                    {
                        xpDoc_TextTagForNewFormat = new XPathDocument(new StringReader(TagValuesForNewFormat));
                        xpNav_TagsForNewFormat = xpDoc_TextTagForNewFormat.CreateNavigator();
                        xpTextTagNodeItr = xpNav_TagsForNewFormat.Select("FORMATCOLLECTION/FORMAT/FORMAT_TEXTELEMENT/TEXTELEMENT");
                        xpGraphicNodeItr = xpNav_TagsForNewFormat.Select("FORMATCOLLECTION/FORMAT/FORMAT_GRAPHICELEMENT/GRAPHICELEMENT");

                        ArtworkTagNode = ArtworkCollectionXMLDoc.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/FORMAT_ARTWORK/FORMAT");
                        if (ArtworkTagNode != null)                        
                            ArtworkTagNode.SelectSingleNode("FORMATID").InnerText = NewFormatId;                            
                        
                        while (xpTextTagNodeItr.MoveNext())
                        {
                            ArtworkTagNode = ArtworkCollectionXMLDoc.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/ARTWORK_TEXTELEMENTVALUE/TEXTELEMENTVALUE/TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENT [TEXTTAGNAME/text()='" + xpTextTagNodeItr.Current.SelectSingleNode("TEXTTAGNAME").Value + "']");
                            if (ArtworkTagNode != null)
                                ArtworkTagNode.SelectSingleNode("TEXTELEMENTID").InnerText = xpTextTagNodeItr.Current.SelectSingleNode("TEXTELEMENTID").Value;
                        }
                        while (xpGraphicNodeItr.MoveNext())
                        {
                            ArtworkTagNode = ArtworkCollectionXMLDoc.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/ARTWORK_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE [GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENT/GRAPHICTAGNAME/text()='" + xpGraphicNodeItr.Current.SelectSingleNode("GRAPHICTAGNAME").Value + "']");
                            NewGraphicElementId = xpGraphicNodeItr.Current.SelectSingleNode("GRAPHICELEMENTID").Value;

                            if (ArtworkTagNode != null)
                            {
                                OldGraphicElementId = ArtworkTagNode.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENT/GRAPHICELEMENTID").InnerText;
                                ArtworkTagNode.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENT/GRAPHICELEMENTID").InnerText = NewGraphicElementId;
                            }
                            ReplacedFileNode = ArtworkCollectionXMLDoc.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/ARTWORK_GRAPHICELEMENT/GRAPHICELEMENT/GRAPHICELEMENT_REPLACEDFILE/GRAPHICELEMENT [GRAPHICELEMENTID/text()='" + OldGraphicElementId + "']");
                            if (ReplacedFileNode != null)
                                ReplacedFileNode.SelectSingleNode("GRAPHICELEMENTID").InnerText = NewGraphicElementId;
                        }
                    }
                }

                #endregion

                if (ArtworkCollectionXMLDoc.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/FORMAT_ARTWORK/FORMAT/FORMATTYPE") != null)
                    FormatType = ArtworkCollectionXMLDoc.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/FORMAT_ARTWORK/FORMAT/FORMATTYPE").InnerText;

                #region Remove primary key nodes

                RelationshipTypeNodes = XPathNav.Select("descendant::RELATEDFACETTYPECOLLECTION/TYPE");
                while (RelationshipTypeNodes.MoveNext())
                {
                    RelationshipName = RelationshipTypeNodes.Current.GetAttribute("RelationshipName", "");
                    PrimaryKeyName = RelationshipTypeNodes.Current.SelectSingleNode("PRIMARYKEY").Value;
                    RemovePrimaryKeyNodes(PrimaryKeyName, RelationshipName, ArtworkCollectionXMLDoc);
                }

                #endregion

                #region update date/required fields and add required relations
                ArtworkNode = ArtworkCollectionXMLDoc.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK");

                ArtworkDigitalAssetRelNode = ArtworkNode.SelectSingleNode("ARTWORK_PREVIEWASSET");
                if (ArtworkDigitalAssetRelNode != null)
                {
                    DigitalAssetIDNodeList = ArtworkNode.SelectNodes("ARTWORK_PREVIEWASSET/DIGITALASSET/DIGITALASSETID");
                    if (DigitalAssetIDNodeList != null)
                    {
                        DigitalAssetIDList = new List<String>();
                        for (int Index = 0; Index < DigitalAssetIDNodeList.Count; Index++)
                            DigitalAssetIDList.Add(DigitalAssetIDNodeList[Index].InnerText);
                        //DigitalAssetID = DigitalAssetIDNode.InnerText;
                    }
                    ArtworkNode.RemoveChild(ArtworkDigitalAssetRelNode);
                }
                                
                ColumnNode = ArtworkNode.SelectSingleNode("ARTWORKID");
                UniqueID.Read(ref NewArtworkID);
                ColumnNode.InnerText = NewArtworkID.ToString();

                ColumnNode = ArtworkNode.SelectSingleNode("ARTWORKNAME");
                if (!String.IsNullOrEmpty(NewArtworkName))
                {
                    ColumnNode.InnerText = NewArtworkName;
                }
                else
                {
                    NewArtworkName = ColumnNode.InnerText;
                }

                ColumnNode = ArtworkNode.SelectSingleNode("ARTWORKSTATUS");
                ColumnNode.InnerText = "Draft";

                ColumnNode = ArtworkNode.SelectSingleNode("DISPLAYSTATUS");                
                helperObj.GetDisplayStatus("Artwork", "Draft", out DisplayStatus);
                ColumnNode.InnerText = DisplayStatus;
                

                ColumnNode = ArtworkNode.SelectSingleNode("CREATEDON");
                ColumnNode.InnerXml = ALCommon.CurrentDateTimeToXml();

                ColumnNode = ArtworkNode.SelectSingleNode("SUBMITTEDON");
                ColumnNode.InnerXml = "";

                ColumnNode = ArtworkNode.SelectSingleNode("MODIFIEDON");
                ColumnNode.InnerXml = ALCommon.CurrentDateTimeToXml();

                ColumnNode = ArtworkNode.SelectSingleNode("FORMAT_ARTWORK");
                FormatID = ColumnNode.SelectSingleNode("FORMAT/FORMATID").InnerText;
                ((XmlElement)ColumnNode).SetAttribute("TYPE", "RELATION");
                ((XmlElement)ColumnNode).SetAttribute("PARENTID", "TRUE");
                ColumnNode.InnerXml = FormatID;

                ColumnNode = ArtworkCollectionXMLDoc.CreateElement("ARTWORK_CREATOR");
                ((XmlElement)ColumnNode).SetAttribute("TYPE", "RELATION");
                ColumnNode.InnerText = UserID.ToString();
                ArtworkNode.AppendChild(ColumnNode);

                ColumnNode = ArtworkCollectionXMLDoc.CreateElement("ARTWORK_MODIFIER");
                ((XmlElement)ColumnNode).SetAttribute("TYPE", "RELATION");
                ColumnNode.InnerText = UserID.ToString();
                ArtworkNode.AppendChild(ColumnNode);

                #endregion

                #region Remove primary key nodes of TE/GE and set required attributes
                ArtworkNode = ArtworkCollectionXMLDoc.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK");

                // Copy TextElementValue
                Artwork_TEVRelNode = ArtworkNode.SelectSingleNode("ARTWORK_TEXTELEMENTVALUE[TEXTELEMENTVALUE]");
                if (Artwork_TEVRelNode != null)
                {
                    ((XmlElement)Artwork_TEVRelNode).SetAttribute("TYPE", "RELATION");
                    TextElementValueNodes = Artwork_TEVRelNode.SelectNodes("TEXTELEMENTVALUE");
                    foreach (XmlNode TextElementValueNode in TextElementValueNodes)
                    {
                        ColumnNode = TextElementValueNode.SelectSingleNode("TEXTELEMENTVALUEID");
                        TextElementValueNode.RemoveChild(ColumnNode);

                        ColumnNode = TextElementValueNode.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE");
                        ((XmlElement)ColumnNode).SetAttribute("TYPE", "RELATION");
                        ((XmlElement)ColumnNode).SetAttribute("PARENTID", "TRUE");
                        ColumnNode.InnerXml = TextElementValueNode.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENT/TEXTELEMENTID").InnerText;
                    }
                }

                // Copy GraphicElementValue
                Artwork_GEVRelNode = ArtworkNode.SelectSingleNode("ARTWORK_GRAPHICELEMENTVALUEINFO[GRAPHICELEMENTVALUE]");
                if (Artwork_GEVRelNode != null)
                {
                    ((XmlElement)Artwork_GEVRelNode).SetAttribute("TYPE", "RELATION");
                    GraphicElementValueNodes = Artwork_GEVRelNode.SelectNodes("GRAPHICELEMENTVALUE");
                    foreach (XmlNode GraphicElementValueNode in GraphicElementValueNodes)
                    {
                        ColumnNode = GraphicElementValueNode.SelectSingleNode("GRAPHICELEMENTVALUEID");
                        GraphicElementValueNode.RemoveChild(ColumnNode);

                        ColumnNode = GraphicElementValueNode.SelectSingleNode("GRAPHICELEMENTVALUEINFO_DIGITALASSET[DIGITALASSET]");
                        if (ColumnNode != null)
                        {
                            ((XmlElement)ColumnNode).SetAttribute("TYPE", "RELATION");
                            ((XmlElement)ColumnNode).SetAttribute("PARENTID", "TRUE");
                            ColumnNode.InnerXml = GraphicElementValueNode.SelectSingleNode("GRAPHICELEMENTVALUEINFO_DIGITALASSET/DIGITALASSET/DIGITALASSETID").InnerText;

                            Artwork_DigitalAsset = ArtworkCollectionXMLDoc.CreateElement("ARTWORK_DIGITALASSET");
                            Artwork_DigitalAsset.SetAttribute("TYPE", "RELATION");
                            Artwork_DigitalAsset.InnerXml = ColumnNode.InnerXml;
                            ArtworkNode.AppendChild(Artwork_DigitalAsset);
                        }

                        ColumnNode = GraphicElementValueNode.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO[GRAPHICELEMENT]");
                        if (ColumnNode != null)
                        {
                            ((XmlElement)ColumnNode).SetAttribute("TYPE", "RELATION");
                            ((XmlElement)ColumnNode).SetAttribute("PARENTID", "TRUE");
                            ColumnNode.InnerXml = GraphicElementValueNode.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENT/GRAPHICELEMENTID").InnerText;
                        }
                    }
                }

                // Copy GraphicElement
                Artwork_GERelNode = ArtworkNode.SelectSingleNode("ARTWORK_GRAPHICELEMENT[GRAPHICELEMENT]");
                if (Artwork_GERelNode != null)
                {
                    ((XmlElement)Artwork_GERelNode).SetAttribute("TYPE", "RELATION");
                    GraphicElementNodes = Artwork_GERelNode.SelectNodes("GRAPHICELEMENT");
                    foreach (XmlNode GraphicElementNode in GraphicElementNodes)
                    {
                        ColumnNode = GraphicElementNode.SelectSingleNode("GRAPHICELEMENTID");
                        GraphicElementNode.RemoveChild(ColumnNode);

                        ColumnNode = GraphicElementNode.SelectSingleNode("CREATEDON");
                        ColumnNode.InnerXml = ALCommon.CurrentDateTimeToXml();

                        ColumnNode = GraphicElementNode.SelectSingleNode("MODIFIEDON");
                        ColumnNode.InnerXml = ALCommon.CurrentDateTimeToXml();

                        ColumnNode = ArtworkCollectionXMLDoc.CreateElement("GRAPHICELEMENT_CREATOR");
                        ((XmlElement)ColumnNode).SetAttribute("TYPE", "RELATION");
                        ColumnNode.InnerText = UserID.ToString();
                        GraphicElementNode.AppendChild(ColumnNode);

                        ColumnNode = ArtworkCollectionXMLDoc.CreateElement("GRAPHICELEMENT_MODIFIER");
                        ((XmlElement)ColumnNode).SetAttribute("TYPE", "RELATION");
                        ColumnNode.InnerText = UserID.ToString();
                        GraphicElementNode.AppendChild(ColumnNode);

                        ColumnNode = GraphicElementNode.SelectSingleNode("GRAPHICELEMENT_REPLACEDFILE[GRAPHICELEMENT]");
                        if (ColumnNode != null)
                        {
                            ((XmlElement)ColumnNode).SetAttribute("TYPE", "RELATION");
                            ((XmlElement)ColumnNode).SetAttribute("PARENTID", "TRUE");
                            ColumnNode.InnerXml = GraphicElementNode.SelectSingleNode("GRAPHICELEMENT_REPLACEDFILE/GRAPHICELEMENT/GRAPHICELEMENTID").InnerText;
                        }
                    }
                }

                ColumnNode = ArtworkNode.SelectSingleNode("ARTWORK_EXTRACTTEMPLATE");
                if (ColumnNode != null)
                {
                    ((XmlElement)ColumnNode).SetAttribute("TYPE", "RELATION");
                    ColumnNode.InnerXml = ArtworkNode.SelectSingleNode("ARTWORK_EXTRACTTEMPLATE/EXTRACTTEMPLATE/EXTRACTTEMPLATEID").InnerText;
                }

                #endregion                

                #region Prepare Authorization XML for Creator of Artwork
                CreateAuthorizationXML = new StringBuilder();
                CreateAuthorizationXML.Append("<AUTHORIZATION>");
                CreateAuthorizationXML.Append("<OBJECTINSTANCE>");
                CreateAuthorizationXML.AppendFormat("<OBJECTID>{0}</OBJECTID>", NewArtworkID);
                CreateAuthorizationXML.Append("<OBJECTTYPE>ENTITY</OBJECTTYPE>");
                CreateAuthorizationXML.Append("<OBJECTTYPENAME>ARTWORK</OBJECTTYPENAME>");
                CreateAuthorizationXML.Append("<ACCESSCONTROLLIST>");
                CreateAuthorizationXML.Append("<ACCESSCONTROLENTRY>");
                CreateAuthorizationXML.AppendFormat("<PRIVILEGENAME>DELETE</PRIVILEGENAME>");
                CreateAuthorizationXML.Append("<ACCESSORTYPE>USER</ACCESSORTYPE>");
                CreateAuthorizationXML.AppendFormat("<ACCESSORID>{0}</ACCESSORID>", UserID);
                CreateAuthorizationXML.Append("</ACCESSCONTROLENTRY>");
                CreateAuthorizationXML.Append("</ACCESSCONTROLLIST>");
                CreateAuthorizationXML.Append("</OBJECTINSTANCE>");
                CreateAuthorizationXML.Append("</AUTHORIZATION>");
                #endregion

                OMQueueObj = new OperationQueue(true);
                OperObj = new Operation("ENTITY", "CREATEDATA");
                OperObj.AddAttribute("COPYFILES", "TRUE");
                OperObj.AddParameter("INPUTXML", ArtworkNode.OuterXml);
                OMQueueObj.AddOperation(OperObj);

                #region Add Operation for creating authorization for Creator of Artwork
                OperObj = new Operation("AUTHORIZATION", "CREATE");
                OperObj.AddParameter("INPUTXML", CreateAuthorizationXML.ToString());
                OMQueueObj.AddOperation(OperObj);
                #endregion

                //Increment Format's GenerationCount value
                ErrorXML = ArtworkProducerCommonObj.IncrementGenerationCount(FormatID, ArtworkProducerCommon.EntityType.Format, false, out OperObj);
                OMQueueObj.AddOperation(OperObj);

                OperationXML = OMQueueObj.Xml;
                ErrorXML = OMIObj.Execute(ref OperationXML);

                if (ErrorXML != null)
                {
                    objErrorCommon.AppendError("APENGINE_ARTWORK_COPY_FAILED", this.GetType().ToString(), "CopyArtwork", null, ErrorXML);
                    return objErrorCommon.ErrorCollectionXML;
                }

                #region Copy LargePreview(DigitalAsset) from OldArtwork to NewArtwork

                #region Get/Create Section for Book
                ErrorXML = Services.Engines.ArtworkProducerEngine.ArtworkProducerCommon.GetSectionForArtwork(UserID, Language, TenantName, NewArtworkID.ToString(), NewArtworkName, out SectionID, "");
                if (ErrorXML != null)
                    return ErrorXML;
                #endregion

                if (DigitalAssetIDList != null && DigitalAssetIDList.Count > 0)
                {
                    for (int Index = 0; Index < DigitalAssetIDList.Count; Index++)
                    {
                        ErrorXML = CopyDigitalAsset(DigitalAssetIDList[Index], NewArtworkID.ToString(), NewArtworkName, "ARTWORK_PREVIEWASSET", SectionID);
                        if (ErrorXML != null)
                        {
                            objErrorCommon.AppendError("APENGINE_ARTWORK_COPY_FAILED", this.GetType().ToString(), "CopyArtwork", null, ErrorXML);
                            return objErrorCommon.ErrorCollectionXML;
                        }
                    }
                }                               
                #endregion

                if (String.Compare(FormatType, "READYTOUSE", true) == 0 && ErrorXML == null)
                {
                    ErrorXML = GenerateReadyToUseArtwork(NewArtworkID.ToString(), "ARTWORK GENERATION JOB");
                }
            }
            catch (Exception)
            {
                throw;
            }
            return ErrorXML;
        }

        public ERRORXML CopyArtwork(String ArtworkID, String NewArtworkName)
        {
            ERRORXML ErrorXML = null;
            EntityID NewArtworkID = 0;
            String NewFormatId = null;
            CopyArtwork(ArtworkID, NewArtworkName, NewFormatId, out NewArtworkID);

            return ErrorXML;
        }

        public ERRORXML UpdateArtworkForDAMChanges(string ArtworkID, string TaxonomyDataXML, bool IsRelatedToBook, ref string UpdatedList)
        {
            ERRORXML ErrorXML = null;
            StringBuilder sbSearchInputXML = null;
            OMInterface OMIObj = null;
            String SearchOutputXML = null;
            XPathDocument xpOutputDoc = null;
            XPathNavigator xpOutputNav = null;
            XPathNodeIterator xpElementIter = null;
            string ElementID = null;
            string JobType = null;
            string ModifiedElem_List = null;
            string OperationList = null;
            string IsMultipleAllowed = null;
            StringBuilder sbArtworkValueXML = null;
            StringBuilder sbTEValue_Asset = null;
            XPathNodeIterator xpAssetIter = null;
            String OptionValue = null;
            String PrevOptionValue = null;
            String ArtworkValueXML = null;
            String DataSource = null;
            XML ArtworkDetails = null;
            String WebServiceName = null;
            String Prefix = null;
            String Suffix = null;
            String TEValue_Asset = null;
            string FormatID = null;
            string FormatType = null;
            String Usage = null;
            XML OptionsXML = null;
            string TagName = null;
            string AssetOrder = null;
            ArtworkProducerEngine APEngineObj = null;
            string TagConfigurationXML = null;
            bool AssetDeleted = false;
            string NewAssetOrder = null;
            String AdditionalInfo = null;
            String DataType = null;
            Boolean LengthValidation = false;
            Boolean RangeValidation = false;
            int MinValue = 0;
            int MaxValue = 0;
            int OptionCount = 1;
            int MaxLength = 0;
            String ErrorString = null;
            string ErrorReport = null;
            string UpdateArtworkXml = null;
            bool PreviousUpdatePresent = false;
            string PreviousUpdateXml = null;
            int SelectedOptionCount = 0;
            int DeleteEventCount = 0;
            int RestoreEventCount = 0;
            int ExpireEventCount = 0;
            int ReviveEventCount = 0;
            string OldTextElementValue = null;
            XPathNodeIterator OldUpdates = null;
            Dictionary<string, string> Elem_List = null;
            String OptionValueRelation = null;
            String OptionValueObject = null;
            String OptionValueColumn = null;

            try
            {
                OMIObj = new OMInterface(UserID, Language, TenantName);

                sbSearchInputXML = new StringBuilder(40);
                sbSearchInputXML.AppendFormat(@"<SEARCH>
                                            <TARGETOBJECT>ARTWORK</TARGETOBJECT>
                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                            <TARGETENTITY>
	                                            <SELECTLIST>
		                                            <EXPRESSION>
			                                            <FIELDNAME>ARTWORKID</FIELDNAME>
			                                            <ALIAS>ARTWORKID</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>REGENERATIONXML</FIELDNAME>
			                                            <ALIAS>REGENERATIONXML</ALIAS>
		                                            </EXPRESSION>
	                                            </SELECTLIST>
	                                            <FILTERLIST>
                                                    <FILTER>
			                                            <EXPRESSION>ARTWORKID</EXPRESSION>
			                                            <OPERATOR>EQUALTO</OPERATOR>
			                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
			                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>			                                            
		                                            </FILTER>
	                                            </FILTERLIST>                                                                                         
                                            </TARGETENTITY>
                                            <SESSIONINFO Language='{1}' TenantName='{2}' UserID='{3}' SessionID='{4}'/>
                                        </SEARCH>", ArtworkID, Language, TenantName, UserID, SessionID);

                ErrorXML = OMIObj.GetHierarchialData(sbSearchInputXML.ToString(), ref SearchOutputXML);
                if (ErrorXML != null)
                {
                    objErrorCommon.AppendError("APENGINE_ARTWORK_GETARTWORKDETAILS_FAILED", this.GetType().ToString(), "UpdateArtworksForAssetChanges", null, ErrorXML);
                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                }

                if (!String.IsNullOrEmpty(SearchOutputXML))
                {
                    xpOutputDoc = new XPathDocument(new StringReader(SearchOutputXML));
                    xpOutputNav = xpOutputDoc.CreateNavigator();
                    xpElementIter = xpOutputNav.Select("ARTWORKCOLLECTION/ARTWORK/REGENERATIONXML/MODIFIED_ELEMENTS/PREPROCESS/ELEMENT");
                    Elem_List = new Dictionary<string, string>();

                    while (xpElementIter.MoveNext())
                    {
                        ElementID = xpElementIter.Current.GetAttribute("ELEMENTID", "");
                        JobType = xpElementIter.Current.GetAttribute("JOBTYPE", "");
                        if (!Elem_List.ContainsKey(ElementID))
                        {
                            Elem_List.Add(ElementID, JobType);
                        }
                        else
                        {
                            Elem_List[ElementID] += "," + JobType;
                        }
                    }

                    if (xpOutputNav.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK/REGENERATIONXML/MODIFIED_ELEMENTS/POSTPROCESS/ELEMENT") != null)
                    {
                        PreviousUpdatePresent = true;
                        OldUpdates = xpOutputNav.Select("ARTWORKCOLLECTION/ARTWORK/REGENERATIONXML/MODIFIED_ELEMENTS/POSTPROCESS/ELEMENT");
                        while (OldUpdates.MoveNext())
                        {
                            if (!Elem_List.ContainsKey(OldUpdates.Current.GetAttribute("ELEMENTID", "")))
                                PreviousUpdateXml += OldUpdates.Current.OuterXml;
                        }
                    }
                }

                if ((Elem_List != null) && (Elem_List.Count > 0))
                {
                    foreach (string Elem_Value in Elem_List.Keys)
                    {
                        ModifiedElem_List += Elem_Value + ",";
                    }

                    ModifiedElem_List = ModifiedElem_List.Substring(0, ModifiedElem_List.Length - 1);

                    sbSearchInputXML = new StringBuilder(40);
                    sbSearchInputXML.AppendFormat(@"<SEARCH>
                                            <TARGETOBJECT>FORMAT</TARGETOBJECT>
                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                            <TARGETENTITY>
	                                            <SELECTLIST>
		                                            <EXPRESSION>
			                                            <FIELDNAME>FORMATID</FIELDNAME>
			                                            <ALIAS>FORMATID</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>FORMATTYPE</FIELDNAME>
			                                            <ALIAS>FORMATTYPE</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>USAGE</FIELDNAME>
			                                            <ALIAS>USAGE</ALIAS>
		                                            </EXPRESSION>
	                                            </SELECTLIST>
	                                            <FILTERLIST>
                                                    <FILTER>
			                                            <EXPRESSION>ARTWORKID</EXPRESSION>
			                                            <OPERATOR>EQUALTO</OPERATOR>
			                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
			                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
			                                            <RELATIONSHIPNAME>FORMAT_ARTWORK</RELATIONSHIPNAME>
		                                            </FILTER>
	                                            </FILTERLIST>
	                                            <RELATIONSHIP NAME='FORMAT_ARTWORKPAGE'>
		                                            <SELECTLIST>
			                                            <EXPRESSION>
				                                            <FIELDNAME>ARTWORKPAGEID</FIELDNAME>
				                                            <ALIAS>ARTWORKPAGEID</ALIAS>
			                                            </EXPRESSION>
		                                            </SELECTLIST>
		                                            <RELATIONSHIP NAME='ARTWORKPAGE_ELEMENTGROUP'>
			                                            <SELECTLIST>
				                                            <EXPRESSION>
					                                            <FIELDNAME>ELEMENTGROUPID</FIELDNAME>
					                                            <ALIAS>ELEMENTGROUPID</ALIAS>
				                                            </EXPRESSION>
			                                            </SELECTLIST>
			                                            <RELATIONSHIP NAME='ELEMENTGROUP_GRAPHICELEMENT'>
				                                            <SELECTLIST>
					                                            <EXPRESSION>
						                                            <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
						                                            <ALIAS>GRAPHICELEMENTID</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
						                                            <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
						                                            <ALIAS>GRAPHICTAGNAME</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
						                                            <FIELDNAME>GRAPHICELEMENTNAME</FIELDNAME>
						                                            <ALIAS>GRAPHICELEMENTNAME</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
						                                            <FIELDNAME>TAGCONFIGURATIONXML</FIELDNAME>
						                                            <ALIAS>TAGCONFIGURATIONXML</ALIAS>
					                                            </EXPRESSION>                                                                                                                                   
				                                            </SELECTLIST>
                                                            <FILTERLIST>
                                                                <FILTER>
			                                                        <EXPRESSION>GRAPHICELEMENTID</EXPRESSION>
			                                                        <OPERATOR>IN</OPERATOR>
			                                                        <EXPRESSIONVALUE>({5})</EXPRESSIONVALUE>
			                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>			                                                        
		                                                        </FILTER>
	                                                        </FILTERLIST>                                                                    
				                                            <RELATIONSHIP NAME='GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO'>
					                                            <SELECTLIST>
						                                            <EXPRESSION>
							                                            <FIELDNAME>GRAPHICELEMENTVALUEID</FIELDNAME>
							                                            <ALIAS>GRAPHICELEMENTVALUEID</ALIAS>
						                                            </EXPRESSION>                                                                    					                                            
					                                            </SELECTLIST>	
                                                                <FILTERLIST>
						                                            <FILTER>
							                                            <EXPRESSION>ARTWORKID</EXPRESSION>
							                                            <OPERATOR>EQUALTO</OPERATOR>
							                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
							                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
							                                            <RELATIONSHIPNAME>ARTWORK_GRAPHICELEMENTVALUEINFO</RELATIONSHIPNAME>
						                                            </FILTER>
					                                            </FILTERLIST>				                                            
					                                            <RELATIONSHIP NAME='GRAPHICELEMENTVALUEINFO_DIGITALASSET'>
						                                            <SELECTLIST>
							                                            <EXPRESSION>
								                                            <FIELDNAME>DIGITALASSETID</FIELDNAME>
								                                            <ALIAS>DIGITALASSETID</ALIAS>
							                                            </EXPRESSION>                                                                        
						                                            </SELECTLIST>
					                                            </RELATIONSHIP>
				                                            </RELATIONSHIP>
			                                            </RELATIONSHIP>
			                                            <RELATIONSHIP NAME='ELEMENTGROUP_TEXTELEMENT'>
				                                            <SELECTLIST>
					                                            <EXPRESSION>
						                                            <FIELDNAME>TEXTELEMENTID</FIELDNAME>
						                                            <ALIAS>TEXTELEMENTID</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
						                                            <FIELDNAME>TEXTTAGNAME</FIELDNAME>
						                                            <ALIAS>TEXTTAGNAME</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
						                                            <FIELDNAME>TEXTELEMENTNAME</FIELDNAME>
						                                            <ALIAS>TEXTELEMENTNAME</ALIAS>
					                                            </EXPRESSION>
					                                            <EXPRESSION>
						                                            <FIELDNAME>EDITABLE</FIELDNAME>
						                                            <ALIAS>EDITABLE</ALIAS>
					                                            </EXPRESSION>
					                                            <EXPRESSION>
						                                            <FIELDNAME>ISMULTIPLEALLOWED</FIELDNAME>
						                                            <ALIAS>ISMULTIPLEALLOWED</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
						                                            <FIELDNAME>TAGCONFIGURATIONXML</FIELDNAME>
						                                            <ALIAS>TAGCONFIGURATIONXML</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
						                                            <FIELDNAME>FIELDNODE</FIELDNAME>
						                                            <ALIAS>FIELDNODE</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
						                                            <FIELDNAME>DATATYPE</FIELDNAME>
						                                            <ALIAS>DATATYPE</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
					                                                <FIELDNAME>DATASOURCE</FIELDNAME>
					                                                <ALIAS>DATASOURCE</ALIAS>
				                                                </EXPRESSION> 	                                                                
				                                            </SELECTLIST>
                                                            <FILTERLIST>
                                                                <FILTER>
			                                                        <EXPRESSION>TEXTELEMENTID</EXPRESSION>
			                                                        <OPERATOR>IN</OPERATOR>
			                                                        <EXPRESSIONVALUE>({5})</EXPRESSIONVALUE>
			                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>			                                                        
		                                                        </FILTER>
	                                                        </FILTERLIST>      
				                                            <RELATIONSHIP NAME='TEXTELEMENT_TEXTELEMENTVALUE'>
					                                            <SELECTLIST>
						                                            <EXPRESSION>
							                                            <FIELDNAME>TEXTELEMENTVALUEID</FIELDNAME>
							                                            <ALIAS>TEXTELEMENTVALUEID</ALIAS>
						                                            </EXPRESSION>
						                                            <EXPRESSION>
							                                            <FIELDNAME>TEXTELEMENTACTUALVALUE</FIELDNAME>
							                                            <ALIAS>TEXTELEMENTACTUALVALUE</ALIAS>
						                                            </EXPRESSION>                                                                     
                                                                    <EXPRESSION>
							                                            <FIELDNAME>ASSETORDER</FIELDNAME>
							                                            <ALIAS>ASSETORDER</ALIAS>
						                                            </EXPRESSION>        
					                                            </SELECTLIST>	
                                                                <FILTERLIST>
						                                            <FILTER>
							                                            <EXPRESSION>ARTWORKID</EXPRESSION>
							                                            <OPERATOR>EQUALTO</OPERATOR>
							                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
							                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
							                                            <RELATIONSHIPNAME>ARTWORK_TEXTELEMENTVALUE</RELATIONSHIPNAME>
						                                            </FILTER>
					                                            </FILTERLIST>				                                           
                                                                <RELATIONSHIP NAME='TEXTELEMENTVALUE_DIGITALASSET'>
                                                                    <SELECTLIST>
						                                                <EXPRESSION>
							                                                <FIELDNAME>DIGITALASSETID</FIELDNAME>
							                                                <ALIAS>DIGITALASSETID</ALIAS>
						                                                </EXPRESSION>                                                                        
					                                                </SELECTLIST>
                                                                </RELATIONSHIP>
				                                            </RELATIONSHIP>
			                                            </RELATIONSHIP>
		                                            </RELATIONSHIP>
	                                            </RELATIONSHIP>
                                            </TARGETENTITY>
                                            <SESSIONINFO Language='{1}' TenantName='{2}' UserID='{3}' SessionID='{4}'/>
                                        </SEARCH>", ArtworkID, Language, TenantName, UserID, SessionID, ModifiedElem_List);

                    ErrorXML = OMIObj.GetHierarchialData(sbSearchInputXML.ToString(), ref SearchOutputXML);

                    if (ErrorXML == null && !String.IsNullOrEmpty(SearchOutputXML))
                    {
                        xpOutputDoc = new XPathDocument(new StringReader(SearchOutputXML));
                        xpOutputNav = xpOutputDoc.CreateNavigator();
                        xpOutputNav = xpOutputNav.SelectSingleNode("FORMATCOLLECTION/FORMAT");

                        if (xpOutputNav != null)
                        {
                            FormatID = xpOutputNav.SelectSingleNode("FORMATID").Value;
                            FormatType = xpOutputNav.SelectSingleNode("FORMATTYPE").Value;
                            Usage = xpOutputNav.SelectSingleNode("USAGE").Value;
                            sbArtworkValueXML = new StringBuilder(40);
                            sbArtworkValueXML.Append("<PAGE><GROUPELEMENT>");

                            //Text element
                            xpElementIter = xpOutputNav.Select("FORMAT_ARTWORKPAGE/ARTWORKPAGE/ARTWORKPAGE_ELEMENTGROUP/ELEMENTGROUP/ELEMENTGROUP_TEXTELEMENT/TEXTELEMENT");
                            while (xpElementIter.MoveNext())
                            {
                                sbTEValue_Asset = new StringBuilder(40);
                                if (xpElementIter.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENTVALUE/ASSETORDER") != null)
                                {
                                    AssetOrder = xpElementIter.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENTVALUE/ASSETORDER").Value;
                                }
                                else
                                {
                                    AssetOrder = null;
                                }
                                OldTextElementValue = xpElementIter.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENTVALUE/TEXTELEMENTACTUALVALUE").InnerXml;
                                if (!String.IsNullOrEmpty(AssetOrder))
                                {
                                    AssetDeleted = false;
                                    TEValue_Asset = null;
                                    NewAssetOrder = null;
                                    OptionValue = null;
                                    AdditionalInfo = null;
                                    ErrorString = null;
                                    SelectedOptionCount = 0;
                                    OperationList = Elem_List[xpElementIter.Current.SelectSingleNode("TEXTELEMENTID").Value];
                                    IsMultipleAllowed = xpElementIter.Current.SelectSingleNode("ISMULTIPLEALLOWED").Value;
                                    DataSource = xpElementIter.Current.SelectSingleNode("DATASOURCE").InnerXml;
                                    if (OperationList.Contains("DELETEASSETS") || (OperationList.Contains("ASSETSEXPIRED")))
                                    {
                                        DeleteEventCount = ComputeStringFrequency(OperationList, "DELETEASSETS");
                                        RestoreEventCount = ComputeStringFrequency(OperationList, "RESTOREASSETS");
                                        if (DeleteEventCount > RestoreEventCount)
                                            AssetDeleted = true;

                                        ExpireEventCount = ComputeStringFrequency(OperationList, "ASSETSEXPIRED");
                                        ReviveEventCount = ComputeStringFrequency(OperationList, "REVIVEASSETS");
                                        if (ExpireEventCount > ReviveEventCount)
                                            AssetDeleted = true;
                                    }
                                    if ((!AssetDeleted) || ((AssetDeleted) && (IsMultipleAllowed == "1")) || (DataSource.ToUpper() == "WEBSERVICE"))
                                    {
                                        TagName = xpElementIter.Current.SelectSingleNode("TEXTTAGNAME").Value;

                                        if (xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGPREFIX") != null)
                                            Prefix = xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGPREFIX").Value;
                                        else
                                            Prefix = String.Empty;

                                        if (xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGSUFFIX") != null)
                                            Suffix = xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGSUFFIX").Value;
                                        else
                                            Suffix = String.Empty;

                                        if (xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/OPTIONVALUEINFO/RELATIONSHIPNAME") != null)
                                            OptionValueRelation = xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/OPTIONVALUEINFO/RELATIONSHIPNAME").Value;

                                        if (xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/OPTIONVALUEINFO/RELATEDOBJECTNAME") != null)
                                            OptionValueObject = xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/OPTIONVALUEINFO/RELATEDOBJECTNAME").Value;

                                        if (xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/OPTIONVALUEINFO/OPTIONVALUECOLUMN") != null)
                                            OptionValueColumn = xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/OPTIONVALUEINFO/OPTIONVALUECOLUMN").Value;

                                        if (String.IsNullOrEmpty(OptionValueRelation) || String.IsNullOrEmpty(OptionValueObject) || String.IsNullOrEmpty(OptionValueColumn))
                                        {
                                            OptionValueRelation = "DIGITALASSET_TAGTAXONOMY";
                                            OptionValueObject = "TAGTAXONOMY";
                                            OptionValueColumn = "OPTIONVALUE";
                                        }

                                        switch (DataSource.ToUpper())
                                        {
                                            case "WEBSERVICE":
                                                if (xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/DATASOURCE_CONFIG/WEBSERVICE_CONFIG/WEBSERVICENAME") != null)
                                                {
                                                    ErrorXML = GetFacetDataForWSCall(ArtworkID, IsRelatedToBook, ref ArtworkDetails);
                                                    WebServiceName = xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/DATASOURCE_CONFIG/WEBSERVICE_CONFIG/WEBSERVICENAME").Value;

                                                    if (IsRelatedToBook)
                                                        ArtworkDetails = ArtworkDetails.Replace("</BOOK>", "<FORMATTYPE>" + FormatType.ToUpper() + "</FORMATTYPE></BOOK>");
                                                    else
                                                        ArtworkDetails = ArtworkDetails.Replace("</ARTWORK>", "<FORMATTYPE>" + FormatType.ToUpper() + "</FORMATTYPE></ARTWORK>");

                                                    ErrorXML = GetOption_Webservice(ArtworkDetails, "TEXT", IsRelatedToBook, WebServiceName, out OptionValue, out AdditionalInfo);
                                                    if (!String.IsNullOrEmpty(OptionValue))
                                                        OptionValue = Prefix + OptionValue + Suffix;

                                                    if (AdditionalInfo.Contains("<ASSETIDLIST>"))
                                                    {
                                                        TEValue_Asset = AdditionalInfo.Replace("<ASSETIDLIST>", "").Replace("</ASSETIDLIST>", "");
                                                        NewAssetOrder = TEValue_Asset;
                                                    }
                                                }
                                                break;

                                            default:
                                                APEngineObj = new ArtworkProducerEngine(UserID, Language, TenantName, SolutionName);
                                                TagConfigurationXML = xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML").InnerXml;
                                                ErrorXML = APEngineObj.GetOption(TagName, TagConfigurationXML, TaxonomyDataXML, IsRelatedToBook, out OptionsXML);

                                                if (ErrorXML == null && !String.IsNullOrEmpty(OptionsXML))
                                                {
                                                    XPathDocument xpOptionsDoc = null;
                                                    XPathNavigator xpOptionsNav = null;

                                                    xpOptionsDoc = new XPathDocument(new StringReader(OptionsXML));
                                                    xpOptionsNav = xpOptionsDoc.CreateNavigator();

                                                    string[] AssetOrderList = AssetOrder.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
                                                    String TempOptionValue = null;
                                                    XPathNavigator xpNav_OptionVal = null;

                                                    foreach (string AssetID in AssetOrderList)
                                                    {
                                                        xpNav_OptionVal = xpOptionsNav.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET[DIGITALASSETID = '" + AssetID + "']/" + OptionValueRelation + "/" + OptionValueObject + "/" + OptionValueColumn);
                                                        if (xpNav_OptionVal != null)
                                                        {
                                                            TempOptionValue = xpNav_OptionVal.InnerXml;

                                                            if (!String.IsNullOrEmpty(TempOptionValue))
                                                            {
                                                                TempOptionValue = Prefix + TempOptionValue + Suffix;

                                                                if (!String.IsNullOrEmpty(OptionValue))
                                                                    TempOptionValue = OptionValue + "\r\n" + TempOptionValue;

                                                                OptionValue = TempOptionValue;

                                                                if (!String.IsNullOrEmpty(AssetID))
                                                                    sbTEValue_Asset.AppendFormat("{0},", AssetID);

                                                                SelectedOptionCount++;
                                                            }
                                                        }
                                                    }

                                                    if (sbTEValue_Asset != null)
                                                    {
                                                        TEValue_Asset = sbTEValue_Asset.ToString();
                                                        NewAssetOrder = sbTEValue_Asset.ToString();
                                                    }
                                                }
                                                break;
                                        }


                                        if (ErrorXML == null)
                                        {
                                            sbArtworkValueXML.Append("<TEXTELEMENT MODIFIED='TRUE'>");
                                            sbArtworkValueXML.AppendFormat("<TEXTELEMENTID>{0}</TEXTELEMENTID>", xpElementIter.Current.SelectSingleNode("TEXTELEMENTID").Value);
                                            sbArtworkValueXML.Append("<TEXTELEMENT_TEXTELEMENTVALUE TYPE='RELATION'>");
                                            if (xpElementIter.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENTVALUE") == null)
                                            {
                                                sbArtworkValueXML.Append("<TEXTELEMENTVALUE>");
                                            }
                                            else
                                            {
                                                sbArtworkValueXML.Append("<TEXTELEMENTVALUE MODIFIED='TRUE'>");
                                                sbArtworkValueXML.AppendFormat("<TEXTELEMENTVALUEID>{0}</TEXTELEMENTVALUEID>", xpElementIter.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENTVALUE/TEXTELEMENTVALUEID").Value);
                                            }
                                            sbArtworkValueXML.AppendFormat("<TEXTELEMENTACTUALVALUE>{0}</TEXTELEMENTACTUALVALUE>", OptionValue);
                                            sbArtworkValueXML.AppendFormat("<ASSETORDER>{0}</ASSETORDER>", NewAssetOrder);

                                            xpAssetIter = xpElementIter.Current.Select("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENTVALUE/TEXTELEMENTVALUE_DIGITALASSET/DIGITALASSET/DIGITALASSETID");
                                            while (xpAssetIter.MoveNext())
                                            {
                                                if (String.IsNullOrEmpty(TEValue_Asset))
                                                    sbArtworkValueXML.AppendFormat("<TEXTELEMENTVALUE_DIGITALASSET  TYPE='RELATION' DISASSOCIATE='TRUE'>{0}</TEXTELEMENTVALUE_DIGITALASSET>", xpAssetIter.Current.Value);
                                                else if (!TEValue_Asset.Contains(xpAssetIter.Current.Value))
                                                    sbArtworkValueXML.AppendFormat("<TEXTELEMENTVALUE_DIGITALASSET  TYPE='RELATION' DISASSOCIATE='TRUE'>{0}</TEXTELEMENTVALUE_DIGITALASSET>", xpAssetIter.Current.Value);
                                                else
                                                    TEValue_Asset = TEValue_Asset.Replace(xpAssetIter.Current.Value + ",", "");
                                            }

                                            if (!String.IsNullOrEmpty(TEValue_Asset))
                                            {
                                                String[] AssetList = TEValue_Asset.Split(new String[] { "," }, StringSplitOptions.RemoveEmptyEntries);
                                                for (int i = 0; i < AssetList.Length; i++)
                                                    sbArtworkValueXML.AppendFormat("<TEXTELEMENTVALUE_DIGITALASSET  TYPE='RELATION'>{0}</TEXTELEMENTVALUE_DIGITALASSET>", AssetList[i].ToString());
                                            }

                                            sbArtworkValueXML.Append("</TEXTELEMENTVALUE>");

                                            sbArtworkValueXML.Append("</TEXTELEMENT_TEXTELEMENTVALUE>");
                                            sbArtworkValueXML.Append("</TEXTELEMENT>");
                                        }
                                        else
                                        {
                                            objErrorCommon.AppendError("APENGINE_ARTWORK_GETOPTIONS_FAILED", this.GetType().ToString(), "PopulateOptions", null, ErrorXML);
                                        }
                                    }
                                    else
                                    {
                                        TagName = xpElementIter.Current.SelectSingleNode("TEXTTAGNAME").Value;
                                        sbArtworkValueXML.Append("<TEXTELEMENT MODIFIED='TRUE'>");
                                        sbArtworkValueXML.AppendFormat("<TEXTELEMENTID>{0}</TEXTELEMENTID>", xpElementIter.Current.SelectSingleNode("TEXTELEMENTID").Value);
                                        sbArtworkValueXML.Append("<TEXTELEMENT_TEXTELEMENTVALUE TYPE='RELATION'>");
                                        if (xpElementIter.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENTVALUE") == null)
                                        {
                                            sbArtworkValueXML.Append("<TEXTELEMENTVALUE>");
                                        }
                                        else
                                        {
                                            sbArtworkValueXML.Append("<TEXTELEMENTVALUE MODIFIED='TRUE'>");
                                            sbArtworkValueXML.AppendFormat("<TEXTELEMENTVALUEID>{0}</TEXTELEMENTVALUEID>", xpElementIter.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENTVALUE/TEXTELEMENTVALUEID").Value);
                                        }
                                        sbArtworkValueXML.AppendFormat("<TEXTELEMENTACTUALVALUE></TEXTELEMENTACTUALVALUE>");
                                        sbArtworkValueXML.Append("<ASSETORDER></ASSETORDER>");
                                        sbArtworkValueXML.Append("</TEXTELEMENTVALUE>");
                                        sbArtworkValueXML.Append("</TEXTELEMENT_TEXTELEMENTVALUE>");
                                        sbArtworkValueXML.Append("</TEXTELEMENT>");
                                    }

                                    ReturnValidationDetails(xpElementIter.Current, out DataType, out LengthValidation, out MaxLength, out RangeValidation, out MinValue, out MaxValue, out OptionCount);

                                    //DataType 
                                    if (!IsValidDataType(DataType, OptionValue))
                                        ErrorString += "$AL$INVALID_DATATYPE$AL$";

                                    //Range
                                    if (RangeValidation)
                                    {
                                        if (!IsValidRange(SelectedOptionCount.ToString(), MinValue.ToString(), MaxValue.ToString()))
                                            ErrorString += "$AL$INVALID_RANGE$AL$";
                                    }

                                    //Length
                                    if (!String.IsNullOrEmpty(OptionValue) && LengthValidation)
                                    {
                                        if (!IsValidLength(OptionValue, MaxLength))
                                            ErrorString += "$AL$INVALID_LENGTH$AL$";
                                    }

                                    if (!String.IsNullOrEmpty(ErrorString))
                                        ErrorReport += "<ELEMENT ELEMENTID=\"" + xpElementIter.Current.SelectSingleNode("TEXTELEMENTID").Value + "\" TAGNAME=\"" + TagName + "\" ELEMENTNAME=\"" + xpElementIter.Current.SelectSingleNode("TEXTELEMENTNAME").Value + "\" TAGTYPE=\"TEXT\" ERRORSTRING=\"" + ErrorString + "\" ERROROCCURED=\"TRUE\" PREVIOUSVALUE=\"" + ALCommon.TrimAndReplaceSpecialChars(OldTextElementValue) + "\" MODIFIEDVALUE=\"" + ALCommon.TrimAndReplaceSpecialChars(OptionValue) + "\"/>";
                                    else
                                        ErrorReport += "<ELEMENT ELEMENTID=\"" + xpElementIter.Current.SelectSingleNode("TEXTELEMENTID").Value + "\" TAGNAME=\"" + TagName + "\" ELEMENTNAME=\"" + xpElementIter.Current.SelectSingleNode("TEXTELEMENTNAME").Value + "\" TAGTYPE=\"TEXT\" ERRORSTRING=\"TEXT_UPDATED\" PREVIOUSVALUE=\"" + ALCommon.TrimAndReplaceSpecialChars(OldTextElementValue) + "\" MODIFIEDVALUE=\"" + ALCommon.TrimAndReplaceSpecialChars(OptionValue) + "\"/>";

                                }
                            }

                            //Graphic element

                            xpElementIter = xpOutputNav.Select("FORMAT_ARTWORKPAGE/ARTWORKPAGE/ARTWORKPAGE_ELEMENTGROUP/ELEMENTGROUP/ELEMENTGROUP_GRAPHICELEMENT/GRAPHICELEMENT");
                            while (xpElementIter.MoveNext())
                            {
                                AssetDeleted = false;
                                OperationList = Elem_List[xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTID").Value];

                                if (OperationList.Contains("DELETEASSETS") || (OperationList.Contains("ASSETSEXPIRED")))
                                {
                                    DeleteEventCount = ComputeStringFrequency(OperationList, "DELETEASSETS");
                                    RestoreEventCount = ComputeStringFrequency(OperationList, "RESTOREASSETS");
                                    if (DeleteEventCount > RestoreEventCount)
                                        AssetDeleted = true;

                                    ExpireEventCount = ComputeStringFrequency(OperationList, "ASSETSEXPIRED");
                                    ReviveEventCount = ComputeStringFrequency(OperationList, "REVIVEASSETS");
                                    if (ExpireEventCount > ReviveEventCount)
                                        AssetDeleted = true;
                                }

                                if (!AssetDeleted)
                                {
                                    //This case handles presence of the asset but some change has been done to the asset 
                                    TEValue_Asset = null;
                                    TagName = xpElementIter.Current.SelectSingleNode("GRAPHICTAGNAME").Value;
                                    TagConfigurationXML = xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML").InnerXml;
                                    APEngineObj = new ArtworkProducerEngine(UserID, Language, TenantName, SolutionName);
                                    ErrorXML = APEngineObj.GetOption(TagName, TagConfigurationXML, TaxonomyDataXML, IsRelatedToBook, out OptionsXML);

                                    if (xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/GRAPHICELEMENTVALUEINFO_DIGITALASSET/DIGITALASSET/DIGITALASSETID") != null)
                                        PrevOptionValue = xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/GRAPHICELEMENTVALUEINFO_DIGITALASSET/DIGITALASSET/DIGITALASSETID").Value;
                                    else
                                        PrevOptionValue = "";

                                    if (String.IsNullOrEmpty(TagConfigurationXML))
                                    {
                                        //UpdateCache only
                                        if (!String.IsNullOrEmpty(PrevOptionValue))
                                        {
                                            ArtworkProducerCache objAPCache = new ArtworkProducerCache(UserID, Language, TenantName);
                                            if (!objAPCache.ModifyGraphicTagValue(ArtworkID, xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTID").Value, PrevOptionValue, ArtworkProducerCache.GraphicTagUpdateType.ASSET, ArtworkProducerCache.GraphicTagUpdateStatus.PENDING, false))
                                            {
                                                objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_UPDATECACHE_FAILED", this.GetType().ToString(), "UpdateArtworkForDAMChanges", null, ErrorXML);
                                                ErrorXML = objErrorCommon.ErrorCollectionXML;
                                            }
                                        }
                                        
                                        ErrorReport += "<ELEMENT ELEMENTID=\"" + xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTID").Value + "\" TAGNAME=\"" + TagName + "\" ELEMENTNAME=\"" + xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTNAME").Value + "\" TAGTYPE=\"GRAPHIC\" ERRORSTRING=\"IMAGE_UPDATED\" ASSETID=\"" + PrevOptionValue + "\"/>";
                                    }
                                    else
                                    {
                                        if (ErrorXML == null && !String.IsNullOrEmpty(OptionsXML) && (OptionsXML.Contains(PrevOptionValue)))
                                        {
                                            //sbArtworkValueXML.Append("<GRAPHICELEMENT MODIFIED='TRUE'>");
                                            //sbArtworkValueXML.AppendFormat("<GRAPHICELEMENTID>{0}</GRAPHICELEMENTID>", xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTID").Value);
                                            //sbArtworkValueXML.Append("<GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO TYPE='RELATION'>");
                                            //if (xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE") == null)
                                            //{
                                            //    sbArtworkValueXML.Append("<GRAPHICELEMENTVALUE>");
                                            //}
                                            //else
                                            //{
                                            //    sbArtworkValueXML.Append("<GRAPHICELEMENTVALUE MODIFIED='TRUE'>");
                                            //    sbArtworkValueXML.AppendFormat("<GRAPHICELEMENTVALUEID>{0}</GRAPHICELEMENTVALUEID>", xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/GRAPHICELEMENTVALUEID").Value);
                                            //}

                                            //UpdateCache only
                                            if (!String.IsNullOrEmpty(PrevOptionValue))
                                            {
                                                ArtworkProducerCache objAPCache = new ArtworkProducerCache(UserID, Language, TenantName);
                                                if (!objAPCache.ModifyGraphicTagValue(ArtworkID, xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTID").Value, PrevOptionValue, ArtworkProducerCache.GraphicTagUpdateType.ASSET, ArtworkProducerCache.GraphicTagUpdateStatus.PENDING, false))
                                                {
                                                    objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_UPDATECACHE_FAILED", this.GetType().ToString(), "UpdateArtworkForDAMChanges", null, ErrorXML);
                                                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                                                }
                                            }
                                            //sbArtworkValueXML.Append("</GRAPHICELEMENTVALUE>");

                                            //sbArtworkValueXML.Append("</GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO>");
                                            //sbArtworkValueXML.Append("</GRAPHICELEMENT>");

                                            ErrorReport += "<ELEMENT ELEMENTID=\"" + xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTID").Value + "\" TAGNAME=\"" + TagName + "\" ELEMENTNAME=\"" + xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTNAME").Value + "\" TAGTYPE=\"GRAPHIC\" ERRORSTRING=\"IMAGE_UPDATED\" ASSETID=\"" + PrevOptionValue + "\"/>";

                                        }
                                        else if (ErrorXML == null)
                                        {
                                            sbArtworkValueXML.Append("<GRAPHICELEMENT MODIFIED='TRUE'>");
                                            sbArtworkValueXML.AppendFormat("<GRAPHICELEMENTID>{0}</GRAPHICELEMENTID>", xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTID").Value);
                                            sbArtworkValueXML.Append("<GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO TYPE='RELATION'>");
                                            if (xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE") == null)
                                            {
                                                sbArtworkValueXML.Append("<GRAPHICELEMENTVALUE>");
                                            }
                                            else
                                            {
                                                sbArtworkValueXML.Append("<GRAPHICELEMENTVALUE MODIFIED='TRUE'>");
                                                sbArtworkValueXML.AppendFormat("<GRAPHICELEMENTVALUEID>{0}</GRAPHICELEMENTVALUEID>", xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/GRAPHICELEMENTVALUEID").Value);
                                            }

                                            sbArtworkValueXML.Append("<NOIMAGE>1</NOIMAGE>");

                                            //if (xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/GRAPHICELEMENTVALUEINFO_DIGITALASSET/DIGITALASSET/DIGITALASSETID") != null)                                    
                                            if (!String.IsNullOrEmpty(PrevOptionValue))
                                                sbArtworkValueXML.AppendFormat("<GRAPHICELEMENTVALUEINFO_DIGITALASSET TYPE='RELATION' DISASSOCIATE='TRUE'>{0}</GRAPHICELEMENTVALUEINFO_DIGITALASSET>", PrevOptionValue);

                                            sbArtworkValueXML.Append("</GRAPHICELEMENTVALUE>");

                                            sbArtworkValueXML.Append("</GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO>");
                                            sbArtworkValueXML.Append("<GRAPHICELEMENT_NOPHOTO></GRAPHICELEMENT_NOPHOTO>");
                                            sbArtworkValueXML.Append("</GRAPHICELEMENT>");

                                            ErrorReport += "<ELEMENT ELEMENTID=\"" + xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTID").Value + "\" TAGNAME=\"" + TagName + "\" ELEMENTNAME=\"" + xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTNAME").Value + "\" TAGTYPE=\"GRAPHIC\" ERRORSTRING=\"IMAGE_REMOVED\" ERROROCCURED=\"TRUE\"/>";

                                        }
                                        else if (ErrorXML != null)
                                        {
                                            objErrorCommon.AppendError("APENGINE_ARTWORK_GETOPTIONS_FAILED", this.GetType().ToString(), "PopulateOptions", null, ErrorXML);
                                        }
                                    }
                                }
                                else
                                {
                                    TagName = xpElementIter.Current.SelectSingleNode("GRAPHICTAGNAME").Value;
                                    //This case occurs when asset has been deleted explicitly from DAM
                                    sbArtworkValueXML.Append("<GRAPHICELEMENT MODIFIED='TRUE'>");
                                    sbArtworkValueXML.AppendFormat("<GRAPHICELEMENTID>{0}</GRAPHICELEMENTID>", xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTID").Value);
                                    sbArtworkValueXML.Append("<GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO TYPE='RELATION'>");
                                    if (xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE") == null)
                                    {
                                        sbArtworkValueXML.Append("<GRAPHICELEMENTVALUE>");
                                    }
                                    else
                                    {
                                        sbArtworkValueXML.Append("<GRAPHICELEMENTVALUE MODIFIED='TRUE'>");
                                        sbArtworkValueXML.AppendFormat("<GRAPHICELEMENTVALUEID>{0}</GRAPHICELEMENTVALUEID>", xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/GRAPHICELEMENTVALUEID").Value);
                                    }
                                    sbArtworkValueXML.Append("<NOIMAGE>1</NOIMAGE>");
                                    sbArtworkValueXML.Append("</GRAPHICELEMENTVALUE>");

                                    sbArtworkValueXML.Append("</GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO>");
                                    sbArtworkValueXML.Append("<GRAPHICELEMENT_NOPHOTO></GRAPHICELEMENT_NOPHOTO>");
                                    sbArtworkValueXML.Append("</GRAPHICELEMENT>");

                                    ErrorReport += "<ELEMENT ELEMENTID=\"" + xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTID").Value + "\" TAGNAME=\"" + TagName + "\" ELEMENTNAME=\"" + xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTNAME").Value + "\" TAGTYPE=\"GRAPHIC\" ERRORSTRING=\"IMAGE_REMOVED\" ERROROCCURED=\"TRUE\"/>";
                                }

                            }
                            //ErrorXML = APEngineObj.GetOption(TagName, TagConfigurationXML, TaxonomyDataXML, BookTaxonomy, out OptionsXML);
                            sbArtworkValueXML.Append("</GROUPELEMENT></PAGE>");
                            ArtworkValueXML = sbArtworkValueXML.ToString();

                            if (!String.IsNullOrEmpty(ErrorReport))
                            {
                                UpdateArtworkXml = "<ARTWORK><ARTWORKID>" + ArtworkID + "</ARTWORKID><REGENERATE>0</REGENERATE><REGENERATIONXML><MODIFIED_ELEMENTS><POSTPROCESS>" + ErrorReport + "</POSTPROCESS></MODIFIED_ELEMENTS></REGENERATIONXML></ARTWORK>";

                                if (ErrorReport.Contains("ERROROCCURED=\"TRUE\""))
                                    UpdateArtworkXml = UpdateArtworkXml.Replace("<REGENERATE>", "<ERRORPRESENT>1</ERRORPRESENT><REGENERATE>");

                                if (PreviousUpdatePresent)
                                {
                                    UpdateArtworkXml = UpdateArtworkXml.Replace("</POSTPROCESS>", PreviousUpdateXml + "</POSTPROCESS>");
                                }

                                ErrorXML = this.UpdateArtwork(ref UpdateArtworkXml, true);
                                if (ErrorXML != null)
                                {
                                    objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_FAILED", this.GetType().ToString(), "UpdateArtworkForDAMChanges", null, ErrorXML);
                                }

                                ErrorXML = this.UpdateArtwork(ArtworkID, ref ArtworkValueXML);
                                if (ErrorXML != null)
                                {
                                    objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_FAILED", this.GetType().ToString(), "UpdateArtworkForDAMChanges", null, ErrorXML);
                                }

                                if (ErrorXML == null && !IsRelatedToBook)
                                {
                                    if (String.Compare(FormatType, "HTML", true) != 0)
                                        ErrorXML = GenerateArtwork(FormatID, FormatType, Usage, ArtworkID);
                                }

                                ErrorXML = objErrorCommon.ErrorCollectionXML;
                                if (ErrorXML != null)
                                {
                                    //Associate Error with artwork
                                    ArtworkProducerCommon ObjCommon = new ArtworkProducerCommon(UserID, Language, TenantName, SolutionName);
                                    ObjCommon.LogError("ArtworkProducer", ErrorXML, ErrorType.ERROR, ArtworkID, "ARTWORK_JOBERROR");
                                }

                                UpdatedList += "$AL$NEW_UPDATES$AL$";
                            }
                        }
                    }
                }

                if (PreviousUpdatePresent)
                    UpdatedList += "$AL$OLD_UPDATES$AL$";
            }
            catch
            {
                throw;
            }
            return null;
        }

        public ERRORXML UpdateArtworksForAssetChanges(string AssetID, string JobType)
        {
            ERRORXML ErrorXML = null;
            StringBuilder sbSearchInputXML = null;
            OMInterface OMIObj = null;
            String SearchOutputXML = null;
            XPathDocument xpOutputDoc = null;
            XPathNavigator xpOutputNav = null;
            XPathNavigator xpNav_BookForArtwork = null;
            XPathNodeIterator xpElementIter = null;
            XPathNodeIterator TE_Collection = null;
            XPathNodeIterator GE_Collection = null;
            StringBuilder sb_UpdateArtworkXml = null;
            String CurrentModificationNode = null;
            XML ArtworkXML = null;
            XML BookXML = null;
            String ErrorType = null;
            String ErrorCollection = null;
            OMIObj = new OMInterface(UserID, Language, TenantName);

            try
            {
                sbSearchInputXML = new StringBuilder(40);
                sbSearchInputXML.AppendFormat(@"<SEARCH>
                                            <TARGETOBJECT>ARTWORK</TARGETOBJECT>
                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                            <TARGETENTITY>
	                                            <SELECTLIST>
		                                            <EXPRESSION>
			                                            <FIELDNAME>ARTWORKID</FIELDNAME>
			                                            <ALIAS>ARTWORKID</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>REGENERATIONXML</FIELDNAME>
			                                            <ALIAS>REGENERATIONXML</ALIAS>
		                                            </EXPRESSION>
	                                            </SELECTLIST>
	                                            <FILTERLIST>
                                                    <FILTER>
                                                        <FILTERID>1</FILTERID>
			                                            <EXPRESSION>DIGITALASSETID</EXPRESSION>
			                                            <OPERATOR>EQUALTO</OPERATOR>
			                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
			                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
			                                            <RELATIONSHIPNAME>ARTWORK_DIGITALASSET</RELATIONSHIPNAME>
		                                            </FILTER>
                                                    <FILTER>
                                                        <FILTERID>2</FILTERID>        
			                                            <EXPRESSION>ASSETORDER</EXPRESSION>
			                                            <OPERATOR>CONTAINS</OPERATOR>
			                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
			                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
			                                            <RELATIONSHIPNAME>ARTWORK_TEXTELEMENTVALUE</RELATIONSHIPNAME>
		                                            </FILTER>
                                                    <FILTERCOMBINATION>((1) OR (2))</FILTERCOMBINATION>
	                                            </FILTERLIST>
                                                <RELATIONSHIP NAME='ARTWORK_GRAPHICELEMENTVALUEINFO'>
                                                    <SELECTLIST>
	                                                    <EXPRESSION>
		                                                    <FIELDNAME>GRAPHICELEMENTVALUEID</FIELDNAME>
		                                                    <ALIAS>GRAPHICELEMENTVALUEID</ALIAS>
	                                                    </EXPRESSION>		                                            
                                                    </SELECTLIST>
                                                     <FILTERLIST>
			                                            <FILTER>
				                                            <EXPRESSION>DIGITALASSETID</EXPRESSION>
				                                            <OPERATOR>EQUALTO</OPERATOR>
				                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
				                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                            <RELATIONSHIPNAME>GRAPHICELEMENTVALUEINFO_DIGITALASSET</RELATIONSHIPNAME>				                                            
			                                            </FILTER>
		                                            </FILTERLIST>
                                                    <RELATIONSHIP NAME='GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO'>
                                                        <SELECTLIST>
	                                                        <EXPRESSION>
		                                                        <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
		                                                        <ALIAS>GRAPHICELEMENTID</ALIAS>
	                                                        </EXPRESSION>		                                            
                                                        </SELECTLIST>            
                                                    </RELATIONSHIP>
                                                </RELATIONSHIP>     
                                                <RELATIONSHIP NAME='ARTWORK_TEXTELEMENTVALUE'>
                                                    <SELECTLIST>
	                                                    <EXPRESSION>
		                                                    <FIELDNAME>TEXTELEMENTVALUEID</FIELDNAME>
		                                                    <ALIAS>TEXTELEMENTVALUEID</ALIAS>
	                                                    </EXPRESSION>		                                            
                                                    </SELECTLIST>
                                                     <FILTERLIST>
			                                            <FILTER>
				                                            <EXPRESSION>ASSETORDER</EXPRESSION>
				                                            <OPERATOR>CONTAINS</OPERATOR>
				                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
				                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>				                                            
			                                            </FILTER>
		                                            </FILTERLIST>
                                                    <RELATIONSHIP NAME='TEXTELEMENT_TEXTELEMENTVALUE'>
                                                        <SELECTLIST>
	                                                        <EXPRESSION>
		                                                        <FIELDNAME>TEXTELEMENTID</FIELDNAME>
		                                                        <ALIAS>TEXTELEMENTID</ALIAS>
	                                                        </EXPRESSION>		                                            
                                                        </SELECTLIST>            
                                                    </RELATIONSHIP>
                                                </RELATIONSHIP> 
                                                <RELATIONSHIP NAME='SPREAD_ARTWORK'>
                                                    <SELECTLIST>
	                                                    <EXPRESSION>
		                                                    <FIELDNAME>SPREADID</FIELDNAME>
		                                                    <ALIAS>SPREADID</ALIAS>
	                                                    </EXPRESSION>
                                                    </SELECTLIST>
                                                    <RELATIONSHIP NAME='BOOK_SPREAD'>
                                                        <SELECTLIST>
                                                            <EXPRESSION>
                                                                <FIELDNAME>BOOKID</FIELDNAME>
                                                                <ALIAS>BOOKID</ALIAS>
                                                            </EXPRESSION>                                                            
                                                        </SELECTLIST>
                                                    </RELATIONSHIP>                
                                                </RELATIONSHIP>                                           
                                            </TARGETENTITY>
                                            <SESSIONINFO Language='{1}' TenantName='{2}' UserID='{3}' SessionID='{4}'/>
                                        </SEARCH>", AssetID, Language, TenantName, UserID, SessionID);

                ErrorXML = OMIObj.GetHierarchialData(sbSearchInputXML.ToString(), ref SearchOutputXML);

                if (ErrorXML != null)
                {
                    objErrorCommon.AppendError("APENGINE_ARTWORK_GETARTWORKDETAILS_FAILED", this.GetType().ToString(), "UpdateArtworksForAssetChanges", null, ErrorXML);
                    ErrorXML = objErrorCommon.ErrorCollectionXML;
                }

                if (!String.IsNullOrEmpty(SearchOutputXML))
                {
                    xpOutputDoc = new XPathDocument(new StringReader(SearchOutputXML));
                    xpOutputNav = xpOutputDoc.CreateNavigator();
                    xpElementIter = xpOutputNav.Select("ARTWORKCOLLECTION/ARTWORK");

                    if (xpElementIter != null)
                    {
                        while (xpElementIter.MoveNext())
                        {
                            //Create update Artwork xml
                            CurrentModificationNode = "";
                            sb_UpdateArtworkXml = new StringBuilder();
                            sb_UpdateArtworkXml.Append("<ARTWORK>");
                            sb_UpdateArtworkXml.AppendFormat("<ARTWORKID>{0}</ARTWORKID>", xpElementIter.Current.SelectSingleNode("ARTWORKID").Value);
                            sb_UpdateArtworkXml.AppendFormat("<REGENERATE>1</REGENERATE>");

                            TE_Collection = xpElementIter.Current.Select("ARTWORK_TEXTELEMENTVALUE/TEXTELEMENTVALUE/TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENT/TEXTELEMENTID");
                            GE_Collection = xpElementIter.Current.Select("ARTWORK_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENT/GRAPHICELEMENTID");
                            if (TE_Collection.Count > 0)
                            {
                                while (TE_Collection.MoveNext())
                                {
                                    CurrentModificationNode += "<ELEMENT PREVIOUSASSETID=\"" + AssetID + "\" JOBTYPE=\"" + JobType + "\" ELEMENTTYPE=\"TEXT\" ELEMENTID=\"" + TE_Collection.Current.Value + "\" />";
                                }
                            }
                            else if (GE_Collection.Count > 0)
                            {
                                while (GE_Collection.MoveNext())
                                {
                                    CurrentModificationNode += "<ELEMENT PREVIOUSASSETID=\"" + AssetID + "\" JOBTYPE=\"" + JobType + "\" ELEMENTTYPE=\"GRAPHIC\" ELEMENTID=\"" + GE_Collection.Current.Value + "\" />";
                                }
                            }

                            if (xpElementIter.Current.SelectSingleNode("REGENERATIONXML/MODIFIED_ELEMENTS/PREPROCESS") != null)
                            {
                                sb_UpdateArtworkXml.AppendFormat("<REGENERATIONXML><MODIFIED_ELEMENTS><PREPROCESS>{0}{1}</PREPROCESS></MODIFIED_ELEMENTS></REGENERATIONXML>", xpElementIter.Current.SelectSingleNode("REGENERATIONXML/MODIFIED_ELEMENTS/PREPROCESS").InnerXml, CurrentModificationNode);
                            }
                            else
                            {
                                sb_UpdateArtworkXml.AppendFormat("<REGENERATIONXML><MODIFIED_ELEMENTS><PREPROCESS>{0}</PREPROCESS></MODIFIED_ELEMENTS></REGENERATIONXML>", CurrentModificationNode);
                            }

                            if (xpElementIter.Current.SelectSingleNode("REGENERATIONXML/MODIFIED_ELEMENTS/POSTPROCESS") != null)
                            {
                                sb_UpdateArtworkXml.Replace("</MODIFIED_ELEMENTS>", xpElementIter.Current.SelectSingleNode("REGENERATIONXML/MODIFIED_ELEMENTS/POSTPROCESS").OuterXml + "</MODIFIED_ELEMENTS>");
                            }
                            sb_UpdateArtworkXml.Append("</ARTWORK>");

                            ArtworkXML = sb_UpdateArtworkXml.ToString();

                            ErrorXML = ArtworkProducerEngine.CreateAndUpdateDataWithoutAccessControlCheck(ref ArtworkXML, "UPDATEDATA", UserID, Language, TenantName);

                            if (!String.IsNullOrEmpty(ErrorXML))
                            {
                                ErrorType = ALCommon.GetInnerValue(ref ErrorXML, "<ERRORTYPE>", "</ERRORTYPE>");

                                if ((ErrorType != null) && (ErrorType.ToUpper() == "ERROR"))
                                    ErrorCollection += ErrorXML;
                            }
                            else
                            {
                                //Update corresponding book and mark it for regeneration in case the artwork belongs to a book
                                xpNav_BookForArtwork = xpElementIter.Current.SelectSingleNode("SPREAD_ARTWORK/SPREAD/BOOK_SPREAD/BOOK/BOOKID");
                                if (xpNav_BookForArtwork != null)
                                {
                                    BookXML = "<BOOK><BOOKID>" + xpNav_BookForArtwork.Value + "</BOOKID><REGENERATE>1</REGENERATE></BOOK>";
                                    ErrorXML = ArtworkProducerEngine.CreateAndUpdateDataWithoutAccessControlCheck(ref BookXML, "UPDATEDATA", UserID, Language, TenantName);
                                }
                            }
                        }
                    }

                    if (!String.IsNullOrEmpty(ErrorCollection))
                        ErrorCollection = "<ERRORCOLLECTION>" + ErrorCollection + "</ERRORCOLLECTION>";
                }
            }
            catch
            {
                throw;
            }

            return ErrorCollection;
        }

        public ERRORXML UpdateArtworksForMultipleAssetChanges(string AssetIDList, string JobType)
        {
            string ErrorXML = null;
            string ErrorCollection = null;
            string ErrorType = null;
            string[] arr_AssetID = null;

            try
            {
                AssetIDList = ALCommon.GetInnerValue(ref AssetIDList, "<DIGITALASSETID>", "</DIGITALASSETID>");
                arr_AssetID = AssetIDList.Split(new String[] { "," }, StringSplitOptions.RemoveEmptyEntries);

                foreach (string AssetID in arr_AssetID)
                {
                    ErrorXML = UpdateArtworksForAssetChanges(AssetID, JobType);

                    if (!String.IsNullOrEmpty(ErrorXML))
                    {
                        ErrorType = ALCommon.GetInnerValue(ref ErrorXML, "<ERRORTYPE>", "</ERRORTYPE>");

                        if ((ErrorType != null) && (ErrorType.ToUpper() == "ERROR"))
                            ErrorCollection += ErrorXML;
                    }
                }

                if (!String.IsNullOrEmpty(ErrorCollection))
                    ErrorCollection = "<ERRORCOLLECTION>" + ErrorCollection + "</ERRORCOLLECTION>";
            }
            catch
            {
                throw;
            }
            return ErrorCollection;
        }
        # endregion

        internal ERRORXML CopyDigitalAsset(String DigitalAssetID, String ArtworkID, String DestArtworkName,
            String Artwork_DARel, String SectionID)
        {
            StringBuilder sbSearchXML = null;
            String DigitalAssetXML = null;
            OMInterface OMIObj = null;
            ERRORXML ErrorXML = null;
            XPathDocument DigitalAssetXMLDoc = null;
            XPathNavigator xpNav_RootDigitalAsset = null;
            XPathNavigator RenditionNode = null;
            XPathNavigator ThumbRenditionNode = null;
            String OrgFilePath = null;
            String AssetDescription = null;
            String OrgFileFormat = null;
            String ThumbFilePath = null;
            String ThumbFileFormat = null;
            String PrvFilePath = null;
            String PrvFileFormat = null;

            try
            {
                sbSearchXML = new StringBuilder(100);
                OMIObj = new OMInterface(UserID, Language, TenantName);

                #region Get DigitalAsset and its renditions

                sbSearchXML.Remove(0, sbSearchXML.Length);
                sbSearchXML.AppendFormat(@"<SEARCH>
                                            <TARGETOBJECT>DIGITALASSET</TARGETOBJECT>
                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                            <AUTHORIZATIONREQUIRED>False</AUTHORIZATIONREQUIRED>
                                            <TARGETENTITY>
                                                <FILTERLIST>
                                                    <FILTER>
                                                        <EXPRESSION>DIGITALASSETID</EXPRESSION>
                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                    </FILTER>
                                                </FILTERLIST>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>DIGITALASSETID</FIELDNAME>
                                                        <ALIAS>DIGITALASSETID</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>DESCRIPTION</FIELDNAME>
                                                        <ALIAS>DESCRIPTION</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                                <RELATIONSHIP NAME='DIGITALASSET_CURRENTORIGINALRENDITION'>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>ASSETRENDITIONID</FIELDNAME>
                                                            <ALIAS>ASSETRENDITIONID</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>FILEFORMAT</FIELDNAME>
                                                            <ALIAS>FILEFORMAT</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>HEIGHT</FIELDNAME>
                                                            <ALIAS>HEIGHT</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>WIDTH</FIELDNAME>
                                                            <ALIAS>WIDTH</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME FileRequired='Yes'>ACTUALFILE</FIELDNAME>
                                                            <ALIAS>ACTUALFILE</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                </RELATIONSHIP>
                                                <RELATIONSHIP NAME='DIGITALASSET_CURRENTPREVIEWRENDITION'>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>ASSETRENDITIONID</FIELDNAME>
                                                            <ALIAS>ASSETRENDITIONID</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>FILEFORMAT</FIELDNAME>
                                                            <ALIAS>FILEFORMAT</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>HEIGHT</FIELDNAME>
                                                            <ALIAS>HEIGHT</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>WIDTH</FIELDNAME>
                                                            <ALIAS>WIDTH</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME FileRequired='Yes'>ACTUALFILE</FIELDNAME>
                                                            <ALIAS>ACTUALFILE</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                </RELATIONSHIP>
                                                <RELATIONSHIP NAME='DIGITALASSET_CURRENTTHUMBNAILRENDITION'>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>ASSETRENDITIONID</FIELDNAME>
                                                            <ALIAS>ASSETRENDITIONID</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>FILEFORMAT</FIELDNAME>
                                                            <ALIAS>FILEFORMAT</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>HEIGHT</FIELDNAME>
                                                            <ALIAS>HEIGHT</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>WIDTH</FIELDNAME>
                                                            <ALIAS>WIDTH</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME FileRequired='Yes'>ACTUALFILE</FIELDNAME>
                                                            <ALIAS>ACTUALFILE</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                </RELATIONSHIP>
                                            </TARGETENTITY>
                                          </SEARCH>", DigitalAssetID);

                ErrorXML = OMIObj.GetHierarchialData(sbSearchXML.ToString(), ref DigitalAssetXML);

                if (ErrorXML != null)
                {
                    return ErrorXML;
                }

                DigitalAssetXMLDoc = new XPathDocument(new StringReader(DigitalAssetXML));
                xpNav_RootDigitalAsset = DigitalAssetXMLDoc.CreateNavigator();

                if (xpNav_RootDigitalAsset.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/DESCRIPTION") != null)
                    AssetDescription = xpNav_RootDigitalAsset.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/DESCRIPTION").InnerXml;

                // Org File
                RenditionNode = xpNav_RootDigitalAsset.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/DIGITALASSET_CURRENTORIGINALRENDITION/ASSETRENDITION");
                if (RenditionNode != null)
                {
                    if (RenditionNode.SelectSingleNode("FILESYSTEM/FILE/PATH") != null)
                    {
                        OrgFilePath = RenditionNode.SelectSingleNode("FILESYSTEM/FILE/PATH").Value;
                        OrgFileFormat = RenditionNode.SelectSingleNode("FILEFORMAT").Value;
                    }
                }

                // Prv File
                RenditionNode = xpNav_RootDigitalAsset.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/DIGITALASSET_CURRENTPREVIEWRENDITION/ASSETRENDITION");
                if (RenditionNode != null)
                {
                    if (RenditionNode.SelectSingleNode("FILESYSTEM/FILE/PATH") != null)
                    {
                        PrvFilePath = RenditionNode.SelectSingleNode("FILESYSTEM/FILE/PATH").Value;
                        PrvFileFormat = RenditionNode.SelectSingleNode("FILEFORMAT").Value;
                    }
                }

                // Thumb File
                ThumbRenditionNode = xpNav_RootDigitalAsset.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/DIGITALASSET_CURRENTTHUMBNAILRENDITION/ASSETRENDITION");
                if (ThumbRenditionNode != null)
                {
                    if (RenditionNode.SelectSingleNode("FILESYSTEM/FILE/PATH") != null)
                    {
                        ThumbFilePath = ThumbRenditionNode.SelectSingleNode("FILESYSTEM/FILE/PATH").Value;
                        ThumbFileFormat = ThumbRenditionNode.SelectSingleNode("FILEFORMAT").Value;
                    }
                }
                #endregion

                if (ThumbRenditionNode != null)
                    ErrorXML = CreateDigitalAsset(ThumbRenditionNode.CreateNavigator(), OrgFilePath, OrgFileFormat, PrvFilePath, PrvFileFormat,
                    ThumbFilePath, ThumbFileFormat, SectionID, ArtworkID, Artwork_DARel, AssetDescription, ref DigitalAssetID);
                else
                    ErrorXML = CreateDigitalAsset(OrgFilePath, OrgFileFormat, PrvFilePath, PrvFileFormat,
                    ThumbFilePath, ThumbFileFormat, "", "", SectionID, ArtworkID, Artwork_DARel, AssetDescription, ref DigitalAssetID);

                return ErrorXML;
            }
            catch (Exception)
            {
                throw;
            }
        }

        internal ERRORXML CopyDigitalAsset(String DigitalAssetID, String SectionID, out String NewDigitalAssetID)
        {
            StringBuilder sbSearchXML = null;
            String DigitalAssetXML = null;
            OMInterface OMIObj = null;
            ERRORXML ErrorXML = null;
            XPathDocument DigitalAssetXMLDoc = null;
            XPathNavigator xpNav_RootDigitalAsset = null;
            XPathNavigator RenditionNode = null;
            XPathNavigator ThumbRenditionNode = null;
            String OrgFilePath = null;
            String OrgFileFormat = null;
            String ThumbFilePath = null;
            String ThumbFileFormat = null;
            String PrvFilePath = null;
            String PrvFileFormat = null;
            String AssetDescription = null;
            //String SectionID = null;

            try
            {
                NewDigitalAssetID = null;

                sbSearchXML = new StringBuilder(100);
                OMIObj = new OMInterface(UserID, Language, TenantName);

                #region Get DigitalAsset and its renditions

                sbSearchXML.Remove(0, sbSearchXML.Length);
                sbSearchXML.AppendFormat(@"<SEARCH>
                                            <TARGETOBJECT>DIGITALASSET</TARGETOBJECT>
                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                            <AUTHORIZATIONREQUIRED>False</AUTHORIZATIONREQUIRED>
                                            <TARGETENTITY>
                                                <FILTERLIST>
                                                    <FILTER>
                                                        <EXPRESSION>DIGITALASSETID</EXPRESSION>
                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                    </FILTER>
                                                </FILTERLIST>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>DIGITALASSETID</FIELDNAME>
                                                        <ALIAS>DIGITALASSETID</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>DESCRIPTION</FIELDNAME>
                                                        <ALIAS>DESCRIPTION</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                                <RELATIONSHIP NAME='DIGITALASSET_CURRENTORIGINALRENDITION'>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>ASSETRENDITIONID</FIELDNAME>
                                                            <ALIAS>ASSETRENDITIONID</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>FILEFORMAT</FIELDNAME>
                                                            <ALIAS>FILEFORMAT</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>HEIGHT</FIELDNAME>
                                                            <ALIAS>HEIGHT</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>WIDTH</FIELDNAME>
                                                            <ALIAS>WIDTH</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME FileRequired='Yes'>ACTUALFILE</FIELDNAME>
                                                            <ALIAS>ACTUALFILE</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                </RELATIONSHIP>
                                                <RELATIONSHIP NAME='DIGITALASSET_CURRENTPREVIEWRENDITION'>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>ASSETRENDITIONID</FIELDNAME>
                                                            <ALIAS>ASSETRENDITIONID</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>FILEFORMAT</FIELDNAME>
                                                            <ALIAS>FILEFORMAT</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>HEIGHT</FIELDNAME>
                                                            <ALIAS>HEIGHT</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>WIDTH</FIELDNAME>
                                                            <ALIAS>WIDTH</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME FileRequired='Yes'>ACTUALFILE</FIELDNAME>
                                                            <ALIAS>ACTUALFILE</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                </RELATIONSHIP>
                                                <RELATIONSHIP NAME='DIGITALASSET_CURRENTTHUMBNAILRENDITION'>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>ASSETRENDITIONID</FIELDNAME>
                                                            <ALIAS>ASSETRENDITIONID</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>FILEFORMAT</FIELDNAME>
                                                            <ALIAS>FILEFORMAT</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>HEIGHT</FIELDNAME>
                                                            <ALIAS>HEIGHT</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME>WIDTH</FIELDNAME>
                                                            <ALIAS>WIDTH</ALIAS>
                                                        </EXPRESSION>
                                                        <EXPRESSION>
                                                            <FIELDNAME FileRequired='Yes'>ACTUALFILE</FIELDNAME>
                                                            <ALIAS>ACTUALFILE</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                </RELATIONSHIP>
                                            </TARGETENTITY>
                                          </SEARCH>", DigitalAssetID);

                ErrorXML = OMIObj.GetHierarchialData(sbSearchXML.ToString(), ref DigitalAssetXML);

                if (ErrorXML != null)
                {
                    return ErrorXML;
                }

                DigitalAssetXMLDoc = new XPathDocument(new StringReader(DigitalAssetXML));
                xpNav_RootDigitalAsset = DigitalAssetXMLDoc.CreateNavigator();

                if (xpNav_RootDigitalAsset.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/DESCRIPTION") != null)
                    AssetDescription = xpNav_RootDigitalAsset.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/DESCRIPTION").InnerXml;

                // Org File
                RenditionNode = xpNav_RootDigitalAsset.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/DIGITALASSET_CURRENTORIGINALRENDITION/ASSETRENDITION");
                if (RenditionNode != null)
                {
                    if (RenditionNode.SelectSingleNode("FILESYSTEM/FILE/PATH") != null)
                    {
                        OrgFilePath = RenditionNode.SelectSingleNode("FILESYSTEM/FILE/PATH").Value;
                        OrgFileFormat = RenditionNode.SelectSingleNode("FILEFORMAT").Value;
                    }
                }

                // Prv File
                RenditionNode = xpNav_RootDigitalAsset.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/DIGITALASSET_CURRENTPREVIEWRENDITION/ASSETRENDITION");
                if (RenditionNode != null)
                {
                    if (RenditionNode.SelectSingleNode("FILESYSTEM/FILE/PATH") != null)
                    {
                        PrvFilePath = RenditionNode.SelectSingleNode("FILESYSTEM/FILE/PATH").Value;
                        PrvFileFormat = RenditionNode.SelectSingleNode("FILEFORMAT").Value;
                    }
                }

                // Thumb File
                ThumbRenditionNode = xpNav_RootDigitalAsset.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/DIGITALASSET_CURRENTTHUMBNAILRENDITION/ASSETRENDITION");
                if (ThumbRenditionNode != null)
                {
                    if (RenditionNode.SelectSingleNode("FILESYSTEM/FILE/PATH") != null)
                    {
                        ThumbFilePath = ThumbRenditionNode.SelectSingleNode("FILESYSTEM/FILE/PATH").Value;
                        ThumbFileFormat = ThumbRenditionNode.SelectSingleNode("FILEFORMAT").Value;
                    }
                }
                #endregion

                if (ThumbRenditionNode != null)
                    ErrorXML = CreateDigitalAsset(ThumbRenditionNode.CreateNavigator(), OrgFilePath, OrgFileFormat, PrvFilePath, PrvFileFormat,
                    ThumbFilePath, ThumbFileFormat, SectionID, null, null, AssetDescription, ref NewDigitalAssetID);
                else
                    ErrorXML = CreateDigitalAsset(OrgFilePath, OrgFileFormat, PrvFilePath, PrvFileFormat,
                    ThumbFilePath, ThumbFileFormat, "", "", SectionID, null, null, AssetDescription, ref NewDigitalAssetID);

                return ErrorXML;
            }
            catch (Exception)
            {
                throw;
            }
        }

        private ERRORXML CreateDigitalAsset(XPathNavigator xpOutPutXMLNav, String OrgFilePath,
        String OrgFileFormat, String PrvFilePath, String PrvFileFormat, String ThbFilePath,
            String ThbFileFormat, String SectionID, String ArtworkID,
            String RelationshipName, String  AssetDesc, ref String DigitalAssetID)
        {
            String Height = null;
            String Width = null;
            ERRORXML ErrorXML = null;
            String ThbFileName = null;

            if (!String.IsNullOrEmpty(ThbFilePath))
            {
                ThbFileName = Path.GetFileNameWithoutExtension(ThbFilePath);
                if (xpOutPutXMLNav != null)
                {
                    XPathNodeIterator xpOutPutIter = xpOutPutXMLNav.Select("OUTPUTLIST/OUTPUT [translate(@FILEFORMAT, 'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='" + ThbFileFormat.ToUpper() + "']");
                    while (xpOutPutIter.MoveNext())
                    {
                        if (xpOutPutIter.Current.GetAttribute("FILENAME", "").EndsWith("\\" + ThbFileName + "." + ThbFileFormat))
                        {
                            Height = xpOutPutIter.Current.GetAttribute("HEIGHT", "");
                            Width = xpOutPutIter.Current.GetAttribute("WIDTH", "");
                            break;
                        }
                    }
                }

                if (String.IsNullOrEmpty(Height) || String.IsNullOrEmpty(Width))
                {
                    try
                    {
                        System.Drawing.Image ImgObj = System.Drawing.Image.FromFile(ThbFilePath);
                        Height = ImgObj.Height.ToString();
                        Width = ImgObj.Width.ToString();
                    }
                    catch (Exception)
                    {
                    }
                }
            }
            ErrorXML = CreateDigitalAsset(OrgFilePath, OrgFileFormat, PrvFilePath, PrvFileFormat, ThbFilePath, ThbFileFormat, Height, Width, SectionID, ArtworkID, RelationshipName, AssetDesc, ref DigitalAssetID);
            return ErrorXML;
        }

        private ERRORXML CreateDigitalAsset(String OrgFilePath,
        String OrgFileFormat, String PrvFilePath, String PrvFileFormat, String ThbFilePath,
            String ThbFileFormat, String ThumbnailHeight, String ThumbnailWidth, String SectionID, String ArtworkID,
            String RelationshipName, String AssetDesc, ref String DigitalAssetID)
        {
            #region variable
            StringBuilder DigitalAssetXML = null;
            ERRORXML ErrorXML = null;
            String DAM_ForumID = null;
            String str_VaultMountPoint = null;
            String OrgFileName = Path.GetFileNameWithoutExtension(OrgFilePath);

            #endregion
            ulong iContentRevisionID = 0;
            ulong iOrgRenditionID = 0;
            ulong iPrvRenditionID = 0;
            ulong iThmRenditionID = 0;
            ulong iAssetID = 0;
            DigitalAssetID = null;

            #region Get Forum ID

            if (String.IsNullOrEmpty(DAM_ForumID))
            {
                Assetlink.Services.Engines.SearchHelper SearchObj = new Assetlink.Services.Engines.SearchHelper(UserID, Language, TenantName);
                SearchObj.SearchAllFacets = false;
                SearchObj.SortColumnName = "FORUMID";
                SearchObj.SortOrder = "ASC";
                SearchObj.MaxResultSize = 1;

                SearchObj.Init("FORUM", "ENTITY");
                SearchObj.AddFilter("MESSAGEBOARDTITLE", "EQUALTO", "DAM_MESSAGEBOARD",
                                    false, "FORUM.MESSAGEBOARDFORUM");
                SearchObj.AddFilter("FORUMTITLE", "EQUALTO", "DAM_FORUM",
                                    false, "FORUM");

                SearchObj.AddFields("FORUMID", null, "FORUM");
                String ForumXML = null;
                ErrorXML = SearchObj.DoSearch(ref ForumXML);

                if (ErrorXML != null)
                    return ErrorXML;

                DAM_ForumID = ALCommon.GetInnerValue(ref ForumXML, "<FORUMID>", "</FORUMID>");
            }

            #endregion

            #region Get Vault Path

            if (String.IsNullOrEmpty(str_VaultMountPoint))
            {
                XPathDocument xpDoc_TenantXML = new XPathDocument(ALCommon.GetTenantXMLPath(TenantName));
                String VaultName = xpDoc_TenantXML.CreateNavigator().SelectSingleNode("CONFIGURATION/ARTWORKPRODUCER/VAULTNAME").InnerXml;
                StringBuilder sbVaultSearch = new StringBuilder();
                String str_VaultDetails = null;

                sbVaultSearch.AppendFormat(@"  <SEARCH>
                                                <TARGETOBJECT>VAULT</TARGETOBJECT>
                                                <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                <AUTHORIZATIONREQUIRED>False</AUTHORIZATIONREQUIRED>
                                                <TARGETENTITY>
                                                    <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                    <FILTERLIST>
                                                        <FILTER>
                                                            <EXPRESSION>NAME</EXPRESSION>
                                                             <OPERATOR>EQUALTO</OPERATOR>
                                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                            <FILTERID>1</FILTERID>
                                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                        </FILTER>
                                                    </FILTERLIST>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
	                                                        <FIELDNAME>MOUNTPOINT</FIELDNAME>
	                                                        <ALIAS>MOUNTPOINT</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                </TARGETENTITY>
                                                <SESSIONINFO Language='{1}' TenantName='{2}' UserID='{3}' SessionID='{4}'></SESSIONINFO>
                                            </SEARCH>
            ", VaultName, Language, TenantName, UserID, SessionID);

                Assetlink.Services.Engines.AssetlinkSearch objSearch = new Assetlink.Services.Engines.AssetlinkSearch(UserID, Language, TenantName);
                ErrorXML = objSearch.GetHierarchialData(sbVaultSearch.ToString(), ref str_VaultDetails);
                if (ErrorXML != null)
                    return ErrorXML;

                str_VaultMountPoint = ALCommon.GetInnerValue(ref str_VaultDetails, "<MOUNTPOINT>", "</MOUNTPOINT>");
            }

            #endregion

            //#region Create Directory Engine ref
            //if (DirServicesObj == null)
            //{
            //    String CurrentAssemblyPath = Assembly.GetExecutingAssembly().Location;
            //    Assembly ComponentAssembly = Assembly.LoadFrom(System.IO.Path.GetDirectoryName(CurrentAssemblyPath) + "\\" + "DirectoryEngine.dll");
            //    Type ComponentType = ComponentAssembly.GetType("Assetlink.Services.Engines.DirectoryEngine.DirectoryOperations", true);

            //    Object[] ConstructorParameters = { iUserID, str_Language, str_TenantName };
            //    DirServicesObj = Activator.CreateInstance(ComponentType, ConstructorParameters);
            //}
            //#endregion

            UInt64 iOrgFileSize = 0;

            //Object[] Parameters = { OrgFilePath, iOrgFileSize };
            //System.Type[] arrTypes = new System.Type[2];
            //arrTypes.SetValue(Type.GetType("System.String"), 0);
            //arrTypes.SetValue(Type.GetType("System.UInt64&"), 1);

            //ErrorXML = (String)DirServicesObj.GetType().GetMethod("GetFileSize", arrTypes).Invoke(DirServicesObj, Parameters);
            FileInfo fileInfoObj = new FileInfo(OrgFilePath);
            iOrgFileSize = (ulong)fileInfoObj.Length;

            DigitalAssetXML = new StringBuilder();
            DigitalAssetXML.Append("<DIGITALASSET>");
            ALCommon.GenerateUniqueID(ref iAssetID);
            DigitalAssetXML.AppendFormat("<DIGITALASSETID>{0}</DIGITALASSETID>", iAssetID);
            DigitalAssetXML.AppendFormat("<NAME>{0}</NAME>", OrgFileName);
            if (String.IsNullOrEmpty(AssetDesc))
                DigitalAssetXML.AppendFormat("<DESCRIPTION>Asset Created For {0}</DESCRIPTION>", OrgFileName);
            else
            {
                try
                {
                    int.Parse(AssetDesc);
                    DigitalAssetXML.AppendFormat("<DESCRIPTION>{0}</DESCRIPTION>", AssetDesc);
                }
                catch (Exception)
                {
                    DigitalAssetXML.AppendFormat("<DESCRIPTION>Asset Created For {0}</DESCRIPTION>", OrgFileName);
                }                
            }
            DigitalAssetXML.AppendFormat("<CREATEDON><DAY>{0}</DAY><MONTH>{1}</MONTH><YEAR>{2}</YEAR><HOUR>{3}</HOUR><MINUTE>{4}</MINUTE><SECOND>{5}</SECOND></CREATEDON>",
                                            DateTime.Now.Day.ToString(), DateTime.Now.Month.ToString(), DateTime.Now.Year.ToString(), DateTime.Now.Hour.ToString(), DateTime.Now.Minute.ToString(), DateTime.Now.Second.ToString());
            DigitalAssetXML.Append("<MARKEDASDELETED>0</MARKEDASDELETED>");
            DigitalAssetXML.Append("<DIGITALASSET_CONTENTREVISION  TYPE='RELATION'>");
            DigitalAssetXML.Append("<CONTENTREVISION>");
            ALCommon.GenerateUniqueID(ref iContentRevisionID);
            DigitalAssetXML.AppendFormat("<CONTENTREVISIONID>{0}</CONTENTREVISIONID>", iContentRevisionID);
            DigitalAssetXML.Append("<VERSIONNUMBER>1</VERSIONNUMBER>");
            //Original
            DigitalAssetXML.Append("<CONTENTREVISION_RENDITION TYPE='RELATION'>");
            DigitalAssetXML.Append("<ASSETRENDITION>");
            ALCommon.GenerateUniqueID(ref iOrgRenditionID);
            DigitalAssetXML.AppendFormat("<ASSETRENDITIONID>{0}</ASSETRENDITIONID>", iOrgRenditionID);
            DigitalAssetXML.AppendFormat("<NAME>{0}</NAME>", OrgFileName);
            DigitalAssetXML.Append("<RENDITIONTYPE>Original</RENDITIONTYPE>");
            DigitalAssetXML.AppendFormat("<FILENAME>{0}</FILENAME>", OrgFileName);
            if (OrgFileFormat != null)
                DigitalAssetXML.AppendFormat("<FILEFORMAT>{0}</FILEFORMAT>", OrgFileFormat);
            DigitalAssetXML.Append("<MEDIACREATEDIN>Printer</MEDIACREATEDIN>");
            DigitalAssetXML.AppendFormat("<FILESIZE>{0}</FILESIZE>", iOrgFileSize);
            if (String.IsNullOrEmpty(str_VaultMountPoint))
                DigitalAssetXML.Append("<FILESYSTEM><FILE>");
            else
                DigitalAssetXML.AppendFormat("<FILESYSTEM><FILE MountPoint='{0}'>", ALSpecialCharHandling.TrimAndReplaceSpecialChars(str_VaultMountPoint));
            DigitalAssetXML.Append("<FIELDNAME>ACTUALFILE</FIELDNAME>");
            DigitalAssetXML.AppendFormat("<DESCRIPTION>File For {0}</DESCRIPTION>", OrgFileName);
            DigitalAssetXML.AppendFormat("<NAME>{0}.{1}</NAME>", OrgFileName, OrgFileFormat);
            DigitalAssetXML.AppendFormat("<DISPLAYNAME>{0}.{1}</DISPLAYNAME>", OrgFileName, OrgFileFormat);
            DigitalAssetXML.AppendFormat("<PATH>{0}</PATH>", OrgFilePath);
            DigitalAssetXML.Append("</FILE></FILESYSTEM>");
            DigitalAssetXML.Append("</ASSETRENDITION>");

            //Preview            
            if (!String.IsNullOrEmpty(PrvFilePath))
            {
                UInt64 iPrvFileSize = 0;
                //Object[] Parameters_prv = { PrvFilePath, iPrvFileSize };
                //ErrorXML = (String)DirServicesObj.GetType().GetMethod("GetFileSize", arrTypes).Invoke(DirServicesObj, Parameters_prv);
                fileInfoObj = new FileInfo(PrvFilePath);
                iPrvFileSize = (ulong)fileInfoObj.Length;

                String PrvFileName = Path.GetFileNameWithoutExtension(PrvFilePath);
                DigitalAssetXML.Append("<ASSETRENDITION>");
                ALCommon.GenerateUniqueID(ref iPrvRenditionID);
                DigitalAssetXML.AppendFormat("<ASSETRENDITIONID>{0}</ASSETRENDITIONID>", iPrvRenditionID);
                DigitalAssetXML.AppendFormat("<NAME>{0}</NAME>", PrvFileName);
                DigitalAssetXML.Append("<RENDITIONTYPE>Preview</RENDITIONTYPE>");
                DigitalAssetXML.AppendFormat("<FILENAME>{0}</FILENAME>", PrvFileName);
                if (PrvFileFormat != null)
                    DigitalAssetXML.AppendFormat("<FILEFORMAT>{0}</FILEFORMAT>", PrvFileFormat);
                DigitalAssetXML.Append("<MEDIACREATEDIN>Printer</MEDIACREATEDIN>");
                DigitalAssetXML.AppendFormat("<FILESIZE>{0}</FILESIZE>", iPrvFileSize);
                if (String.IsNullOrEmpty(str_VaultMountPoint))
                    DigitalAssetXML.Append("<FILESYSTEM><FILE>");
                else
                    DigitalAssetXML.AppendFormat("<FILESYSTEM><FILE MountPoint='{0}'>", ALSpecialCharHandling.TrimAndReplaceSpecialChars(str_VaultMountPoint));
                DigitalAssetXML.Append("<FIELDNAME>ACTUALFILE</FIELDNAME>");
                DigitalAssetXML.AppendFormat("<DESCRIPTION>File For {0}</DESCRIPTION>", PrvFileName);
                DigitalAssetXML.AppendFormat("<NAME>{0}.{1}</NAME>", PrvFileName, PrvFileFormat);
                DigitalAssetXML.AppendFormat("<DISPLAYNAME>{0}.{1}</DISPLAYNAME>", PrvFileName, PrvFileFormat);
                DigitalAssetXML.AppendFormat("<PATH>{0}</PATH>", PrvFilePath);
                DigitalAssetXML.Append("</FILE></FILESYSTEM>");
                DigitalAssetXML.Append("</ASSETRENDITION>");
            }

            //Thumbnail            
            if (!String.IsNullOrEmpty(ThbFileFormat))
            {
                UInt64 iThbFileSize = 0;
                //Object[] Parameters_thb = { ThbFilePath, iThbFileSize };
                //ErrorXML = (String)DirServicesObj.GetType().GetMethod("GetFileSize", arrTypes).Invoke(DirServicesObj, Parameters_thb);
                fileInfoObj = new FileInfo(ThbFilePath);
                iThbFileSize = (ulong)fileInfoObj.Length;

                //String Height = null;
                //String Width = null;
                String ThbFileName = Path.GetFileNameWithoutExtension(ThbFilePath);


                //XPathNodeIterator xpOutPutIter = xpOutPutXMLNav.Select("OUTPUTLIST/OUTPUT [translate(@FILEFORMAT, 'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='" + ThbFileFormat.ToUpper() + "']");
                //while (xpOutPutIter.MoveNext())
                //{
                //    if (xpOutPutIter.Current.GetAttribute("FILENAME", "").EndsWith("\\" + ThbFileName + "." + ThbFileFormat))
                //    {
                //        Height = xpOutPutIter.Current.GetAttribute("HEIGHT", "");
                //        Width = xpOutPutIter.Current.GetAttribute("WIDTH", "");
                //        break;
                //    }
                //}

                DigitalAssetXML.Append("<ASSETRENDITION>");
                ALCommon.GenerateUniqueID(ref iThmRenditionID);
                DigitalAssetXML.AppendFormat("<ASSETRENDITIONID>{0}</ASSETRENDITIONID>", iThmRenditionID);
                DigitalAssetXML.AppendFormat("<NAME>{0}</NAME>", ThbFileName);
                DigitalAssetXML.Append("<RENDITIONTYPE>Thumbnail</RENDITIONTYPE>");
                DigitalAssetXML.AppendFormat("<FILENAME>{0}</FILENAME>", ThbFileName);
                if (ThbFileFormat != null)
                    DigitalAssetXML.AppendFormat("<FILEFORMAT>{0}</FILEFORMAT>", ThbFileFormat);
                DigitalAssetXML.Append("<MEDIACREATEDIN>Printer</MEDIACREATEDIN>");
                DigitalAssetXML.AppendFormat("<FILESIZE>{0}</FILESIZE>", iThbFileSize);

                if (!String.IsNullOrEmpty(ThumbnailHeight))
                    DigitalAssetXML.AppendFormat("<HEIGHT>{0}</HEIGHT>", Convert.ToInt32(Double.Parse(ThumbnailHeight)).ToString());
                if (!String.IsNullOrEmpty(ThumbnailWidth))
                    DigitalAssetXML.AppendFormat("<WIDTH>{0}</WIDTH>", Convert.ToInt32(Double.Parse(ThumbnailWidth)).ToString());

                if (String.IsNullOrEmpty(str_VaultMountPoint))
                    DigitalAssetXML.Append("<FILESYSTEM><FILE>");
                else
                    DigitalAssetXML.AppendFormat("<FILESYSTEM><FILE MountPoint='{0}'>", ALSpecialCharHandling.TrimAndReplaceSpecialChars(str_VaultMountPoint));
                DigitalAssetXML.Append("<FIELDNAME>ACTUALFILE</FIELDNAME>");
                DigitalAssetXML.AppendFormat("<DESCRIPTION>File For {0}</DESCRIPTION>", ThbFileName);
                DigitalAssetXML.AppendFormat("<NAME>{0}.{1}</NAME>", ThbFileName, ThbFileFormat);
                DigitalAssetXML.AppendFormat("<DISPLAYNAME>{0}.{1}</DISPLAYNAME>", ThbFileName, ThbFileFormat);
                DigitalAssetXML.AppendFormat("<PATH>{0}</PATH>", ThbFilePath);
                DigitalAssetXML.Append("</FILE></FILESYSTEM>");
                DigitalAssetXML.Append("</ASSETRENDITION>");
            }



            DigitalAssetXML.Append("</CONTENTREVISION_RENDITION>");
            DigitalAssetXML.Append("</CONTENTREVISION>");
            DigitalAssetXML.Append("</DIGITALASSET_CONTENTREVISION>");
            DigitalAssetXML.AppendFormat("<SECTION_DIGITALASSET TYPE='RELATION' PARENTID='TRUE'>{0}</SECTION_DIGITALASSET>", SectionID);
            DigitalAssetXML.AppendFormat(@" <DIGITALASSET_TOPIC TYPE='RELATION'>
                                                <TOPIC>
                                                    <TOPICTITLE>{0}_{1}</TOPICTITLE>
                                                    <TOPICSTATUS>Active</TOPICSTATUS>
                                                    <TOPICOWNER TYPE='RELATION'>{2}</TOPICOWNER>
                                                    <FORUMTOPIC TYPE='RELATION'>{3}</FORUMTOPIC>
                                                </TOPIC>
                                            </DIGITALASSET_TOPIC>", iAssetID, OrgFileName, UserID, DAM_ForumID);

            if (!String.IsNullOrEmpty(RelationshipName) && !String.IsNullOrEmpty(ArtworkID))
                DigitalAssetXML.AppendFormat(@"<{0} TYPE='RELATION'>{1}</{0}>", RelationshipName, ArtworkID);

            DigitalAssetXML.AppendFormat(@"<DIGITALASSET_CURRENTORIGINALRENDITION TYPE='RELATION'>{0}</DIGITALASSET_CURRENTORIGINALRENDITION>", iOrgRenditionID);
            if (iPrvRenditionID > 0)
                DigitalAssetXML.AppendFormat(@"<DIGITALASSET_CURRENTPREVIEWRENDITION TYPE='RELATION'>{0}</DIGITALASSET_CURRENTPREVIEWRENDITION>", iPrvRenditionID);
            if (iThmRenditionID > 0)
                DigitalAssetXML.AppendFormat(@"<DIGITALASSET_CURRENTTHUMBNAILRENDITION TYPE='RELATION'>{0}</DIGITALASSET_CURRENTTHUMBNAILRENDITION>", iThmRenditionID);
            DigitalAssetXML.AppendFormat(@"<DIGITALASSET_CURRENTREVISION TYPE='RELATION'>{0}</DIGITALASSET_CURRENTREVISION>", iContentRevisionID);
            DigitalAssetXML.Append("</DIGITALASSET>");


            XML DigitalAssetInputXML = DigitalAssetXML.ToString();

            OMInterface objOMI = new OMInterface(UserID, Language, TenantName);
            Assetlink.Services.Engines.OperationQueue OpQueue = new Assetlink.Services.Engines.OperationQueue(true);
            Assetlink.Services.Engines.Operation OperationNode = new Assetlink.Services.Engines.Operation("ENTITY", "CREATEDATA");
            OperationNode.AddParameter("INPUTXML", DigitalAssetInputXML);
            OpQueue.AddOperation(OperationNode);
            String OMInputXML = OpQueue.Xml;
            ErrorXML = objOMI.Execute(ref OMInputXML);
            if (ErrorXML == null)
            {
                DigitalAssetID = ALCommon.GetInnerValue(ref  OMInputXML, "<DIGITALASSETID>", "</DIGITALASSETID>");
                ErrorXML = Services.Engines.ArtworkProducerEngine.ArtworkProducerCommon.UpdateDAMSections(UserID, TenantName, Language, SectionID, "INCREMENT", 1);
            }
            else
            {
                return ErrorXML;
            }

            return ErrorXML;
        }

        private ERRORXML PopulateOptions(String ArtworkID, String TextElementFilterXML, String TaxonomyDataXML, Boolean IsRelatedToBook, String OptionMode, out String FormatID, out String FormatType, out String Usage, out Boolean OptionsPopulated)
        {
            ERRORXML ErrorXML = null;
            StringBuilder sbSearchInputXML = null;
            OMInterface OMIObj = null;
            String SearchOutputXML = null;
            StringBuilder sbArtworkValueXML = null;
            XPathDocument xpOutputDoc = null;
            XPathNavigator xpOutputNav = null;
            XPathNodeIterator xpElementIter = null;
            XPathNodeIterator xpAssetIter = null;
            String OptionValue = null;
            String PrevOptionValue = null;
            String ArtworkValueXML = null;
            String DataSource = null;
            XML ArtworkDetails = null;
            String WebServiceName = null;
            String Prefix = null;
            String Suffix = null;
            String TEValue_Asset = null;
            String AdditionalInfo = null;
            String AssetOrder = null;
            String ReplacedGEID = null;

            FormatType = null;
            Usage = null;
            FormatID = null;
            OptionsPopulated = false;

            sbSearchInputXML = new StringBuilder(40);
            sbSearchInputXML.AppendFormat(@"<SEARCH>
                                            <TARGETOBJECT>FORMAT</TARGETOBJECT>
                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                            <TARGETENTITY>
	                                            <SELECTLIST>
		                                            <EXPRESSION>
			                                            <FIELDNAME>FORMATID</FIELDNAME>
			                                            <ALIAS>FORMATID</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>FORMATTYPE</FIELDNAME>
			                                            <ALIAS>FORMATTYPE</ALIAS>
		                                            </EXPRESSION>
		                                            <EXPRESSION>
			                                            <FIELDNAME>USAGE</FIELDNAME>
			                                            <ALIAS>USAGE</ALIAS>
		                                            </EXPRESSION>
	                                            </SELECTLIST>
	                                            <FILTERLIST>
                                                    <FILTER>
			                                            <EXPRESSION>ARTWORKID</EXPRESSION>
			                                            <OPERATOR>EQUALTO</OPERATOR>
			                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
			                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
			                                            <RELATIONSHIPNAME>FORMAT_ARTWORK</RELATIONSHIPNAME>
		                                            </FILTER>
	                                            </FILTERLIST>
	                                            <RELATIONSHIP NAME='FORMAT_ARTWORKPAGE'>
		                                            <SELECTLIST>
			                                            <EXPRESSION>
				                                            <FIELDNAME>ARTWORKPAGEID</FIELDNAME>
				                                            <ALIAS>ARTWORKPAGEID</ALIAS>
			                                            </EXPRESSION>
		                                            </SELECTLIST>
		                                            <RELATIONSHIP NAME='ARTWORKPAGE_ELEMENTGROUP'>
			                                            <SELECTLIST>
				                                            <EXPRESSION>
					                                            <FIELDNAME>ELEMENTGROUPID</FIELDNAME>
					                                            <ALIAS>ELEMENTGROUPID</ALIAS>
				                                            </EXPRESSION>
			                                            </SELECTLIST>
			                                            <RELATIONSHIP NAME='ELEMENTGROUP_GRAPHICELEMENT'>
				                                            <SELECTLIST>
					                                            <EXPRESSION>
						                                            <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
						                                            <ALIAS>GRAPHICELEMENTID</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
						                                            <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
						                                            <ALIAS>GRAPHICTAGNAME</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
						                                            <FIELDNAME>TAGCONFIGURATIONXML</FIELDNAME>
						                                            <ALIAS>TAGCONFIGURATIONXML</ALIAS>
					                                            </EXPRESSION>                                                                    
				                                            </SELECTLIST>
                                                            <FILTERLIST>
					                                            <FILTER>
						                                            <EXPRESSION>AUTOPOPULATE</EXPRESSION>
						                                            <OPERATOR>EQUALTO</OPERATOR>
						                                            <EXPRESSIONVALUE>1</EXPRESSIONVALUE>
						                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>							                                            
					                                            </FILTER>
					                                        </FILTERLIST>
                                                            <RELATIONSHIP NAME='GRAPHICELEMENT_REPLACEDFILE'>
                                                                <SELECTLIST>
					                                                <EXPRESSION>
						                                                <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
						                                                <ALIAS>GRAPHICELEMENTID</ALIAS>
					                                                </EXPRESSION>
                                                                    <EXPRESSION>
						                                                <FIELDNAME>GRAPHICTAGNAME</FIELDNAME>
						                                                <ALIAS>GRAPHICTAGNAME</ALIAS>
					                                                </EXPRESSION>
				                                                </SELECTLIST>
                                                                <FILTERLIST>
						                                            <FILTER>
							                                            <EXPRESSION>ARTWORKID</EXPRESSION>
							                                            <OPERATOR>EQUALTO</OPERATOR>
							                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
							                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
							                                            <RELATIONSHIPNAME>ARTWORK_GRAPHICELEMENT</RELATIONSHIPNAME>
						                                            </FILTER>
					                                            </FILTERLIST>
                                                            </RELATIONSHIP>
				                                            <RELATIONSHIP NAME='GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO'>
					                                            <SELECTLIST>
						                                            <EXPRESSION>
							                                            <FIELDNAME>SHOWURL</FIELDNAME>
							                                            <ALIAS>SHOWURL</ALIAS>
						                                            </EXPRESSION>
						                                            <EXPRESSION>
							                                            <FIELDNAME>URL</FIELDNAME>
							                                            <ALIAS>URL</ALIAS>
						                                            </EXPRESSION>
						                                            <EXPRESSION>
							                                            <FIELDNAME>URLTARGET</FIELDNAME>
							                                            <ALIAS>URLTARGET</ALIAS>
						                                            </EXPRESSION>
						                                            <EXPRESSION>
							                                            <FIELDNAME>URLTYPE</FIELDNAME>
							                                            <ALIAS>URLTYPE</ALIAS>
						                                            </EXPRESSION>
						                                            <EXPRESSION>
							                                            <FIELDNAME>URLPROTOCOL</FIELDNAME>
							                                            <ALIAS>URLPROTOCOL</ALIAS>
						                                            </EXPRESSION>
                                                                    <EXPRESSION>
							                                            <FIELDNAME>NOIMAGE</FIELDNAME>
							                                            <ALIAS>NOIMAGE</ALIAS>
						                                            </EXPRESSION>
					                                            </SELECTLIST>
					                                            <FILTERLIST>
						                                            <FILTER>
							                                            <EXPRESSION>ARTWORKID</EXPRESSION>
							                                            <OPERATOR>EQUALTO</OPERATOR>
							                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
							                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
							                                            <RELATIONSHIPNAME>ARTWORK_GRAPHICELEMENTVALUEINFO</RELATIONSHIPNAME>
						                                            </FILTER>
					                                            </FILTERLIST>
					                                            <RELATIONSHIP NAME='GRAPHICELEMENTVALUEINFO_DIGITALASSET'>
						                                            <SELECTLIST>
							                                            <EXPRESSION>
								                                            <FIELDNAME>DIGITALASSETID</FIELDNAME>
								                                            <ALIAS>DIGITALASSETID</ALIAS>
							                                            </EXPRESSION>
						                                            </SELECTLIST>
					                                            </RELATIONSHIP>
				                                            </RELATIONSHIP>
			                                            </RELATIONSHIP>
			                                            <RELATIONSHIP NAME='ELEMENTGROUP_TEXTELEMENT'>
				                                            <SELECTLIST>
					                                            <EXPRESSION>
						                                            <FIELDNAME>TEXTELEMENTID</FIELDNAME>
						                                            <ALIAS>TEXTELEMENTID</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
						                                            <FIELDNAME>TEXTTAGNAME</FIELDNAME>
						                                            <ALIAS>TEXTTAGNAME</ALIAS>
					                                            </EXPRESSION>
					                                            <EXPRESSION>
						                                            <FIELDNAME>EDITABLE</FIELDNAME>
						                                            <ALIAS>EDITABLE</ALIAS>
					                                            </EXPRESSION>
					                                            <EXPRESSION>
						                                            <FIELDNAME>ISMULTIPLEALLOWED</FIELDNAME>
						                                            <ALIAS>ISMULTIPLEALLOWED</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
						                                            <FIELDNAME>TAGCONFIGURATIONXML</FIELDNAME>
						                                            <ALIAS>TAGCONFIGURATIONXML</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
						                                            <FIELDNAME>FIELDNODE</FIELDNAME>
						                                            <ALIAS>FIELDNODE</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
						                                            <FIELDNAME>DATATYPE</FIELDNAME>
						                                            <ALIAS>DATATYPE</ALIAS>
					                                            </EXPRESSION>
                                                                <EXPRESSION>
						                                            <FIELDNAME>DATASOURCE</FIELDNAME>
						                                            <ALIAS>DATASOURCE</ALIAS>
					                                            </EXPRESSION>
				                                            </SELECTLIST>
                                                            <FILTERLIST>
					                                            <FILTER>
						                                            <EXPRESSION>AUTOPOPULATE</EXPRESSION>
						                                            <OPERATOR>EQUALTO</OPERATOR>
						                                            <EXPRESSIONVALUE>1</EXPRESSIONVALUE>
						                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>							                                            
					                                            </FILTER>
                                                                {5}
					                                        </FILTERLIST>
                                                            <RELATIONSHIP NAME='TEXTELEMENT_OPTION'>
                                                                <SELECTLIST>
						                                            <EXPRESSION>
							                                            <FIELDNAME>OPTIONDISPLAYTEXT</FIELDNAME>
							                                            <ALIAS>OPTIONDISPLAYTEXT</ALIAS>
						                                            </EXPRESSION>
						                                            <EXPRESSION>
							                                            <FIELDNAME>OPTIONVALUE</FIELDNAME>
							                                            <ALIAS>OPTIONVALUE</ALIAS>
						                                            </EXPRESSION> 
						                                            <EXPRESSION>
							                                            <FIELDNAME>ISDEFAULTOPTION</FIELDNAME>
							                                            <ALIAS>ISDEFAULTOPTION</ALIAS>
						                                            </EXPRESSION> 
					                                            </SELECTLIST>
                                                            </RELATIONSHIP>
				                                            <RELATIONSHIP NAME='TEXTELEMENT_TEXTELEMENTVALUE'>
					                                            <SELECTLIST>
						                                            <EXPRESSION>
							                                            <FIELDNAME>TEXTELEMENTVALUEID</FIELDNAME>
							                                            <ALIAS>TEXTELEMENTVALUEID</ALIAS>
						                                            </EXPRESSION>
						                                            <EXPRESSION>
							                                            <FIELDNAME>TEXTELEMENTACTUALVALUE</FIELDNAME>
							                                            <ALIAS>TEXTELEMENTACTUALVALUE</ALIAS>
						                                            </EXPRESSION> 
					                                            </SELECTLIST>
					                                            <FILTERLIST>
						                                            <FILTER>
							                                            <EXPRESSION>ARTWORKID</EXPRESSION>
							                                            <OPERATOR>EQUALTO</OPERATOR>
							                                            <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
							                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
							                                            <RELATIONSHIPNAME>ARTWORK_TEXTELEMENTVALUE</RELATIONSHIPNAME>
						                                            </FILTER>
					                                            </FILTERLIST>
                                                                <RELATIONSHIP NAME='TEXTELEMENTVALUE_DIGITALASSET'>
                                                                    <SELECTLIST>
						                                                <EXPRESSION>
							                                                <FIELDNAME>DIGITALASSETID</FIELDNAME>
							                                                <ALIAS>DIGITALASSETID</ALIAS>
						                                                </EXPRESSION>
					                                                </SELECTLIST>
                                                                </RELATIONSHIP>
				                                            </RELATIONSHIP>
			                                            </RELATIONSHIP>
		                                            </RELATIONSHIP>
	                                            </RELATIONSHIP>
                                            </TARGETENTITY>
                                            <SESSIONINFO Language='{1}' TenantName='{2}' UserID='{3}' SessionID='{4}'/>
                                        </SEARCH>", ArtworkID, Language, TenantName, UserID, SessionID, TextElementFilterXML);

            OMIObj = new OMInterface(UserID, Language, TenantName);
            ErrorXML = OMIObj.GetHierarchialData(sbSearchInputXML.ToString(), ref SearchOutputXML);
            if (ErrorXML == null && !String.IsNullOrEmpty(SearchOutputXML))
            {
                xpOutputDoc = new XPathDocument(new StringReader(SearchOutputXML));
                xpOutputNav = xpOutputDoc.CreateNavigator();
                xpOutputNav = xpOutputNav.SelectSingleNode("FORMATCOLLECTION/FORMAT");

                if (xpOutputNav != null)
                {
                    FormatID = xpOutputNav.SelectSingleNode("FORMATID").Value;
                    FormatType = xpOutputNav.SelectSingleNode("FORMATTYPE").Value;
                    Usage = xpOutputNav.SelectSingleNode("USAGE").Value;
                    sbArtworkValueXML = new StringBuilder(40);
                    sbArtworkValueXML.Append("<PAGE><GROUPELEMENT>");

                    //Text element
                    xpElementIter = xpOutputNav.Select("FORMAT_ARTWORKPAGE/ARTWORKPAGE/ARTWORKPAGE_ELEMENTGROUP/ELEMENTGROUP/ELEMENTGROUP_TEXTELEMENT/TEXTELEMENT");
                    while (xpElementIter.MoveNext())
                    {
                        DataSource = xpElementIter.Current.SelectSingleNode("DATASOURCE").InnerXml;
                        TEValue_Asset = null;
                        AdditionalInfo = null;

                        if (xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGPREFIX") != null)
                            Prefix = xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGPREFIX").Value;
                        else
                            Prefix = String.Empty;

                        if (xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGSUFFIX") != null)
                            Suffix = xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGSUFFIX").Value;
                        else
                            Suffix = String.Empty;

                        switch (DataSource.ToUpper())
                        {
                            case "DATABASEFIELD":
                                ErrorXML = PopulateTextTagMappedToDatabaseField(FormatID, ArtworkID, xpElementIter.Current, out OptionValue);
                                if (!String.IsNullOrEmpty(OptionValue))
                                    OptionValue = Prefix + OptionValue + Suffix;
                                break;

                            case "WEBSERVICE":
                                if (xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/DATASOURCE_CONFIG/WEBSERVICE_CONFIG/WEBSERVICENAME") != null)
                                {
                                    ErrorXML = GetFacetDataForWSCall(ArtworkID, IsRelatedToBook, ref ArtworkDetails);
                                    WebServiceName = xpElementIter.Current.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/DATASOURCE_CONFIG/WEBSERVICE_CONFIG/WEBSERVICENAME").Value;

                                    if (IsRelatedToBook)
                                        ArtworkDetails = ArtworkDetails.Replace("</BOOK>", "<FORMATTYPE>" + FormatType.ToUpper() + "</FORMATTYPE></BOOK>");
                                    else
                                        ArtworkDetails = ArtworkDetails.Replace("</ARTWORK>", "<FORMATTYPE>" + FormatType.ToUpper() + "</FORMATTYPE></ARTWORK>");

                                    ErrorXML = GetOption_Webservice(ArtworkDetails, "TEXT", IsRelatedToBook, WebServiceName, out OptionValue, out AdditionalInfo);
                                    if (!String.IsNullOrEmpty(OptionValue))
                                        OptionValue = Prefix + OptionValue + Suffix;

                                    if (AdditionalInfo.Contains("<ASSETIDLIST>"))
                                    {
                                        TEValue_Asset = AdditionalInfo.Replace("<ASSETIDLIST>", "").Replace("</ASSETIDLIST>", "");
                                    }
                                }
                                break;

                            default:
                                if (String.Compare(OptionMode, "BOTH", true) == 0)
                                {
                                    ErrorXML = GetTextOption(xpElementIter.Current, out OptionValue);
                                    if (OptionValue == null)
                                    {
                                        ErrorXML = GetOption(xpElementIter.Current, "TEXT", TaxonomyDataXML, IsRelatedToBook, out OptionValue, out TEValue_Asset);
                                    }
                                }
                                else if (String.Compare(OptionMode, "OPTIONS", true) == 0)
                                {
                                    if (xpElementIter.Current.SelectSingleNode("TEXTELEMENT_OPTION/TEXTELEMENTOPTION") != null)
                                        ErrorXML = GetTextOption(xpElementIter.Current, out OptionValue);
                                }
                                else if (String.Compare(OptionMode, "DAM", true) == 0)
                                {
                                    ErrorXML = GetOption(xpElementIter.Current, "TEXT", TaxonomyDataXML, IsRelatedToBook, out OptionValue, out TEValue_Asset);
                                }
                                break;
                        }


                        if (ErrorXML == null)
                        {
                            OptionsPopulated = true;
                            AssetOrder = TEValue_Asset;
                            sbArtworkValueXML.Append("<TEXTELEMENT MODIFIED='TRUE'>");
                            sbArtworkValueXML.AppendFormat("<TEXTELEMENTID>{0}</TEXTELEMENTID>", xpElementIter.Current.SelectSingleNode("TEXTELEMENTID").Value);
                            sbArtworkValueXML.Append("<TEXTELEMENT_TEXTELEMENTVALUE TYPE='RELATION'>");
                            if (xpElementIter.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENTVALUE") == null)
                            {
                                sbArtworkValueXML.Append("<TEXTELEMENTVALUE>");
                            }
                            else
                            {
                                sbArtworkValueXML.Append("<TEXTELEMENTVALUE MODIFIED='TRUE'>");
                                sbArtworkValueXML.AppendFormat("<TEXTELEMENTVALUEID>{0}</TEXTELEMENTVALUEID>", xpElementIter.Current.SelectSingleNode("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENTVALUE/TEXTELEMENTVALUEID").Value);
                            }
                            sbArtworkValueXML.AppendFormat("<TEXTELEMENTACTUALVALUE>{0}</TEXTELEMENTACTUALVALUE>", ALSpecialCharHandling.TrimAndReplaceSpecialChars(OptionValue));
                            sbArtworkValueXML.Append("<FILTERVALUEXML></FILTERVALUEXML>");
                            sbArtworkValueXML.AppendFormat("<ASSETORDER>{0}</ASSETORDER>", AssetOrder);

                            xpAssetIter = xpElementIter.Current.Select("TEXTELEMENT_TEXTELEMENTVALUE/TEXTELEMENTVALUE/TEXTELEMENTVALUE_DIGITALASSET/DIGITALASSET/DIGITALASSETID");
                            while (xpAssetIter.MoveNext())
                            {
                                if (String.IsNullOrEmpty(TEValue_Asset))
                                    sbArtworkValueXML.AppendFormat("<TEXTELEMENTVALUE_DIGITALASSET  TYPE='RELATION' DISASSOCIATE='TRUE'>{0}</TEXTELEMENTVALUE_DIGITALASSET>", xpAssetIter.Current.Value);
                                else if (!TEValue_Asset.Contains(xpAssetIter.Current.Value))
                                    sbArtworkValueXML.AppendFormat("<TEXTELEMENTVALUE_DIGITALASSET  TYPE='RELATION' DISASSOCIATE='TRUE'>{0}</TEXTELEMENTVALUE_DIGITALASSET>", xpAssetIter.Current.Value);
                                else
                                    TEValue_Asset = TEValue_Asset.Replace(xpAssetIter.Current.Value + ",", "");
                            }

                            if (!String.IsNullOrEmpty(TEValue_Asset))
                            {
                                String[] AssetList = TEValue_Asset.Split(new String[] { "," }, StringSplitOptions.RemoveEmptyEntries);
                                for (int i = 0; i < AssetList.Length; i++)
                                    sbArtworkValueXML.AppendFormat("<TEXTELEMENTVALUE_DIGITALASSET  TYPE='RELATION'>{0}</TEXTELEMENTVALUE_DIGITALASSET>", AssetList[i].ToString());
                            }

                            sbArtworkValueXML.Append("</TEXTELEMENTVALUE>");

                            sbArtworkValueXML.Append("</TEXTELEMENT_TEXTELEMENTVALUE>");
                            sbArtworkValueXML.Append("</TEXTELEMENT>");
                        }
                        else
                        {
                            objErrorCommon.AppendError("APENGINE_ARTWORK_GETOPTIONS_FAILED", this.GetType().ToString(), "PopulateOptions", null, ErrorXML);
                        }
                    }

                    //Graphic element
                    if (String.Compare(OptionMode, "OPTIONS", true) != 0)
                    {
                        xpElementIter = xpOutputNav.Select("FORMAT_ARTWORKPAGE/ARTWORKPAGE/ARTWORKPAGE_ELEMENTGROUP/ELEMENTGROUP/ELEMENTGROUP_GRAPHICELEMENT/GRAPHICELEMENT");
                        while (xpElementIter.MoveNext())
                        {
                            TEValue_Asset = null;
                            ReplacedGEID = null;
                            if (String.Compare(OptionMode, "BOTH", true) == 0)
                            {
                                ErrorXML = GetGraphicOption(xpElementIter.Current, out OptionValue);
                                if (OptionValue == null)
                                {
                                    ErrorXML = GetOption(xpElementIter.Current, "GRAPHIC", TaxonomyDataXML, IsRelatedToBook, out OptionValue, out TEValue_Asset);
                                }
                            }
                            else if (String.Compare(OptionMode, "OPTIONS", true) == 0)
                            {
                                ErrorXML = GetGraphicOption(xpElementIter.Current, out OptionValue);
                            }
                            else if (String.Compare(OptionMode, "DAM", true) == 0)
                            {
                                ErrorXML = GetOption(xpElementIter.Current, "GRAPHIC", TaxonomyDataXML, IsRelatedToBook, out OptionValue, out TEValue_Asset);
                            }
                            if (xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/GRAPHICELEMENTVALUEINFO_DIGITALASSET/DIGITALASSET/DIGITALASSETID") != null)
                                PrevOptionValue = xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/GRAPHICELEMENTVALUEINFO_DIGITALASSET/DIGITALASSET/DIGITALASSETID").Value;
                            else
                            {
                                if (xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_REPLACEDFILE/GRAPHICELEMENT/GRAPHICELEMENTID") != null)
                                {
                                    ReplacedGEID = xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_REPLACEDFILE/GRAPHICELEMENT/GRAPHICELEMENTID").Value;
                                    PrevOptionValue = "";
                                }
                                else
                                    PrevOptionValue = "";
                            }

                            if (ErrorXML == null && !String.IsNullOrEmpty(OptionValue) && OptionValue != PrevOptionValue)
                            {
                                OptionsPopulated = true;
                                sbArtworkValueXML.Append("<GRAPHICELEMENT MODIFIED='TRUE'>");
                                sbArtworkValueXML.AppendFormat("<GRAPHICELEMENTID>{0}</GRAPHICELEMENTID>", xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTID").Value);
                                sbArtworkValueXML.Append("<GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO TYPE='RELATION'>");
                                if (xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE") == null)
                                {
                                    sbArtworkValueXML.Append("<GRAPHICELEMENTVALUE>");
                                }
                                else
                                {
                                    sbArtworkValueXML.Append("<GRAPHICELEMENTVALUE MODIFIED='TRUE'>");
                                    sbArtworkValueXML.AppendFormat("<GRAPHICELEMENTVALUEID>{0}</GRAPHICELEMENTVALUEID>", xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/GRAPHICELEMENTVALUEID").Value);
                                }

                                //if (xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/GRAPHICELEMENTVALUEINFO_DIGITALASSET/DIGITALASSET/DIGITALASSETID") != null)                                    
                                if (!String.IsNullOrEmpty(PrevOptionValue))
                                    sbArtworkValueXML.AppendFormat("<GRAPHICELEMENTVALUEINFO_DIGITALASSET TYPE='RELATION' DISASSOCIATE='TRUE'>{0}</GRAPHICELEMENTVALUEINFO_DIGITALASSET>", PrevOptionValue);

                                sbArtworkValueXML.AppendFormat("<GRAPHICELEMENTVALUEINFO_DIGITALASSET TYPE='RELATION'>{0}</GRAPHICELEMENTVALUEINFO_DIGITALASSET>", OptionValue);
                                //By default all the fields are set to no value.
                                sbArtworkValueXML.Append("<SHOWURL>0</SHOWURL><URL></URL><URLTARGET></URLTARGET><URLTYPE></URLTYPE><URLPROTOCOL></URLPROTOCOL><NOIMAGE>0</NOIMAGE><FILTERVALUEXML></FILTERVALUEXML>");

                                sbArtworkValueXML.Append("</GRAPHICELEMENTVALUE>");

                                sbArtworkValueXML.Append("</GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO>");

                                if (!String.IsNullOrEmpty(ReplacedGEID))
                                    sbArtworkValueXML.AppendFormat("<DELETE_GRAPHICELEMENTVALUE>{0}</DELETE_GRAPHICELEMENTVALUE>", ReplacedGEID);                                

                                sbArtworkValueXML.Append("</GRAPHICELEMENT>");
                            }
                            else if (ErrorXML == null && String.IsNullOrEmpty(OptionValue))
                            {
                                //NOIMAGE Case
                                OptionsPopulated = true;
                                sbArtworkValueXML.Append("<GRAPHICELEMENT MODIFIED='TRUE'>");
                                sbArtworkValueXML.AppendFormat("<GRAPHICELEMENTID>{0}</GRAPHICELEMENTID>", xpElementIter.Current.SelectSingleNode("GRAPHICELEMENTID").Value);
                                sbArtworkValueXML.Append("<GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO TYPE='RELATION'>");
                                if (xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE") == null)
                                {
                                    sbArtworkValueXML.Append("<GRAPHICELEMENTVALUE>");
                                }
                                else
                                {
                                    sbArtworkValueXML.Append("<GRAPHICELEMENTVALUE MODIFIED='TRUE'>");
                                    sbArtworkValueXML.AppendFormat("<GRAPHICELEMENTVALUEID>{0}</GRAPHICELEMENTVALUEID>", xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/GRAPHICELEMENTVALUEID").Value);
                                }

                                //if (xpElementIter.Current.SelectSingleNode("GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO/GRAPHICELEMENTVALUE/GRAPHICELEMENTVALUEINFO_DIGITALASSET/DIGITALASSET/DIGITALASSETID") != null)                                    
                                if (!String.IsNullOrEmpty(PrevOptionValue))
                                    sbArtworkValueXML.AppendFormat("<GRAPHICELEMENTVALUEINFO_DIGITALASSET TYPE='RELATION' DISASSOCIATE='TRUE'>{0}</GRAPHICELEMENTVALUEINFO_DIGITALASSET>", PrevOptionValue);

                                //By default all the fields are set to no value.
                                sbArtworkValueXML.Append("<SHOWURL>0</SHOWURL><URL></URL><URLTARGET></URLTARGET><URLTYPE></URLTYPE><URLPROTOCOL></URLPROTOCOL><NOIMAGE>1</NOIMAGE><FILTERVALUEXML></FILTERVALUEXML>");

                                sbArtworkValueXML.Append("</GRAPHICELEMENTVALUE>");

                                sbArtworkValueXML.Append("</GRAPHICELEMENT_GRAPHICELEMENTVALUEINFO>");
                                sbArtworkValueXML.Append("<GRAPHICELEMENT_NOPHOTO></GRAPHICELEMENT_NOPHOTO>");

                                if (!String.IsNullOrEmpty(ReplacedGEID))
                                    sbArtworkValueXML.AppendFormat("<DELETE_GRAPHICELEMENTVALUE>{0}</DELETE_GRAPHICELEMENTVALUE>", ReplacedGEID);                                

                                sbArtworkValueXML.Append("</GRAPHICELEMENT>");
                            }
                            else if (ErrorXML != null)
                            {
                                objErrorCommon.AppendError("APENGINE_ARTWORK_GETOPTIONS_FAILED", this.GetType().ToString(), "PopulateOptions", null, ErrorXML);
                            }
                        }
                    }

                    if (OptionsPopulated)
                    {
                        sbArtworkValueXML.Append("</GROUPELEMENT></PAGE>");
                        ArtworkValueXML = sbArtworkValueXML.ToString();

                        ErrorXML = this.UpdateArtwork(ArtworkID, ref ArtworkValueXML, true);
                        if (ErrorXML != null)
                        {
                            OptionsPopulated = false;
                            objErrorCommon.AppendError("APENGINE_ARTWORK_UPDATEARTWORK_FAILED", this.GetType().ToString(), "PopulateOptions", null, ErrorXML);
                        }
                        ErrorXML = objErrorCommon.ErrorCollectionXML;
                        if (ErrorXML != null)
                        {
                            //Associate Error with artwork
                            ArtworkProducerCommon ObjCommon = new ArtworkProducerCommon(UserID, Language, TenantName, SolutionName);
                            ObjCommon.LogError("ArtworkProducer", ErrorXML, ErrorType.ERROR, ArtworkID, "ARTWORK_JOBERROR");
                        }
                    }
                }
            }
            else
            {
                objErrorCommon.AppendError("APENGINE_ARTWORK_GETFORMATDETAILS_FAILED", this.GetType().ToString(), "PopulateOptions", null, ErrorXML);
                ErrorXML = objErrorCommon.ErrorCollectionXML;
            }
            return ErrorXML;
        }

        private ERRORXML GenerateArtwork(String FormatID, String FormatType, String Usage, String ArtworkID)
        {
            StringBuilder sbJobXML = null;
            MediaJobEngine MJEngObj = null;
            String JobXML = null;
            ERRORXML ErrorXML = null;
            String ArtworkStatus = null;
            String DisplayStatus = null;
            ArtworkProducerCommon ArtworkProducerCommonObj = null;
            String JobType = null;
            String JobName = null;

            sbJobXML = new StringBuilder(40);
            ArtworkProducerCommonObj = new ArtworkProducerCommon(UserID, Language, TenantName, SolutionName);
            if (Usage == "2")
            {
                ArtworkStatus = "Artwork Generation In Progress";
                JobName = "ArtworkJob";
                JobType = "ARTWORK GENERATION JOB";
            }
            else
            {
                ArtworkStatus = "Preview Generation In Progress";
                JobName = "PreviewJob";
                JobType = "PREVIEW GENERATION JOB";
            }
            ArtworkProducerCommonObj.GetDisplayStatus(ArtworkStatus, out DisplayStatus);

            sbJobXML.AppendFormat(@"<MEDIAJOB>
                                        <JOBNAME>{9}</JOBNAME>
                                        <CREATEDON>
                                            <DAY>{0}</DAY>
                                            <MONTH>{1}</MONTH>
                                            <YEAR>{2}</YEAR>
                                            <HOUR>{3}</HOUR>
                                            <MINUTE>{4}</MINUTE>
                                            <SECOND>{5}</SECOND>
                                        </CREATEDON>
                                        <MODIFIEDON>
                                            <DAY>{0}</DAY>
                                            <MONTH>{1}</MONTH>
                                            <YEAR>{2}</YEAR>
                                            <HOUR>{3}</HOUR>
                                            <MINUTE>{4}</MINUTE>
                                            <SECOND>{5}</SECOND>
                                        </MODIFIEDON>
                                        <SUBMITTEDON>
                                            <DAY>{0}</DAY>
                                            <MONTH>{1}</MONTH>
                                            <YEAR>{2}</YEAR>
                                            <HOUR>{3}</HOUR>
                                            <MINUTE>{4}</MINUTE>
                                            <SECOND>{5}</SECOND>
                                        </SUBMITTEDON>
                                        <JOBTYPE>{10}</JOBTYPE>
                                        <JOBHANDLER>{6}</JOBHANDLER>
                                        <JOBSTATUS>New</JOBSTATUS>
                                        <MEDIAJOB_CREATOR TYPE='RELATION'>{7}</MEDIAJOB_CREATOR>                                            
                                        <ARTWORK_MEDIAJOB TYPE='RELATION'>{8}</ARTWORK_MEDIAJOB>
                                        <ARTWORK_MEDIAJOB TYPE='RELATION'>
                                            <ARTWORK MODIFIED='TRUE'>
                                                <ARTWORKID>{8}</ARTWORKID>
                                                <ARTWORKSTATUS>{11}</ARTWORKSTATUS>
                                                <DISPLAYSTATUS>{12}</DISPLAYSTATUS>
                                            </ARTWORK>
                                        </ARTWORK_MEDIAJOB>
                                    </MEDIAJOB>", DateTime.Now.Day.ToString(), DateTime.Now.Month.ToString(), DateTime.Now.Year.ToString(), DateTime.Now.Hour.ToString(),
                            DateTime.Now.Minute.ToString(), DateTime.Now.Second.ToString(), FormatType, UserID, ArtworkID, JobName, JobType, ArtworkStatus, DisplayStatus);

            MJEngObj = new MediaJobEngine(SessionID, UserID, Language, TenantName);
            JobXML = sbJobXML.ToString();
            ErrorXML = MJEngObj.StampMediaJob(FormatID, ArtworkID, SolutionName, ref JobXML);
            if (ErrorXML != null)
            {
                objErrorCommon.AppendError("APENGINE_ARTWORK_STAMPMEDIAJOB_FAILED", this.GetType().ToString(), "PopulateOptionsAndGenerateArtwork", null, ErrorXML);
                ErrorXML = objErrorCommon.ErrorCollectionXML;
            }
            return ErrorXML;
        }

        private ERRORXML GetOption(XPathNavigator xpElementNav, String TagType, String TaxonomyDataXML, Boolean BookTaxonomy, out String OptionValue, out String TE_DigitalAsset)
        {
            String TagName = null;
            String TagConfigurationXML = null;
            ERRORXML ErrorXML = null;
            String OptionsXML = null;
            String Prefix = null;
            String Suffix = null;
            StringBuilder sbTEValue_Asset = null;
            String DigitalAssetID = null;
            ArtworkProducerEngine APEngineObj = new ArtworkProducerEngine(UserID, Language, TenantName, SolutionName);
            String OptionValueRelation = null;
            String OptionValueObject = null;
            String OptionValueColumn = null;

            try
            {
                if (xpElementNav.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGPREFIX") != null)
                    Prefix = xpElementNav.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGPREFIX").Value;
                else
                    Prefix = String.Empty;

                if (xpElementNav.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGSUFFIX") != null)
                    Suffix = xpElementNav.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGSUFFIX").Value;
                else
                    Suffix = String.Empty;

                if (xpElementNav.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/OPTIONVALUEINFO/RELATIONSHIPNAME") != null)
                    OptionValueRelation = xpElementNav.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/OPTIONVALUEINFO/RELATIONSHIPNAME").Value;

                if (xpElementNav.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/OPTIONVALUEINFO/RELATEDOBJECTNAME") != null)
                    OptionValueObject = xpElementNav.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/OPTIONVALUEINFO/RELATEDOBJECTNAME").Value;

                if (xpElementNav.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/OPTIONVALUEINFO/OPTIONVALUECOLUMN") != null)
                    OptionValueColumn = xpElementNav.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/OPTIONVALUEINFO/OPTIONVALUECOLUMN").Value;

                if (String.IsNullOrEmpty(OptionValueRelation) || String.IsNullOrEmpty(OptionValueObject) || String.IsNullOrEmpty(OptionValueColumn))
                {
                    OptionValueRelation = "DIGITALASSET_TAGTAXONOMY";
                    OptionValueObject = "TAGTAXONOMY";
                    OptionValueColumn = "OPTIONVALUE";
                }

                if (String.Compare(TagType, "GRAPHIC", true) == 0)
                    TagName = xpElementNav.SelectSingleNode("GRAPHICTAGNAME").Value;
                else
                {
                    TagName = xpElementNav.SelectSingleNode("TEXTTAGNAME").Value;
                    sbTEValue_Asset = new StringBuilder(20);
                }
                TagConfigurationXML = xpElementNav.SelectSingleNode("TAGCONFIGURATIONXML").InnerXml;

                OptionValue = String.Empty;
                TE_DigitalAsset = null;
                ErrorXML = APEngineObj.GetOption(TagName, TagConfigurationXML, TaxonomyDataXML, BookTaxonomy, out OptionsXML);
                if (ErrorXML == null && !String.IsNullOrEmpty(OptionsXML))
                {
                    String DataType = null;
                    Boolean LengthValidation = false;
                    Boolean RangeValidation = false;
                    int MinValue = 0;
                    int MaxValue = 0;
                    int OptionCount = 1;
                    int MaxLength = 0;

                    if (String.Compare(TagType, "TEXT", true) == 0)
                        GetValidationDetails(xpElementNav, out DataType, out LengthValidation, out MaxLength, out RangeValidation, out MinValue, out MaxValue, out OptionCount);

                    XPathDocument xpOptionsDoc = null;
                    XPathNavigator xpOptionsNav = null;
                    XPathNodeIterator xpOptionIter = null;

                    xpOptionsDoc = new XPathDocument(new StringReader(OptionsXML));
                    xpOptionsNav = xpOptionsDoc.CreateNavigator();

                    if (OptionCount == 1)
                    {
                        xpOptionsNav = xpOptionsNav.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET");
                        if (String.Compare(TagType, "GRAPHIC", true) == 0)
                        {
                            OptionValue = xpOptionsNav.SelectSingleNode("DIGITALASSETID").Value;
                            OptionCount--;
                        }
                        else
                        {
                            if (xpOptionsNav.SelectSingleNode(OptionValueRelation + "/" + OptionValueObject + "/" + OptionValueColumn) != null)
                            {
                                OptionValue = xpOptionsNav.SelectSingleNode(OptionValueRelation + "/" + OptionValueObject + "/" + OptionValueColumn).Value;
                                DigitalAssetID = xpOptionsNav.SelectSingleNode("DIGITALASSETID").Value;
                            }

                            OptionCount--;
                            if (!String.IsNullOrEmpty(OptionValue))
                            {
                                //DataType 
                                if (IsValidDataType(DataType, OptionValue))
                                {
                                    //Range
                                    if (RangeValidation)
                                    {
                                        if (!IsValidRange(OptionValue, MinValue.ToString(), MaxValue.ToString()))
                                            OptionValue = null;
                                    }

                                    //Length
                                    if (!String.IsNullOrEmpty(OptionValue) && LengthValidation)
                                    {
                                        if (!IsValidLength(OptionValue, MaxLength))
                                            OptionValue = null;
                                    }
                                }
                                else
                                    OptionValue = null;

                                if (!String.IsNullOrEmpty(OptionValue))
                                {
                                    OptionValue = Prefix + OptionValue + Suffix;
                                    if (!String.IsNullOrEmpty(DigitalAssetID))
                                        sbTEValue_Asset.AppendFormat("{0},", DigitalAssetID);
                                }
                            }
                        }
                    }
                    else
                    {
                        xpOptionIter = xpOptionsNav.Select("DIGITALASSETCOLLECTION/DIGITALASSET");
                        if (String.Compare(TagType, "TEXT", true) == 0)
                        {
                            String TempOptionValue = null;
                            String TempValue = null;
                            while (xpOptionIter.MoveNext())
                            {
                                TempOptionValue = xpOptionIter.Current.SelectSingleNode(OptionValueRelation + "/" + OptionValueObject + "/" + OptionValueColumn).Value;
                                DigitalAssetID = xpOptionIter.Current.SelectSingleNode("DIGITALASSETID").Value;

                                if (!String.IsNullOrEmpty(TempOptionValue))
                                {
                                    //DataTypeValidation                                        
                                    if (IsValidDataType(DataType, TempOptionValue))
                                    {
                                        TempOptionValue = Prefix + TempOptionValue + Suffix;
                                        //LengthValidation                                        
                                        if (!String.IsNullOrEmpty(OptionValue))
                                            TempOptionValue = OptionValue + "\r\n" + TempOptionValue;
                                        if (LengthValidation)
                                        {
                                            if (!String.IsNullOrEmpty(Prefix) && !String.IsNullOrEmpty(Suffix))
                                                TempValue = TempOptionValue.Replace(Prefix, "").Replace(Suffix, "");
                                            else if (!String.IsNullOrEmpty(Prefix))
                                                TempValue = TempOptionValue.Replace(Prefix, "");
                                            else if (!String.IsNullOrEmpty(Suffix))
                                                TempValue = TempOptionValue.Replace(Suffix, "");

                                            if (IsValidLength(TempValue.Replace("\r\n", ""), MaxLength))
                                            {
                                                OptionValue = TempOptionValue;
                                                OptionCount--;
                                                if (!String.IsNullOrEmpty(DigitalAssetID))
                                                    sbTEValue_Asset.AppendFormat("{0},", DigitalAssetID);
                                            }
                                        }
                                        else
                                        {
                                            OptionValue = TempOptionValue;
                                            OptionCount--;
                                            if (!String.IsNullOrEmpty(DigitalAssetID))
                                                sbTEValue_Asset.AppendFormat("{0},", DigitalAssetID);
                                        }

                                        if (OptionCount == 0)
                                            break;
                                    }
                                }
                            }
                        }
                    }

                    if (OptionCount > 0)
                        OptionValue = null;
                    if (sbTEValue_Asset != null)
                        TE_DigitalAsset = sbTEValue_Asset.ToString();
                }
            }
            catch (Exception)
            {
                OptionValue = null;
                TE_DigitalAsset = null;
            }
            return ErrorXML;
        }

        private ERRORXML GetTextOption(XPathNavigator xpElementNav, out String OptionValue)
        {
            ERRORXML ErrorXML = null;
            String Prefix = null;
            String Suffix = null;
            String DataType = null;
            Boolean LengthValidation = false;
            Boolean RangeValidation = false;
            int MinValue = 0;
            int MaxValue = 0;
            int OptionCount = 1;
            int MaxLength = 0;

            try
            {
                if (xpElementNav.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGPREFIX") != null)
                    Prefix = xpElementNav.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGPREFIX").Value;
                else
                    Prefix = String.Empty;

                if (xpElementNav.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGSUFFIX") != null)
                    Suffix = xpElementNav.SelectSingleNode("TAGCONFIGURATIONXML/TAGCONFIGURATION/ADDITIONALINFO/TAGSUFFIX").Value;
                else
                    Suffix = String.Empty;

                GetValidationDetails(xpElementNav, out DataType, out LengthValidation, out MaxLength, out RangeValidation, out MinValue, out MaxValue, out OptionCount);

                OptionValue = null;

                if (OptionCount == 1)
                {
                    if (xpElementNav.SelectSingleNode("TEXTELEMENT_OPTION/TEXTELEMENTOPTION[ISDEFAULTOPTION='1']/OPTIONVALUE") != null)
                        OptionValue = xpElementNav.SelectSingleNode("TEXTELEMENT_OPTION/TEXTELEMENTOPTION[ISDEFAULTOPTION='1']/OPTIONVALUE").Value;
                    OptionCount--;

                    if (!String.IsNullOrEmpty(OptionValue))
                    {
                        //DataType 
                        if (IsValidDataType(DataType, OptionValue))
                        {
                            //Range
                            if (RangeValidation)
                            {
                                if (!IsValidRange(OptionValue, MinValue.ToString(), MaxValue.ToString()))
                                    OptionValue = null;
                            }

                            //Length
                            if (!String.IsNullOrEmpty(OptionValue) && LengthValidation)
                            {
                                if (!IsValidLength(OptionValue, MaxLength))
                                    OptionValue = null;
                            }
                        }
                        else
                            OptionValue = null;

                        if (!String.IsNullOrEmpty(OptionValue))
                            OptionValue = Prefix + OptionValue + Suffix;
                    }
                }
                else
                {
                    XPathNodeIterator xpOptionIter = xpElementNav.Select("TEXTELEMENT_OPTION/TEXTELEMENTOPTION");
                    String TempOptionValue = null;
                    String TempValue = null;

                    while (xpOptionIter.MoveNext())
                    {
                        TempOptionValue = xpOptionIter.Current.SelectSingleNode("OPTIONVALUE").Value;
                        if (!String.IsNullOrEmpty(TempOptionValue))
                        {
                            //DataTypeValidation                                        
                            if (IsValidDataType(DataType, TempOptionValue))
                            {
                                TempOptionValue = Prefix + TempOptionValue + Suffix;
                                //LengthValidation
                                if (!String.IsNullOrEmpty(OptionValue))
                                    TempOptionValue = OptionValue + "\r\n" + TempOptionValue;
                                if (LengthValidation)
                                {
                                    if (!String.IsNullOrEmpty(Prefix) && !String.IsNullOrEmpty(Suffix))
                                        TempValue = TempOptionValue.Replace(Prefix, "").Replace(Suffix, "");
                                    else if (!String.IsNullOrEmpty(Prefix))
                                        TempValue = TempOptionValue.Replace(Prefix, "");
                                    else if (!String.IsNullOrEmpty(Suffix))
                                        TempValue = TempOptionValue.Replace(Suffix, "");

                                    if (IsValidLength(TempValue.Replace("\r\n", ""), MaxLength))
                                    {
                                        OptionValue = TempOptionValue;
                                        OptionCount--;
                                    }
                                }
                                else
                                {
                                    OptionValue = TempOptionValue;
                                    OptionCount--;
                                }

                                if (OptionCount == 0)
                                    break;
                            }
                        }
                    }
                }

                if (OptionCount > 0)
                    OptionValue = null;
            }
            catch (Exception)
            {
                OptionValue = null;
            }
            return ErrorXML;
        }

        private ERRORXML GetGraphicOption(XPathNavigator xpElementNav, out String OptionValue)
        {
            String GraphicElementName = "";
            String GraphicElementID = "";
            ulong SectionID;
            String SectionPath = null;
            OMInterface OMIObj = null;
            String SearchOutputXML = null;
            ERRORXML ErrorXML = null;
            ERRORXML ErrorXMLSection = null;
            XPathDocument xpOutputDoc = null;
            XPathNavigator xpOutputNav = null;

            StringBuilder sbSearchInputXML = null;
            sbSearchInputXML = new StringBuilder();

            GraphicElementName = xpElementNav.SelectSingleNode("GRAPHICTAGNAME").Value;
            GraphicElementID = xpElementNav.SelectSingleNode("GRAPHICELEMENTID").Value;

            Assetlink.Services.Engines.ArtworkProducerEngine.GraphicElementEngine SectionObj = null;
            SectionObj = new Assetlink.Services.Engines.ArtworkProducerEngine.GraphicElementEngine(UserID, Language, TenantName);

            ErrorXMLSection = SectionObj.CreateSectionForGraphicElement(GraphicElementID, out SectionID, out SectionPath);

            sbSearchInputXML.AppendFormat(@"<SEARCH>
	                                        <TARGETOBJECT>DIGITALASSET</TARGETOBJECT>
	                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
	                                        <TARGETENTITY>
		                                        <PRIVILEGENAME>READ</PRIVILEGENAME>  
                                                <SELECTLIST>
								                    <EXPRESSION>
									                    <FIELDNAME>DIGITALASSETID</FIELDNAME>
									                    <ALIAS>DIGITALASSETID</ALIAS>
								                    </EXPRESSION>
							                    </SELECTLIST>                                          
                                                <FILTERLIST>
                                                <FILTERCOMBINATION>(((1) AND (2)) OR ((1) AND (3)))</FILTERCOMBINATION>
                                                    <FILTER>
                                                        <EXPRESSION>MARKEDASDELETED</EXPRESSION>
                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>0</EXPRESSIONVALUE>
                                                        <FILTERID>1</FILTERID>
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                    </FILTER>
                                                    <FILTER>
                                                        <EXPRESSION>ASSETEXPIRED</EXPRESSION>
                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>0</EXPRESSIONVALUE>
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                    
                                                    </FILTER>
                                                    <FILTER>
                                                        <EXPRESSION>SECTIONID</EXPRESSION>
                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                        <FILTERID>2</FILTERID>
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                        <RELATIONSHIPNAME>SECTION_DIGITALASSET</RELATIONSHIPNAME>
                                                    </FILTER>
                                                    <FILTER>
                                                        <EXPRESSION>GRAPHICELEMENTID</EXPRESSION>
                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>{1}</EXPRESSIONVALUE>   
                                                        <FILTERID>3</FILTERID>                                                         
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                        <RELATIONSHIPNAME>GRAPHICELEMENT_LINKEDOPTIONS</RELATIONSHIPNAME>                                                            
                                                     </FILTER>
                                                </FILTERLIST>  
                                            </TARGETENTITY>
                                            <SESSIONINFO SessionID='{2}' UserID='{3}' TenantName='{4}' Language='{5}'/>
                                            </SEARCH>", SectionID, GraphicElementID, SessionID, UserID, TenantName, Language);

            OMIObj = new OMInterface(UserID, Language, TenantName);
            ErrorXML = OMIObj.GetHierarchialData(sbSearchInputXML.ToString(), ref SearchOutputXML);

            xpOutputDoc = new XPathDocument(new StringReader(SearchOutputXML));
            xpOutputNav = xpOutputDoc.CreateNavigator();
            if (xpOutputNav.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/DIGITALASSETID") != null)
                OptionValue = xpOutputNav.SelectSingleNode("DIGITALASSETCOLLECTION/DIGITALASSET/DIGITALASSETID").Value;
            else
                OptionValue = null;

            return ErrorXML;
        }

        private void GetValidationDetails(XPathNavigator xpElementNav, out String DataType, out Boolean LengthValidation, out int MaxLength, out Boolean RangeValidation, out int MinValue, out int MaxValue, out int OptionCount)
        {
            String FieldNodeXML = null;
            XPathDocument xpFieldNodeDoc = null;
            XPathNavigator xpFieldNodeNav = null;
            XPathNavigator xpValidatorNav = null;
            String IsMultipleAllowed = null;

            DataType = "STRING";
            LengthValidation = false;
            RangeValidation = false;
            MinValue = 0;
            MaxValue = 0;
            OptionCount = 1;
            MaxLength = 0;

            DataType = xpElementNav.SelectSingleNode("DATATYPE").InnerXml;
            FieldNodeXML = xpElementNav.SelectSingleNode("FIELDNODE").InnerXml;
            IsMultipleAllowed = xpElementNav.SelectSingleNode("ISMULTIPLEALLOWED").InnerXml;

            if (!String.IsNullOrEmpty(FieldNodeXML))
            {
                xpFieldNodeDoc = new XPathDocument(new StringReader(FieldNodeXML));
                xpFieldNodeNav = xpFieldNodeDoc.CreateNavigator();
                xpFieldNodeNav = xpFieldNodeNav.SelectSingleNode("Field");
            }

            if (xpFieldNodeNav != null)
            {
                xpValidatorNav = xpFieldNodeNav.SelectSingleNode("Validators/Validator [translate(@Type,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ') = 'RANGE']");
                if (xpValidatorNav != null)
                {
                    if (!String.IsNullOrEmpty(xpValidatorNav.GetAttribute("MinValue", "")))
                    {
                        try
                        {
                            if (IsMultipleAllowed == "1")
                            {
                                OptionCount = int.Parse(xpValidatorNav.GetAttribute("MinValue", ""));
                                RangeValidation = false;
                            }
                            else
                            {
                                MinValue = int.Parse(xpValidatorNav.GetAttribute("MinValue", "")); ;
                                RangeValidation = true;
                            }
                        }
                        catch (Exception)
                        {
                            OptionCount = 1;
                            MinValue = 0;
                            RangeValidation = false;
                        }

                        if (RangeValidation)
                        {
                            if (!String.IsNullOrEmpty(xpValidatorNav.GetAttribute("MaxValue", "")))
                            {
                                try
                                {
                                    MaxValue = int.Parse(xpValidatorNav.GetAttribute("MaxValue", "")); ;
                                    RangeValidation = true;
                                }
                                catch (Exception)
                                {
                                    MaxValue = 0;
                                    RangeValidation = false;
                                }
                            }
                        }
                    }
                }

                xpValidatorNav = xpFieldNodeNav.SelectSingleNode("Validators/Validator [translate(@Type,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ') = 'LENGTH']");
                if (xpValidatorNav != null)
                {
                    if (!String.IsNullOrEmpty(xpValidatorNav.GetAttribute("MaxLength", "")))
                    {
                        try
                        {
                            MaxLength = int.Parse(xpValidatorNav.GetAttribute("MaxLength", ""));
                            LengthValidation = true;
                        }
                        catch (Exception)
                        {
                            LengthValidation = false;
                            MaxLength = 0;
                        }
                    }
                }
            }
        }

        private void ReturnValidationDetails(XPathNavigator xpElementNav, out String DataType, out Boolean LengthValidation, out int MaxLength, out Boolean RangeValidation, out int MinValue, out int MaxValue, out int OptionCount)
        {
            String FieldNodeXML = null;
            XPathDocument xpFieldNodeDoc = null;
            XPathNavigator xpFieldNodeNav = null;
            XPathNavigator xpValidatorNav = null;
            String IsMultipleAllowed = null;

            DataType = "STRING";
            LengthValidation = false;
            RangeValidation = false;
            MinValue = 0;
            MaxValue = 0;
            OptionCount = 1;
            MaxLength = 0;

            DataType = xpElementNav.SelectSingleNode("DATATYPE").InnerXml;
            FieldNodeXML = xpElementNav.SelectSingleNode("FIELDNODE").InnerXml;
            IsMultipleAllowed = xpElementNav.SelectSingleNode("ISMULTIPLEALLOWED").InnerXml;

            if (!String.IsNullOrEmpty(FieldNodeXML))
            {
                xpFieldNodeDoc = new XPathDocument(new StringReader(FieldNodeXML));
                xpFieldNodeNav = xpFieldNodeDoc.CreateNavigator();
                xpFieldNodeNav = xpFieldNodeNav.SelectSingleNode("Field");
            }

            if (xpFieldNodeNav != null)
            {
                xpValidatorNav = xpFieldNodeNav.SelectSingleNode("Validators/Validator [translate(@Type,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ') = 'RANGE']");
                if (xpValidatorNav != null)
                {
                    if (!String.IsNullOrEmpty(xpValidatorNav.GetAttribute("MinValue", "")))
                    {
                        try
                        {
                            MinValue = int.Parse(xpValidatorNav.GetAttribute("MinValue", "")); ;
                            RangeValidation = true;
                        }
                        catch (Exception)
                        {
                            MinValue = 0;
                            RangeValidation = false;
                        }
                    }


                    if (!String.IsNullOrEmpty(xpValidatorNav.GetAttribute("MaxValue", "")))
                    {
                        try
                        {
                            MaxValue = int.Parse(xpValidatorNav.GetAttribute("MaxValue", "")); ;
                            RangeValidation = true;
                        }
                        catch (Exception)
                        {
                            MaxValue = 0;
                            RangeValidation = false;
                        }
                    }
                }

                xpValidatorNav = xpFieldNodeNav.SelectSingleNode("Validators/Validator [translate(@Type,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ') = 'LENGTH']");
                if (xpValidatorNav != null)
                {
                    if (!String.IsNullOrEmpty(xpValidatorNav.GetAttribute("MaxLength", "")))
                    {
                        try
                        {
                            MaxLength = int.Parse(xpValidatorNav.GetAttribute("MaxLength", ""));
                            LengthValidation = true;
                        }
                        catch (Exception)
                        {
                            LengthValidation = false;
                            MaxLength = 0;
                        }
                    }
                }
            }
        }

        private Boolean IsValidDataType(String DataType, String InputString)
        {
            try
            {
                if (!String.IsNullOrEmpty(InputString))
                {
                    switch (DataType.ToUpper())
                    {
                        case "INTEGER": Convert.ToInt32(InputString);
                            break;

                        case "FLOAT": Convert.ToDouble(InputString);
                            break;

                        default:
                            break;
                    }
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        private Boolean IsValidRange(String InputString, String MinValue, String MaxValue)
        {
            Double double_MaxVal = 0;
            Double double_MinVal = 0;
            Double double_Input = 0;

            if (!String.IsNullOrEmpty(MinValue) && !String.IsNullOrEmpty(MaxValue))
            {
                if (String.IsNullOrEmpty(InputString))
                    return false;

                double_MaxVal = Convert.ToDouble(MaxValue);
                double_MinVal = Convert.ToDouble(MinValue);
                double_Input = Convert.ToDouble(InputString);

                if (double_Input > double_MaxVal)
                    return false;

                if (Convert.ToDouble(InputString) < double_MinVal)
                    return false;
            }

            return true;
        }

        private Boolean IsValidLength(String InputString, int MaxLength)
        {
            if (MaxLength > 0)
            {
                if (InputString.Length > MaxLength)
                    return false;
            }

            return true;
        }

        private ERRORXML GetOption_Webservice(String ArtworkDetails, string TagType, Boolean BookTaxonomy, String WebServiceName, out String OptionValue, out String AdditionalInfo)
        {
            ERRORXML ErrorXML = null;
            String OptionsXML = null;
            String WSInputXML = null;
            String Error_WS = null;
            ArtworkProducerEngine APEngineObj = new ArtworkProducerEngine(UserID, Language, TenantName, SolutionName);
            WebServiceParamList OptionValueList = null;

            OptionValue = String.Empty;
            AdditionalInfo = String.Empty;
            WSInputXML = "<DATAINPUT><ARTWORKDETAILS>" + ArtworkDetails + "</ARTWORKDETAILS><ISRELATEDTOBOOK>" + BookTaxonomy.ToString() + "</ISRELATEDTOBOOK></DATAINPUT>";

            ALWebServiceInterfaceClass obj_ALWebServiceInterfaceClass = new ALWebServiceInterfaceClass(SessionID.ToString(), UserID.ToString(), TenantName, Language, SolutionName);
            ErrorXML = obj_ALWebServiceInterfaceClass.InvokeWebService(WebServiceName, WSInputXML, out OptionsXML, out Error_WS);
            if ((ErrorXML == null) && (String.IsNullOrEmpty(Error_WS)) && !String.IsNullOrEmpty(OptionsXML))
            {
                if (obj_ALWebServiceInterfaceClass.ListOfWSOutputParams.TryGetValue("OptionValue", out OptionValueList))
                    OptionValue = OptionValueList.ParameterValue;

                if (obj_ALWebServiceInterfaceClass.ListOfWSOutputParams.TryGetValue("AdditionalInfo", out OptionValueList))
                    AdditionalInfo = OptionValueList.ParameterValue;
            }

            return ErrorXML;
        }

        private ERRORXML GetFacetDataForWSCall(String ArtworkID, Boolean IsRelatedToBook, ref XML ArtworkDetails)
        {
            ERRORXML ErrorXML = null;
            StringBuilder sbSearchInputXML = null;
            OMInterface OMIObj = null;
            String SearchOutputXML = null;
            XPathDocument xpSearchOutput = null;
            XPathNavigator xpNav_SearchOpt = null;
            XPathNavigator xpNavReturnNode = null;

            sbSearchInputXML = new StringBuilder();
            if (!IsRelatedToBook)
            {
                sbSearchInputXML.AppendFormat(@"<SEARCH>
                                        <TARGETOBJECT>ARTWORK</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <AUTHORIZATIONREQUIRED>False</AUTHORIZATIONREQUIRED>
                                        <SEARCHALLFACETS>True</SEARCHALLFACETS>
                                        <TARGETENTITY>
                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                            <FILTERLIST>
                                                <FILTER>
                                                    <EXPRESSION>ARTWORKID</EXPRESSION>
                                                     <OPERATOR>EQUALTO</OPERATOR>
                                                    <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                    <FILTERID>1</FILTERID>
                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                </FILTER>
                                             </FILTERLIST>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>ARTWORKID</FIELDNAME>
                                                    <ALIAS>ARTWORKID</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>                                            
                                        </TARGETENTITY>                                  
                                        <SESSIONINFO Language='{1}' TenantName='{2}' UserID='{3}' SessionID='{4}'/>
                                        </SEARCH>", ArtworkID, Language, TenantName, UserID, SessionID);
            }
            else
            {
                sbSearchInputXML.AppendFormat(@"<SEARCH>
                                            <TARGETOBJECT>SPREAD</TARGETOBJECT>
                                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                            <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                            <SEARCHALLFACETS>TRUE</SEARCHALLFACETS>
                                            <SESSIONINFO UserID='{0}' TenantName='{1}' Language='{2}'/>
                                            <TARGETENTITY>
                                                <FILTERLIST>
                                                    <FILTER>
                                                        <EXPRESSION>ARTWORKID</EXPRESSION>
                                                        <OPERATOR>EQUALTO</OPERATOR>
                                                        <EXPRESSIONVALUE>{3}</EXPRESSIONVALUE>
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                        <RELATIONSHIPNAME>SPREAD_ARTWORK</RELATIONSHIPNAME>
                                                    </FILTER>
                                                </FILTERLIST>"
                                                + "  <RELATIONSHIP NAME=\"BOOK_SPREAD\">"
                                                + @"<SEARCHALLFACETS>TRUE</SEARCHALLFACETS>
                                                    </RELATIONSHIP>
                                             </TARGETENTITY>
                                            </SEARCH>
                                                    ", UserID, TenantName, Language, ArtworkID);
            }

            OMIObj = new OMInterface(UserID, Language, TenantName);
            ErrorXML = OMIObj.GetHierarchialData(sbSearchInputXML.ToString(), ref SearchOutputXML);

            xpSearchOutput = new XPathDocument(new StringReader(SearchOutputXML));
            xpNav_SearchOpt = xpSearchOutput.CreateNavigator();
            if (!IsRelatedToBook)
                xpNavReturnNode = xpNav_SearchOpt.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK");
            else
                xpNavReturnNode = xpNav_SearchOpt.SelectSingleNode("SPREADCOLLECTION/SPREAD/BOOK_SPREAD/BOOK");

            if (xpNavReturnNode != null)
                ArtworkDetails = xpNavReturnNode.OuterXml;

            return ErrorXML;
        }

        /// <alsummary>  
        /// <createddate>11 June 2010</createddate>
        /// <author>Meenal Joshi</author>
        /// <purpose>
        /// This method is used update the Artwork input xml resides in artwork folder (cache)
        /// If the artwork folder is not present in cache it will be created. It used for inline editing
        /// of artwork.
        /// </purpose>
        /// <param name="ArtworkID"></param>
        /// <param name="TagValueXML">
        /// <xmlstructure><![CDATA[
        /// <TAGVALUECOLLECTION FormatFileName=[Orignal file name for format] ApplicationType=''>
        ///     <TAG>
        ///	        <TAGTYPE>TEXT</TAGTYPE>
        ///	        <VALUE>aaaa</VALUE>
        ///	        <NAME>Advisor_Caption_1_6</NAME>
        ///	        <TAGID>46647880973410</TAGID>
        ///     </TAG>
        ///     <TAG>
        ///	        <TAGTYPE>TEXT</TAGTYPE>
        ///	        <VALUE>bbbb</VALUE>
        ///	        <NAME>AdvisorInfo1_2</NAME>
        ///	        <TAGID>51019375904127</TAGID>
        ///     </TAG>
        ///     <TAG>
        ///	        <TAGTYPE>GRAPHIC</TAGTYPE>
        ///	        <VALUE>D:\Assetlink\MediaProcessor\ArtworkCache\coke\246010176598391\Input\WFPname_8X10_CL_D.JPG.Original.102178497401999.JPEG</VALUE>
        ///	        <NAME>P2_I1_Image1</NAME>
        ///	        <TAGID>17362729651486</TAGID>
        ///	        <VALUETYPE>ASSET</VALUETYPE>
        ///     </TAG>
        ///</TAGVALUECOLLECTION>
        /// ]]></xmlstructure>
        /// </param>
        /// <returns>ERRORXML</returns>
        /// </alsummary>
        public ERRORXML ModifyArtworkForUpdatedTagElements(String FormatID, String ArtworkID, String ArtworkName)
        {
            #region Member Variable


            String str_ArtworkCacheFolder = null;
            String str_ArtworkValueXMLPath = null;
            ArtworkProducerCache objAPCache = null;
            String str_ArtworkName_WithoutSpecialChars = null;
            String ErrorXML = null;
            String JobPath = null;

            #endregion

            try
            {
                #region Initialization
                //xp_TagValueXML          = new XPathDocument(new StringReader(TagValueXML));
                //xpNav_TagValueXML       = xp_TagValueXML.CreateNavigator();
                str_ArtworkCacheFolder = ArtworkProducerCache.GetCacheFolderPath(TenantName, ArtworkID, ArtworkProducerCache.CacheType.ArtworkCache);
                JobPath = str_ArtworkCacheFolder + "\\Input\\";
                str_ArtworkValueXMLPath = ArtworkProducerCache.GetArtworkValuesXMLPath(TenantName, ArtworkID);
                objAPCache = new ArtworkProducerCache(UserID, Language, TenantName);
                //FormatFileName          = xpNav_TagValueXML.SelectSingleNode("TAGVALUECOLLECTION").GetAttribute("FormatFileName", "");
                //ApplicationType         = xpNav_TagValueXML.SelectSingleNode("TAGVALUECOLLECTION").GetAttribute("ApplicationType", "");

                str_ArtworkName_WithoutSpecialChars = ArtworkName;
                ArtworkProducerEngine.ConvertToAlphaNumericString(ref str_ArtworkName_WithoutSpecialChars);
                #endregion

                if (!File.Exists(str_ArtworkValueXMLPath))
                {
                    objAPCache.ArtworkCachePath = str_ArtworkCacheFolder;
                    ErrorXML = objAPCache.ArtworkCacheForEditMode(ArtworkID);
                    if (!String.IsNullOrEmpty(ErrorXML))
                        return ErrorXML;
                }
                //Prepare Output folder if not exists
                String str_OutputFolder = str_ArtworkCacheFolder + "\\Output";
                if (!Directory.Exists(str_OutputFolder))
                    Directory.CreateDirectory(str_OutputFolder);

                ErrorXML = PrepareArtworkTagCollection(FormatID, ArtworkID, str_ArtworkName_WithoutSpecialChars, str_ArtworkValueXMLPath, JobPath);
                if (!String.IsNullOrEmpty(ErrorXML))
                    return ErrorXML;
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_ARTWORK_MODIFYARTWORKFORUPDATEDTAGELEMENTS_EXCEPTION", this.GetType().ToString(), "ModifyArtworkForUpdatedTagElements", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                return ReturnErrorXML;
            }

            return null;
        }

        public ERRORXML PrepareArtworkTagCollection(String FormatID, String ArtworkID, String str_ArtworkName_WithoutSpecialChars, String ArtworkXMLPath, String JobPath)
        {
            List<String> GEIDList = null;
            List<String> DEIDList = null;
            XPathDocument xpDoc_ArtworkXML = null;
            XPathNavigator xpNav_ArtworkRoot = null;
            Dictionary<String, String> objDictionary = null;
            XPathNodeIterator xpIter_GE = null;
            XPathNodeIterator xpIter_TE = null;
            String str_Status = null;
            String str_Type = null;
            String GraphicElementID = null;
            String GEFilePath = null;
            String FileName = null;
            String GESearch = null;
            String DESearch = null;
            StringBuilder sbGESearch = null;
            String str_GEIDList = null;
            String str_DEIDList = null;
            StringBuilder sbDESearch = null;
            String ArtworkValues = null;
            StringBuilder sbSearchXML = null;
            XPathDocument xpDoc_ArtworkValues = null;
            XPathNodeIterator xpIter_GEs = null;
            String GraphicFileXML = null;
            XPathNavigator tmpNode = null;
            XPathNodeIterator xpIter_DEs = null;
            XPathNavigator xpNav_FilSystemNode = null;
            String str_PrintReadyInputXMLPath = null;
            String str_PrintReadyOutputXMLPath = null;
            String str_NetworkPath = null;
            String str_ReplacablePath = null;
            ArtworkProducerCache objAPCache = null;
            OMInterface OMIObject = null;
            ERRORXML ErrorXML = null;
            String str_GraphicElementTagValue = null;
            MediaRequestRouter objRouter = null;
            String FormatFileName = null;
            XPathDocument xpDoc_FormatDetails = null;
            XPathNavigator xpNav_FormatRoot = null;

            XPathDocument xpTmpDoc_ArtworkXML = null;
            XPathNavigator xpTmpNav_ArtworkRoot = null;
            XPathNavigator xpTmpNav_GE = null;
            DateTime STARTTIME;
            DateTime CurrentTime;
            TimeSpan DateDiff;
            Double TotalMinutes;
            bool isDone = false;

            xpDoc_FormatDetails = new XPathDocument(ArtworkProducerCache.GetFormatDetailsXMLPath(TenantName, FormatID));
            xpNav_FormatRoot = xpDoc_FormatDetails.CreateNavigator();
            xpNav_FormatRoot = xpNav_FormatRoot.SelectSingleNode("FORMATDETAILS");
            FormatFileName = xpNav_FormatRoot.SelectSingleNode("FORMAT/FILESYSTEM/FILE[FIELDNAME/text()='ORIGINALFILE']/DISPLAYNAME").Value;

            GEIDList = new List<String>();
            DEIDList = new List<String>();
            objDictionary = new Dictionary<String, String>();
            objAPCache = new ArtworkProducerCache(UserID, Language, TenantName);

            xpDoc_ArtworkXML = new XPathDocument(ArtworkXMLPath);
            xpNav_ArtworkRoot = xpDoc_ArtworkXML.CreateNavigator();

            xpIter_TE = xpNav_ArtworkRoot.Select("ARTWORK/ARTWORKVALUES/TEXTTAGS/TAG");

            while (xpIter_TE.MoveNext())
            {
                if (xpIter_TE.Current.GetAttribute("ISHIDDEN", "") != "1")
                {
                    objDictionary.Add(xpIter_TE.Current.GetAttribute("NAME", "").ToUpper(), xpIter_TE.Current.SelectSingleNode("VALUE").Value);
                }
                else
                {
                    if (xpIter_TE.Current.GetAttribute("AUTOPOPULATE", "") == "1")
                        objDictionary.Add(xpIter_TE.Current.GetAttribute("NAME", "").ToUpper(), xpIter_TE.Current.SelectSingleNode("VALUE").Value);
                }
            }

            xpIter_GE = xpNav_ArtworkRoot.Select("ARTWORK/ARTWORKVALUES/GRAPHICTAGS/TAG");
            while (xpIter_GE.MoveNext())
            {
                str_Status = xpIter_GE.Current.GetAttribute("STATUS", "");
                str_Type = xpIter_GE.Current.GetAttribute("VALUETYPE", "");
                if (str_Type == "DEFAULT")
                {
                    if (String.Compare(str_Status, "PENDING", true) == 0)
                    {
                        GraphicElementID = xpIter_GE.Current.GetAttribute("ID", "");
                        GEFilePath = ArtworkProducerCache.GetGraphicElementOriginalFilePath(TenantName, FormatID, GraphicElementID);
                        FileName = Path.GetFileName(GEFilePath);

                        File.Copy(GEFilePath, JobPath + FileName, true);
                        objDictionary.Add(xpIter_GE.Current.GetAttribute("NAME", "").ToUpper(), JobPath + FileName);

                        objAPCache.ModifyGraphicTagValue(ArtworkID, GraphicElementID, JobPath + FileName, ArtworkProducerCache.GraphicTagUpdateType.DEFAULT, ArtworkProducerCache.GraphicTagUpdateStatus.DONE);
                    }
                    else if (String.Compare(str_Status, "DONE", true) == 0)
                    {
                        objDictionary.Add(xpIter_GE.Current.GetAttribute("NAME", "").ToUpper(), xpIter_GE.Current.Value);
                    }
                }
                else if (str_Type == "UPLOAD")
                {
                    if (String.Compare(str_Status, "IN_OUTBOX", true) == 0)
                    {
                        FileName = Path.GetFileName(xpIter_GE.Current.Value);
                        File.Copy(xpIter_GE.Current.Value, JobPath + FileName, true);
                        objDictionary.Add(xpIter_GE.Current.GetAttribute("NAME", "").ToUpper(), JobPath + FileName);

                        GraphicElementID = xpIter_GE.Current.GetAttribute("ID", "");
                        objAPCache.ModifyGraphicTagValue(ArtworkID, GraphicElementID, JobPath + FileName, ArtworkProducerCache.GraphicTagUpdateType.UPLOAD, ArtworkProducerCache.GraphicTagUpdateStatus.DONE);
                        //Update Artwork XML
                    }
                    else if (String.Compare(str_Status, "DONE", true) != 0)
                    {
                        GEIDList.Add(xpIter_GE.Current.Value);
                    }
                    else
                    {
                        objDictionary.Add(xpIter_GE.Current.GetAttribute("NAME", "").ToUpper(), xpIter_GE.Current.Value);
                    }
                }
                else if (str_Type == "ASSET")
                {
                    if (String.Compare(str_Status, "IN_OUTBOX", true) == 0)
                    {
                        FileName = Path.GetFileName(xpIter_GE.Current.Value);
                        File.Copy(xpIter_GE.Current.Value, JobPath + FileName, true);
                        objDictionary.Add(xpIter_GE.Current.GetAttribute("NAME", "").ToUpper(), JobPath + FileName);

                        GraphicElementID = xpIter_GE.Current.GetAttribute("ID", "");
                        objAPCache.ModifyGraphicTagValue(ArtworkID, GraphicElementID, JobPath + FileName, ArtworkProducerCache.GraphicTagUpdateType.ASSET, ArtworkProducerCache.GraphicTagUpdateStatus.DONE);
                    }
                    else if (String.Compare(str_Status, "DONE", true) != 0)
                    {
                        if (String.Compare(str_Status, "PENDING", true) == 0)
                        {
                            if (!String.IsNullOrEmpty(xpIter_GE.Current.GetAttribute("STARTTIME", "")))
                            {
                                GraphicElementID = xpIter_GE.Current.GetAttribute("ID", "");
                                STARTTIME = DateTime.Parse(xpIter_GE.Current.GetAttribute("STARTTIME", ""));
                                CurrentTime = DateTime.Now;
                                DateDiff = CurrentTime.Subtract(STARTTIME);
                                TotalMinutes = DateDiff.TotalMinutes;
                                isDone = false;

                                while (TotalMinutes < 5)
                                {
                                    System.Threading.Thread.Sleep(3000);
                                    xpTmpDoc_ArtworkXML = new XPathDocument(ArtworkXMLPath);
                                    xpTmpNav_ArtworkRoot = xpTmpDoc_ArtworkXML.CreateNavigator();
                                    xpTmpNav_GE = xpTmpNav_ArtworkRoot.SelectSingleNode("ARTWORK/ARTWORKVALUES/GRAPHICTAGS/TAG[@ID='" + GraphicElementID + "']");

                                    if (xpTmpNav_GE != null)
                                    {
                                        str_Status = xpTmpNav_GE.GetAttribute("STATUS", "");

                                        if (String.Compare(str_Status, "IN_OUTBOX", true) == 0)
                                        {
                                            FileName = Path.GetFileName(xpTmpNav_GE.Value);
                                            File.Copy(xpTmpNav_GE.Value, JobPath + FileName, true);
                                            objDictionary.Add(xpTmpNav_GE.GetAttribute("NAME", "").ToUpper(), JobPath + FileName);

                                            objAPCache.ModifyGraphicTagValue(ArtworkID, GraphicElementID, JobPath + FileName, ArtworkProducerCache.GraphicTagUpdateType.ASSET, ArtworkProducerCache.GraphicTagUpdateStatus.DONE);
                                            isDone = true;
                                            break;
                                        }
                                    }
                                    else
                                        break;

                                    CurrentTime = DateTime.Now;
                                    DateDiff = CurrentTime.Subtract(STARTTIME);
                                    TotalMinutes = DateDiff.TotalMinutes;
                                }

                                if (!isDone)
                                    DEIDList.Add(xpIter_GE.Current.Value);
                            }
                            else
                                DEIDList.Add(xpIter_GE.Current.Value);
                        }
                        else
                            DEIDList.Add(xpIter_GE.Current.Value);
                    }
                    else
                    {
                        objDictionary.Add(xpIter_GE.Current.GetAttribute("NAME", "").ToUpper(), xpIter_GE.Current.Value);
                    }
                }
                else if (str_Type == "NOIMAGE")
                {
                    if (String.Compare(str_Status, "DONE", true) != 0)
                    {
                        GEIDList.Add(xpIter_GE.Current.Value);
                    }
                    else
                    {
                        objDictionary.Add(xpIter_GE.Current.GetAttribute("NAME", "").ToUpper(), xpIter_GE.Current.Value);
                    }
                }
            }

            if (DEIDList.Count > 0 || GEIDList.Count > 0)
            {
                if (GEIDList.Count > 0)
                {
                    str_GEIDList = GEIDList[0];
                    GEIDList.Remove(GEIDList[0]);
                    foreach (String GE in GEIDList)
                    {
                        str_GEIDList += "," + GE;
                    }

                    sbGESearch = new StringBuilder();
                    sbGESearch.AppendFormat(@"<RELATIONSHIP NAME='ARTWORK_GRAPHICELEMENT'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>GRAPHICELEMENTID</FIELDNAME>
                                                        <ALIAS>GRAPHICELEMENTID</ALIAS>
                                                    </EXPRESSION>
                                                    <EXPRESSION>
                                                        <FIELDNAME>ORIGINALFILE</FIELDNAME>
                                                        <ALIAS>ORIGINALFILE</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                                <FILTERLIST>
                                                    <FILTER>
                                                        <EXPRESSION>GRAPHICELEMENTID</EXPRESSION>
                                                        <OPERATOR>IN</OPERATOR>
                                                        <EXPRESSIONVALUE>({0})</EXPRESSIONVALUE>
                                                        <FILTERID>1</FILTERID>
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                    </FILTER>
                                                </FILTERLIST>
                                            </RELATIONSHIP>", str_GEIDList);

                    GESearch = sbGESearch.ToString();
                }

                if (DEIDList.Count > 0)
                {
                    str_DEIDList = DEIDList[0];
                    DEIDList.Remove(DEIDList[0]);
                    foreach (String DE in DEIDList)
                    {
                        str_DEIDList += "," + DE;
                    }

                    sbDESearch = new StringBuilder();
                    sbDESearch.AppendFormat(@"<RELATIONSHIP NAME='ARTWORK_DIGITALASSET'>
                                                <SELECTLIST>
                                                    <EXPRESSION>
                                                        <FIELDNAME>DIGITALASSETID</FIELDNAME>
                                                        <ALIAS>DIGITALASSETID</ALIAS>
                                                    </EXPRESSION>
                                                </SELECTLIST>
                                                <RELATIONSHIP NAME='DIGITALASSET_CONTENTREVISION'>
                                                    <RELATIONSHIP NAME='CONTENTREVISION_RENDITION'>
                                                        <FILTERLIST>
                                                            <FILTER>
                                                                <EXPRESSION>RENDITIONTYPE</EXPRESSION>
                                                                <OPERATOR>EQUALTO</OPERATOR>
                                                                <EXPRESSIONVALUE>ORIGINAL</EXPRESSIONVALUE>
                                                                <FILTERID>1</FILTERID>
                                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                            </FILTER>
                                                        </FILTERLIST>
                                                        <SELECTLIST>
                                                            <EXPRESSION>
                                                                <FIELDNAME>ACTUALFILE</FIELDNAME>
                                                                <ALIAS>ACTUALFILE</ALIAS>
                                                            </EXPRESSION>
                                                            <EXPRESSION>
                                                                <FIELDNAME>RENDITIONTYPE</FIELDNAME>
                                                                <ALIAS>RENDITIONTYPE</ALIAS>
                                                            </EXPRESSION>
                                                        </SELECTLIST>
                                                    </RELATIONSHIP>
                                                </RELATIONSHIP>
                                                <FILTERLIST>
                                                    <FILTER>
                                                        <EXPRESSION>DIGITALASSETID</EXPRESSION>
                                                        <OPERATOR>IN</OPERATOR>
                                                        <EXPRESSIONVALUE>({0})</EXPRESSIONVALUE>
                                                        <FILTERID>1</FILTERID>
                                                        <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                    </FILTER>
                                                </FILTERLIST>
                                            </RELATIONSHIP>", str_DEIDList);

                    DESearch = sbDESearch.ToString();
                }

                sbSearchXML = new StringBuilder();
                sbSearchXML.AppendFormat(@"<SEARCH>
                                                <TARGETOBJECT>ARTWORK</TARGETOBJECT>
                                                <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                                <TARGETENTITY>
                                                    <PRIVILEGENAME>READ</PRIVILEGENAME>
                                                    <FILTERLIST>
                                                        <FILTER>
                                                            <EXPRESSION>ARTWORKID</EXPRESSION>
                                                            <OPERATOR>EQUALTO</OPERATOR>
                                                            <EXPRESSIONVALUE>{2}</EXPRESSIONVALUE>
                                                            <FILTERID>1</FILTERID>
                                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                        </FILTER>
                                                    </FILTERLIST>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
                                                            <FIELDNAME>ARTWORKID</FIELDNAME>
                                                            <ALIAS>ARTWORKID</ALIAS>
                                                        </EXPRESSION>
                                                    </SELECTLIST>
                                                    {0}
                                                    {1}
                                                </TARGETENTITY>   
                                            </SEARCH>", DESearch, GESearch, ArtworkID);


                OMIObject = new OMInterface(UserID, Language, TenantName);
                ErrorXML = OMIObject.GetHierarchialData(sbSearchXML.ToString(), ref ArtworkValues);

                xpDoc_ArtworkValues = new XPathDocument(new StringReader(ArtworkValues));
                xpIter_GEs = xpDoc_ArtworkValues.CreateNavigator().Select("ARTWORKCOLLECTION/ARTWORK/ARTWORK_GRAPHICELEMENT/GRAPHICELEMENT");
                while (xpIter_GEs.MoveNext())
                {
                    GraphicFileXML = xpIter_GEs.Current.SelectSingleNode("FILESYSTEM").OuterXml;
                    ErrorXML = GetFile(GraphicFileXML, "ORIGINALFILE", JobPath, null);
                    if (ErrorXML != null)
                        return ErrorXML;

                    str_GraphicElementTagValue = JobPath + "\\" + xpIter_GEs.Current.SelectSingleNode("FILESYSTEM/FILE/DISPLAYNAME").InnerXml;
                    tmpNode = xpNav_ArtworkRoot.SelectSingleNode("ARTWORK/ARTWORKVALUES/GRAPHICTAGS/TAG [text() = '" + xpIter_GEs.Current.SelectSingleNode("GRAPHICELEMENTID").InnerXml + "']");
                    objDictionary.Add(tmpNode.GetAttribute("NAME", "").ToUpper(), str_GraphicElementTagValue);

                    objAPCache.ModifyGraphicTagValue(ArtworkID, tmpNode.GetAttribute("ID", ""), str_GraphicElementTagValue, ArtworkProducerCache.GraphicTagUpdateType.UPLOAD, ArtworkProducerCache.GraphicTagUpdateStatus.DONE);
                }


                xpIter_DEs = xpDoc_ArtworkValues.CreateNavigator().Select("ARTWORKCOLLECTION/ARTWORK/ARTWORK_DIGITALASSET/DIGITALASSET");
                while (xpIter_DEs.MoveNext())
                {
                    xpNav_FilSystemNode = xpIter_DEs.Current.SelectSingleNode("DIGITALASSET_CONTENTREVISION/CONTENTREVISION/CONTENTREVISION_RENDITION/ASSETRENDITION[RENDITIONTYPE/text()='Original' or RENDITIONTYPE/text()='ORIGINAL']/FILESYSTEM");
                    GraphicFileXML = xpNav_FilSystemNode.OuterXml;
                    ErrorXML = GetFile(GraphicFileXML, "ACTUALFILE", JobPath, null);
                    if (ErrorXML != null)
                        return ErrorXML;

                    tmpNode = xpNav_ArtworkRoot.SelectSingleNode("ARTWORK/ARTWORKVALUES/GRAPHICTAGS/TAG [text() = '" + xpIter_DEs.Current.SelectSingleNode("DIGITALASSETID").InnerXml + "']");

                    if (!String.IsNullOrEmpty(xpNav_FilSystemNode.SelectSingleNode("FILE/DISPLAYNAME").Value))
                        str_GraphicElementTagValue = JobPath + "\\" + tmpNode.GetAttribute("ID", "") + "_" + xpNav_FilSystemNode.SelectSingleNode("FILE/DISPLAYNAME").Value;
                    else
                        str_GraphicElementTagValue = JobPath + "\\" + tmpNode.GetAttribute("ID", "") + "_" + xpNav_FilSystemNode.SelectSingleNode("FILE/NAME").Value;

                    objDictionary.Add(tmpNode.GetAttribute("NAME", "").ToUpper(), str_GraphicElementTagValue);
                    objAPCache.ModifyGraphicTagValue(ArtworkID, tmpNode.GetAttribute("ID", ""), str_GraphicElementTagValue, ArtworkProducerCache.GraphicTagUpdateType.ASSET, ArtworkProducerCache.GraphicTagUpdateStatus.DONE);
                }
            }


            str_PrintReadyInputXMLPath = JobPath + "\\" + Path.GetFileNameWithoutExtension(FormatFileName) + ".xml";
            str_PrintReadyOutputXMLPath = JobPath + "\\" + str_ArtworkName_WithoutSpecialChars + "_Inp.xml";

            objRouter = new MediaRequestRouter(TenantName, UserID.ToString(), SessionID.ToString(), Language, SolutionName);
            ErrorXML = objRouter.GetNetworkPath("INDESIGN", out str_NetworkPath);
            if (ErrorXML != null)
                return ErrorXML;

            ErrorXML = objRouter.GetJobPath("INDESIGN", out str_ReplacablePath);
            if (ErrorXML != null)
                return ErrorXML;

            PrepareTagValueList_InputXML(objDictionary, str_PrintReadyInputXMLPath, str_PrintReadyOutputXMLPath, "PROOF", "", str_ReplacablePath, str_NetworkPath);
            return null;
        }

        void PrepareTagValueList_InputXML(XPathNavigator xpNav_TagValueXML, String PrintReadyXML, String OutputXMLPath, String SpecialTagName, String SpecialTagValue, String JobPath, String NetworkPath,
            String str_ArtworkCacheFolder, bool VerifyInputXMLWithArtworkXML)
        {
            #region Member Variable
            XmlNodeList ListNodes = null;
            XmlNode RootNode = null;
            XmlDocument OutXML = new XmlDocument(); //This is the XML that we will get from the DB which represents the format in which In Design expects the XML.            
            XPathNavigator xpNav_Tag = null;
            bool isOutputXMLExists = false;
            String str_InputFolderPath = null;
            XPathNodeIterator xpItr_TagList = null;
            String str_TagType = null;
            String str_TagName = null;
            //XPathDocument   xp_ArtworkXml           = null;
            XmlDocument xp_ArtworkXml = null;
            XPathNavigator xpNav_ArtworkXmlRoot = null;
            XPathNodeIterator xpItr_TextTagItr = null;
            XPathNodeIterator xpItr_GraphicTagItr = null;
            XPathNavigator xpNav_TagPresent = null;
            //XmlNode           xml_ArtworkRoot       = null;
            XmlNodeList UpdatedTagNodeList = null;
            #endregion

            if (File.Exists(OutputXMLPath))
                isOutputXMLExists = true;

            if (!isOutputXMLExists)
            {
                OutXML.Load(PrintReadyXML);
                RootNode = OutXML.SelectSingleNode("Root");
                str_InputFolderPath = str_ArtworkCacheFolder + "\\Input\\";

                #region Iterate In Design required XML

                ListNodes = RootNode.SelectNodes("//*");

                foreach (XmlNode LeafNode in ListNodes)
                {
                    if ((LeafNode.Attributes.Count == 0))//If it is a TEXT Tag
                    {
                        if (LeafNode.SelectNodes("child::*").Count == 0)
                        {
                            if (LeafNode.Name.ToUpper() != SpecialTagName.ToUpper())
                            {
                                xpNav_Tag = xpNav_TagValueXML.SelectSingleNode("TAGVALUECOLLECTION/TAG[NAME='" + LeafNode.Name + "']");

                                if (xpNav_Tag != null)
                                {
                                    LeafNode.InnerText = xpNav_Tag.SelectSingleNode("VALUE").Value;
                                }
                                else
                                {
                                    //if(!VerifyInputXMLWithArtworkXML)
                                    LeafNode.InnerText = "";
                                }
                            }
                            else
                            {
                                LeafNode.InnerText = SpecialTagValue;
                            }
                        }
                    }
                    else //If it is a GRAPHIC Tag
                    {
                        xpNav_Tag = xpNav_TagValueXML.SelectSingleNode("TAGVALUECOLLECTION/TAG[NAME='" + LeafNode.Name + "']");
                        if (xpNav_Tag != null)
                        {
                            if (!String.IsNullOrEmpty(xpNav_Tag.SelectSingleNode("VALUE").Value))
                                ((XmlElement)LeafNode).SetAttribute("href", "file:///" + xpNav_Tag.SelectSingleNode("VALUE").Value.Replace(JobPath, NetworkPath));
                            else
                            {
                                if (LeafNode.Attributes["href"] != null)
                                    ((XmlElement)LeafNode).RemoveAttribute("href");
                            }
                        }
                        else
                        {
                            //if (LeafNode.Attributes["href"] != null)
                            //    ((XmlElement)LeafNode).RemoveAttribute("href");

                            if (LeafNode.Name != "Root")
                            {
                                String str_TempFileName = ((XmlElement)LeafNode).GetAttribute("href");
                                str_TempFileName = str_TempFileName.Substring(str_TempFileName.LastIndexOf("/") + 1);
                                str_TempFileName = str_InputFolderPath + str_TempFileName;
                                ((XmlElement)LeafNode).SetAttribute("href", "file:///" + str_TempFileName.Replace(JobPath, NetworkPath));
                            }
                        }

                    }
                }
                #endregion

                StreamWriter OutputXMLWriter = new StreamWriter(OutputXMLPath);
                String GeneratedOutputXML = null;
                GeneratedOutputXML = OutXML.InnerXml;
                GeneratedOutputXML = GeneratedOutputXML.Replace("$lt;", "<");
                GeneratedOutputXML = GeneratedOutputXML.Replace("$gt;", ">");

                OutputXMLWriter.Write(GeneratedOutputXML);
                OutputXMLWriter.Close();


            }//If Artwork_Inp.xml not exists
            else
            {
                OutXML.Load(OutputXMLPath);
                RootNode = OutXML.SelectSingleNode("Root");
                if (RootNode != null)
                {
                    xpItr_TagList = xpNav_TagValueXML.Select("TAGVALUECOLLECTION/TAG");
                    foreach (XPathNavigator xpNav_TagNode in xpItr_TagList)
                    {
                        str_TagType = xpNav_TagNode.SelectSingleNode("TAGTYPE").Value;
                        str_TagName = xpNav_TagNode.SelectSingleNode("NAME").Value;

                        //TagNode         = RootNode.SelectNodes(str_TagName);
                        UpdatedTagNodeList = RootNode.SelectNodes(str_TagName);
                        if (UpdatedTagNodeList != null)
                        {
                            switch (str_TagType)
                            {
                                case "TEXT":
                                    foreach (XmlNode Tag_Node in UpdatedTagNodeList)
                                        Tag_Node.InnerText = xpNav_TagNode.SelectSingleNode("VALUE").Value;
                                    break;

                                case "GRAPHIC":
                                    foreach (XmlNode Tag_Node in UpdatedTagNodeList)
                                        ((XmlElement)Tag_Node).SetAttribute("href", "file:///" + xpNav_TagNode.SelectSingleNode("VALUE").Value.Replace(JobPath, NetworkPath));
                                    break;
                            }
                        }
                    }
                }//if (RootNode != null)
                File.WriteAllText(OutputXMLPath, OutXML.OuterXml);
            }//else if Artwork_Inp.xml exists

            //This means that the Artwork job folder is created now (it may not present if explicitly deleted for clean up)
            //VerifyInputXMLWithArtworkXML: Flag specifies that the input xml created needs to be verified with the Artwork.xml
            //Tag values. New values gets replaced but the old values for that Tag will be verified with Tag values in artwork xml,
            //so that it has latest values always.
            if (VerifyInputXMLWithArtworkXML)
            {
                #region VerifyInputXMLWithArtworkXML
                OutXML = new XmlDocument();
                OutXML.Load(OutputXMLPath);
                RootNode = OutXML.SelectSingleNode("Root");
                ListNodes = RootNode.SelectNodes("//*");

                //xp_ArtworkXml           = new XPathDocument(str_ArtworkCacheFolder + "\\Artwork.xml");
                xp_ArtworkXml = new XmlDocument();
                xp_ArtworkXml.Load(str_ArtworkCacheFolder + "\\Artwork.xml");
                xpNav_ArtworkXmlRoot = xp_ArtworkXml.CreateNavigator();

                //Verify Text Tags
                xpItr_TextTagItr = xpNav_ArtworkXmlRoot.Select("ARTWORK/ARTWORKVALUES/TEXTTAGS/TAG[@ISHIDDEN='0']");
                foreach (XPathNavigator xpNav_TextTag in xpItr_TextTagItr)
                {
                    xpNav_TagPresent = xpNav_TagValueXML.SelectSingleNode("TAGVALUECOLLECTION/TAG[NAME='" + xpNav_TextTag.GetAttribute("NAME", "") + "']");
                    //If the Tag is not from updated Tag list, then verify its value wit Artwork xml values
                    if (xpNav_TagPresent == null)
                    {
                        updateTextTagValue(xpNav_TextTag, ref ListNodes);
                    }
                }

                //Verify Graphic Tags
                xpItr_GraphicTagItr = xpNav_ArtworkXmlRoot.Select("ARTWORK/ARTWORKVALUES/GRAPHICTAGS/TAG");
                foreach (XPathNavigator xpNav_GraphicTag in xpItr_GraphicTagItr)
                {
                    xpNav_TagPresent = xpNav_TagValueXML.SelectSingleNode("TAGVALUECOLLECTION/TAG[NAME='" + xpNav_GraphicTag.GetAttribute("NAME", "") + "']");
                    //If the Tag is not from updated Tag list, then verify its value wit Artwork xml values
                    if (xpNav_TagPresent == null)
                    {
                        updateGraphicTagValue(xpNav_GraphicTag, ref ListNodes);
                    }
                    else
                    {
                        XmlNodeList xml_UpdatedTags = null;
                        //xml_ArtworkRoot = xp_ArtworkXml.SelectSingleNode("ARTWORK/ARTWORKVALUES/GRAPHICTAGS/TAG[@NAME='" + xpNav_GraphicTag.GetAttribute("NAME", "") + "']");
                        xml_UpdatedTags = xp_ArtworkXml.SelectNodes("ARTWORK/ARTWORKVALUES/GRAPHICTAGS/TAG[@NAME='" + xpNav_GraphicTag.GetAttribute("NAME", "") + "']");
                        if (xml_UpdatedTags != null && xml_UpdatedTags.Count > 0)
                        {
                            foreach (XmlNode xml_ArtworkRoot in xml_UpdatedTags)
                            {
                                ((XmlElement)xml_ArtworkRoot).SetAttribute("STATUS", "DONE");
                                xml_ArtworkRoot.InnerText = xpNav_TagPresent.SelectSingleNode("VALUE").Value;
                            }
                        }
                    }
                }

                xp_ArtworkXml.Save(str_ArtworkCacheFolder + "\\Artwork.xml");
                File.WriteAllText(OutputXMLPath, OutXML.OuterXml);
                #endregion
            }
        }

        void PrepareTagValueList_InputXML(Dictionary<String, String> objDictionary, String PrintReadyXML, String OutputXMLPath, String SpecialTagName, String SpecialTagValue, String JobPath, String NetworkPath)
        {
            XmlNodeList ListNodes = null;
            XmlDocument OutXML = new XmlDocument(); // This is the XML that we will get from the DB which represents the format in which In Design expects the XML.
            OutXML.PreserveWhitespace = true;
            OutXML.Load(PrintReadyXML);

            XmlNode RootNode = OutXML.SelectSingleNode("Root");

            #region Iterate In Design required XML

            ListNodes = RootNode.SelectNodes("//*");

            foreach (XmlNode LeafNode in ListNodes)
            {
                string TagValue = null;

                if ((LeafNode.Attributes.Count == 0))
                {
                    if (LeafNode.SelectNodes("child::*").Count == 0)
                    {
                        if (LeafNode.Name.ToUpper() != SpecialTagName.ToUpper())
                        {
                            if (objDictionary.TryGetValue(LeafNode.Name.ToUpper(), out TagValue) == true)
                            {
                                LeafNode.InnerText = TagValue;
                            }
                        }
                        else
                        {
                            LeafNode.InnerText = SpecialTagValue;
                        }
                    }
                }
                else
                {
                    if (objDictionary.TryGetValue(LeafNode.Name.ToUpper(), out TagValue) == true)
                    {
                        //LeafNode.InnerText=TagValue;
                        if (TagValue != "")
                            ((XmlElement)LeafNode).SetAttribute("href", "file:///" + TagValue.Replace(JobPath, NetworkPath));
                        else
                        {
                            if (LeafNode.Attributes["href"] != null)
                                ((XmlElement)LeafNode).RemoveAttribute("href");
                        }
                    }
                }
            }
            #endregion

            StreamWriter OutputXMLWriter = new StreamWriter(OutputXMLPath);
            String GeneratedOutputXML = null;
            GeneratedOutputXML = OutXML.InnerXml;

            GeneratedOutputXML = GeneratedOutputXML.Replace("$lt;", "<");
            GeneratedOutputXML = GeneratedOutputXML.Replace("$gt;", ">");

            OutputXMLWriter.Write(GeneratedOutputXML);
            OutputXMLWriter.Close();
        }

        private ERRORXML GetFile(String FileXML, String FieldName, String DownloadPath, String OrgDisplayFileName)
        {
            #region variable
            FileRetrival FileRetrivalObj = null;
            FileTransferInfrastructure FileTransferObj = null;
            String FileSystemXML = null;
            String ErrorXML = null;
            XPathDocument XPathDoc = null;
            StringReader XPathDocXML = null;
            XPathNavigator XPathNav = null;
            XPathNavigator FileNode = null;
            XPathNodeIterator FileNodeList = null;

            String FilePath = null;
            String FileName = null;
            String DisplayFileName = null;
            String ObjectID = null;
            int Index = 0;
            #endregion

            #region Get File

            FileRetrivalObj = new FileRetrival(SessionID.ToString(), UserID.ToString(), TenantName, Language);
            FileTransferObj = new FileTransferInfrastructure(SessionID.ToString(), UserID.ToString(), TenantName, Language);


            XPathDocXML = new StringReader(FileXML);
            XPathDoc = new XPathDocument(XPathDocXML);
            XPathNav = XPathDoc.CreateNavigator();

            FileNodeList = XPathNav.Select("//FIELDNAME[text()='" + FieldName + "']");

            if (FileNodeList.Count > 0)
            {
                while (FileNodeList.MoveNext())
                {

                    FileNode = FileNodeList.Current;
                    FileNode.MoveToParent();
                    FileSystemXML = FileNode.OuterXml;
                    if (FileSystemXML.StartsWith("<FILE>"))
                    {
                        //Get File Name
                        if (FileNode.SelectSingleNode("NAME") != null)
                            FileName = FileNode.SelectSingleNode("NAME").InnerXml;

                        if (FileNode.SelectSingleNode("DISPLAYNAME") != null)
                            DisplayFileName = FileNode.SelectSingleNode("DISPLAYNAME").InnerXml;

                        #region Get File From Engine to WebService
                        ObjectID = FileSystemXML.Substring(FileSystemXML.LastIndexOf("<OBJECTID>"), FileSystemXML.IndexOf("</OBJECTID>") - FileSystemXML.IndexOf("<OBJECTID>"));
                        ObjectID = ObjectID.Replace("<OBJECTID>", "");
                        FileSystemXML = FileSystemXML.Replace("<FILE>", "<FILE FileRequired=\"Yes\">");
                        FileSystemXML = "<FILESYSTEM ObjectID=\"" + ObjectID + "\">" + FileSystemXML + "</FILESYSTEM>";
                        try
                        {
                            ErrorXML = FileRetrivalObj.GetFile(ref FileSystemXML);
                            if (ErrorXML != null)
                                return ErrorXML;
                        }
                        catch (Exception)
                        {
                        }
                        #endregion

                        #region Bug fix for MediaServer File
                        String str_FilePresentOn = null;
                        StringBuilder sb_GetFileCollectionXML = new StringBuilder(100);
                        FileTransferInfrastructure ObjFT_MS = null;

                        ObjFT_MS = new FileTransferInfrastructure(SessionID.ToString(), UserID.ToString(), TenantName, Language);
                        ObjFT_MS.ToUseServiceURL = ALCommon.GetWebServiceURL("ALMEDIASERVERMTOMWS");

                        str_FilePresentOn = ALCommon.GetInnerValue(ref FileSystemXML, "<ONMEDIASERVER>", "</ONMEDIASERVER>");
                        if (String.Compare(str_FilePresentOn, "FALSE", true) == 0)
                            str_FilePresentOn = "SERVICES";
                        else
                            str_FilePresentOn = "MEDIASERVER";

                        if (str_FilePresentOn == "MEDIASERVER")
                        {
                            FilePath = ALCommon.GetInnerValue(ref FileSystemXML, "<PATH>", "</PATH>");
                            sb_GetFileCollectionXML.AppendFormat(@"<FILECOLLECTION><FILE>                                                                                
                                                                                <SOURCEPATH>{0}</SOURCEPATH>
                                                                                <DESTINATIONPATH>{0}</DESTINATIONPATH>
                                                                            </FILE></FILECOLLECTION>", ALCommon.TrimAndReplaceSpecialChars(FilePath));

                            FileSystemXML = sb_GetFileCollectionXML.ToString();
                            ErrorXML = ObjFT_MS.GetFile(ref FileSystemXML);
                            if (String.IsNullOrEmpty(ErrorXML))
                            {
                                FileSystemXML = FileSystemXML.Replace("<NEWPATH>", "<PATH>");
                                FileSystemXML = FileSystemXML.Replace("</NEWPATH>", "</PATH>");
                            }
                            else
                                return ErrorXML;
                        }

                        #endregion

                        #region Renaming of file to actual file name
                        bool bFileMoved = false;
                        if (String.IsNullOrEmpty(OrgDisplayFileName))
                        {
                            if (FileName.ToUpper() != DisplayFileName.ToUpper() && !String.IsNullOrEmpty(DisplayFileName))
                            {
                                FilePath = ALCommon.GetInnerValue(ref FileSystemXML, "<PATH>", "</PATH>");
                                if (File.Exists(ALSpecialCharHandling.DecodeSpecialCharacters(FilePath)))
                                {
                                    Index = FilePath.LastIndexOf("\\");
                                    FilePath = FilePath.Remove(Index + 1);
                                    {
                                        if (File.Exists(ALSpecialCharHandling.DecodeSpecialCharacters(DownloadPath + DisplayFileName)))
                                            File.Delete(ALSpecialCharHandling.DecodeSpecialCharacters(DownloadPath + DisplayFileName));
                                        File.Move(ALSpecialCharHandling.DecodeSpecialCharacters(FilePath + FileName), ALSpecialCharHandling.DecodeSpecialCharacters(DownloadPath + DisplayFileName));
                                        FileSystemXML = FileSystemXML.Replace(FilePath + FileName, FilePath + DisplayFileName);
                                        FileSystemXML = FileSystemXML.Replace("<NAME>" + FileName, "<NAME>" + DisplayFileName);
                                        bFileMoved = true;
                                    }
                                }
                            }
                        }
                        else
                        {
                            if (String.Compare(FileName, OrgDisplayFileName, true) != 0)
                            {
                                FilePath = ALCommon.GetInnerValue(ref FileSystemXML, "<PATH>", "</PATH>");
                                if (File.Exists(ALSpecialCharHandling.DecodeSpecialCharacters(FilePath)))
                                {
                                    Index = FilePath.LastIndexOf("\\");
                                    FilePath = FilePath.Remove(Index + 1);
                                    {
                                        if (File.Exists(ALSpecialCharHandling.DecodeSpecialCharacters(DownloadPath + OrgDisplayFileName)))
                                            File.Delete(ALSpecialCharHandling.DecodeSpecialCharacters(DownloadPath + OrgDisplayFileName));
                                        File.Move(ALSpecialCharHandling.DecodeSpecialCharacters(FilePath + FileName), ALSpecialCharHandling.DecodeSpecialCharacters(DownloadPath + OrgDisplayFileName));
                                        FileSystemXML = FileSystemXML.Replace(FilePath + FileName, FilePath + OrgDisplayFileName);
                                        FileSystemXML = FileSystemXML.Replace("<NAME>" + FileName, "<NAME>" + OrgDisplayFileName);
                                        FileSystemXML = FileSystemXML.Replace("<DISPLAYNAME>" + DisplayFileName, "<DISPLAYNAME>" + OrgDisplayFileName);
                                        bFileMoved = true;
                                    }
                                }
                            }
                        }
                        #endregion

                        #region Get File from WebService to local Machine
                        if (ErrorXML == null && !bFileMoved)
                        {
                            //FileSystemXML = FileSystemXML.Replace("<PATH>", "<SOURCEPATH>");
                            //FileSystemXML = FileSystemXML.Replace("</PATH>", "</SOURCEPATH>");
                            //FileTransferObj.GetFilePath = DownloadPath;
                            //ErrorXML = FileTransferObj.GetFile(ref FileSystemXML);
                            //if (ErrorXML != null)
                            //    return ErrorXML;

                            FilePath = ALCommon.GetInnerValue(ref FileSystemXML, "<PATH>", "</PATH>");
                            FilePath = ALSpecialCharHandling.DecodeSpecialCharacters(FilePath);
                            String str_FileName = Path.GetFileName(FilePath);

                            if (File.Exists(DownloadPath + str_FileName))
                                File.Delete(DownloadPath + str_FileName);
                            File.Move(FilePath, DownloadPath + str_FileName);
                        }
                        #endregion

                    }
                }//end of while

            }
            #endregion

            return null;
        }

        private void updateTextTagValue(XPathNavigator xpNav_TextTag, ref XmlNodeList ListNodes)
        {
            #region Member variable
            XPathNavigator xpNav_TagValue = null;
            #endregion

            foreach (XmlNode LeafNode in ListNodes)
            {
                if ((LeafNode.Attributes.Count == 0))//If it is a TEXT Tag
                {
                    if (LeafNode.SelectNodes("child::*").Count == 0)
                    {
                        if (String.Compare(LeafNode.Name, xpNav_TextTag.GetAttribute("NAME", ""), false) == 0)
                        {
                            xpNav_TagValue = xpNav_TextTag.SelectSingleNode("VALUE");
                            if (xpNav_TagValue != null && LeafNode.InnerXml != xpNav_TagValue.InnerXml)
                                LeafNode.InnerXml = xpNav_TagValue.InnerXml;
                            break;
                        }
                    }
                }
            }
            return;
        }

        private void updateGraphicTagValue(XPathNavigator xpNav_GraphicTag, ref XmlNodeList ListNodes)
        {
            #region member variable
            String str_FileNameIn_InpFile = null;
            String str_FileNameIn_ArtworkXMLFile = null;
            String str_NewFileName = null;
            String str_InputFolderPath = null;
            #endregion

            foreach (XmlNode LeafNode in ListNodes)
            {
                if ((LeafNode.Attributes.Count != 0))//If it is a GRAPHIC Tag
                {
                    if (String.Compare(LeafNode.Name, xpNav_GraphicTag.GetAttribute("NAME", ""), false) == 0)
                    {
                        str_FileNameIn_InpFile = ((XmlElement)LeafNode).GetAttribute("href");
                        str_FileNameIn_ArtworkXMLFile = xpNav_GraphicTag.InnerXml;

                        str_InputFolderPath = str_FileNameIn_InpFile.Substring(0, str_FileNameIn_InpFile.LastIndexOf("\\"));
                        str_FileNameIn_InpFile = str_FileNameIn_InpFile.Substring(str_FileNameIn_InpFile.LastIndexOf("\\") + 1);
                        str_FileNameIn_ArtworkXMLFile = str_FileNameIn_ArtworkXMLFile.Substring(str_FileNameIn_ArtworkXMLFile.LastIndexOf("\\") + 1);

                        if (String.Compare(str_FileNameIn_InpFile, str_FileNameIn_ArtworkXMLFile, true) != 0)
                        {
                            str_NewFileName = String.Format("{0}\\{1}", str_InputFolderPath, str_FileNameIn_ArtworkXMLFile);
                            ((XmlElement)LeafNode).SetAttribute("href", str_NewFileName);
                        }
                        break;
                    }
                }
            }
            return;
        }

        private void SaveHTMLArtwork(object data)
        {
            String ArtworkID = null;
            String FormatID = null;
            String ArtworkName = null;
            String JobID = null;
            String SolutionName = null;
            String JobType = null;
            String ZipFilePath = null;
            List<String> param = null;

            //System.Security.Principal.WindowsImpersonationContext NewContext = null;
            try
            {
                param = (List<String>)data;
                SolutionName = param[0];
                ArtworkID = param[1];
                ArtworkName = param[2];
                FormatID = param[3];
                JobType = param[4];
                JobID = param[5];
                ZipFilePath = param[6];

                //System.Threading.Thread.CurrentPrincipal = new System.Security.Principal.WindowsPrincipal(OrgIdentity);
                //NewContext = OrgIdentity.Impersonate();

                String AG_ConfigXML = ALCommon.GetTenantConfigurationPath(TenantName) + "\\ArtworkGeneration_Config.xml";
                XPathDocument xpDoc_GenerationConfig = new XPathDocument(AG_ConfigXML);
                XPathNavigator xpNav_ArtworkConfigRoot = xpDoc_GenerationConfig.CreateNavigator();
                XPathNodeIterator xpIter_Outputs = xpNav_ArtworkConfigRoot.Select("SOLUTIONLIST/SOLUTION[translate(@NAME,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='" + SolutionName.ToUpper() + "']/BUTTON/JOBTYPE [translate(@TYPE,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='" + JobType.ToUpper() + "']/FORMATTYPE [@TYPE='HTML']/OUTPUTLIST/OUTPUT");
                List<String> lstRelations = new List<string>(xpIter_Outputs.Count);
                while (xpIter_Outputs.MoveNext())
                {
                    lstRelations.Add(xpIter_Outputs.Current.GetAttribute("RELATIONSHIPNAME", ""));
                }

                if (lstRelations.Count > 0)
                {
                    String ErrorXML = ArtworkProducerCommon.DeleteExistingAssets(UserID, TenantName, Language, ArtworkID, lstRelations);
                    if (ErrorXML != null)
                    {
                        ArtworkProducerCommon.UpdateJobStatus(UserID, Language, TenantName, SessionID, "Error", JobID);
                        ArtworkProducerCommon.UpdateArtworkStatus(UserID, Language, TenantName, SolutionName, "Error", ArtworkID);
                        return;
                    }

                    String str_SectionID = null;
                    ErrorXML = ArtworkProducerCommon.GetSectionForArtwork(UserID, Language, TenantName, ArtworkID, ArtworkName, out str_SectionID, JobType);
                    if (ErrorXML != null)
                    {
                        ArtworkProducerCommon.UpdateJobStatus(UserID, Language, TenantName, SessionID, "Error", JobID);
                        ArtworkProducerCommon.UpdateArtworkStatus(UserID, Language, TenantName, SolutionName, "Error", ArtworkID);
                        return;
                    }

                    xpIter_Outputs = xpNav_ArtworkConfigRoot.Select("SOLUTIONLIST/SOLUTION[translate(@NAME,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='" + SolutionName.ToUpper() + "']/BUTTON/JOBTYPE [translate(@TYPE,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='" + JobType.ToUpper() + "']/FORMATTYPE [@TYPE='HTML']/OUTPUTLIST/OUTPUT");
                    while (xpIter_Outputs.MoveNext())
                    {
                        XPathNavigator xpNav_Current = xpIter_Outputs.Current;
                        String RelationName = xpNav_Current.GetAttribute("RELATIONSHIPNAME", "");
                        String AssetID = null;
                        ErrorXML = ArtworkProducerCommon.CreateDigitalAssetForArtwork(JobID, UserID, TenantName, Language, SessionID.ToString(), SolutionName, ZipFilePath, "ZIP", null, null, null, null, str_SectionID, ArtworkID, RelationName, JobType, "", ArtworkName, "", out AssetID);
                        if (ErrorXML != null)
                        {
                            ArtworkProducerCommon.UpdateJobStatus(UserID, Language, TenantName, SessionID, "Error", JobID);
                            ArtworkProducerCommon.UpdateArtworkStatus(UserID, Language, TenantName, SolutionName, "Error", ArtworkID);
                            return;
                        }
                    }

                    ArtworkProducerCommon.UpdateJobStatus(UserID, Language, TenantName, SessionID, "Done", JobID);
                    ArtworkProducerCommon.UpdateArtworkStatus(UserID, Language, TenantName, SolutionName, "Artwork Generated", ArtworkID);
                }
                else
                {
                    ArtworkProducerCommon.UpdateJobStatus(UserID, Language, TenantName, SessionID, "Error", JobID);
                    ArtworkProducerCommon.UpdateArtworkStatus(UserID, Language, TenantName, SolutionName, "Error", ArtworkID);
                    return;
                }
            }
            catch
            {
                try
                {
                    ArtworkProducerCommon.UpdateJobStatus(UserID, Language, TenantName, SessionID, "Error", JobID);
                    ArtworkProducerCommon.UpdateArtworkStatus(UserID, Language, TenantName, SolutionName, "Error", ArtworkID);
                }
                catch
                { }
            }
            //finally
            //{
            //    NewContext.Undo();
            //}
        }

        public ERRORXML GetFormatIDForArtwork(String ArtworkID, out String FormatID)
        {
            FormatID = null;
            String ArtworkXML = null;
            StringBuilder sbSearchXml = new StringBuilder();
            sbSearchXml.AppendFormat(@"<SEARCH>
                                        <TARGETOBJECT>ARTWORK</TARGETOBJECT>
                                        <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                        <AUTHORIZATIONREQUIRED>False</AUTHORIZATIONREQUIRED>
                                        <TARGETENTITY>
                                            <PRIVILEGENAME>READ</PRIVILEGENAME>
                                            <FILTERLIST>
                                                <FILTER>
                                                    <EXPRESSION>ARTWORKID</EXPRESSION>
                                                     <OPERATOR>EQUALTO</OPERATOR>
                                                    <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                    <FILTERID>1</FILTERID>
                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                </FILTER>
                                             </FILTERLIST>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>ARTWORKID</FIELDNAME>
                                                    <ALIAS>ARTWORKID</ALIAS>
                                                </EXPRESSION>
                                            </SELECTLIST>
                                            <RELATIONSHIP NAME='FORMAT_ARTWORK'>   
                                               <SELECTLIST>
                                                   <EXPRESSION>
                                                       <FIELDNAME>FORMATID</FIELDNAME>
                                                       <ALIAS>FORMATID</ALIAS>
                                                   </EXPRESSION>
                                               </SELECTLIST>                                                         
                                            </RELATIONSHIP>
                                        </TARGETENTITY>
                                  </SEARCH>", ArtworkID);
            OMInterface OMIObj = new OMInterface(UserID, Language, TenantName);
            String ErrorXML = OMIObj.GetHierarchialData(sbSearchXml.ToString(), ref ArtworkXML);

            if (ErrorXML == null)
            {
                FormatID = ALCommon.GetInnerValue(ref ArtworkXML, "<FORMATID>", "</FORMATID>");

                if (String.IsNullOrEmpty(FormatID))
                    return "Format ID could not be found.";
                else
                    return null;
            }
            else
                return ErrorXML;
        }

        public ERRORXML OpenIllustratorSession(String ArtworkID)
        {
            //String CachePath = ArtworkProducerCache.GetCacheFolderPath(TenantName, ArtworkID, ArtworkProducerCache.CacheType.ArtworkCache);
            StringBuilder sbInputXML = new StringBuilder();
            String OrgFilePath = ArtworkProducerCache.GetArtworkOriginalFilePath(TenantName, ArtworkID);
            String FileFormat = Path.GetExtension(OrgFilePath).Substring(1);
            sbInputXML.AppendFormat(@"  <FILECONVERTER>
                                            <INPUT FILENAME='{0}' FILEFORMAT='{1}' DOCACHING='YES' ILLUSTRATORSESSIONID='{2}' SENDRESPONSE='FALSE' />
                                        </FILECONVERTER>"
                                    , OrgFilePath, FileFormat, ArtworkID);
            MediaRequestRouter objRouter = new MediaRequestRouter(TenantName, UserID.ToString(), SessionID.ToString(), Language, SolutionName);
            ERRORXML ErrorXML = objRouter.SendRequestToMediaProcessor(sbInputXML.ToString(), "ILLUSTRATOR", null, null);
            return ErrorXML;
        }

        private ERRORXML CreateTextElementValueForTE(String FormatID, String ArtworkID)
        {
            StringBuilder sbSearchXml = null;
            StringBuilder sbUpdateXML = null;
            String FormatXML = null;
            ERRORXML ErrorXML = null;
            XPathDocument xpDoc_FormatXML = null;
            XPathNavigator xpNav_FormatXML = null;

            sbSearchXml = new StringBuilder(16);
            sbSearchXml.Append(@"<SEARCH>
                            <TARGETOBJECT>FORMAT</TARGETOBJECT>
                            <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                            <TARGETENTITY>
                                    <PRIVILEGENAME>READ</PRIVILEGENAME>
                                    <FILTERLIST>
                                        <FILTER>
                                            <EXPRESSION>FORMATID</EXPRESSION>
                                            <OPERATOR>EQUALTO</OPERATOR>
                                            <EXPRESSIONVALUE>" + FormatID + @"</EXPRESSIONVALUE>
                                            <FILTERID>1</FILTERID>
                                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                        </FILTER>
                                     </FILTERLIST>
                                     <SELECTLIST>
                                        <EXPRESSION>
                                            <FIELDNAME>FORMATID</FIELDNAME>
                                            <ALIAS>FORMATID</ALIAS>
                                        </EXPRESSION>                                       
                                    </SELECTLIST>                                     
                                    <RELATIONSHIP NAME='FORMAT_TEXTELEMENT'>
                                        <SELECTLIST>
                                            <EXPRESSION>
                                                <FIELDNAME>TEXTELEMENTID</FIELDNAME>
                                                <ALIAS>TEXTELEMENTID</ALIAS>
                                            </EXPRESSION>                                           
                                        </SELECTLIST>
                                        <FILTERLIST>
                                            <FILTER>
                                                <EXPRESSION>ISDEFAULTOPTION</EXPRESSION>
                                                <OPERATOR>EQUALTO</OPERATOR>
                                                <EXPRESSIONVALUE>1</EXPRESSIONVALUE>
                                                <FILTERID>1</FILTERID>
                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                <RELATIONSHIPNAME>TEXTELEMENT_OPTION</RELATIONSHIPNAME>
                                            </FILTER>
                                         </FILTERLIST>   
                                        <RELATIONSHIP NAME='TEXTELEMENT_OPTION'>
                                            <SELECTLIST>
                                                <EXPRESSION>
                                                    <FIELDNAME>OPTIONVALUE</FIELDNAME>
                                                    <ALIAS>OPTIONVALUE</ALIAS>
                                                </EXPRESSION>                                                    
                                            </SELECTLIST>
                                            <FILTERLIST>
                                                <FILTER>
                                                    <EXPRESSION>ISDEFAULTOPTION</EXPRESSION>
                                                    <OPERATOR>EQUALTO</OPERATOR>
                                                    <EXPRESSIONVALUE>1</EXPRESSIONVALUE>
                                                    <FILTERID>1</FILTERID>
                                                    <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                
                                                </FILTER>
                                            </FILTERLIST>                                                                                                                                              
                                        </RELATIONSHIP>                                                                                                     
                                    </RELATIONSHIP>                                     
                             </TARGETENTITY>
                          </SEARCH>");
            OMInterface OMIObj = new OMInterface(UserID, Language, TenantName);
            ErrorXML = OMIObj.GetHierarchialData(sbSearchXml.ToString(), ref FormatXML);
            if (ErrorXML == null)
            {
                sbUpdateXML = new StringBuilder(16);
                xpDoc_FormatXML = new XPathDocument(new StringReader(FormatXML));
                xpNav_FormatXML = xpDoc_FormatXML.CreateNavigator();
                if (xpNav_FormatXML != null)
                {
                    XPathNodeIterator xpItr_TE = xpNav_FormatXML.Select("FORMATCOLLECTION/FORMAT/FORMAT_TEXTELEMENT/TEXTELEMENT");
                    if (xpItr_TE.Count > 0)
                    {
                        //sbUpdateXML.AppendFormat(@"<TEXTELEMENTVALUECOLLECTION>");
                        while (xpItr_TE.MoveNext())
                        {
                            XPathNavigator xpNav_TEOption = xpItr_TE.Current.SelectSingleNode("TEXTELEMENT_OPTION/TEXTELEMENTOPTION/OPTIONVALUE");
                            if (xpNav_TEOption != null)
                            {
                                sbUpdateXML.Remove(0, sbUpdateXML.Length);
                                sbUpdateXML.AppendFormat(@"<TEXTELEMENTVALUE>
                                                                <TEXTELEMENTACTUALVALUE>{0}</TEXTELEMENTACTUALVALUE>
                                                                <ARTWORK_TEXTELEMENTVALUE TYPE='RELATION'>{1}</ARTWORK_TEXTELEMENTVALUE>
                                                                <TEXTELEMENT_TEXTELEMENTVALUE TYPE='RELATION'>{2}</TEXTELEMENT_TEXTELEMENTVALUE>
                                                            </TEXTELEMENTVALUE>", xpNav_TEOption.InnerXml, ArtworkID, xpItr_TE.Current.SelectSingleNode("TEXTELEMENTID").InnerXml);
                            }
                            String UpdateXML = sbUpdateXML.ToString();
                            ErrorXML = ArtworkProducerEngine.CreateAndUpdateData(ref UpdateXML, "CREATEDATA", UserID, Language, TenantName);
                        }
                        //sbUpdateXML.AppendFormat(@"</TEXTELEMENTVALUECOLLECTION>");                       
                    }
                }
            }
            return ErrorXML;
        }

        private ERRORXML GetArtworkLastModifier(String ArtworkID, out String LastModifierID)
        {
            LastModifierID = null;
            String SearchOP = null;
            OMInterface objOMI = new OMInterface(UserID, Language, TenantName);
            StringBuilder sbSearchXML = new StringBuilder();
            sbSearchXML.AppendFormat(@"<SEARCH>
                                    <TARGETOBJECT>USER</TARGETOBJECT>
                                    <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE> 
                                    <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                    <TARGETENTITY>
                                        <PRIVILEGENAME>READ</PRIVILEGENAME>
                                        <FILTERLIST>
                                            <FILTER>
                                                <EXPRESSION>ARTWORKID</EXPRESSION>
                                                 <OPERATOR>EQUALTO</OPERATOR>
                                                <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                <FILTERID>1</FILTERID>
                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>
                                                <RELATIONSHIPNAME>ARTWORK_MODIFIER</RELATIONSHIPNAME>
                                            </FILTER>
                                        </FILTERLIST>
                                        <SELECTLIST>
                                            <EXPRESSION>
                                                <FIELDNAME>USERID</FIELDNAME>
                                                <ALIAS>USERID</ALIAS>
                                            </EXPRESSION>
                                        </SELECTLIST>
                                     </TARGETENTITY>
                                  </SEARCH>", ArtworkID);
            ERRORXML ErrorXML = objOMI.GetHierarchialData(sbSearchXML.ToString(), ref SearchOP);
            LastModifierID = ALCommon.GetInnerValue(ref SearchOP, "<USERID>", "</USERID>");
            return ErrorXML;
        }

        private ERRORXML GenerateArtwork_Cache_Section_TextElements(String FormatID, ref XML ArtworkXML)
        {
            ERRORXML ErrorXML = null;
            ArtworkProducerCache objAPCache = new ArtworkProducerCache(UserID, Language, TenantName);
            String ArtworkID = ALCommon.GetInnerValue(ref ArtworkXML, "<ARTWORKID>", "</ARTWORKID>");

            objErrorCommon = new ALErrorCommon();

            #region Create and Validate format cache
            bool bretVal = objAPCache.ValidateFormatCache(FormatID);
            if (!bretVal)
            {
                ErrorXML = objAPCache.PrepareFormatCache(FormatID);
                if (ErrorXML != null)
                {
                    objErrorCommon.AppendError("APENGINE_ARTWORK_CREATEARTWORK_PREPARECACHE_FAILED", this.GetType().ToString(), "CreateArtwork", null, ErrorXML);
                    ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                    return ReturnErrorXML;
                }
            }
            #endregion

            #region Create Artwork cache
            ErrorXML = objAPCache.PrepareArtworkCache(ArtworkID, "CREATE");
            #endregion

            #region Create Artwork Section
            String tempArtworkName = null;
            String ArtworkName = ALCommon.GetInnerValue(ref ArtworkXML, "<ARTWORKNAME>", "</ARTWORKNAME>");
            ArtworkProducerCommon ArtCommonObj = new ArtworkProducerCommon(UserID, Language, TenantName);
            String SectionID = null;
            ArtCommonObj.Mode = ArtworkProducerCommon.ModeType.Generation;

            tempArtworkName = ArtworkName;
            ErrorXML = ArtworkProducerEngine.ConvertToAlphaNumericString(ref tempArtworkName);

            if (String.IsNullOrEmpty(ErrorXML))
                ErrorXML = ArtCommonObj.CreateSection(tempArtworkName + "_" + ArtworkID, null, out SectionID);
            else
                ErrorXML = ArtCommonObj.CreateSection(ArtworkName + "_" + ArtworkID, null, out SectionID);

            if (ErrorXML != null)
            {
                objErrorCommon.AppendError("APENGINE_ARTWORK_CREATEARTWORK_CREATESECTION_FAILED", this.GetType().ToString(), "CreateArtwork", null, ErrorXML);
                ErrorXML = objErrorCommon.ErrorCollectionXML;
            }

            StringBuilder sbAssoSectionArtwork = new StringBuilder(50);
            sbAssoSectionArtwork.AppendFormat(@"<ARTWORK><ARTWORKID>{1}</ARTWORKID><ARTWORK_SECTION TYPE='RELATION'>{0}</ARTWORK_SECTION></ARTWORK>", SectionID, ArtworkID);
            String UpdateXML = sbAssoSectionArtwork.ToString();
            ErrorXML = UpdateArtwork(ref UpdateXML, false);

            #endregion

            #region Get Format's TextElement Options
            ErrorXML = CreateTextElementValueForTE(FormatID, ArtworkID);
            if (ErrorXML != null)
            {
                objErrorCommon.AppendError("APENGINE_ARTWORK_CREATEARTWORK_FAILED", this.GetType().ToString(), "CreateArtwork", null, ErrorXML);
                ErrorXML = objErrorCommon.ErrorCollectionXML;
            }
            #endregion

            return ErrorXML;
        }

        internal String getExternalFileFromDB(String GEID)
        {
            String SearchOP = null;
            OMInterface objOMI = new OMInterface(UserID, Language, TenantName);
            StringBuilder sbSearchXML = new StringBuilder();
            String ExternalFilePath = null;

            sbSearchXML.AppendFormat(@"<SEARCH>
                                    <TARGETOBJECT>GRAPHICELEMENT</TARGETOBJECT>
                                    <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE> 
                                    <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                    <TARGETENTITY>
                                        <PRIVILEGENAME>READ</PRIVILEGENAME>
                                        <FILTERLIST>
                                            <FILTER>
                                                <EXPRESSION>GRAPHICELEMENTID</EXPRESSION>
                                                 <OPERATOR>EQUALTO</OPERATOR>
                                                <EXPRESSIONVALUE>{0}</EXPRESSIONVALUE>
                                                <FILTERID>1</FILTERID>
                                                <ISCASESENSITIVE>False</ISCASESENSITIVE>                                                
                                            </FILTER>
                                        </FILTERLIST>
                                        <SELECTLIST>
                                            <EXPRESSION>
                                                <FIELDNAME FileRequired='Yes'>ORIGINALFILE</FIELDNAME>
                                                <ALIAS>ORIGINALFILE</ALIAS>
                                            </EXPRESSION>
                                        </SELECTLIST>
                                     </TARGETENTITY>
                                  </SEARCH>", GEID);
            ERRORXML ErrorXML = objOMI.GetHierarchialData(sbSearchXML.ToString(), ref SearchOP);
            if (String.IsNullOrEmpty(ErrorXML))
                ExternalFilePath = ALCommon.GetInnerValue(ref SearchOP, "<PATH>", "</PATH>");

            return ExternalFilePath;
        }

        internal ERRORXML PopulateTextTagMappedToDatabaseField(String FormatID, String ArtworkID, XPathNavigator TextElementNav, out String TextElementValue)
        {
            StringBuilder sbSearch = new StringBuilder();
            OMInterface oOMI = null;
            XML ErrorXML = null;
            XPathNavigator TagXPathNav = null;
            String TagConfigXML = null;
            XPathDocument TagConfigXMLDoc = null;
            String SearchConfiguration = null;
            XPathNodeIterator FieldNodes = null;
            String TagValue = null;
            String EnitityXML = null;
            XPathDocument EnitityXMLDoc = null;
            String XPath = null;
            XPathNavigator TagValueNode = null;
            String TextElementID = null;
            StringBuilder TextElementValueXML = null;

            TextElementValue = null;
            try
            {
                TextElementValueXML = new StringBuilder(100);
                oOMI = new OMInterface(UserID, Language, TenantName);
                TextElementID = TextElementNav.SelectSingleNode("TEXTELEMENTID").Value;
                TagConfigXML = TextElementNav.SelectSingleNode("TAGCONFIGURATIONXML").InnerXml;
                if (!String.IsNullOrEmpty(TagConfigXML))
                {
                    TagConfigXMLDoc = new XPathDocument(new StringReader(TagConfigXML));
                    TagXPathNav = TagConfigXMLDoc.CreateNavigator();
                    SearchConfiguration = TagXPathNav.SelectSingleNode("TAGCONFIGURATION/DATASOURCE_CONFIG/DATABASEFIELD_CONFIG/SEARCHCONFIGURATION").InnerXml;
                    SearchConfiguration = SearchConfiguration.Replace("$ARTWORKID$", ArtworkID);

                    ErrorXML = oOMI.GetHierarchialData(SearchConfiguration, ref EnitityXML);
                    if (ErrorXML != null)
                        return ErrorXML;

                    EnitityXMLDoc = new XPathDocument(new StringReader(EnitityXML));
                    TagValue = null;
                    FieldNodes = TagXPathNav.Select("TAGCONFIGURATION/DATASOURCE_CONFIG/DATABASEFIELD_CONFIG/TAGVALUE_FIELD_MAPPING/FIELD");

                    //While loop is used because multiple field node with prefix and suffix may be required.
                    while (FieldNodes.MoveNext())
                    {
                        XPath = FieldNodes.Current.GetAttribute("XPath", "");
                        TagValueNode = EnitityXMLDoc.CreateNavigator().SelectSingleNode(XPath);
                        if (TagValueNode != null)
                        {
                            TagValue += TagValueNode.InnerXml;
                        }
                    }

                    TextElementValue = TagValue;
                }
                return null;
            }
            catch (Exception ExceptionObj)
            {
                throw ExceptionObj;
            }
        }

        private ERRORXML RemovePrimaryKeyNodes(String PrimaryKeyName, String RelationshipName, XmlDocument ObjectCollectionXMLDoc)
        {
            StringBuilder XPath = null;
            XmlNodeList TempRelationshipNodes = null;
            XmlNodeList PrimaryKeyNodes = null;
            ALEngineError ALEngineErrorLocalObj = null;

            try
            {
                XPath = new StringBuilder(15);
                XPath.AppendFormat("//{0}", RelationshipName);
                TempRelationshipNodes = ObjectCollectionXMLDoc.SelectNodes(XPath.ToString());

                // Remove ID columns
                foreach (XmlNode TempRelationshipNode in TempRelationshipNodes)
                {
                    ((XmlElement)TempRelationshipNode).SetAttribute("TYPE", "RELATION");
                    XPath.Remove(0, XPath.Length);
                    XPath.AppendFormat("*/{0}", PrimaryKeyName);
                    PrimaryKeyNodes = TempRelationshipNode.SelectNodes(XPath.ToString());
                    foreach (XmlNode PrimaryKeyNode in PrimaryKeyNodes)
                    {
                        PrimaryKeyNode.ParentNode.RemoveChild(PrimaryKeyNode);
                    }
                }
            }
            catch (Exception ExceptionObj)
            {
                ALEngineErrorLocalObj = new ALEngineError();
                ALEngineErrorLocalObj.AppendError("ARTWORKPRODUCERENGINE_REMOVING_PRIMARYKEY_NODES_FAILED", this.GetType().ToString(), "RemovePrimaryKeyNodes", ExceptionObj, null);
                return ALEngineErrorLocalObj.ErrorCollectionXML;
            }
            return null;
        }

        private int ComputeStringFrequency(string InputString, string SearchString)
        {
            int start;
            int at;
            int end;
            int count;
            int frequency = 0;

            end = InputString.Length;
            start = 0;
            count = 0;
            at = 0;

            while ((start <= end) && (at > -1))
            {
                // start+count must be a position within -str-.
                count = end - start;
                at = InputString.IndexOf(SearchString, start, count);
                if (at == -1)
                    break;
                else
                    frequency++;
                start = at + 1;
            }
            return frequency;
        }

        public ERRORXML GenerateReadyToUseArtwork(String ArtworkID, String JobType)
        {
            StringBuilder sbSearchInputXML = null;
            OMInterface OMIObj = null;
            String SearchOutputXML = null;
            ERRORXML ErrorXML = null;
            XPathDocument xpArtworkDoc = null;
            XPathNavigator xpArtworkNav = null;
            String ArtworkName = null;
            String SectionID = null;
            String AG_ConfigXML = null;
            XPathDocument xpDoc_GenerationConfig = null;
            XPathNavigator xpNav_ArtworkConfigRoot = null;
            XPathNodeIterator xpIter_Outputs = null;
            List<String> lstRelations = null;
            MediaJobEngine MJEngObj = null;
            StringBuilder CreateJobXML = null;
            String str_CreateJobXML = null;
            String FormatID = null;
            String JobID = null;
            ArtworkProducerCommon helperObj = null;
            String DisplayStatus = null;

            try
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine GenerateReadyToUseArtwork Starts");
                helperObj = new ArtworkProducerCommon(UserID, Language, TenantName, SolutionName);
                helperObj.GetDisplayStatus("Artwork", "Artwork Generation In Progress", out DisplayStatus);

                sbSearchInputXML = new StringBuilder(20);
                sbSearchInputXML.AppendFormat(@"<SEARCH>
                                                <TARGETOBJECT>ARTWORK</TARGETOBJECT>
                                                <TARGETOBJECTTYPE>ENTITY</TARGETOBJECTTYPE>
                                                <AUTHORIZATIONREQUIRED>FALSE</AUTHORIZATIONREQUIRED>
                                                <TARGETENTITY>
                                                    <SELECTLIST>
                                                        <EXPRESSION>
							                                <FIELDNAME>ARTWORKID</FIELDNAME>
							                                <ALIAS>ARTWORKID</ALIAS>
						                                </EXPRESSION>
						                                <EXPRESSION>
							                                <FIELDNAME>ARTWORKNAME</FIELDNAME>
							                                <ALIAS>ARTWORKNAME</ALIAS>
						                                </EXPRESSION>
                                                    </SELECTLIST>
                                                    <FILTERLIST>
							                            <FILTER>
								                            <EXPRESSION>ARTWORKID</EXPRESSION>
								                            <OPERATOR>EQUALTO</OPERATOR>
								                            <EXPRESSIONVALUE>{4}</EXPRESSIONVALUE>
								                            <ISCASESENSITIVE>False</ISCASESENSITIVE>
							                            </FILTER>
                                                    </FILTERLIST>
                                                    <RELATIONSHIP NAME='FORMAT_ARTWORK'>
                                                        <SELECTLIST>
								                            <EXPRESSION>
									                            <FIELDNAME>FORMATID</FIELDNAME>
									                            <ALIAS>FORMATID</ALIAS>
								                            </EXPRESSION>
								                            <EXPRESSION>
									                            <FIELDNAME FileRequired='Yes'>ORIGINALFILE</FIELDNAME>
									                            <ALIAS>ORIGINALFILE</ALIAS>
								                            </EXPRESSION>
								                            <EXPRESSION>
									                            <FIELDNAME FileRequired='Yes'>PREVIEWFILE</FIELDNAME>
									                            <ALIAS>PREVIEWFILE</ALIAS>
								                            </EXPRESSION>
								                            <EXPRESSION>
									                            <FIELDNAME FileRequired='Yes'>THUMBNAILFILE</FIELDNAME>
									                            <ALIAS>THUMBNAILFILE</ALIAS>
								                            </EXPRESSION>
								                            <EXPRESSION>
									                            <FIELDNAME>THUMBNAILACTUALHEIGHT</FIELDNAME>
									                            <ALIAS>THUMBNAILACTUALHEIGHT</ALIAS>
								                            </EXPRESSION>
								                            <EXPRESSION>
									                            <FIELDNAME>THUMBNAILACTUALWIDTH</FIELDNAME>
									                            <ALIAS>THUMBNAILACTUALWIDTH</ALIAS>
								                            </EXPRESSION>
							                            </SELECTLIST>
                                                    </RELATIONSHIP>
                                                </TARGETENTITY>
                                                <SESSIONINFO Language='{0}' TenantName='{1}' UserID='{2}' SessionID='{3}'></SESSIONINFO>
                                            </SEARCH>", Language, TenantName, UserID, SessionID, ArtworkID);

                OMIObj = new OMInterface(UserID, Language, TenantName);
                ErrorXML = OMIObj.GetHierarchialData(sbSearchInputXML.ToString(), ref SearchOutputXML);

                if (ErrorXML == null)
                {
                    xpArtworkDoc = new XPathDocument(new StringReader(SearchOutputXML));
                    xpArtworkNav = xpArtworkDoc.CreateNavigator();
                    xpArtworkNav = xpArtworkNav.SelectSingleNode("ARTWORKCOLLECTION/ARTWORK");
                    if (xpArtworkNav != null)
                    {
                        ArtworkName = xpArtworkNav.SelectSingleNode("ARTWORKNAME").Value;
                        FormatID = xpArtworkNav.SelectSingleNode("FORMAT_ARTWORK/FORMAT/FORMATID").Value;

                        //Stamp Job
                        MJEngObj = new MediaJobEngine(SessionID, UserID, Language, TenantName);
                        CreateJobXML = new StringBuilder();
                        CreateJobXML.AppendFormat(@"<MEDIAJOB>");
                        if (ArtworkName != null)
                            CreateJobXML.AppendFormat("<JOBNAME>{0}</JOBNAME>", ALSpecialCharHandling.TrimAndReplaceSpecialChars(ArtworkName) + " Job");
                        else
                            CreateJobXML.AppendFormat("<JOBNAME>{0}</JOBNAME>", " Job");

                        CreateJobXML.AppendFormat(@"<CREATEDON>
                                        <DAY>{0}</DAY>
                                        <MONTH>{1}</MONTH>
                                        <YEAR>{2}</YEAR>
                                        <HOUR>{3}</HOUR>
                                        <MINUTE>{4}</MINUTE>
                                        <SECOND>{5}</SECOND>
                                    </CREATEDON>
                                    <MODIFIEDON>
                                        <DAY>{0}</DAY>
                                        <MONTH>{1}</MONTH>
                                        <YEAR>{2}</YEAR>
                                        <HOUR>{3}</HOUR>
                                        <MINUTE>{4}</MINUTE>
                                        <SECOND>{5}</SECOND>
                                    </MODIFIEDON>
                                    <SUBMITTEDON>
                                        <DAY>{0}</DAY>
                                        <MONTH>{1}</MONTH>
                                        <YEAR>{2}</YEAR>
                                        <HOUR>{3}</HOUR>
                                        <MINUTE>{4}</MINUTE>
                                        <SECOND>{5}</SECOND>
                                    </SUBMITTEDON>
                                    <JOBTYPE>{8}</JOBTYPE>
                                    <JOBHANDLER>READYTOUSE</JOBHANDLER>
                                    <JOBSTATUS>New</JOBSTATUS>
                                    <MEDIAJOB_CREATOR TYPE='RELATION'>{6}</MEDIAJOB_CREATOR>
                                    <ARTWORK_MEDIAJOB TYPE='RELATION'>{7}</ARTWORK_MEDIAJOB>
                                    <ARTWORK_MEDIAJOB TYPE='RELATION'>
                                        <ARTWORK MODIFIED='TRUE'>
                                            <ARTWORKID>{7}</ARTWORKID>
                                            <ARTWORKSTATUS>Artwork Generation In Progress</ARTWORKSTATUS>
                                            <DISPLAYSTATUS>{9}</DISPLAYSTATUS>
                                        </ARTWORK>
                                    </ARTWORK_MEDIAJOB>
                                </MEDIAJOB>", DateTime.Now.Day, DateTime.Now.Month, DateTime.Now.Year, DateTime.Now.Hour,
                                                            DateTime.Now.Minute, DateTime.Now.Second, UserID, ArtworkID, JobType, DisplayStatus);

                        str_CreateJobXML = CreateJobXML.ToString();
                        ErrorXML = MJEngObj.StampMediaJob(FormatID, ArtworkID, SolutionName, ref str_CreateJobXML);

                        if (ErrorXML == null)
                        {
                            ArtworkProducerEngine.ConvertToAlphaNumericString(ref ArtworkName);
                            JobID = ALCommon.GetInnerValue(ref str_CreateJobXML, "<JOBID>", "</JOBID>");

                            ErrorXML = ArtworkProducerCommon.GetSectionForArtwork(UserID, Language, TenantName, ArtworkID, ArtworkName, out SectionID, JobType);
                            if (ErrorXML == null)
                            {
                                AG_ConfigXML = ALCommon.GetTenantConfigurationPath(TenantName) + "\\ArtworkGeneration_Config.xml";
                                xpDoc_GenerationConfig = new XPathDocument(AG_ConfigXML);
                                xpNav_ArtworkConfigRoot = xpDoc_GenerationConfig.CreateNavigator();
                                xpIter_Outputs = xpNav_ArtworkConfigRoot.Select("SOLUTIONLIST/SOLUTION[translate(@NAME,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='" + SolutionName.ToUpper() + "']/BUTTON/JOBTYPE [translate(@TYPE,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='" + JobType.ToUpper() + "']/FORMATTYPE [@TYPE='READYTOUSE']/OUTPUTLIST/OUTPUT");

                                lstRelations = new List<string>();
                                while (xpIter_Outputs.MoveNext())
                                    lstRelations.Add(xpIter_Outputs.Current.GetAttribute("RELATIONSHIPNAME", ""));

                                //Delete existing Assets
                                if (lstRelations.Count > 0)
                                    ArtworkProducerCommon.DeleteExistingAssets(UserID, TenantName, Language, ArtworkID, lstRelations);

                                xpIter_Outputs = xpNav_ArtworkConfigRoot.Select("SOLUTIONLIST/SOLUTION[translate(@NAME,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='" + SolutionName.ToUpper() + "']/BUTTON/JOBTYPE [translate(@TYPE,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')='" + JobType.ToUpper() + "']/FORMATTYPE [@TYPE='READYTOUSE']/OUTPUTLIST/OUTPUT");
                                while (xpIter_Outputs.MoveNext())
                                {
                                    //Create Asset
                                    XPathNavigator xpReditionTypeNav = null;
                                    XPathNavigator xpFileNav = null;
                                    XPathNavigator xpHeightWidthNav = null;
                                    String OrgFilePath = null;
                                    String OrgFileFormat = null;
                                    String PrvFilePath = null;
                                    String PrvFileFormat = null;
                                    String ThbFilePath = null;
                                    String ThbFileFormat = null;
                                    String DigitalAssetID = null;
                                    String ThbHeight = null;
                                    String ThbWidth = null;
                                    String FieldName = null;
                                    String FileName = null;
                                    String NewPath = null;

                                    xpReditionTypeNav = xpIter_Outputs.Current.SelectSingleNode("FORMAT [@RENDITIONTYPE='ORIGINAL']");
                                    if (xpReditionTypeNav != null)
                                        FieldName = xpReditionTypeNav.GetAttribute("FIELDNAME", "");
                                    else
                                        FieldName = "ORIGINALFILE";

                                    xpFileNav = xpArtworkNav.SelectSingleNode("FORMAT_ARTWORK/FORMAT/FILESYSTEM/FILE [FIELDNAME/text()='" + FieldName + "']");
                                    if (xpFileNav != null)
                                    {
                                        OrgFilePath = xpFileNav.SelectSingleNode("PATH").Value;
                                        FileName = Path.GetFileNameWithoutExtension(OrgFilePath);
                                        NewPath = OrgFilePath.Replace(FileName, ArtworkName + "_" + xpIter_Outputs.Current.GetAttribute("QUALITY", ""));
                                        File.Copy(OrgFilePath, NewPath, true);
                                        OrgFilePath = NewPath;
                                        OrgFileFormat = Path.GetExtension(OrgFilePath);
                                        OrgFileFormat = OrgFileFormat.Replace(".", "");
                                    }

                                    xpReditionTypeNav = xpIter_Outputs.Current.SelectSingleNode("FORMAT [@RENDITIONTYPE='PREVIEW']");
                                    if (xpReditionTypeNav != null)
                                        FieldName = xpReditionTypeNav.GetAttribute("FIELDNAME", "");
                                    else
                                        FieldName = "PREVIEWFILE";

                                    xpFileNav = xpArtworkNav.SelectSingleNode("FORMAT_ARTWORK/FORMAT/FILESYSTEM/FILE [FIELDNAME/text()='" + FieldName + "']");
                                    if (xpFileNav != null)
                                    {
                                        PrvFilePath = xpFileNav.SelectSingleNode("PATH").Value;
                                        FileName = Path.GetFileNameWithoutExtension(PrvFilePath);
                                        NewPath = PrvFilePath.Replace(FileName, ArtworkName + "_PRV_" + xpIter_Outputs.Current.GetAttribute("QUALITY", ""));
                                        File.Copy(PrvFilePath, NewPath, true);
                                        PrvFilePath = NewPath;
                                        PrvFileFormat = Path.GetExtension(PrvFilePath);
                                        PrvFileFormat = PrvFileFormat.Replace(".", "");
                                    }

                                    xpReditionTypeNav = xpIter_Outputs.Current.SelectSingleNode("FORMAT [@RENDITIONTYPE='THUMBNAIL']");
                                    if (xpReditionTypeNav != null)
                                        FieldName = xpReditionTypeNav.GetAttribute("FIELDNAME", "");
                                    else
                                        FieldName = "THUMBNAILFILE";

                                    xpFileNav = xpArtworkNav.SelectSingleNode("FORMAT_ARTWORK/FORMAT/FILESYSTEM/FILE [FIELDNAME/text()='" + FieldName + "']");
                                    if (xpFileNav != null)
                                    {
                                        ThbFilePath = xpFileNav.SelectSingleNode("PATH").Value;
                                        FileName = Path.GetFileNameWithoutExtension(ThbFilePath);
                                        NewPath = ThbFilePath.Replace(FileName, ArtworkName + "_THB_" + xpIter_Outputs.Current.GetAttribute("QUALITY", ""));
                                        File.Copy(ThbFilePath, NewPath, true);
                                        ThbFilePath = NewPath;
                                        ThbFileFormat = Path.GetExtension(ThbFilePath);
                                        ThbFileFormat = ThbFileFormat.Replace(".", "");
                                    }

                                    xpHeightWidthNav = xpArtworkNav.SelectSingleNode("FORMAT_ARTWORK/FORMAT/THUMBNAILACTUALHEIGHT");
                                    if (xpHeightWidthNav != null)
                                        ThbHeight = xpHeightWidthNav.Value;

                                    xpHeightWidthNav = xpArtworkNav.SelectSingleNode("FORMAT_ARTWORK/FORMAT/THUMBNAILACTUALWIDTH");
                                    if (xpHeightWidthNav != null)
                                        ThbWidth = xpHeightWidthNav.Value;

                                    ErrorXML = CreateDigitalAsset(OrgFilePath, OrgFileFormat, PrvFilePath, PrvFileFormat, ThbFilePath, ThbFileFormat, ThbHeight, ThbWidth, SectionID, ArtworkID, xpIter_Outputs.Current.GetAttribute("RELATIONSHIPNAME", ""), "", ref DigitalAssetID);
                                    if (ErrorXML != null)
                                        break;
                                }
                            }
                        }
                    }
                }

                if (ErrorXML == null)
                {
                    //Done
                    if (String.Compare(JobType, "ARTWORK GENERATION JOB", true) == 0)
                    {
                        ArtworkProducerCommon.UpdateArtworkStatus(UserID, Language, TenantName, SolutionName, "Artwork Generated", ArtworkID);
                    }
                    else if (String.Compare(JobType, "PREVIEW GENERATION JOB", true) == 0)
                    {
                        ArtworkProducerCommon.UpdateArtworkStatus(UserID, Language, TenantName, SolutionName, "Draft", ArtworkID);
                    }
                    else
                    {
                        ArtworkProducerCommon.UpdateArtworkStatus(UserID, Language, TenantName, SolutionName, "Proof Generated", ArtworkID);
                    }
                    ArtworkProducerCommon.UpdateJobStatus(UserID, Language, TenantName, SessionID, "Done", JobID);
                }
                else
                {
                    //Error
                    ArtworkProducerCommon.UpdateArtworkStatus(UserID, Language, TenantName, SolutionName, "Error", ArtworkID);
                    ArtworkProducerCommon.UpdateJobStatus(UserID, Language, TenantName, SessionID, "Error", JobID);
                }
            }
            catch (Exception ExcepObj)
            {
                objErrorCommon.AppendError("APENGINE_ARTWORK_GENERATEREADYTOUSEARTWORK_EXCEPTION", this.GetType().ToString(), "GenerateReadyToUseArtwork", ExcepObj, ReturnErrorXML);
                ReturnErrorXML = objErrorCommon.ErrorCollectionXML;
                return ReturnErrorXML;
            }
            finally
            {
                ALCommon.WriteLog(TenantName, "ArtworkProducer", "Artwork.cs", SessionID.ToString(), "Engine GenerateReadyToUseArtwork Ends");
            }
            return ErrorXML;
        }

        public ERRORXML PrepareArtworkCache(String ArtworkID)
        {
            ERRORXML ErrorXML = null;
            String FormatID = null;

            GetFormatIDForArtwork(ArtworkID, out FormatID);
            ArtworkProducerCache objAPCache = new ArtworkProducerCache(UserID, Language, TenantName);

            bool bretVal = objAPCache.ValidateFormatCache(FormatID);
            if (bretVal)
            {
                bretVal = objAPCache.ValidateArtworkCacheWithFormatCache(ArtworkID, FormatID);
                if (!bretVal)
                    ErrorXML = objAPCache.PrepareArtworkCache(ArtworkID, "EDIT");
            }
            else
            {
                objAPCache.PrepareFormatCache(FormatID);
                ErrorXML = objAPCache.PrepareArtworkCache(ArtworkID, "EDIT");
            }

            return ErrorXML;
        }
    }
}